Type: Package
Package: BayesSIM
Title: Integrated Interface of Bayesian Single Index Models using
        'nimble'
Version: 1.0.4
Authors@R: c(
    person("Seowoo", "Jung", , "jsw1347@ewha.ac.kr", role = c("aut", "cre")),
    person("Eun-kyung", "Lee", , "lee.eunk@ewha.ac.kr", role = "aut")
  )
Author: Seowoo Jung [aut, cre],
  Eun-kyung Lee [aut]
Maintainer: Seowoo Jung <jsw1347@ewha.ac.kr>
Description: Provides tools for fitting Bayesian single index models with
    flexible choices of priors for both the index and the link function.
    The package implements model estimation and posterior inference using
    efficient MCMC algorithms built on the 'nimble' framework, allowing
    users to specify, extend, and simulate models in a unified and
    reproducible manner. The following methods are implemented in the
    package: Antoniadis et al. (2004)
    <https://www.jstor.org/stable/24307224>, Wang (2009)
    <doi:10.1016/j.csda.2008.12.010>, Choi et al. (2011)
    <doi:10.1080/10485251003768019>, Dhara et al. (2019)
    <doi:10.1214/19-BA1170>, McGee et al. (2023) <doi:10.1111/biom.13569>.
License: GPL (>= 2)
Depends: R (>= 4.1), nimble
Imports: coda, magrittr, MASS, methods, mvtnorm, patchwork, tidyr,
        ggplot2, dplyr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-11 09:01:43 UTC; Seowoo
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Repository: CRAN
Date/Publication: 2026-05-14 02:40:02 UTC
Built: R 4.6.0; ; 2026-05-14 05:29:12 UTC; unix

Type: Package
Package: BayesSIM
Title: Integrated Interface of Bayesian Single Index Models using
        'nimble'
Version: 1.0.4
Authors@R: c(
    person("Seowoo", "Jung", , "jsw1347@ewha.ac.kr", role = c("aut", "cre")),
    person("Eun-kyung", "Lee", , "lee.eunk@ewha.ac.kr", role = "aut")
  )
Author: Seowoo Jung [aut, cre],
  Eun-kyung Lee [aut]
Maintainer: Seowoo Jung <jsw1347@ewha.ac.kr>
Description: Provides tools for fitting Bayesian single index models with
    flexible choices of priors for both the index and the link function.
    The package implements model estimation and posterior inference using
    efficient MCMC algorithms built on the 'nimble' framework, allowing
    users to specify, extend, and simulate models in a unified and
    reproducible manner. The following methods are implemented in the
    package: Antoniadis et al. (2004)
    <https://www.jstor.org/stable/24307224>, Wang (2009)
    <doi:10.1016/j.csda.2008.12.010>, Choi et al. (2011)
    <doi:10.1080/10485251003768019>, Dhara et al. (2019)
    <doi:10.1214/19-BA1170>, McGee et al. (2023) <doi:10.1111/biom.13569>.
License: GPL (>= 2)
Depends: R (>= 4.1), nimble
Imports: coda, magrittr, MASS, methods, mvtnorm, patchwork, tidyr,
        ggplot2, dplyr
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-11 09:01:43 UTC; Seowoo
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Repository: CRAN
Date/Publication: 2026-05-14 02:40:02 UTC
Built: R 4.5.2; ; 2026-05-14 03:58:44 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(GOF,bsim)
S3method(as.data.frame,bsimPred)
S3method(as_bsim,bsimSetup)
S3method(coef,bsim)
S3method(fitted,bsim)
S3method(plot,bsim)
S3method(plot,bsimPred)
S3method(plot,bsimSetup)
S3method(predict,bsim)
S3method(print,bsim)
S3method(print,bsimPred)
S3method(print,bsimSetup)
S3method(print,summary.bsim)
S3method(print,summary.bsimPred)
S3method(print,summary.bsimSetup)
S3method(residuals,bsim)
S3method(summary,bsim)
S3method(summary,bsimPred)
S3method(summary,bsimSetup)
export(BayesSIM)
export(BayesSIM_setup)
export(GOF)
export(as_bsim)
export(bsFisher)
export(bsFisher_setup)
export(bsPolar)
export(bsPolar_setup)
export(bsSphere)
export(bsSphere_setup)
export(bsSpike)
export(bsSpike_setup)
export(compileModelAndMCMC)
export(getInit)
export(getModel)
export(getModelDef)
export(getSampler)
export(getVarMonitor)
export(gpFisher)
export(gpFisher_setup)
export(gpPolar)
export(gpPolarHigh)
export(gpPolarHigh_setup)
export(gpPolar_setup)
export(gpSphere)
export(gpSphere_setup)
export(gpSpike)
export(gpSpike_setup)
export(init_param)
export(nimTraceplot)
export(prior_param)
import(ggplot2)
import(methods)
import(nimble)
import(patchwork)
importFrom(MASS,mvrnorm)
importFrom(dplyr,select)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(magrittr,"%>%")
importFrom(mvtnorm,rmvnorm)
importFrom(stats,dbeta)
importFrom(stats,dnorm)
importFrom(stats,knots)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,ppr)
importFrom(stats,qnorm)
importFrom(stats,qpois)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(tidyr,all_of)
importFrom(tidyr,pivot_longer)
importFrom(utils,modifyList)
importFrom(utils,str)

#' Bayesian Single-Index Regression with Gaussian Process Link and von Mises-Fisher Prior
#'
#' @description
#' Fits a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}, where
#' the index \eqn{\theta} lies on the unit sphere with von Mises-Fisher prior, and the link \eqn{f(\cdot)} is represented
#' with Gaussian process.
#'
#' @inheritParams bsFisher
#'
#' @details
#' \strong{Model} The single-index model uses Gaussian process with zero mean and and covariance kernel \eqn{\eta \cdot \text{exp}(-\frac{(t_i-t_j)^2}{l})} as a link function, where \eqn{t_i, t_j, j = 1, \ldots, n} is index value.
#' Index vector should be length 1.
#'
#' \strong{Priors}
#' \itemize{
#' \item von Mises–Fisher prior on the index \eqn{\theta} with direction and concentration.
#' \item Covariance kernel: Amplitude, \eqn{\eta}, follows log normal distribution with mean \eqn{a_\eta} and variance \eqn{b_\eta}.
#' Length-scale parameter follows gamma distribution with shape parameter \eqn{\alpha_l} and rate parameter \eqn{\beta_l}.
#' \item Inverse gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.
#'
#' }
#'
#' \strong{Sampling} All sampling parameters' samplers are assigned by nimble.
#'
#' \strong{Prior hyper-parameters}
#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
#' \enumerate{
#'   \item Index vector: von Mises--Fisher prior for the projection vector \eqn{\theta} (\code{index}).
#'         \code{index_direction} is a unit direction vector of the von Mises--Fisher distribution.
#'         By default, the estimated vector from projection pursuit regression is assigned.
#'         \code{index_dispersion} is the positive concentration parameter. By default, \code{150} is assigned.
#'
#' \item Link function:
#' \itemize{
#'    \item{Length-scale:Gamma distribution is assigned for length-scale parameter, \eqn{l}.
#'     \code{link_lengthscale_shape} is shape parameter (default \code{1/8}) and \code{link_lengthscale_rate} is rate parameter of \code{lengthscale}. (default \code{1/8})}
#'     \item{Amplitude: Log-normal distribution is assigned for amplitude parameter, \eqn{\eta}.
#'      \code{link_amp_a} is mean (default \code{-1}), and \code{link_amp_b} is variance. (default \code{1})}
#' }
#'
#' \item Error variance (\code{sigma2}): An inverse-gamma prior is assigned to \eqn{\sigma^2}
#'         where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
#'         (default \code{sigma2_shape = 1, sigma2_rate = 1})
#'         }
#'
#'
#' \strong{Initial values}
#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
#' \enumerate{
#'       \item Index vector (\code{index}): Initial unit index vector \eqn{\theta}. By default, the vector is sampled from the von Mises--Fisher prior.
#'      \item Link function: \code{link_lengthscale} is initial scalar length-scale parameter. (default: \code{0.1})
#'      \code{link_amp} is initial scalar amplitude parameter. (default: \code{1})
#'      \item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{1})
#' }
#'
#' @inherit bsFisher return
#'
#' @examples
#' \donttest{
#' set.seed(123)
#' N <- 60; p <- 2
#' x1 <- runif(N, -3, 5)
#' x2 <- runif(N, -3, 5)
#' beta1 <- 0.45; beta2 <- sqrt(1-beta1^2)
#' sigma <- sqrt(0.0036)
#' xlin <- x1*beta1 + x2*beta2
#' eta <- 0.1*xlin + sin(0.5*xlin)^2
#' y <- rnorm(N, eta, sigma)
#' x <- matrix(c(x1, x2), ncol = 2)
#' simdata <- data.frame(x = x, y = y)
#' colnames(simdata) <- c("X1", "X2", "y")
#'
#' # One tool version
#' fit1 <- gpFisher(y ~ ., data = simdata, nchain = 1, niter = 1000, nburnin = 100)
#'
#' # Split version
#' models <- gpFisher_setup(y ~ ., data = simdata, nchain = 1)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 1000, nburnin = 100, thin = 1,
#'                    nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' summary(fit2)
#' }
#'
#' @references
#'
#' Antoniadis, A., Grégoire, G., & McKeague, I. W. (2004).
#' Bayesian estimation in single-index models. \emph{Statistica Sinica}, 1147-1164.
#'
#' Choi, T., Shi, J. Q., & Wang, B. (2011).
#' A Gaussian process regression approach to a single-index model.
#' \emph{Journal of Nonparametric Statistics}, 23(1), 21-36.
#'
#' Hornik, K., & Grün, B. (2014). movMF: an R package for fitting mixtures of von Mises-Fisher distributions.
#' \emph{Journal of Statistical Software}, 58, 1-31.
#'
#' @name gpFisher
#' @export
gpFisher <- function(formula, data,
                     prior = NULL,
                     init = NULL,
                     monitors = NULL, niter = 10000, nburnin=1000,
                     thin = 1, nchain = 1, setSeed = FALSE){
  return(gpFisher.default(formula = formula, data = data,
                          prior = prior,
                          init = init, sampling = TRUE,
                          monitors = monitors, niter = niter, nburnin = nburnin,
                          thin = thin, nchain = nchain, setSeed = setSeed))
}

#' @rdname gpFisher
#' @export
gpFisher_setup <- function(formula, data,
                           prior = NULL,
                           init = NULL,
                           monitors = NULL, niter = 10000, nburnin=1000,
                           thin = 1, nchain = 1, setSeed = FALSE){
  return(gpFisher.default(formula = formula, data = data,
                          prior = prior,
                          init = init, sampling = FALSE,
                          monitors = monitors, niter = niter, nburnin = nburnin,
                          thin = thin, nchain = nchain, setSeed = setSeed))
}

gpFisher.default <- function(formula, data,
                             prior = NULL,
                             init = NULL,
                             sampling = TRUE, monitors = NULL, niter = 10000, nburnin=1000,
                             thin = 1, nchain = 1, setSeed = FALSE){
  start1 <- Sys.time()
  index_temp <- 0; log_amp <- 0; lengthscale <- 0; sigma2 <- 0; Id <- 0
  amp <- 0

  # check sampling, prior, init parameters for independent execution
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin, nchain,
    prior, init, "fisher", "gp"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  # environment
  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))

  .fns <- c(
    "nimNorm","postll_bspline_fisher","nimNorm","rW","besselI_nimble","Stheta",
   "estBeta_fisher","gvcCV",
    # gpSphere
    "expcov_gpSphere","expcovTest_gpSphere","sigma2Sampler_gpSphere",

   # distribution
   "dvMFnim", "rvMFnim",

    # utils
    "pred_fitted"
  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())

  suppressMessages(
    nimble::registerDistributions(list(
      dvMFnim = list(
        BUGSdist = "dvMFnim(theta)",
        types    = c("value = double(1)", "theta = double(1)"),
        discrete = FALSE
      )
    ), verbose = FALSE)
  )


  # check data dimension
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
           length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  }else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  # data dimension
  N <- length(Y)
  p <- ncol(X)

  # model code
  modelCode <- nimbleCode({
    # index
    index_temp[1:p] ~ dvMFnim(theta = theta_prior[1:p])
    index[1:p] <- index_temp[1:p]/sqrt(sum(index_temp[1:p]^2))

    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i,1:p] * index[1:p])
    }

    # hyperprior
    lengthscale ~ dgamma(shape, rate)
    amp ~ dlnorm(a_amp, b_amp)

    # linkFunction
    cov[1:N, 1:N] <- expcov_gpSphere(Xlin[1:N], lengthscale, amp)
    linkFunction[1:N] ~ dmnorm(mu0[1:N], cov = cov[1:N, 1:N])

    # likelihood - gibbs
    sigma2 ~ dinvgamma(a_sig, b_sig)
    Sigma[1:N,1:N] <- sigma2 * Id[1:N,1:N]
    Y[1:N] ~ dmnorm(linkFunction[1:N], cov = Sigma[1:N,1:N])
  })

  # Check hyper-parameters
  ## index
  if (length(prior$index$dispersion) >= 2|| !is.numeric(prior$index$dispersion)){
    stop("Dispersion of index vector should be scalar.")
  }
  if (prior$index$dispersion < 0){
    stop("Dispersion of index vector should be postive")
  }

  dispersion_prior <- prior$index$dispersion

  if (!is.null(prior$index$direction) & length(prior$index$direction) != (p)){
    stop("Prior index vector has incorrect dimension.")
  }
  if (!is.null(prior$index$direction) & sum((prior$index$direction)^2) != 1){
    stop("Prior index vector should be unit vector.")
  }

  if (is.null(prior$index$direction)){
    # index_direct_input <- c(rep(0, p-1), 1)
    index_direct_input <- as.vector(ppr(X, Y, nterms = 1)$alpha)
    index_direct_input <- index_direct_input/sqrt(sum(index_direct_input^2))
  } else{
    index_direct_input <- prior$index$direction
  }
  index_direction <- index_direct_input * dispersion_prior


  # lengthscale
  ## shape
  if (is.null(prior$link$lengthscale$shape)||
      length(prior$link$lengthscale$shape) >= 2 ||
      prior$link$lengthscale$shape < 0){
    stop("Prior lengthscale (shape) has incorrect value.")
  } else{
    shape <- prior$link$lengthscale$shape
  }

  ## rate
  if (is.null(prior$link$lengthscale$rate)||
      length(prior$link$lengthscale$rate) >= 2 ||
      prior$link$lengthscale$rate < 0){
    stop("Prior lengthscale (rate) has incorrect value.")
  } else{
    rate <- prior$link$lengthscale$rate
  }

  # amp - log normal
  ## a_amp: mean
  if (is.null(prior$link$amp$a_amp)||
      length(prior$link$amp$a_amp) >= 2 ){
    stop("Prior amp (a_amp) has incorrect value.")
  } else{
    a_amp <- prior$link$amp$a_amp
  }

  ## b_amp: sd
  if (is.null(prior$link$amp$b_amp)||
      length(prior$link$amp$b_amp) >= 2 ||
      prior$link$amp$b_amp < 0){
    stop("Prior amp (b_amp) has incorrect value.")
  } else{
    b_amp <- prior$link$amp$b_amp
  }

  # sigma2: inverse gamma
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (shape) has incorrect value.")
  } else{
    a_sig <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (rate) has incorrect value.")
  } else{
    b_sig <- prior$sigma2$rate
  }

  # initialize - function

  # seed
  seedNum <- rep(FALSE, nchain)
  seedNum <- rep(FALSE, nchain)

  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }

  inits_list <- lapply(seq_len(nchain),
                       function(j) initFunction_gpFisher(X = X, Y = as.vector(Y),
                                                                        index = init$index,
                                                                        lengthscale = init$link$lengthscale,
                                                                        amp  = init$link$amp,
                                                                        sigma2  = init$sigma2,
                                                                        setSeed = seedNum[j]))
  firstInit <- inits_list[[1]]

  message("== Building model ==")
  suppressMessages(simpleModel <- nimbleModel(modelCode,
                             data = list(X = X, Y = as.vector(Y)),
                             constants = list(theta_prior = index_direction,
                                              shape = shape, rate = rate,
                                              a_sig = a_sig, b_sig = b_sig,
                                              a_amp = a_amp, b_amp = b_amp,
                                              p = p, N = N,
                                              Id = diag(1, N), mu0 = rep(0, N)),
                             inits = firstInit))

  # sampling
  monitorsList <- c("index", "sigma2", "linkFunction", "Xlin", "lengthscale", "amp",
                    "index_temp")

  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
                                             monitors = monitorsList,
                                             print = FALSE))

  # mcmcConf$removeSamplers('lengthscale')
  # mcmcConf$addSampler(target = "lengthscale", type = "RW",
  #                     control = list(log = TRUE))
  #
  mcmcConf$removeSamplers('sigma2')
  mcmcConf$addSampler(target = "sigma2",
                      type = sigma2Sampler_gpSphere)
  #
  # mcmcConf$setSamplerExecutionOrder(c(2, 1, 4, 3, 5))

  message("== Assigning samplers ==")
  mcmc1 <- buildMCMC(mcmcConf)


  if (!sampling){
    samples <- NULL

  } else{
    start2 <- Sys.time()
    message("== Compiling model ==")
    suppressMessages(CModel <- compileNimble(simpleModel,
                                             resetFunctions = TRUE))

    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1))
    end2 <- Sys.time()

    message("== Running MCMC ==")
    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                        thin = thin,
                        nchains = nchain, setSeed = seedNum, inits = inits_list,
                        summary = FALSE, samplesAsCodaMCMC = TRUE)
    # sampMCMC <- mcmc.out
    samples <- sampleBind(mcmc.out, nchain)


  }
  end1 <- Sys.time()

  # inputOptions <- NULL
  if (!sampling){
    time <- NULL
  } else{
    time <- difftime(end1, start1, units = "secs") - difftime(end2, start2, units = "secs")
  }

  # Results ------------------------------------------------------------------
  # Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  # input options
  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = list(direction = index_direct_input, dispersion = dispersion_prior),
                                    link= list(lengthscale = list(shape = shape, rate = rate),
                                               amp = list(a_amp = a_amp, b_amp = b_amp)),
                                    sigma2 = list(shape = a_sig, rate = b_sig)),
                       init = inits_list,
                       samplingOptions = list(sampling = sampling,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       time = time)

  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = mcmc.out, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "gpFisher")

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "gpFisher")

    class(out) <- "bsimSetup"

  }
  return(out)



}

#' @importFrom stats dbeta dnorm median knots pnorm ppr qnorm qpois quantile
#' @importFrom stats rbeta rbinom rgamma rnorm rpois runif sd
#' @importFrom MASS mvrnorm

# Sorting algorithm: quick sort
quickSortOrderIndexOnly <- nimble::nimbleFunction(
  run = function(x = double(1), decreasing = logical(0, default = FALSE)) {
    n <- length(x)
    idx <- integer(n)
    for(i in 1:n) idx[i] <- i  # initial index

    # stack
    stackLeft <- integer(n)
    stackRight <- integer(n)
    top <- 1
    stackLeft[top] <- 1
    stackRight[top] <- n

    while(top > 0) {
      left <- stackLeft[top]
      right <- stackRight[top]
      top <- top - 1

      if(left < right) {
        pivotIdx <- floor((left + right) / 2)
        pivotVal <- x[idx[pivotIdx]]
        i <- left
        j <- right

        if(decreasing) {
          while(i <= j) {
            while(x[idx[i]] > pivotVal) i <- i + 1
            while(x[idx[j]] < pivotVal) j <- j - 1
            if(i <= j) {
              tmp <- idx[i]; idx[i] <- idx[j]; idx[j] <- tmp
              i <- i + 1; j <- j - 1
            }
          }
        } else {
          while(i <= j) {
            while(x[idx[i]] < pivotVal) i <- i + 1
            while(x[idx[j]] > pivotVal) j <- j - 1
            if(i <= j) {
              tmp <- idx[i]; idx[i] <- idx[j]; idx[j] <- tmp
              i <- i + 1; j <- j - 1
            }
          }
        }

        if(left < j) {
          top <- top + 1
          stackLeft[top] <- left
          stackRight[top] <- j
        }
        if(i < right) {
          top <- top + 1
          stackLeft[top] <- i
          stackRight[top] <- right
        }
      }
    }

    returnType(integer(1))
    return(idx)
  }
)

nimOrder <- nimble::nimbleFunction(
  run = function(x = double(1), decreasing = logical(0, default = FALSE)) {
    returnType(integer(1))
    return(quickSortOrderIndexOnly(x, decreasing))
  }
)


nimSort <- nimble::nimbleFunction(
  run = function(x = double(1), decreasing = logical(0, default = FALSE)) {
    idx <- nimOrder(x, decreasing)
    out <- numeric(length(x))
    for(i in 1:length(x)) {
      out[i] <- x[idx[i]]
    }
    returnType(double(1))
    return(out)
  }
)


# Quantile function
sampleQuantile_nim <- nimble::nimbleFunction(
  run = function(x   = double(1),
                 prob= double(1)) {
    returnType(double(1))
    n   <- nimDim(x)[1]
    sortx <- nimSort(x)
    np  <- nimDim(prob)[1]
    out <- numeric(np)
    for(p in 1:np) {
      k <- ceiling(prob[p] * (n + 1))
      if(k < 1)        k <- 1
      if(k > n)        k <- n
      out[p] <- sortx[k]
    }
    return(out)
  }
)



quantile_nimble <- nimble::nimbleFunction(
  run = function(x   = double(1),
                 prob= double(1)) {
    returnType(double(1))

    n   <- nimDim(x)[1]
    sortx <- nimSort(x)
    np  <- nimDim(prob)[1]
    out <- numeric(np)

    for (j in 1:np){

      h  <- (n - 1) * prob[j]
      i  <- floor(h)          # 0-based
      a  <- h - i             # alpha

      x_left  <- sortx[i + 1]
      x_right <- sortx[i + 2]

      if (prob[j] == 0){
        out[j] <- sortx[1]}
      else if (prob[j] == 1) {
        out[j] <- sortx[n]}
      else{
        out[j] <- (1 - a) * x_left + a * x_right
      }
    }

    return(out)
  }
)

#' Bayesian Single-Index Regression with Gaussian Process Link and Spike-and-Slab Prior
#'
#' @description Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n},
#' where index vector \eqn{\theta} has a spike and slab prior and
#' the link \eqn{f(\cdot)} is represented by Gaussian process and the
#'
#' @inheritParams bsFisher
#'
#' @details
#' \strong{Model} The single–index model is specified as \eqn{Y_i = f(X_i' \theta) + \epsilon_i},
#' where \eqn{\theta} is a p-dimensional index vector subject to a spike-and-slab
#' prior for variable selection. The link function \eqn{f(\cdot)} is modeled
#' using a Gaussian process prior with zero mean and squared exponential covariance
#' kernel \eqn{K(x_1, x_2) = \exp\{-\rho {(x_1 - x_2)^{T}\theta}^2\}},
#' where \eqn{\rho} determines the smoothness of \eqn{f}.
#' The covariance kernel is re-parameterized to \eqn{\exp\{-{(x_1 - x_2)^{T}\theta^{*}}^2\}} where
#' \eqn{\rho = ||\theta^{*}||} and
#' \eqn{\theta = ||\theta||^{-1}\theta^{*}}.
#' Therefore, \eqn{\theta^{*}} is sampled in MCMC.
#'
#' \strong{Priors}
#' \itemize{
#' \item The variable selection indicator \eqn{\nu} has Beta–Bernoulli hierarchy prior. Beta hyper-prior on the Bernoulli–inclusion probability \eqn{w},
#'     inducing \eqn{p(\nu) \propto \mathrm{Beta}(r_1 + n_\nu, r_2 + p - n_\nu)} where  \eqn{n_\nu = \Sigma_{i=1}^{p}I(\nu_i = 1)}.
#'     \eqn{r_1, r_2} are shape and rate parameter of beta distribution.
#' \item Slab coefficients \eqn{\theta} have normal distribution with zero mean and \eqn{\sigma_\theta^2} variance.
#'   \item GP precision \eqn{\lambda^{-1}} follows gamma distribution with shape parameter \eqn{a_\lambda}, and rate parameter \eqn{b_\lambda}.
#'   \item Error precision \eqn{(\sigma^2)^{-1}} follows gamma distribution with shape parameter \eqn{a_\sigma}, and rate parameter \eqn{b_\sigma}.
#' }
#'
#' \strong{Sampling} A random walk Metropolis algorithm is used to sample \eqn{\lambda^{-1}}
#' and a Metropolis-Hastings algorithm is used for the main parameters \eqn{(\theta^{*}, \nu)}.
#' The variance \eqn{\sigma^2} is directly sampled from posterior distribution.
#' \eqn{f} is not directly sampled by MCMC.
#'
#' \strong{Prior hyper-parameters}
#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
#' \enumerate{
#' \item Index vector: \code{index_r1, index_r2} gives the shape and rate parameter of beta-binomial prior, respectively.
#'     For slab prior, normal distribution with zero mean is assigned for selected variables \eqn{\theta}. \code{index_sigma_theta} is for variance of \eqn{\theta}, and it is assigned 0.25 by default.
#'     \item Link function: Inverse gamma prior is assigned for hyper-parameters \eqn{\lambda^{-1}}
#'     `link_inv_lambda_shape` is shape parameter and `link_inv_lambda_rate` is rate parameter of inverse gamma distribution.
#'     (default \code{link_inv_lambda_shape = 1, link_inv_lambda_rate = 0.1})
#'     \item Error variance (\code{sigma2}): An Inverse gamma prior is assigned to \eqn{\sigma^2}
#'         where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
#'         (default \code{sigma2_shape = 0.001, sigma2_rate = 100})
#' }
#'
#' \strong{Initial values}
#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}
#' \enumerate{
#' \item Index vector:
#' \itemize{
#'      \item{\code{index_pi}: Initial selecting variable probability. (default: \code{0.5})}
#'      \item{\code{index_nu}: Initial vector of inclusion indicators . By default, each \code{index_nu} is randomly drawn by  \eqn{Bernoulli(1/2)}}
#'      \item{\code{index}: Initial vector of index. By default, each element of index vector, which is chosen by indicator, is proposed by normal distribution.}
#' }
#' \item Link function: Initial scalar of lambda (\code{link_inv_lambda}) for covariance kernel of Gaussian process.
#' \item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{0.01})
#'
#' }
#'
#'
#'@inherit bsFisher return
#'
#' @examples
#' \donttest{
#' set.seed(123)
#' n <- 200; d <- 4
#' theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
#' f <- function(u) u^2 * exp(u)
#' sigma <- 0.5
#' X <- matrix(runif(n * d, -1, 1), nrow = n)
#' index_vals <- as.vector(X %*% theta)
#' y <- f(index_vals) + rnorm(n, 0, sigma)
#' simdata <- data.frame(x = X, y = y)
#' colnames(simdata) <- c(paste0("X", 1:4), "y")
#'
#' # One tool version
#' fit1 <- gpSpike(y ~ ., data = simdata,
#'                niter = 5000, nburnin = 1000)
#'
#' # Split version
#' models <- gpSpike_setup(y ~ ., data = simdata)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' summary(fit2)
#' }
#'
#' @references
#' McGee, G., Wilson, A., Webster, T. F., & Coull, B. A. (2023).
#' Bayesian multiple index models for environmental mixtures.
#' \emph{Biometrics}, 79(1), 462-474.
#'
#' @name gpSpike
#' @export
gpSpike <- function(formula, data,
                    prior = NULL,
                    init = NULL,
                    monitors = NULL, niter = 10000, nburnin=1000,
                    thin = 1, nchain = 1, setSeed = FALSE
){
  return(gpSpike.default(formula = formula, data = data,
                         prior = prior,
                         init = init,
                         sampling = TRUE, monitors = monitors,
                         niter = niter, nburnin=nburnin,
                         thin = thin, nchain = nchain, setSeed = setSeed))
}

#' @rdname gpSpike
#' @export
gpSpike_setup <- function(formula, data,
                          prior = NULL,
                          init = NULL,
                          monitors = NULL, niter = 10000, nburnin=1000,
                          thin = 1, nchain = 1, setSeed = FALSE
){
  return(gpSpike.default(formula = formula, data = data,
                         prior = prior,
                         init = init,
                         sampling = FALSE, monitors = monitors,
                         niter = niter, nburnin=nburnin,
                         thin = thin, nchain = nchain, setSeed = setSeed))
}

gpSpike.default <- function(formula, data,
                    prior = NULL,
                    init = NULL,
                    sampling = TRUE,
                    monitors = NULL, niter = 10000, nburnin=1000,
                    thin = 1, nchain = 1, setSeed = FALSE
){
  start1 <- Sys.time()
  index_raw <- 0; nu <- 0; invlambda <- 0; sigma2 <- 0

  # check sampling, prior, init parameters for independent execution
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin, nchain,
    prior, init, "spike", "gp"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  # environment
  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))


  # aa_bspline_ver3
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # gpspike
    "expcov_gpSpike","expcovnn_gpSpike","computeA1","computeA2",
    "llFunLambda","llFunThetaV1","llFunThetaV2","transitionTheta",
    "multiplyMatrixByConstant","pred_gpSpike",
    "SamplingLambda_gp_spike","SamplingThetaV_gp_spike",
    "gibbsSigma2_gp_spike","gibbsGam_gp_spike",


    # utils
    "pred_fitted"
  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())



  # Data
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
           length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  }else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  # data dimension
  N <- length(Y)
  p <- ncol(X)


  # standardized X
  # meanX <- apply(X, 2, mean); sdX <- apply(X, 2, sd)
  # scaleX <- matrix(0, ncol = p, nrow = N)
  # for (i in 1:p){
  #   scaleX[,i] <- (X[,i] - meanX[i])/sdX[i]
  # }
  scaleX <- X

  ## zero mean
  # z <- 1
  # Z <- matrix(rep(0, N), nrow = N)
  # init_gam <- rep(0, z)
  # init_Zlin <- as.vector(Z %*% matrix(init_gam, nrow = z))


  # Prior parameters
  ## check data dimension and save
  ## theta
  if (is.null(prior$index$r1)||
      length(prior$index$r1) >= 2 ||
      prior$index$r1 < 0){
    stop("Prior index (r1) has incorrect value.")
  } else{
    a0 <- prior$index$r1
  }

  if (is.null(prior$index$r2)||
      length(prior$index$r2) >= 2 ||
      prior$index$r2 < 0){
    stop("Prior index (r2) has incorrect value.")
  } else{
    b0 <- prior$index$r2
  }

  if (is.null(prior$index$sigma_theta)||
      length(prior$index$sigma_theta) >= 2 ||
      prior$index$sigma_theta < 0){
    stop("Prior index (sigma_theta) has incorrect value.")
  } else{
    sigma_theta <- prior$index$sigma_theta
  }

  ## link
  if ((!is.null(prior$link$inv_lambda_shape))&
      (length(prior$link$inv_lambda_shape) >= 2 ||
      prior$link$inv_lambda_shape < 0)){
    stop("Prior inverse lambda (a) has incorrect value.")
  }
  a_lam <- prior$link$inv_lambda_shape


  if ((!is.null(prior$link$inv_lambda_rate))&
      (length(prior$link$inv_lambda_rate) >= 2 ||
      prior$link$inv_lambda_rate < 0)){
    stop("Prior inverse lambda (b) has incorrect value.")
  }
  b_lam <- prior$link$inv_lambda_rate


  ## sigma
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (a) has incorrect value.")
  } else{
    a_sig <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (b) has incorrect value.")
  } else{
    b_sig <- prior$sigma2$rate
  }

  # seed
  seedNum <- rep(FALSE, nchain)
  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }


  # initial value
  inits_list <- lapply(seq_len(nchain),
                       function(j) initfunction_gpSpike(x = scaleX, y = Y, p = p,
                                                        pi = init$index$pi, nu = init$index$nu,
                                                        index = init$index$index, inv_lambda = init$link$inv_lambda,
                                                        sigma2 = init$sigma2,
                                                        setSeed = seedNum[j]))
  firstInit <- inits_list[[1]]



  # Model code
  Rmodelcode <- nimbleCode({
    # thetastar: spike and slab
    pi ~ dbeta(a0, b0)
    for (j in 1:p) {
      nu[j] ~ dbern(pi)
      index_raw[j] ~ dnorm(0, sd = sigma_theta)
      indexstar[j] <- index_raw[j] * nu[j]
    }
    index[1:p] <- indexstar[1:p]/sqrt(sum(indexstar[1:p]^2))

    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i,1:p] * indexstar[1:p])
    }

    # likelihood
    invlambda ~ dgamma(a_lam, rate = b_lam)
    sigma2 ~ dinvgamma(a_sig, rate = b_sig)
    Ki[1:N, 1:N] <- expcov_gpSpike(Xlin[1:N], invlambda) # invlambda * (I + lambda^(-1)*K)
    cov[1:N, 1:N] <- sigma2 * Ki[1:N, 1:N]

    Y[1:N, 1] ~ dmnorm(mu0[1:N], cov = cov[1:N, 1:N])

  })

  # Build model
  message("== Building model ==")
  suppressMessages(simpleModel <- nimbleModel(Rmodelcode,
                        data = list(X = scaleX, Y = Y),
                        constants = list(p = p, N = N,
                                         a0 = a0, b0 = b0,
                                         a_lam = a_lam, b_lam = b_lam,
                                         a_sig = a_sig, b_sig = b_sig,
                                         sigma_theta = sigma_theta,
                                         mu0 = rep(0, N)),
                        inits = firstInit))


  # Assign samplers
  message("== Assigning samplers ==")
  monitorsList <- c("nu", "index", "sigma2", "Xlin", "invlambda", "pi",
                    "indexstar")

  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
                                             monitors = monitorsList,
                                             print = FALSE))


  # 1. invlambda
  mcmcConf$removeSamplers(c("invlambda"))
  mcmcConf$addSampler(target = c("invlambda"),
                       type   = SamplingLambda_gp_spike)

  # 2. index_raw, v, (indexstar)
  mcmcConf$removeSamplers(c("index_raw", "nu"))
  mcmcConf$addSampler(target = c("index_raw", "nu"),
                       type   = SamplingThetaV_gp_spike)

  # 3. sigma^2
  mcmcConf$removeSamplers(c("sigma2"))
  mcmcConf$addSampler(target = c("sigma2"),
                       type   = gibbsSigma2_gp_spike)
  # mcmcConf1$printSamplers(executionOrder= TRUE)

  # 4. gam
  # mcmcConf$removeSamplers(c("gam"))
  # mcmcConf$addSampler(target = c("gam"),
  #                      type   = "gibbsGam_gp_spike")


  mcmc1 <- buildMCMC(mcmcConf)


  if (!sampling){
    mcmc.out <- NULL
    samples <- NULL

  } else{
    # Compile
    start2 <- Sys.time()
    message("== Compiling model ==")
    suppressMessages(CsimpleModel <- compileNimble(simpleModel))
    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1, project = simpleModel,
                                            resetFunctions = TRUE))
    end2 <- Sys.time()

    # Sampling
    message("== Running MCMC ==")
    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                        thin = thin,
                        nchains = nchain, setSeed = seedNum,
                        inits = inits_list,
                        summary = FALSE, samplesAsCodaMCMC = TRUE)


    samples <- NULL

    if (nchain > 1){
      for (i in 1:nchain){
        samples <- rbind(samples, mcmc.out[[i]])
      }
    } else if (nchain == 1){
      samples <- mcmc.out
    }

    # # standardize index (make length = 1)
    # namesIndex <- paste0("indexstar", 1:p)
    # samples[, namesIndex] <- t(apply(samples[, namesIndex, drop = FALSE], 1, function(x) {
    #   x / sqrt(sum(x^2))
    # }))


  }
  end1 <- Sys.time()

  ## Input options
  ## Input options
  if (!sampling){
    time <- NULL
  } else{
    time <- difftime(end1, start1, units = "secs") - difftime(end2, start2, units = "secs")
  }


  # results ---------------------------------------------------------------------
  #Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = list(r1 = a0, r2 = b0, sigma_theta = sigma_theta),
                                    link = list(inv_lambda_shape = a_lam, inv_lambda_rate = b_lam),
                                    sigma2 = list(shape = a_sig, rate = b_sig)),
                       samplingOptions = list(sampling = sampling,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       init = inits_list, # initial value for MCMC
                       time = time)


  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = mcmc.out, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "gpSpike")

    # sampling the fitted values--

    samples <- sampleBind(out$samples, nchain)
    nsamp <- nrow(samples)
    namesIndex <- paste0("indexstar[", 1:p, "]")
    namesXlin <- paste0("Xlin[", 1:N, "]")

    indexstarSample <- samples[, namesIndex]
    XlinSample <- samples[, namesXlin]
    sigma2_samples <- samples[, "sigma2"]
    invlambdaSample <- samples[,"invlambda"]

    message("== Compiling functions for fitted values ==")
    suppressMessages(
      cpred_gpSpike <- compileNimble(pred_gpSpike)
    )

    testPred <- cpred_gpSpike(X, nsamp, Y, indexstarSample,
                              XlinSample, sigma2_samples, invlambdaSample,
                              prediction = 1)

    out$fitted.values <- apply(testPred, 2, mean)
    out$residuals <- out$fitted.values - as.vector(Y)
    out$gof <- mean(out$residuals^2)

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "gpSpike")

    class(out) <- "bsimSetup"

  }
  return(out)

}

SamplingLambda_gp_spike<- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(target)
    p <- model$getConstants()$p
    N <- model$getConstants()$N
    RllFunLambda <- llFunLambda(model)
    oldinvlambda <- model$invlambda
  },
  run = function(){
    # nimPrint("################# Start iteration #################")
    # nimPrint("Sampling Lambda")
    beforell <- RllFunLambda$run() # compute
    # nimCat("beforell:", beforell, "\n")
    logproposal <- log(oldinvlambda) + rnorm(1)
    # nimCat("Log proposal:", logproposal, "\n")
    model$invlambda <<- nimC(exp(logproposal))
    model$calculate(calcNodes)
    afterll <- RllFunLambda$run()
    # nimCat("afterll:", afterll, "\n")

    L <- (afterll - beforell)
    # nimCat("logratio:", L, "\n")
    # nimPrint(L)
    ratio <- exp(L)
    # nimCat("ratio:", ratio, "\n")
    u <- runif(1)

    # accept/reject
    if(u < ratio) { # accept
      # nimPrint("accept")
      # nimPrint(nimMatrix(nimC(model$thetastar, mvSaved[["thetastar"]][[1]]),
      #                    nrow = 18))
      # nimCat("model:", model$thetastar, "\n")
      # nimPrint("mvSaved: ")
      # nimPrint(mvSaved[["thetastar"]][[1]])
      nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
    }
    if(u > ratio){
      # nimPrint("reject")
      # nimPrint(nimMatrix(nimC(model$thetastar, mvSaved[["thetastar"]][[1]]),
      #                    nrow = 18))
      nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
    }
  },
  methods = list(reset = function(){})
)

## 2. Draw Joint theta, v
SamplingThetaV_gp_spike <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(target)
    p <- model$getConstants()$p
    N <- model$getConstants()$N
    v <- model$nu
    theta_raw <- model$index_raw
    theta_sig <- model$getConstants()$sigma_theta
    jump <- 0.35
    RllFunThetaV1 <- llFunThetaV1(model)
    RllFunThetaV2 <- llFunThetaV2(model)
  },
  run = function(){
    # nimPrint("--------------------------------------")
    # nimPrint("Sampling v and theta")
    # nimPrint(beforell)
    # initialize
    newV <- model$nu
    newTheta <- model$index_raw
    proposalTheta <- 0
    sumv <- 0
    for (i in 1:p){
      sumv <- sumv + v[i]
    }
    # nimCat("old sumv: ", sumv)

    u1 <- runif(1)
    if (u1 <= 0.5 | sumv == 0){
      # nimPrint("move 1")
      # 1) MH
      idx <- rcat(1, prob = nimRep(1/p, p))
      beforell <- RllFunThetaV1$run(idx) # compute
      beforeTrans <- transitionTheta(theta_raw[idx], theta_sig, v[idx], sumv)
      newV[idx] <- 1 - newV[idx]

      if (newV[idx] == 1){
        proposalTheta <- rnorm(1, 0, theta_sig)
        sumv <- sumv + 1
      } else{
        proposalTheta <- 0
        sumv <- sumv - 1
      }
      newTheta[idx] <- proposalTheta

      # newIndex <- newTheta/sqrt(sum(newTheta^2))

      # if (sum(newTheta) < 0){
      if (newTheta[1] < 0){
        newTheta <- (-1) * newTheta
      }

      model$nu <<- newV
      model$index_raw <<- newTheta
      model$indexstar <<- newTheta
      model$calculate(calcNodes)

      afterll <- RllFunThetaV1$run(idx)
      afterTrans <- transitionTheta(proposalTheta, theta_sig, newV[idx], sumv)
      # nimCat("idx:", idx, "\n")
      # nimCat("beforell:", beforell, "\n")
      # nimCat("beforeTrans:", beforeTrans, "\n")
      # nimCat("proposalTheta:", proposalTheta, "\n")
      # nimCat("new sumv:", sumv, "\n")
      # nimCat("afterll:", afterll, "\n")
      # nimCat("afterTrans:", afterTrans, "\n")

    } else{
      # 2) RW-MH
      # nimPrint("move 2")
      beforeTrans <- 1; afterTrans <- 1
      idx <- rcat(1, prob = v/sumv)
      # idx <- 1
      # count <- 0
      # while(idx <= p & count < idxtemp) {
      #   idx <- idx + 1
      #   # print(paste("idx:", idx))
      #   if (v[idx] == 1) {
      #     # print("in")
      #     count <- count + 1
      #     # print(paste("count:",count))
      #   }
      # }

      beforell <- RllFunThetaV2$run(idx) # compute
      proposalTheta <- rnorm(1, theta_raw[idx], jump)
      newTheta[idx] <- proposalTheta
      # newIndex <- newTheta/sqrt(sum(newTheta^2))
      if (newTheta[1] < 0){
        newTheta <- (-1) * newTheta
      }

      model$nu <<- newV
      model$index_raw <<- newTheta
      model$indexstar <<- newTheta
      model$calculate(calcNodes)
      afterll <- RllFunThetaV2$run(idx)
      # nimCat("idx:", idx, "\n")
      # nimCat("beforell:", beforell, "\n")
      # nimCat("beforeTrans:", beforeTrans, "\n")
      # nimCat("proposalTheta:", proposalTheta, "\n")
      # nimCat("new sumv:", sumv, "\n")
      # nimCat("afterll:", afterll, "\n")
      # nimCat("afterTrans:", afterTrans, "\n")

    }


    ## acceptance rate - log
    L <- (afterll - beforell) - (afterTrans - beforeTrans)
    # nimPrint(L)
    ratio <- exp(L)
    # nimPrint(ratio)
    u2 <- runif(1)

    # nimCat("log acceptance rate:", L, "\n")
    # nimCat("acceptance rate:", ratio, "\n")

    # accept/reject
    if(u2 < ratio) { # accept
      # nimPrint("accept")
      # nimPrint(nimMatrix(nimC(model$thetastar, mvSaved[["thetastar"]][[1]]),
      #                    nrow = 18))
      nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
    }
    if(u2 > ratio){
      # nimPrint("reject")
      # nimPrint(nimMatrix(nimC(model$thetastar, mvSaved[["thetastar"]][[1]]),
      #                    nrow = 18))
      nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
    }


  },
  methods = list(reset = function(){})
)

## 3. Draw sigma2
gibbsSigma2_gp_spike <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(target)
    p <- model$getConstants()$p
    N <- model$getConstants()$N
    y <- model$Y
    # z <- model$Z
    # pz <- model$getConstants()$z
    a_sig <- model$getConstants()$a_sig
    b_sig <- model$getConstants()$b_sig
  },
  run = function(){
    # nimPrint("--------------------------------------")
    # nimPrint("Sampling sigma")
    v <- model$nu
    expcovtemp <- model$Ki
    alpha <- a_sig + (N)/2
    beta <- b_sig + 0.5*(computeA1(y, expcovtemp))
    newSig <- rgamma(1, alpha, rate = beta)
    propsal <- nimC(1/newSig)
    model$sigma2 <<- propsal
    model$calculate(calcNodes)
    # nimCat("\n")
    # nimPrint(nimMatrix(nimC(model$thetastar, mvSaved[["thetastar"]][[1]]),
    #                    nrow = 18))
    nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)


    # nimCat("alpha:", alpha, "\n")
    # nimCat("beta:", 1/beta, "\n")
    # nimCat("propsal:", propsal, "\n")
    # nimCat("stored sigma:", model$sigma2, "\n")
    },
  methods = list(reset = function(){})
)

## 4. Draw gam
gibbsGam_gp_spike <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(target)
    p <- model$getConstants()$p
    N <- model$getConstants()$N
    y <- model$Y
    z <- model$Z
  },
  run = function(){
    # nimPrint("--------------------------------------")
    # nimPrint("Sampling gamma")
    expcovtemp <- model$Ki
    sigma2 <- model$sigma2
    invCov <- inverse(expcovtemp)
    part1 <- inverse(t(z) %*% invCov %*% z)
    part2 <- t(z) %*% invCov %*% y
    meanGam <- part1 %*% part2
    covGam <- multiplyMatrixByConstant(part1, sigma2[1])
    # covGam <- part1 * sigma2
    cholGam <- chol(covGam)


    newGamma <- rmnorm_chol(1, mean = meanGam[,1], cholesky = cholGam,
                            prec_param = FALSE)
    # nimPrint(newGamma)

    # nimCat("meanGam:", meanGam, "\n")
    # nimCat("covGam: \n")
    # nimCat(covGam, "\n")
    # nimCat("covGam: (no sigma) \n")
    # nimCat(part1, "\n")
    # nimCat("cholGam: \n")
    # nimCat(cholGam, "\n")


    model$gam <<- newGamma
    model$calculate(calcNodes)
    # nimCat("\n")
    # nimPrint(nimMatrix(nimC(model$thetastar, mvSaved[["thetastar"]][[1]]),
    #                    nrow = 18))
    nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)

    # nimPrint("################# End iteration #################")

  },
  methods = list(reset = function(){})
)

# R/zzz.R
#' @keywords internal
"_PACKAGE"
.onLoad <- function(lib, pkg) {
  loadNamespace("nimble")
}
.onAttach <- function(libname, pkgname) {
  packageStartupMessage("Loading BayesSIM...\n")
    }

#' Build an Initial Value List for BayesSIM Models
#'
#' @description
#' `init_param` is a convenience helper that constructs a nested initial value list
#' for a given combination of index vector and link function.
#' It starts from the model-specific default prior, and then overwrites only those components for
#' which the user supplies non-`NULL` arguments.
#'
#' This allows users to modify selected hyper-parameters without having to know
#' or manually reconstruct the full nested prior list structure.
#'
#' @param indexprior Character scalar indicating the prior for the index.
#'   Typically one of `"fisher"`, `"sphere"`, `"polar"`, or `"spike"`.
#'   The valid options mirror those used in the corresponding model functions.
#' @param link Character scalar indicating the link function family.
#'   Typically `"bspline"` for B-spline link functions or `"gp"` for Gaussian
#'   process link functions. The valid options mirror those used in the
#'   corresponding model functions.
#'
#' @param index,index_nu,index_psi,index_pi
#'   Optional initial values for index and related parameter values.
#'
#' @param link_beta,link_k,link_knots,link_lengthscale,link_amp,link_kappa,link_inv_lambda
#'   Optional initial values for components under link functions.
#'
#' @param sigma2 Optional numeric scalar giving the initial value of \eqn{\sigma^2}.
#'
#' @details
#' \code{init_param(indexprior, link)} can be used to obtain the random initial values
#' list for the requested combination of index prior and link function.
#' For any argument that is not `NULL`, the corresponding field in the nested prior list is overwritten.
#'
#' The detailed meaning and recommended choices for each initial values depend
#' on the specific model, index vector and link function.
#' For those details, please refer to the documentation of the corresponding
#' model-fitting functions.
#'
#' @seealso [bsFisher()], [bsSphere()], [bsPolar()], [bsSpike()],
#' [gpFisher()], [gpSphere()], [gpPolar()], [gpPolarHigh()], [gpSpike()]
#'
#' @return A nested list with components \code{index}, \code{link}, and
#'   \code{sigma2}.
#' @examples
#' ## Default initial values for Fisher index + B-spline link:
#' i0 <- init_param("fisher", "bspline")
#'
#' ## Modify only a few initial values:
#' i1 <- init_param(
#'   indexprior = "fisher",
#'   link       = "bspline",
#'   index      = c(1, 0, 0),      # initial direction of the index
#'   link_beta  = rep(0, 21),      # initial values for spline coefficients
#'   sigma2     = 0.1              # initial value for sigma^2
#' )
#'
#' ## Example with GP link:
#' i2 <- init_param(
#'   indexprior        = "sphere",
#'   link              = "gp",
#'   link_lengthscale  = 0.2,      # initial GP length-scale
#'   link_amp          = 1.5,      # initial GP amplitude
#'   sigma2            = 1         # initial variance
#' )
#'
#' @export
init_param <- function(
    indexprior, link, index = NULL, index_nu = NULL, index_psi = NULL,
    index_pi  = NULL, link_beta = NULL,link_k  = NULL,
    link_knots  = NULL, link_lengthscale = NULL, link_amp = NULL,
    link_kappa  = NULL, link_inv_lambda  = NULL, sigma2 = NULL
) {

  ## 1. default init list
  init <- init_param_default(indexprior = indexprior,
                             link       = link)

  ## 2. mapping flat args → nested components
  arg_map <- c(
    index        = "index",
    index_nu     = "index.nu",
    index_psi    = "index.psi",
    index_pi     = "index.pi",

    link_beta        = "link.beta",
    link_k           = "link.k",
    link_knots       = "link.knots",
    link_lengthscale = "link.lengthscale",
    link_amp         = "link.amp",
    link_kappa       = "link.kappa",
    link_inv_lambda  = "link.inv_lambda",

    sigma2       = "sigma2"
  )

  if (indexprior == "spike"){ #bsSpike, gpSpike
    arg_map["index"] <- "index.index"
  }

  if (indexprior == "sphere" & link == "bspline"){
    arg_map["index"] <- "index.index"
  }

  ## 3. apply overrides
  env <- environment()

  for (arg_name in names(arg_map)) {
    val <- get(arg_name, envir = env, inherits = FALSE)

    if (!is.null(val)) {
      path <- strsplit(arg_map[[arg_name]], ".", fixed = TRUE)[[1L]]
      init <- set_nested(init, path, val, strict = TRUE)
    }
  }

  init
}

init_param_default <- function(indexprior, link){
  initList <- list(index = NULL, link = NULL, sigma2 = NULL)
  if (link == "bspline"){
    if (indexprior == "fisher"){
      initList <- list(index = NULL, link = list(beta = NULL), sigma2 = 0.01)

    } else if (indexprior == "sphere"){
      initList <-list(index = list(nu = NULL, index = NULL),
                      link = list(k = NULL, knots = NULL, beta = NULL),
                      sigma2 = 0.01)

    } else if (indexprior == "polar"){
      initList <- list(index = list(psi = NULL), link = list(beta = NULL), sigma2 = 0.01)

    } else if (indexprior == "spike"){
      initList <- list(index = list(pi = 0.5, nu = NULL, index = NULL),
                       link = list(beta = NULL),
                       sigma2 = 0.01)
    } else{
      stop("Error: Wrong index prior name!")
    }

  } else if (link == "gp"){
    if (indexprior == "fisher"){
      initList <- list(index = NULL,
                       link = list(lengthscale = 0.1, amp = 1),
                       sigma2 = 1)

    } else if (indexprior == "sphere"){
      initList <- list(index = NULL,
                       link = list(lengthscale = 0.1, amp = 1),
                       sigma2 = 1)

    } else if (indexprior == "polar"){
      initList <- list(index = list(psi = NULL),
                       link = list(kappa = 2),
                       sigma2 = 0.01)

    } else if (indexprior == "spike"){
      initList <- list(index = list(pi = 0.5, nu = NULL, index = NULL),
                       link = list(inv_lambda = NULL),
                       sigma2 = NULL)

    } else{
      stop("Error: Wrong index prior name!")
    }

  } else{
    stop("Error: Wrong link function name!")
  }

  return(initList)

}

#' Build a Prior List for BayesSIM Models
#'
#' @description
#' `prior_param` is a convenience helper that constructs a nested prior list
#' for a given combination of index prior and link function.
#' It starts from the model-specific default prior, and then overwrites only those components for
#' which the user supplies non-`NULL` arguments.
#'
#' This allows users to modify selected hyper-parameters without having to know
#' or manually reconstruct the full nested prior list structure.
#'
#' @param indexprior Character scalar indicating the prior for the index.
#'   Typically one of `"fisher"`, `"sphere"`, `"polar"`, or `"spike"`.
#'   The valid options mirror those used in the corresponding model functions.
#' @param link Character scalar indicating the link function family.
#'   Typically `"bspline"` for B-spline link functions or `"gp"` for Gaussian
#'   process link functions. The valid options mirror those used in the
#'   corresponding model functions.
#'
#' @param index_direction,index_dispersion,index_nu_r1,index_nu_r2,index_psi_alpha,index_sigma_theta,index_r1,index_r2
#'   Optional overrides for hyper-parameters related to the index prior.
#'
#' @param link_basis_df,link_basis_degree,link_basis_delta
#'   Optional overrides for the B-spline basis hyper-parameters, such as the effective degrees of freedom,
#'   spline degree, and penalty parameter.
#' @param link_knots_lambda_k,link_knots_maxknots
#'   Optional overrides for the B-spline knot-selection hyper-parameters in, used for models with adaptive knot placement.
#'
#' @param link_beta_mu,link_beta_cov,link_beta_tau,link_beta_Sigma0
#'   Optional overrides for the prior on spline coefficients. The detailed interpretation of these
#'   hyper-parameters depends on the specific model and is described in the
#'   documentation of each model-fitting function.
#'
#' @param link_lengthscale_shape,link_lengthscale_rate
#'   Optional overrides for the hyper-parameters of the GP length-scale prior.
#'
#' @param link_amp_a,link_amp_b
#'   Optional overrides for the hyper-parameters of the GP amplitude prior.
#'
#' @param link_kappa_min,link_kappa_max,link_kappa_grid_width
#'   Optional overrides for the hyper-parameters in used in models with polar index and GP-type link,
#'   to control the grid or support for the concentration parameter \eqn{\kappa} in `gpPolar`.
#'
#' @param link_inv_lambda_shape,link_inv_lambda_rate
#'   Optional overrides for spike-and-slab–type GP link priors.
#'
#' @param sigma2_shape,sigma2_rate
#'   Optional overrides for the inverse-gamma prior on the observation
#'   variance \eqn{\sigma^2}.
#'
#' @details
#'  \code{prior_param(indexprior, link)} can be used to obtain the default prior
#' list for the requested combination of index prior and link function.
#' For any argument that is not `NULL`, the corresponding field in the nested prior list is overwritten.
#'
#'
#' The detailed meaning and recommended choices for each hyper-parameter depend
#' on the specific model, prior of index vector and link function.
#' For those details, please refer to the documentation of the corresponding
#' model-fitting functions.
#'
#' @return
#' A nested list with top-level elements \code{index}, \code{link}, and
#' \code{sigma2}, suitable for passing to the \code{prior} argument of the
#' various BayesSIM model fitting functions.
#'
#' @seealso [bsFisher()], [bsSphere()], [bsPolar()], [bsSpike()],
#' [gpFisher()], [gpSphere()], [gpPolar()], [gpPolarHigh()], [gpSpike()]
#'
#' @examples
#' ## Default prior for Fisher index + B-spline link:
#' p0 <- prior_param("fisher", "bspline")
#'
#' ## Modify only a few hyper-parameters:
#' p1 <- prior_param(
#'   indexprior          = "fisher",
#'   link                = "bspline",
#'   sigma2_shape        = 0.5,
#'   link_basis_df       = 30,
#'   index_direction     = c(1, 0, 0)
#' )
#' @export
prior_param <- function(indexprior, link,
                        index_direction = NULL, index_dispersion = NULL,
                        index_nu_r1 = NULL, index_nu_r2 = NULL,
                        index_psi_alpha = NULL, index_sigma_theta = NULL,
                        index_r1 = NULL, index_r2 = NULL,
                        link_basis_df = NULL, link_basis_degree = NULL,
                        link_basis_delta = NULL, link_knots_lambda_k = NULL,
                        link_knots_maxknots = NULL, link_beta_mu = NULL,
                        link_beta_cov = NULL, link_beta_tau = NULL,
                        link_beta_Sigma0 = NULL, link_lengthscale_shape = NULL,
                        link_lengthscale_rate = NULL, link_amp_a = NULL,
                        link_amp_b = NULL, link_kappa_min = NULL,
                        link_kappa_max = NULL, link_kappa_grid_width = NULL,
                        link_inv_lambda_shape = NULL, link_inv_lambda_rate = NULL,
                        sigma2_shape = NULL, sigma2_rate = NULL) {

  ## 1. default prior
  prior <- prior_param_default(indexprior = indexprior,
                               link       = link)

  ## 2. mapping table
  arg_map <- c(
    index_direction        = "index.direction",
    index_dispersion       = "index.dispersion",
    index_nu_r1            = "index.nu.r1",
    index_nu_r2            = "index.nu.r2",
    index_psi_alpha        = "index.psi.alpha",
    index_r1               = "index.r1",
    index_r2               = "index.r2",
    index_sigma_theta      = "index.sigma_theta",

    link_basis_df          = "link.basis.df",
    link_basis_degree      = "link.basis.degree",
    link_basis_delta       = "link.basis.delta",

    link_knots_lambda_k    = "link.knots.lambda_k",
    link_knots_maxknots    = "link.knots.maxknots",

    link_beta_mu           = "link.beta.mu",
    link_beta_cov          = "link.beta.cov",
    link_beta_tau          = "link.beta.tau",
    link_beta_Sigma0       = "link.beta.Sigma0",

    link_lengthscale_shape = "link.lengthscale.shape",
    link_lengthscale_rate  = "link.lengthscale.rate",
    link_amp_a         = "link.amp.a_amp",
    link_amp_b         = "link.amp.b_amp",

    link_kappa_min   = "link.kappa.min",
    link_kappa_max   = "link.kappa.max",
    link_kappa_grid_width  = "link.kappa.grid_width",

    link_inv_lambda_shape  = "link.inv_lambda_shape",
    link_inv_lambda_rate   = "link.inv_lambda_rate",

    sigma2_shape           = "sigma2.shape",
    sigma2_rate            = "sigma2.rate"
  )

  env <- environment()


  for (arg_name in names(arg_map)) {
    val <- get(arg_name, envir = env, inherits = FALSE)
    if (!is.null(val)) {
      path <- strsplit(arg_map[[arg_name]], ".", fixed = TRUE)[[1L]]
      prior <- set_nested(
        lst   = prior,
        path  = path,
        value = val,
        strict = TRUE
      )
    }
  }


  return(prior)


}



set_nested <- function(lst, path, value, parent_path = NULL, strict = TRUE) {
  nm <- path[1L]
  full_path <- c(parent_path, nm)

  # Last stage: assignment
  if (length(path) == 1L) {
    if (strict && !is.null(lst) && !nm %in% names(lst)) {
      stop("Unknown prior field: ",
           paste(full_path, collapse = " -> "),
           call. = FALSE)
    }
    if (is.null(lst)) lst <- list()
    lst[[nm]] <- value
    return(lst)
  }

  # middle state: check the path
  if (is.null(lst[[nm]])) {
    if (strict) {
      stop("Unknown prior path: ",
           paste(full_path, collapse = " -> "),
           call. = FALSE)
    } else {
      lst[[nm]] <- list()
    }
  } else if (!is.list(lst[[nm]])) {
    stop("Cannot descend into non-list prior field: ",
         paste(full_path, collapse = " -> "),
         call. = FALSE)
  }

  lst[[nm]] <- set_nested(
    lst[[nm]],
    path        = path[-1L],
    value       = value,
    parent_path = full_path,
    strict      = strict
  )
  lst
}

prior_param_default <- function(indexprior, link){
  priorList <- list(index = NULL, link = NULL, sigma2 = NULL)
  if (link == "bspline"){
    if (indexprior == "fisher"){
      priorList <- list(index = list(direction = NULL, dispersion = 150),
                        link = list(basis = list(df = 21, degree = 2, delta = 0.001),
                                    beta = list(mu = NULL, cov = NULL)),
                        sigma2 = list(shape = 0.001, rate = 100))

    } else if (indexprior == "sphere"){
      priorList <- list(index = list(nu = list(r1 = 1, r2 = 1)),
                        link = list(knots = list(lambda_k = 5, maxknots = NULL),
                                    basis = list(degree = 2),
                                    beta = list(mu = NULL, tau = NULL,Sigma0 = NULL)),
                        sigma2 = list(shape = 0.001, rate = 0.001))

    } else if (indexprior == "polar"){
      priorList <- list(index = list(psi = list(alpha = NULL)),
                        link = list(basis = list(df = 21,degree = 2, delta = 0.001),
                                    beta = list(mu = NULL, cov = NULL)),
                        sigma2 = list(shape = 0.001, rate = 100))

    } else if (indexprior == "spike"){
      priorList <- list(index = list(nu = list(r1 = 1, r2 = 1), sigma_theta = 0.25),
                        link = list(basis = list(df = 21, degree = 2, delta = 0.01),
                                    beta = list(mu = NULL, cov = NULL)),
                        sigma2 = list(shape = 0.001, rate = 100))


    } else{
      stop("Error: Wrong index prior name!")
    }

  } else if (link == "gp"){
    if (indexprior == "fisher"){
      priorList <- list(index = list(direction = NULL, dispersion = 150),
                        link = list(lengthscale = list(shape = 1/8, rate = 1/8),
                                    amp = list(a_amp = -1, b_amp = 1)),
                        sigma2 = list(shape = 1, rate = 1))

    } else if (indexprior == "sphere"){
      priorList <- list(index = NULL,
                        link = list(lengthscale = list(shape = 1/8, rate = 1/8),
                                    amp = list(a_amp = -1, b_amp = 1)),
                        sigma2 = list(shape = 1, rate = 1))

    } else if (indexprior == "polar"){
      priorList <- list(index = list(psi = list(alpha = NULL)),
                        link = list(kappa = list(min= 0.5, max = 4, grid_width = 0.1)),
                        sigma2 = list(shape = 2, rate = 0.01))

    } else if (indexprior == "spike"){
      priorList <- list(index = list(r1 = 1, r2 = 1, sigma_theta = 0.25),
                        link = list(inv_lambda_shape = 1, inv_lambda_rate = 0.1),
                        sigma2 = list(shape = 0.001, rate = 0.001))

    } else{
      stop("Error: Wrong index prior name!")
    }

  } else{
    stop("Error: Wrong link function name!")
  }

  return(priorList)

}


# prior1 <- prior_param(
#   indexprior = "fisher",
#   link       = "bspline",
#   sigma2_shape    = 0.5,               # sigma2$shape
#   sigma2_rate     = 10,                # sigma2$rate
#   link_basis_df   = 30,                # link$basis$df
#   index_direction = c(1, 0, 0, 0)      # index$direction
# )
#
# str(prior1)

#' Bayesian Single-Index Regression with Gaussian Process Link and Unit Sphere Prior
#'
#' @description
#' Fits a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}, where
#' the index \eqn{\theta} lies on the unit sphere, and the link \eqn{f(\cdot)} is represented
#' with Gaussian process.
#'
#' @inheritParams bsFisher
#' @param method Character, `gpSphere` model has 3 different types of sampling method, fully Bayesian method (\code{"FB"}), empirical Bayes approach (\code{"EB"}), and empirical Gibbs sampler (\code{"EG"}).
#' Assign one sampler method. Empirical sampling approach is recommended in high-dimensional data. By default, fully Bayesian approach is assigned.
#' @param lowerB This parameter is only for `gpSphere` model. Numeric vector of element-wise lower bounds for the \code{"L-BFGS-B"} method.
#' When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
#' The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
#' By default, the lower bounds are \code{-1} for the index vector and \code{-1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.
#' @param upperB This parameter is only for `gpSphere` model. Numeric vector of element-wise upper bounds for the \code{"L-BFGS-B"} method.
#' When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
#' The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
#' By default, the upper bounds are \code{1} for the index vector and \code{1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.
#'
#' @details
#' \strong{Model} The single-index model uses Gaussian process with zero mean and and covariance kernel \eqn{\eta \cdot \text{exp}(-\frac{(t_i-t_j)^2}{l})} as a link function, where \eqn{t_i, t_j, j = 1, \ldots, n} is index value.
#' Index vector should be length 1.
#'
#' \strong{Priors}
#' \itemize{
#' \item von Mises–Fisher prior on the index \eqn{\theta} with direction and concentration.
#' \item Covariance kernel: Amplitude, \eqn{\eta}, follows log normal distribution with mean \eqn{a_\eta} and variance \eqn{b_\eta}.
#' Length-scale parameter follows gamma distribution with shape parameter \eqn{\alpha_l} and rate parameter \eqn{\beta_l}.
#' \item Inverse-Gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.
#'
#' }
#'
#' \strong{Sampling} In the fully Bayesian approach, \eqn{\theta}, \eqn{l}, and \eqn{\eta}
#' are updated via the Metropolis–Hastings algorithm, while \eqn{f} and
#' \eqn{\sigma^2} are sampled using Gibbs sampling.
#'
#' In the empirical Bayes approach, \eqn{\theta}, \eqn{l}, \eqn{\eta},
#' and \eqn{\sigma^2} are estimated by maximum a posteriori (MAP), and
#' \eqn{f} is sampled from its full conditional posterior distribution.
#'
#' In the empirical Gibbs sampler, \eqn{\theta}, \eqn{l}, and \eqn{\eta}
#' are estimated by MAP, whereas \eqn{f} and \eqn{\sigma^2} are sampled
#' via Gibbs sampling.
#'
#' For estimation via MAP, effective sample size or potential scale reduction factor is meaningless.
#'
#' \strong{Prior hyper-parameters}
#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
#' \enumerate{
#' \item Index vector: Nothing to assign.
#' \item Link function:
#' \itemize{
#'    \item{Length-scale:Gamma distribution is assigned for length-scale parameter, \eqn{l}.
#'     \code{link_lengthscale_shape} is shape parameter (default \code{1/8}) and \code{link_lengthscale_rate} is rate parameter of \code{lengthscale}. (default \code{1/8})}
#'     \item{Amplitude: Log-normal distribution is assigned for amplitude parameter, \eqn{\eta}.
#'      \code{link_amp_a} is mean (default \code{-1}), and \code{link_amp_b} is variance. (default \code{1})}
#' }
#'
#' \item Error variance (\code{sigma2}): inverse gamma prior is assigned to \eqn{\sigma^2}
#'         where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
#'         (default \code{sigma2_shape = 1, sigma2_rate = 1})
#'
#' }
#'
#' \strong{Initial values}
#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
#' \enumerate{
#' \item Index vector (\code{index}): Initial unit index vector. By default, vector is randomly drawn from normal distribution and standardized.
#'      \item Link function: \code{link_lengthscale} is initial scalar length-scale parameter. (default: \code{0.1})
#'      \code{link_amp} is initial scalar amplitude parameter. (default: \code{1})
#'      \item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{1})
#'
#' }
#'
#'
#' @inherit bsFisher return
#'
#' @examples
#' \donttest{
#' set.seed(123)
#' n <- 200; d <- 4
#' theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
#' f <- function(u) u^2 * exp(u)
#' sigma <- 0.5
#' X <- matrix(runif(n * d, -1, 1), nrow = n)
#' index_vals <- as.vector(X %*% theta)
#' y <- f(index_vals) + rnorm(n, 0, sigma)
#' simdata <- data.frame(x = X, y = y)
#' colnames(simdata) <- c(paste0("X", 1:4), "y")
#'
#' # One tool version
#' fit1 <- gpSphere(y ~ ., method = "EB", data = simdata,
#'                  niter = 1000, nburnin = 100)
#'
#' # Split version
#' models <- gpSphere_setup(y ~ ., method = "EB", data = simdata)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 1000, nburnin = 100, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' # The estimates computed by MAP - standard error of the esitmate is meaningless.
#' summary(fit2)
#' }
#' @references
#' Choi, T., Shi, J. Q., & Wang, B. (2011).
#' A Gaussian process regression approach to a single-index model.
#' \emph{Journal of Nonparametric Statistics}, 23(1), 21-36.
#' @name gpSphere
#' @export
gpSphere <- function(formula, data,
                     prior = NULL,
                     init = NULL,
                     method = "FB",
                     lowerB = NULL, upperB = NULL, monitors = NULL, niter = 10000,
                     nburnin=1000, thin = 1, nchain = 1, setSeed = FALSE){
  return(gpSphere.default(formula = formula, data = data,
                          prior = prior, init = init,
                          sampling = TRUE, method = method,
                          lowerB = lowerB, upperB = upperB, monitors = monitors,
                          niter = niter,
                          nburnin=nburnin, thin = thin, nchain = nchain, setSeed = setSeed))

}

#' @rdname gpSphere
#' @export
gpSphere_setup <- function(formula, data,
                           prior = NULL,
                           init = NULL,
                           method = "FB",
                           lowerB = NULL, upperB = NULL, monitors = NULL, niter = 10000,
                           nburnin=1000, thin = 1, nchain = 1, setSeed = FALSE){

  return(gpSphere.default(formula = formula, data = data,
                          prior = prior, init = init,
                          sampling = FALSE, method = method,
                          lowerB = lowerB, upperB = upperB, monitors = monitors,
                          niter = niter,
                          nburnin=nburnin, thin = thin, nchain = nchain, setSeed = setSeed))
}


gpSphere.default <- function(formula, data,
                             prior = NULL,
                             init = NULL,
                             sampling = TRUE, method = "FB",
                             lowerB = NULL, upperB = NULL, monitors = NULL, niter = 10000,
                             nburnin=1000, thin = 1, nchain = 1, setSeed = FALSE){

  start1 <- Sys.time()
  index <- 0; log_amp <- 0; lengthscale <- 0; sigma2 <- 0; Id <- 0
  index0 <- 0; amp <- 0

  # check sampling, prior, init parameters for independent execution
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin, nchain,
    prior, init, "sphere", "gp"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  # environment
  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))


  # aa_bspline_ver3
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # gpSphere
    "expcov_gpSphere","expcovTest_gpSphere","conBeta","obj_btt",
    "obj_btt_EB","pred_gpSphere","indexSampler_gpSphere","sigma2Sampler_gpSphere",
    "optSampler",

    # distribution
    "dunitSphere", "runitSphere",

    # utils
    "pred_fitted"
  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())

  suppressMessages(
    nimble::registerDistributions(list(
      dunitSphere = list(
        BUGSdist     = "dunitSphere(dim)",
        types        = c("value = double(1)",
                         "dim   = double(0)"),
        discrete     = FALSE
      )
    ), verbose = FALSE)

  )



  # check data dimension
  # Data
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
           length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  }else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  if (method == "EB" & nchain > 1){
    stop("Only single chain is allowed.")
  }

  # data dimension
  N <- length(Y)
  p <- ncol(X)

  # model code
  modelCode_FB <- nimbleCode({
    # beta - MH
    index0[1:p] ~ dunitSphere(p)
    index[1:p] <- index0[1:p]/sqrt(sum(index0[1:p]^2))

    # hyperprior - MH
    lengthscale ~ dgamma(shape, rate)
    amp ~ dlnorm(a_amp, b_amp)

    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i,1:p] * index[1:p])
    }

    sigma2 ~ dinvgamma(a_sig, b_sig)

    # linkFunction
    cov[1:N, 1:N] <- expcov_gpSphere(Xlin[1:N], lengthscale, amp)
    linkFunction[1:N] ~ dmnorm(mu0[1:N], cov = cov[1:N, 1:N])

    # likelihood - gibbs
    Sigma[1:N,1:N] <- sigma2 * Id[1:N,1:N]
    Y[1:N] ~ dmnorm(linkFunction[1:N], cov = Sigma[1:N,1:N])

  })

  modelCode_EG <- nimbleCode({
    # index - MH
    index[1:p] ~ dunitSphere(p)

    # hyperprior - MH
    lengthscale ~ dgamma(shape, rate)
    amp ~ dlnorm(a_amp, b_amp)

    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i,1:p] * index[1:p])
    }

    sigma2 ~ dinvgamma(a_sig, b_sig)

    # linkFunction
    cov[1:N, 1:N] <- expcov_gpSphere(Xlin[1:N], lengthscale, amp)
    linkFunction[1:N] ~ dmnorm(mu0[1:N], cov = cov[1:N, 1:N])

    # likelihood - gibbs
    Sigma[1:N,1:N] <- sigma2 * Id[1:N,1:N]
    Y[1:N] ~ dmnorm(linkFunction[1:N], cov = Sigma[1:N,1:N])

  })

  modelCode_EB <- nimbleCode({
    # MAP method is used
    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i,1:p] * index[1:p])
    }
    # linkFunction
    cov[1:N, 1:N] <- expcov_gpSphere(Xlin[1:N], lengthscale, amp)
    linkFunction[1:N] ~ dmnorm(mu0[1:N], cov = cov[1:N, 1:N])

    # likelihood - gibbs
    Sigma[1:N,1:N] <- sigma2 * Id[1:N,1:N]
    Y[1:N] ~ dmnorm(linkFunction[1:N], cov = Sigma[1:N,1:N])

  })


  # Check hyper-parameters
  # lengthscale
  ## shape
  if (is.null(prior$link$lengthscale$shape)||
      length(prior$link$lengthscale$shape) >= 2 ||
      prior$link$lengthscale$shape < 0){
    stop("Prior lengthscale (shape) has incorrect value.")
  } else{
    shape <- prior$link$lengthscale$shape
  }

  ## rate
  if (is.null(prior$link$lengthscale$rate)||
      length(prior$link$lengthscale$rate) >= 2 ||
      prior$link$lengthscale$rate < 0){
    stop("Prior lengthscale (rate) has incorrect value.")
  } else{
    rate <- prior$link$lengthscale$rate
  }

  # amp - log normal
  ## a_amp: mean
  if (is.null(prior$link$amp$a_amp)||
      length(prior$link$amp$a_amp) >= 2 ){
    stop("Prior amp (a_amp) has incorrect value.")
  } else{
    a_amp <- prior$link$amp$a_amp
  }

  ## b_amp: sd
  if (is.null(prior$link$amp$b_amp)||
      length(prior$link$amp$b_amp) >= 2 ||
      prior$link$amp$b_amp < 0){
    stop("Prior amp (b_amp) has incorrect value.")
  } else{
    b_amp <- prior$link$amp$b_amp
  }

  # sigma2: inverse gamma
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (shape) has incorrect value.")
  } else{
    a_sig <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (rate) has incorrect value.")
  } else{
    b_sig <- prior$sigma2$rate
  }

  # seed
  seedNum <- rep(FALSE, nchain)
  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }



  # For all initial values
  inits_list <- lapply(seq_len(nchain),
                       function(j) initFunction_gpSphere(N = N, p = p, X = X, Y = as.vector(Y),
                                                         index = init$index,
                                                         lengthscale = init$link$lengthscale,
                                                         amp  = init$link$amp,
                                                         sigma2  = init$sigma2, method = method,
                                                         setSeed = seedNum[j]))

  firstInit <- inits_list[[1]]

  # boundarys for optimizing (using MAP) ---------------------------------------
  if (method %in% c("EB", "EG")){
    # lower bound
    if (!is.null(lowerB) & method == "EB" &  length(lowerB) != (p+3)){
      stop("The length of lower bound (lowerB) is not appropriate!")
    }

    if (!is.null(lowerB) & method == "EG" &  length(lowerB) != (p+2)){
      stop("The length of lower bound (lowerB) is not appropriate!")
    }


    if (is.null(lowerB) & method == "EB"){ # index, lengthscale, amp, sigma2
      lowerB <- c(rep(-1, p),-1e3, -1e3, -1e3)
    } else if (is.null(lowerB) & method == "EG"){  # index, lengthscale, amp
      lowerB <- c(rep(-1, p),-1e3, -1e3)
    }


    # Upper bound
    if (!is.null(upperB) & method == "EB" & length(upperB) != (p+3)){
      stop("The length of upper bound (upperB) is not appropriate!")
    }

    if (!is.null(upperB) & method == "EG" & length(upperB) != (p+2)){
      stop("The length of upper bound (upperB) is not appropriate!")
    }


    if (is.null(upperB) & method == "EB"){ # index, lengthscale, amp, sigma2
      upperB <- c(rep(1, p),1e3, 1e3, 1e3)
    } else if (is.null(upperB) & method == "EG"){  # index, lengthscale, amp
      upperB <- c(rep(1, p),1e3, 1e3)
    }
  }

  # Initial values -------------------------------------------------------
  if (method == "EB"){ # only nchain = 1
    obj_fn <- obj_btt_EB(X, Y, p, shape, rate,
                   a_amp, b_amp, a_sig, b_sig)
    # lowerB <- c(rep(-1, p),-1e2, -1e2, -1e2)
    # upperB <- c(rep(1, p),1e2, 1e2, 1e2)
    tempIndex <- as.vector(ppr(X,Y, nterms = 1)$alpha) # initial value for optimizing

    current <- c(tempIndex, log(firstInit$lengthscale), log(firstInit$amp),
                 firstInit$sigma2)

    message("Finding MAP..")

    tryCatch(
      {
        optRes  <- nimOptim(par    = current,
                            fn     = obj_fn$run,
                            method = "L-BFGS-B",
                            lower  = lowerB,
                            upper  = upperB)
      },
      error = function(e) {
        stop(
          paste0(
            "Optimization was not successful.\n",
            "Likely cause: the specified lower/upper bounds are inappropriate for this dataset.\n",
            "Recommendation: adjust the boundarys and verify initial values to enable convergence.\n",
          "Specific error message in optim:"
            ), conditionMessage(e), call. = FALSE)
      }
    )

    proposedIndex <- optRes$par[1:p]
    if (proposedIndex[1] < 0){
      proposedIndex <- (proposedIndex) * (-1)
    }
    map_index  <- proposedIndex/sqrt(sum((proposedIndex)^2))
    map_lengthscale <- exp(optRes$par[(p+1)])
    map_amp   <- exp(optRes$par[(p+2)])
    map_sigma2 <- exp(optRes$par[(p+3)])

    # EB initlist
    init_Xlin <- as.vector(X %*% matrix(map_index, nrow = p))
    init_cov <-  expcov_gpSphere(init_Xlin, map_lengthscale, map_amp)
    firstInit <- list(Xlin = init_Xlin,
                      cov = init_cov,
                      linkFunction = mvtnorm::rmvnorm(1, rep(0, N), sigma = init_cov)[1,],
                      Sigma = map_sigma2 * diag(1, N))
    inits_list <- firstInit
    modelCode <- modelCode_EB

  } else{ # FB, EG

    if (method == "FB"){
      modelCode <- modelCode_FB
    } else if (method == "EG"){
      modelCode <- modelCode_EG
    }
  }

  message("== Building model ==")

  if (method %in% c("FB", "EG")){
    suppressMessages(
      simpleModel <- nimbleModel(modelCode,
                               data = list(X = X, Y = as.vector(Y)),
                               constants = list(shape = shape, rate = rate,
                                                a_sig = a_sig, b_sig = b_sig,
                                                a_amp = a_amp, b_amp = b_amp,
                                                p = p, N = N,
                                                Id = diag(1, N), mu0 = rep(0, N)),
                               inits = firstInit)
    )
    monitorsList <- c("index", "sigma2", "linkFunction", "Xlin", "lengthscale", "amp")

  } else{ # EB
    suppressMessages(
      simpleModel <- nimbleModel(modelCode,
                                 data = list(X = X, Y = as.vector(Y)),
                                 constants = list(p = p, N = N, index = map_index,
                                                  lengthscale = map_lengthscale,
                                                  amp = map_amp, sigma2 = map_sigma2,
                                                  Id = diag(1, N), mu0 = rep(0, N)),
                                 inits = firstInit)
    )
    monitorsList <- c("linkFunction", "Xlin")
  }

  # sampling
  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
                                             monitors = monitorsList,
                                             print = FALSE))

  if (method == "FB"){
    mcmcConf$removeSamplers('lengthscale')
    mcmcConf$addSampler(target = "lengthscale", type = "RW",
                        control = list(log = TRUE))

    mcmcConf$removeSamplers('sigma2')
    mcmcConf$addSampler(target = "sigma2",
                        type = sigma2Sampler_gpSphere)
    mcmcConf$setSamplerExecutionOrder(c(1, 2, 4, 3, 5))

  } else if (method == "EG"){ # Empirical gibbs
    mcmcConf$removeSamplers(c('index','lengthscale','amp'))
    mcmcConf$addSampler(target = c('index','lengthscale','amp'),
                        type   = optSampler,
                        control = list(lowerB = lowerB,
                                       upperB = upperB))
    mcmcConf$setSamplerExecutionOrder(c(3, 2, 1))

  } else if (method == "EB"){ # Empirical bayes
    # mcmcConf$removeSamplers(c('index','lengthscale','amp','sigma2'))
    # mcmcConf$addSampler(target = c('index','lengthscale','amp','sigma2'),
    #                     type   = optSampler,
    #                     control = list(sigmaTrue = TRUE))
    # mcmcConf$setSamplerExecutionOrder(c(2, 1))

  } else{
    stop("Wrong sampling method!")
  }

  message("== Assigning samplers ==")
  mcmc1 <- buildMCMC(mcmcConf)


  if (!sampling){
    samples <- NULL

  } else{
    start2 <- Sys.time()
    message("== Compiling model ==")
    suppressMessages(
      CModel <- compileNimble(simpleModel, resetFunctions = TRUE)
    )
    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1))
    end2 <- Sys.time()

    message("== Running MCMC ==")
    mcmc.out <- NULL

    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    if (method == "EB"){
      mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                          thin = thin,nchains = nchain, setSeed = seedNum,
                          inits = inits_list,
                          summary = FALSE, samplesAsCodaMCMC = TRUE)
    } else if (method == "EG"){
      tryCatch(
        {
          mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                              thin = thin,nchains = nchain, setSeed = seedNum,
                              inits = inits_list, summary = FALSE, samplesAsCodaMCMC = TRUE)
          },
          error = function(e) {
            stop(paste0(
                "Optimization was not successful.\n",
                "Likely cause: the specified lower/upper bounds are inappropriate for this dataset.\n",
                "Recommendation: adjust the boundarys and verify initial values to enable convergence.\n",
                "Specific error message in optim:"
              ), conditionMessage(e), call. = FALSE)
          }
        )
    } else{ # FB
      mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                          thin = thin, nchains = nchain, setSeed = seedNum,
                          inits = inits_list, summary = FALSE, samplesAsCodaMCMC = TRUE)
      }

    sampMCMC <- mcmc.out
    if (method == "EB"){
      nmcmcsamp <- nrow(sampMCMC)
      sampMCMC <- as.matrix(mcmc.out)
      index_mat <- matrix(rep(map_index, nmcmcsamp), ncol = p, byrow = TRUE)
      colnames(index_mat) <- paste0("index[",1:p, "]")
      hype_mat <- matrix(rep(c(map_lengthscale, map_amp, map_sigma2), nmcmcsamp),
                         ncol = 3, byrow = TRUE)
      colnames(hype_mat) <- c("lengthscale", "amp", "sigma2")
      sampMCMC <- cbind(sampMCMC, index_mat, hype_mat)
    }

    # Make single matrix for estimation
    samples <- NULL
    if (nchain > 1){
      for (i in 1:nchain){
        samples <- rbind(samples, sampMCMC[[i]])
      }
    } else if (nchain == 1){
      samples <- sampMCMC
    }
  }
  end1 <- Sys.time()


  ## Input options
  if (!sampling){
    time <- NULL
  } else{
    time <- difftime(end1, start1, units = "secs") - difftime(end2, start2, units = "secs")
  }

  # Results ------------------------------------------------------------------
  #Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = NULL,
                                    link = list(lengthscale = list(shape = shape, rate = rate),
                                                amp = list(a_amp = a_amp, b_amp = b_amp)),
                                    sigma2 = list(shape = a_sig, rate = b_sig)),
                       init = inits_list,
                       samplingOptions = list(lowerB = lowerB, upperB = upperB,
                                              sampling = sampling, method = method,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       time = time)

  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = sampMCMC, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "gpSphere")

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "gpSphere")

    class(out) <- "bsimSetup"

  }
  return(out)


}

#' Bayesian Single-Index Regression with B-Spline Link and Half-Unit Sphere Prior
#'
#' @description
#' Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}
#' where the link \eqn{f(\cdot)} is represented by B-spline link and the
#' index vector \eqn{\theta} is on half-unit sphere.
#'
#' @inheritParams bsFisher
#'
#' @details
#' \strong{Model} The single–index model uses a \eqn{m}-order polynomial spline with \eqn{k} interior knots and intercept as follows:
#' \deqn{f(t) = \sum_{j=1}^{1+m+k} B_j(t)\,\beta_j} on \eqn{[a, b]} with \eqn{t_i = X_i' \theta, i = 1,\cdots, n}
#' and \eqn{\|\theta\|_2 = 1}. \eqn{\{\beta_j\}_{j=1}^{m+k+1}} are spline coefficients and \eqn{a_\theta}, \eqn{ b_\theta} are boundary knots where \eqn{a_{\theta} = min(t_i, i = 1, \cdots, n)},
#' and \eqn{b_{\theta} = max(t_i, i = 1,\cdots, n)}. Variable selection is encoded by a binary vector \eqn{\nu}, equivalently
#' setting components of \eqn{\theta} to zero when \eqn{\nu_i = 0}.
#'
#' \strong{Priors}
#' \itemize{
#' \item The variable selection indicator \eqn{\nu} has Beta–Bernoulli hierarchy prior. Beta hyper-prior on the Bernoulli–inclusion probability \eqn{w},
#'     inducing \eqn{p(\nu) \propto \mathrm{Beta}(r_1 + n_\nu, r_2 + p - n_\nu)} where  \eqn{n_\nu = \Sigma_{i=1}^{p}I(\nu_i = 1)}.
#'     \eqn{r_1, r_2} are shape and rate parameter of beta distribution.
#'   \item Free‑knot prior: the number of knots \eqn{k} with mean \eqn{\lambda_k}. The knot locations \eqn{\xi_i, i = 1,...,k} have a Dirichlet prior on the scaled interval \eqn{[0, 1]}.
#'   \item Index vector prior is uniform on the half‑sphere of dimension \eqn{n_\nu} with first nonzero positive.
#'   \item Conjugate normal–inverse-gamma on \eqn{(\beta, \sigma^2)} enables analytic integration for RJMCMC with covariance \eqn{\tau \Sigma_0}.
#' }
#'
#' \strong{Sampling} Posterior exploration follows a hybrid RJMCMC with six move types:
#' add/remove predictor \eqn{\nu}, update \eqn{\theta}, add/delete/relocate a knot. The \eqn{\theta} update is a random‑walk
#' Metropolis via local rotations in a two‑coordinate subspace. Knot changes use simple proposals with tractable acceptance ratios.
#' Further sampling method is described in Wang (2009).
#'
#' \strong{Prior hyper-parameters}
#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
#'  \enumerate{
#'  \item Index vector: \code{index_nu_r1, index_nu_r2} gives the shape and rate parameter of beta-binomial prior, respectively. (default \code{index_nu_r1 = 1, index_nu_r2 = 1}).
#' \item Link function: B-spline knots, basis and coefficient setup.
#' \itemize{
#'   \item{knots: Free-knot prior for the spline. \code{link_knots_lambda_k} is the Poisson mean for the number of
#'     interior knot \eqn{k} (default \code{5}). \code{link_knots_maxknots} is the maximum number of interior knots.
#'     If \code{link_knots_maxknots} is \code{NULL}, the number of interior knots is randomly drawn from a Poisson distribution.}
#'   \item{basis: For the basis of B-spline,  \code{link_basis_degree} is the spline
#'     degree (default \code{2}).}
#'   \item{beta: For the coefficient of B-spline,
#'    By default, \code{link_beta_mu} is a zero vector, \code{link_beta_tau} is set to the sample size,
#'     and \code{link_beta_Sigma0} is the identity matrix of dimension \eqn{1 + k + m},
#'     where \eqn{k} is the number of interior knots and \eqn{m} is the spline order.}}
#' \item Error variance (\code{sigma2}): Inverse gamma prior is assigned to \eqn{\sigma^2}
#'  where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
#'     Small values for shape and rate parameters yield a weakly-informative prior on \eqn{\sigma^{2}}. (default \code{sigma2_shape = 0.001, sigma2_rate = 0.001})
#'  }
#'
#' \strong{Initial values}
#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
#'  \enumerate{
#'  \item Index vector: \code{index_nu} is binary vector indicating active predictors for the index.
#' \code{index} is initial unit-norm index vector \eqn{\theta} (automatically normalized if necessary, with the first nonzero element made positive for identifiability).
#'     The vector length must match the number of predictor.
#'     Ideally, positions where \code{index_nu} has a value of 1 should correspond to nonzero elements in \eqn{\theta}; elements corresponding to \code{index_nu} = 0 will be set to zero.
#'  \item Link function: \code{link_k} is initial number of interior knots. \code{link_knots} is initial vector of interior knot positions in \eqn{[0, 1]}, automatically scaled to the true boundary.
#'     Length of this vector should be equal to the initial value of \code{k}.
#'     \code{link_beta} is initial vector of spline coefficients. Length should be equal to the initial number of basis functions with intercept (\eqn{1 + k + m}).
#'   \item Error variance (\code{sigma2}): Initial scalar error variance. (default \code{0.01})
#'  }
#'
#' @inherit bsFisher return
#'
#' @examples
#' \donttest{
#' set.seed(123)
#' n <- 200; d <- 4
#' theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
#' f <- function(u) u^2 * exp(u)
#' sigma <- 0.5
#' X <- matrix(runif(n * d, -1, 1), nrow = n)
#' index_vals <- as.vector(X %*% theta)
#' y <- f(index_vals) + rnorm(n, 0, sigma)
#' simdata <- data.frame(x = X, y = y)
#' colnames(simdata) <- c(paste0("X", 1:4), "y")
#'
#' # One tool version
#' fit1 <- bsSphere(y ~ ., data = simdata,
#'                  niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Split version
#' models <- bsSphere_setup(y ~ ., data = simdata)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                    nchains = 1, setSeed = TRUE, inits = initList,
#'                    samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' summary(fit2)
#' }
#'
#' @references
#' Wang, H.-B. (2009). Bayesian estimation and variable selection for single index models.
#' \emph{Computational Statistics & Data Analysis}, 53, 2617–2627.
#'
#' Hornik, K., & Grün, B. (2014). movMF: an R package for fitting mixtures of von Mises-Fisher distributions.
#' \emph{Journal of Statistical Software}, 58, 1-31.
#'
#' @name bsSphere
#' @export
bsSphere <- function(formula, data,
                     prior = NULL,
                     init = NULL,
                     monitors = NULL, niter = 10000, nburnin=1000,
                     thin = 1, nchain = 1, setSeed = FALSE){

  return(bsSphere.default(formula = formula, data  = data,
                          prior = prior,
                          init = init,
                          sampling = TRUE,
                          monitors = monitors, niter = niter, nburnin = nburnin,
                          thin = thin, nchain = nchain, setSeed = setSeed))
}

#' @rdname bsSphere
#' @export
bsSphere_setup <- function(formula, data,
                           prior = NULL,
                           init = NULL,
                           monitors = NULL, niter = 10000, nburnin=1000,
                           thin = 1, nchain = 1, setSeed = FALSE){
  return(bsSphere.default(formula = formula, data  = data,
                          prior = prior,
                          init = init,
                          sampling = FALSE,
                          monitors = monitors, niter = niter, nburnin = nburnin,
                          thin = thin, nchain = nchain, setSeed = setSeed))
}

bsSphere.default <- function(formula, data,
                     prior = NULL,
                     init = NULL,
                     sampling = TRUE,
                     monitors = NULL, niter = 10000, nburnin=1000,
                     thin = 1, nchain = 1, setSeed = FALSE
){

  start1 <- Sys.time()
  index <- 0; k <- 0; knots <- 0; sigma2 <- 0

  # check sampling, prior, init parameters for independent execution --------
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin, nchain,
    prior, init, "sphere", "bspline"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  # global environment

  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # aa_bspline_ver3
    "SplineState", "any_duplicated", "mat_wo_col1", "update_spline_df", "update_x_index",
    "update_knot_sequence", "get_basis_simple", "simplify_knots", "get_inside_x",
    "gen_default_internal_knots", "SplineBase1", "SplineBase2", "basis", "bsNimble", "bsBasis",

    # bsSphere
    "transX_sp","estBetaInit_sp","computeSig","ComputeS","logdet_nim",
    "postll_bspline_sphere","postll_knots","betaFunction","computeA1_nu1",
    "computeA1_nu0","computeA2_add","computeA2_delete","rnormTrun","newKnots",
    "prop_add","prop_delete","rtruncnorm","pred_bsplineSphere",
    "nuSampler_bspline_sphere","indexSampler_bspline_sphere","knotsSampler_bspline_sphere",
    "betaSampler_bspline_sphere","sigma2Sampler_bspline_sphere", "transX_fisher",

    # distribution
    "dunitSphere", "runitSphere","dKnotsSimple", "rKnotsSimple",

    # utils
    "pred_fitted"
  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())

  suppressMessages(
  nimble::registerDistributions(list(
    dKnotsSimple = list(
      BUGSdist = "dKnotsSimple(a, b, k, alpha)",
      types = c("value = double(1)", "a = double(0)", "b = double(0)", "k = double(0)",
                "alpha = double(1)"),
      discrete = FALSE
    )
  ), verbose = FALSE))

  suppressMessages(
  nimble::registerDistributions(list(
    dunitSphere = list(
      BUGSdist     = "dunitSphere(dim)",
      types        = c("value = double(1)",
                       "dim   = double(0)"),
      discrete     = FALSE
    )
  ), verbose = FALSE)

  )

  # check data dimension
  # Data
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
           length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  }else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  # data dimension
  N <- length(Y)
  p <- ncol(X)


  # Model code
  Rmodelcode <- nimbleCode({
    for (i in 1:p) {
      nu[i] ~ dbern(r1/(r1+r2))
    }

    # index
    index[1:p] ~ dunitSphere(p) ## half unit sphere

    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i, 1:p] * index[1:p])
    }

    # boundary knots
    a_alpha <- min(Xlin[1:N])
    b_alpha <- max(Xlin[1:N])

    # knots
    k ~ dpois(lambda_k)
    knots[1:maxknots] ~ dKnotsSimple(a = a_alpha, b = b_alpha, k = k, alpha = alpha[1:maxknots])

    # Design matrix - b spline basis
    numBasis <- degree + k + 1 # real dimension
    Xmat[1:N, 1:maxBasis] <- transX_sp(Xlin[1:N], degree, knots[1:maxknots], k, maxBasis, a_alpha, b_alpha)

    # likelihood
    sigma2 ~ dinvgamma(a_sig, b_sig)
    covBeta[1:maxBasis, 1:maxBasis] <- (sigma2 * tau) * Sigma0[1:maxBasis, 1:maxBasis]
    beta[1:maxBasis, 1] ~ dmnorm(mubeta[1:maxBasis],
                                 cov = covBeta[1:maxBasis, 1:maxBasis])

    linkFunction[1:N, 1] <- Xmat[1:N, 1:maxBasis] %*% beta[1:maxBasis, 1]
    covY[1:N, 1:N] <- sigma2 * Sigma[1:N, 1:N]
    Y[1:N, 1] ~ dmnorm(linkFunction[1:N, 1], cov = covY[1:N, 1:N])

  })


  # Prior parameters
  ## check data dimension and save
  ## index - nu
  if (is.null(prior$index$nu$r1) || is.null(prior$index$nu$r2)){
    stop("nu prior is not available.")
  }
  if (prior$index$nu$r1 <= 0 || prior$index$nu$r2 <= 0){
    stop("nu prior should be positive.")
  }

  r1 <- prior$index$nu$r1
  r2 <- prior$index$nu$r2

  ## link - knots
  if (is.null(prior$link$knots$lambda_k) || is.null(prior$link$knots$lambda_k)){
    stop("knots prior is not available.")
  }
  if (prior$link$knots$lambda_k <= 0){
    stop("knots prior should be positive.")
  }

  lambda_k <- prior$link$knots$lambda_k

  maxknots <- ifelse(is.null(prior$link$knots$maxknots),
                     qpois(1-0.01, lambda = lambda_k), prior$link$knots$maxknots)

  if (length(maxknots) >= 2 || maxknots < 0){
    stop("Prior link (maxknots) has incorrect value.")
  }

  ## link - basis
 if (is.null(prior$link$basis$degree)||length(prior$link$basis$degree) >= 2 || prior$link$basis$degree< 0){
    stop("Prior basis (degree) has incorrect value.")
  } else{
    degree <- prior$link$basis$degree
  }

  alpha <- rep(1/maxknots, maxknots)
  maxBasis <- degree + 1 + maxknots


  ## beta
  ## tau
  tau <- ifelse(is.null(prior$link$beta$tau), N, prior$link$beta$tau)
  if (length(tau) >= 2 || tau < 0){
    stop("Prior beta (tau) has incorrect value.")
  }

  ## mu
  if (!is.vector(prior$link$beta$mu) & !is.null(prior$link$beta$mu)){
    stop("Prior beta (mu) should be vector.")
  }
  if (!is.null(prior$link$beta$mu) & length(prior$link$beta$mu) < maxBasis ){
    stop("Incorrect dimention on prior beta (mu).")
  }

  if (is.null(prior$link$beta$mu)){
    mubeta <- rep(0, maxBasis)
  } else{
    mubeta <- c(prior$link$beta$mu, rep(0, maxBasis - length(prior$link$beta$mu)))
  }



  ## Sigma0

  if (is.null(prior$link$beta$Sigma0)){
    Sigma0 <- diag(1, maxBasis)
  } else{
    Sigma0 <- prior$link$beta$Sigma0
  }
  if (!is.null(prior$link$beta$Sigma0) & !is.matrix(prior$link$beta$Sigma0)){
    stop("Prior beta (Sigma0) should be matrix.")
  }

  Sigma <- diag(1, N)


  ## sigma
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (shape) has incorrect value.")
  } else{
    a_sig <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (rate) has incorrect value.")
  } else{
    b_sig <- prior$sigma2$rate
  }


  # Initialize
  ## nu
  if (!is.null(init$index$nu) & (length(init$index$nu) != p)){
    stop("Incorrect dimension on initial value of nu.")
  }

  if (!is.null(init$index$nu) & (!all(init$index$nu %in% c(0, 1)))){
    stop("nu should consist of 0 or 1.")
  }

  ## index
  if (!is.null(init$index$index) & (length(init$index$index) != p)){
    stop("Incorrect dimension on initial value of index")
  }

  # seed
  seedNum <- rep(FALSE, nchain)
  seedNum <- rep(FALSE, nchain)

  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }

  inits_list <- lapply(seq_len(nchain),
                       function(j) initFunction_bS(nu = init$index$nu, index = init$index$index,
                                                   k = init$link$k, beta = init$link$beta,
                                                   sigma2 = init$sigma2, knots = init$link$knots,
                                                   p = p, X = X, Y = Y, lambda_k = lambda_k,
                                                   maxknots = maxknots, degree = degree,
                                                   maxBasis = maxBasis, tau = tau,
                                                   Sigma0 = Sigma0,setSeed = seedNum[j]))
  firstInit <- inits_list[[1]]


  # Build model
  message("== Building model ==")
  suppressMessages(simpleModel <- nimbleModel(Rmodelcode,
                                              data = list(X = X, Y = Y),
                                              constants = list(p = p, N = N, r1 = r1, r2 = r2,
                                                               lambda_k = lambda_k,maxknots = maxknots,
                                                               a_sig = a_sig, b_sig = b_sig, alpha= alpha,
                                                               tau = tau, degree = degree, maxBasis = maxBasis,
                                                               mubeta = mubeta, Sigma0 = Sigma0,Sigma = Sigma),
                                              inits = firstInit))

  # Assign samplers
  message("== Assigning samplers ==")
  # List that is needed for fitting the model
  monitorsList <- c("index", "nu", "sigma2", "linkFunction", "beta",
                    "k", "knots", "numBasis", "a_alpha", "b_alpha", "Xlin")

  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
                                             monitors = monitorsList,
                                             print = FALSE))


  # mcmcConf1$printSamplers(executionOrder= TRUE)
  mcmcConf$removeSamplers(c("nu"))
  mcmcConf$addSampler(target = c("nu"),
                       type   = nuSampler_bspline_sphere)

  mcmcConf$removeSamplers(c("index"))
  mcmcConf$addSampler(target = c("index"),
                       type   = indexSampler_bspline_sphere)

  mcmcConf$removeSamplers(c("k", "knots"))
  mcmcConf$addSampler(target = c("k","knots"),
                       type   = knotsSampler_bspline_sphere)

  mcmcConf$removeSamplers(c("beta"))
  mcmcConf$addSampler(target = c("beta"),
                       type   = betaSampler_bspline_sphere)

  mcmcConf$removeSamplers(c("sigma2"))
  mcmcConf$addSampler(target = c("sigma2"),
                       type   = sigma2Sampler_bspline_sphere)


  mcmc1 <- buildMCMC(mcmcConf)



  if (!sampling){ # not mcmc sampling
    samples <- NULL

  } else{
    # Compile
    start2 <- Sys.time()
    message("== Compiling model ==")
    suppressMessages(CsimpleModel <- compileNimble(simpleModel))
    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1,
                                            project = simpleModel,
                                            resetFunctions = TRUE))
    end2 <- Sys.time()

    # Sampling

    message("== Running MCMC ==")
    mcmc.out <- NULL
    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                        thin = thin,
                        nchains = nchain, setSeed = seedNum,
                        inits = inits_list,
                        summary = FALSE, samplesAsCodaMCMC = TRUE)


    # output
    ## combine all chains
    sampMCMC <- mcmc.out
    samples <- sampleBind(sampMCMC, nchain)

  }

  end1 <- Sys.time()

  # inputOptions <- NULL
  if (!sampling){
    time <- NULL
  } else{
    # total time - compile time
    time_wo_compile <- difftime(end1, start1, units = "secs") -
      difftime(end2, start2, units = "secs")

    time <- list(time_wo_compile = time_wo_compile)

  }

  # Results ------------------------------------------------------------------
  #Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  # input options
  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = list(nu = list(r1 = r1, r2 = r2)),
                                    link = list(knots = list(lambda_k = lambda_k, maxknots = maxknots),
                                                basis = list(degree = degree),
                                                beta = list(mu = mubeta, tau = tau, sigma0 = Sigma0)),
                                    sigma2 = list(shape = a_sig, rate = b_sig)),
                       init = inits_list,
                       samplingOptions = list(sampling = sampling,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       time = time)

  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = sampMCMC, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "bsSphere")

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "bsSphere")

    class(out) <- "bsimSetup"

  }
  return(out)

}

dKnotsSimple <- nimble::nimbleFunction(
  run = function(x = double(1), a = double(0), b = double(0), k = double(0),
                 alpha = double(1), log = integer(0, default = 0)) {
    returnType(double(0))
    K <- k
    real_alpha <- alpha[1:K]

    gaps <- numeric(K + 1)
    gaps[1] <- x[1] - a
    if (K > 1){
      for (i in 2:K) {
        gaps[i] <- x[i] - x[i-1]
      }
      gaps[K+1] <- b - x[K]
    }
    gaps <- gaps / (b - a)

    alpha_full <- c(real_alpha, 1.0)
    logProb <- ddirch(gaps, alpha_full, log = TRUE)
    if (log) return(logProb)
    else return(exp(logProb))
  }
)


rKnotsSimple <- nimble::nimbleFunction(
  run = function(n = integer(0), a = double(0), b = double(0), k = double(0),
                 alpha = double(1)) {
    returnType(double(1))
    K <- k
    real_alpha <- alpha[1:K]
    alpha_full <- c(alpha, 1.0)
    gaps <- rdirch(1, alpha_full)
    out <- numeric(K)
    cs <- 0.0
    for (i in 1:K) {
      cs <- cs + gaps[i]
      out[i] <- a + cs * (b - a)
    }
    return(out)
  }
)



dunitSphere <- nimble::nimbleFunction(
  run = function(x = double(1), dim =double(), log = integer(0, default = 0)){
    returnType(double(0))

    norm_x <- 0
    for (i in 1:dim) {
      norm_x <- norm_x + x[i] * x[i]
    }
    norm_x <- sqrt(norm_x)

    ## Uniform Density on the Unit Sphere
    density <- (2*pi^(length(x)/2))/gamma(length(x)/2)

    if(log) return(log(density))
    return(density)
  }
)

runitSphere <- nimble::nimbleFunction(
  run = function(n = integer(0),dim = double()) {
    returnType(double(1))
    if (dim <= 0) return(numeric(0))

    z <- rnorm(dim, 0, 1)
    z <- z / sqrt(sum(z * z))
    return(z)
  }
)

transX_sp <- nimble::nimbleFunction(
 run = function(Xlin = double(1), degree = integer(0), knots = double(1),
                 k = integer(0), maxBasis = integer(0), a_alpha = double(0),
                 b_alpha = double(0)){
    returnType(double(2))
    realKnots <- knots[1:k]
    boundary_knots <- nimC(a_alpha, b_alpha)
    X <- bsBasis(Xlin, knots = realKnots, degree = degree, intercept = TRUE,
                 boundary_knots = boundary_knots)
    N <- nimDim(X)[1]; ncol <- nimDim(X)[2]
    result <- nimMatrix(0, nrow = N, ncol = maxBasis)
    result[1:N, 1:ncol] <- X
    return(result)

  }
)

estBetaInit_sp <-  nimble::nimbleFunction(
  run = function(Xmat = double(2), Y = double(2), numBasis = integer(0)){
    returnType(double(2))
    maxBasis <- nimDim(Xmat)[2]
    Xalpha <- Xmat[,1:numBasis]
    lambda <- 1e-8
    part1 <- inverse(t(Xalpha) %*% Xalpha + diag(rep(lambda, numBasis)))
    part2 <- t(Xalpha) %*% Y
    result <- part1 %*% part2 # p*1

    total <- matrix(0, nrow = maxBasis, ncol = 1)
    total[1:numBasis, 1] <- result

    return (total)
  }
)


# (numBasis＊numBasis)
computeSig <- nimble::nimbleFunction(
  run = function(tau = double(0), Sig0 = double(2), xmat = double(2), numBasis = integer(0)){
    returnType(double(2))
    part1 <- inverse(Sig0) / tau
    part2 <- t(xmat) %*% xmat
    result <- inverse(part1 + part2)
    result2 <- result[1:numBasis, 1:numBasis]
    return(result2)
  }
)


ComputeS <- nimble::nimbleFunction(
  run = function(y = double(2), xmat = double(2), Sig = double(2), numBasis = integer(0)){
    returnType(double(0))
    corXmat <- xmat[, 1:numBasis]
    part1 <- t(y) %*% y
    part2 <- t(y) %*% corXmat
    part3 <- part2 %*% Sig %*% t(part2)
    result <- part1 - part3
    return(result[1,1])
  }
)

# About sampling - ll
logdet_nim <- nimble::nimbleFunction(
  run = function(M = double(2)) {
    returnType(double(0))
    U <- chol(M)
    out <- 2 * sum(log(diag(U)))
    return(out)
  }
)


postll_bspline_sphere <- nimble::nimbleFunction(
  setup = function(model){
    tau2 <- model$getConstants()$tau
    Sigma0 <- model$getConstants()$Sigma0
    y <- model$Y
    N <- model$getConstants()$N
  },
  run = function(){
    returnType(double(0))
    k <- model$k[1]
    numBasis <- model$numBasis
    xmat <- model$Xmat
    b_alpha <- model$b_alpha[1]
    a_alpha <- model$a_alpha[1]

    numBasisI <- as.integer(numBasis[1])
    Sigma <- computeSig(tau2, Sigma0, xmat, numBasisI)
    part1 <- logdet_nim(Sigma)/2
    part2 <- logdet_nim(Sigma0[1:numBasisI, 1:numBasisI])/2
    part3 <- log(ComputeS(y, xmat, Sigma, numBasisI)) * (-(N/2))
    part4 <- -k * log(b_alpha - a_alpha)

    result <- part1 - part2 + part3 + part4


    return(result)
  }
)



postll_knots <- nimble::nimbleFunction(
  setup = function(model){
    tau2 <- model$getConstants()$tau
    Sigma0 <- model$getConstants()$Sigma0
    y <- model$Y
    N <- model$getConstants()$N
    k <- model$k
  },
  run = function(){
    returnType(double(0))
    numBasis <- model$numBasis
    xmat <- model$Xmat
    b_alpha <- model$b_alpha
    a_alpha <- model$a_alpha

    numBasisI <- as.integer(numBasis[1])
    Sigma <- computeSig(tau2, Sigma0, xmat, numBasisI)
    part1 <- logdet_nim(Sigma)/2
    part2 <- logdet_nim(Sigma0[1:numBasisI, 1:numBasisI])/2
    part3 <- log(ComputeS(y, xmat, Sigma, numBasisI)) * (-(N/2))

    result <- part1 - part2 + part3

    return(result)
    # return(0.0)

  }
)

# Compute A1
betaFunction <- nimble::nimbleFunction(
  run = function(a = double(0), b = double(0), log = logical(0, default = TRUE)){
    returnType(double(0))
    result <- lgamma(a) + lgamma(b) - lgamma(a+b)
    if (log){
      return (result)
    } else{
      return(result)
    }
  }
)


computeA1_nu1 <- nimble::nimbleFunction(
  run = function(r1 = double(0), r2 = double(0), nnu = double(0),
                 p = double(0), r3 = double(0), r4 = double(0),
                 alpha = double(0)){
    returnType(double(0))

    part1 <- log(r2 + p - nnu) - log(nnu + r1 - 1)
    part2 <- dbeta(alpha^2, r3, r4, log = TRUE)
    part3 <- betaFunction(0.5, (nnu-1)/2)
    part4 <- -0.5*log(alpha^2)+((nnu-1)/2)*log(1-alpha^2)
    result <- part1 + part2 + part3 - part4
    return(result)
  }
)


computeA1_nu0 <- nimble::nimbleFunction(
  run = function(r1 = double(0), r2 = double(0), nnu = double(0),
                 p = double(0), r3 = double(0), r4 = double(0),
                 eta = double(0)){
    returnType(double(0))

    part1 <- log(nnu + r1) - log(r2 + p - nnu -1)
    part2 <- -dbeta(eta, r3, r4, log = TRUE)
    part3 <- -0.5*log(eta)+(nnu/2)*log(1-eta)
    part4 <- betaFunction(0.5, (nnu)/2)

    result <- part1 + part2 + part3 - part4
    return(result)
  }
)

# compute A2
computeA2_add <- nimble::nimbleFunction(
  run = function(k = double(0), minKnot = double(0), maxKnot = double(0),
                 minBound = double(0), maxBound = double(0)){
    returnType(double(0))
    part1 <- log(k+1)
    part2 <- log(maxKnot - minKnot)
    part3 <- log(maxBound - minBound)
    result <- part1 + part2 - part3
    return(result)
  }
)



computeA2_delete <- nimble::nimbleFunction(
  run = function(k = double(0), minKnot = double(0), maxKnot = double(0),
                 minBound = double(0), maxBound = double(0)){
    returnType(double(0))
    part1 <- -log(k)
    part2 <- log(maxKnot - minKnot)
    part3 <- log(maxBound - minBound)
    result <- part1 + part3 - part2
    return(result)
  }
)


# sampling truncated normal
rnormTrun <- nimble::nimbleFunction(
  run = function(mean = double(0), sd = double(0)){
    returnType(double(0))
    samp <- rnorm(1, mean, sd)

    while(samp <= -3.141593 | samp >= 3.141593){
      samp <- rnorm(1, mean, sd)
    }

    return(samp)

  }
)

# calculate knots
newKnots <- nimble::nimbleFunction(
  run = function(oldknots = double(1), olda = double(0), oldb = double(0),
                 newa = double(0), newb = double(0), k = double(0)){
    returnType(double(1))
    newknots <- numeric(nimDim(oldknots)[1])
    for (i in 1:k){
      old <- (oldknots[i] - olda) / (oldb - olda)
      newknots[i] <- newa + old * (newb - newa)
    }
    return(newknots)
  }
)

# nimSort - in bSpline2
# bk, dk, mk
prop_add <- nimble::nimbleFunction(
  run = function(lambda = double(0), k = double(0), tau = double(0)){
    returnType(double(0))
    if (k == 0){
      return(1)
    } else{
      calc <- lambda/((k+1)*sqrt(tau))
      result <- 0.4 * min(1, calc)
      return(result)
    }
  }
)


prop_delete <- nimble::nimbleFunction(
  run = function(lambda = double(0), k = double(0), tau = double(0)){
    returnType(double(0))
    if (k == 0){
      return(0)
    } else{
      calc <- (k*sqrt(tau))/lambda
      result <- 0.4 * min(1, calc)
      return(result)
    }
  }
)

# truncated normal
rtruncnorm <- nimble::nimbleFunction(
  run = function(n = integer(0),
                 mean = double(0), sigma = double(0),
                 lower = double(0), upper = double(0)) {
    returnType(double(1))

    # if (sigma <= 0) stop("sigma must be > 0")
    # if (lower >= upper) stop("need lower < upper")

    out <- numeric(n)

    zL <- (lower - mean) / sigma
    zU <- (upper - mean) / sigma

    pL <- 0.0
    if (lower <= -1.0e300) {
      pL <- 0.0
    } else {
      pL <- pnorm(zL, 0.0, 1.0)
    }

    pU <- 1.0
    if (upper >=  1.0e300) {
      pU <- 1.0
    } else {
      pU <- pnorm(zU, 0.0, 1.0)
    }

    for (i in 1:n) {
      u <- runif(1, pL, pU)
      out[i] <- mean + sigma * qnorm(u, 0.0, 1.0)
    }
    return(out)
  }
)

# initFunction_bS
initFunction_bS <- function(nu, index, k, beta, sigma2, knots,
                            p, X, Y, lambda_k, maxknots, degree, maxBasis,
                            tau, Sigma0, setSeed){
  init <- 0
  if (setSeed != FALSE){
    set.seed(setSeed)
  }

  # init_nu
  if (is.null(nu) & is.null(index)){
    init_nu <- c(1, rbinom(p-1, 1, 0.5))
  } else if (is.null(nu) & !is.null(index)){
    init_nu <- as.numeric(index != 0)
  } else{
    init_nu <- nu
  }

  # index
  if (is.null(index)){
    nonzero <- sum(init_nu)
    nonzeroidx <- which(init_nu == 1)
    temp <- runitSphere(1, nonzero)
    init_index <- rep(0, p)
    init_index[nonzeroidx] <-temp
  } else{
    bad_zero  <- which(init_nu == 0 & index != 0)
    bad_nonzero <- which(init_nu == 1 & index == 0)

    if (length(bad_zero) > 0) {
      warning(sprintf(
        "Theta is nonzero at positions where nu = 0: %s",
        paste(bad_zero, collapse = ", ")
      ))
    }

    if (length(bad_nonzero) > 0) {
      warning(sprintf(
        "Theta is zero at positions where nu = 1: %s",
        paste(bad_nonzero, collapse = ", ")
      ))
    }

    init_index <- index * init_nu
  }

  if (sum((init_index)^2) != 1){
    init_index <- init_index/sqrt(sum(init_index^2))
  }

  if(init_index[1] < 0){
    init_index[1] <- -init_index[1]
  }

  init_Xlin <- as.vector(X %*% matrix(init_index, ncol = 1))
  init_a_alpha <- min(init_Xlin); init_b_alpha <- max(init_Xlin)

  ## knots
  ## k (# of knots)
  if (is.null(k)){
    init_k <- rpois(1, lambda_k) + 1
  } else{
    init_k <- k
  }

  if ((!is.null(k)) & (length(init_k) >= 2 || !is.numeric(init_k))){
    stop("Error: Initial value of the number of knots(k) should be scalar.")
  }
  if ((!is.null(k)) & (init_k < 0)){
    stop("Error: Initial value of the number of knots(k) should be positive.")
  }
  if (!is.null(init_k) & !is.null(knots)){
    if(length(knots) != init_k){
      stop("Error: Number of knots and k is not the same.")
    }
  }

  ## location of knots (weights)
  if (!is.null(knots) & (knots <= 0 || knots >= 1)){
    stop("Error: Initial knots should be bigger than 0, less than 1.")
  }

  if (is.null(knots)){
    init_weights <- rdirch(1, rep(1, init_k))
  } else{
    init_weights <- knots
  }
  cumsum_w <- cumsum(init_weights); epsilon <- 0.05
  init_knots <- init_a_alpha +
    (init_b_alpha - init_a_alpha) * (epsilon + (1 - 2 * epsilon) * cumsum_w)
  init_knots <- c(init_knots, rep(0, maxknots - init_k))
  init_numBasis <- init_k + degree + 1
  init_Xmat <- transX_sp(init_Xlin, degree = degree, knots = init_knots, init_k, maxBasis, init_a_alpha, init_b_alpha)

  if (is.null(beta)){
    init_beta <- estBetaInit_sp(init_Xmat, Y, init_numBasis)
  } else{
    init_beta <- c(beta, rep(0, maxBasis - init_numBasis))
  }

  if ((!is.null(beta) & (length(init_beta) > maxBasis)) ||
      (!is.null(beta) & (length(init_beta) != init_numBasis))){
    stop("Error: Incorrect dimension on initial value of beta.")
  }

  ## sigma2
  init_sigma2 <- sigma2
  if (length(sigma2) >= 2 || !is.numeric(sigma2)){
    stop("Error: Initial value of sigma2 should be scalar.")
  }
  if (sigma2 < 0){
    stop("Error: Initial value of sigma2 should be positive.")
  }
  init_linkFunction <- init_Xmat %*%  init_beta
  init_covbeta <- init_sigma2 * tau * Sigma0
  N <- nrow(X)
  init_covY <- diag(rep(init_sigma2, N))

  return(list(nu = init_nu, index = init_index,
              Xlin = init_Xlin,a_alpha = init_a_alpha,
              b_alpha = init_b_alpha, k = init_k,
              knots = init_knots, numBasis = init_numBasis,
              Xmat = init_Xmat, sigma2 = init_sigma2,
              beta = init_beta, covBeta = init_covbeta,
              linkFunction = init_linkFunction))

}

# prediction
pred_bsplineSphere <- nimbleFunction(
  run = function(newdata = double(2), indexSample = double(2),
                 knotsSample = double(2), XlinSample = double(2),
                 betaSample = double(2), sigma2_samples = double(1),
                 k = double(1), dfSample = double(1),
                 mina = double(1), maxb = double(1), nsamp = double(0),
                 degree = double(0), prediction = integer(0)){
    returnType(double(2))

    # prediction = 1: latent, prediction = 2: response
    new_ncol <- nimDim(newdata)[1]
    testPred <- nimMatrix(0, nrow = nsamp, ncol = new_ncol)

    for (i in 1:nsamp){
      sampleZ <- newdata %*% matrix(indexSample[i, ], ncol = 1)
      numKnots <- k[i]
      Xmat <- bsBasis(sampleZ[,1],
                      degree = degree, knots = knotsSample[i,1:numKnots],
                      intercept = TRUE,
                      boundary_knots = c(mina[i], maxb[i]))
      linkPred <- Xmat %*% betaSample[i,1:dfSample[i]]
      if (prediction == 1){ # latent
        testPred[i,] <- linkPred[ ,1]
      } else if (prediction == 2){ # response
        # predsigma <- chol(diag(rep(sqrt(sigma2_samples[i]), new_ncol)))
        # pred <- t(rmnorm_chol(1, mean = linkPred[,1], cholesky = predsigma, prec_param  = FALSE))

        std_dev <- sqrt(sigma2_samples[i])
        pred <- nimMatrix(0, ncol = new_ncol, nrow = 1)
        for(j in 1:new_ncol) {
          pred[1, j] <- linkPred[j, 1] + rnorm(1, 0, std_dev)
        }
        idxmin <- (sampleZ[,1] < (mina[i]))
        idxmax <- (sampleZ[,1] > (maxb[i]))

        if (sum(idxmin) != 0){
          cat("pass min \n")
          minVal_idx <- which(XlinSample[i,] == min(XlinSample[i,]))
          predmin <- min(pred[1, minVal_idx])
          pred[1, idxmin] <- nimRep(predmin, sum(idxmin))
        }

        if (sum(idxmax) != 0){
          cat("pass max \n")
          maxVal_idx <- which(XlinSample[i,] == max(XlinSample[i,]))
          predmax <- max(pred[1, maxVal_idx])
          pred[1, idxmax] <- nimRep(predmax, sum(idxmax))
        }

        testPred[i,] <- pred[1,]
      }

    }
    return(testPred)
  }
)

## Define samplers
## Gibbs sigma
gibbsSampler_sigma2 <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control) {
    calcNodes <- model$getDependencies(target)
    N <- model$getConstants()$N
    a <- model$getConstants()$a
    b <- model$getConstants()$b
    y <- model$y
  },
  run = function() {
    # nimPrint("============Sigma=================")
    eta <- model$linkFunction
    # nimPrint(eta)
    err <- sum((y - eta)^2)/2
    # nimCat("err:", err, "\n")
    newSigma <- 1/rgamma(1, shape = a + (N/2), rate = (b + err))
    # nimCat("sigma2: ", newSigma, "\n")

    model[[target]] <<- newSigma
    model$calculate(calcNodes)

    nimCopy(from = model, to = mvSaved, row = 1, nodes = calcNodes, logProb = TRUE)
  },
  methods = list(reset = function() {})
)

## Gibbs kappa
gibbsSampler_kappa <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control) {
    calcNodes <- model$getDependencies(target)
    kappas <- nimSeq(model$getConstants()$kappa_a,
                     model$getConstants()$kappa_b,
                     control$grid.width)
    N <- model$getConstants()$N
  },
  run = function() {
    # nimPrint("============kappa=================")
    # nimPrint("Xlinear")
    # nimPrint(model$Xlin)
    # nimPrint("linkFunction")
    # nimPrint(model$linkFunction)

    probs <- nimNumeric(length(kappas), init = TRUE)
    idx <- nimOrder(model$Xlin)
    sortXlin <- model$Xlin[idx]
    eta <- model$linkFunction ; etaMat <- nimMatrix(eta[idx], ncol = 1) # n * 1
    # eta <- model$linkFunction; eta <- eta[idx]

    # nimPrint("sorted Xlinear")
    # nimPrint(sortXlin[1:10])
    # nimPrint("sorted linkFunction")
    # nimPrint(etaMat[1:10, 1])

    for (i in 1:length(kappas)) {
      C_inv <- invcov(sortXlin, kappas[i])
      # cat("kappa: ", kappas[i], "\n")
      # print(C_inv[1:5, 1:5])
      lp1 <- matrix(0.5 * log(det(C_inv)), nrow = 1, ncol = 1)
      lp2 <- -0.5 * (t(etaMat) %*% C_inv %*% etaMat)
      lp <- lp1 + lp2
      if (any(is.nan(lp))) {
        nimPrint("NaN detected in log posterior at kappa index ", i)
        lp <- matrix(-1e8, nrow = 1, ncol = 1)
      }
      if (lp[1,1] > 1e10){
        lp <- matrix(1e10, nrow = 1, ncol = 1)
      }
      # if (any(is.infinite(lp))){ ## 여기
      #   lp <- matrix(1e8, nrow = 1, ncol = 1)
      # }
      probs[i] <- lp[1, 1]
      # probs[i] <- lp
    }
    # nimCat("chosenIdx: ", probs, "\n")
    # nimCat("probability", probs[1:20], "\n")
    maxlp <- max(probs)
    probs <- exp(probs-maxlp)
    # probs <- exp(probs)

    chosen <- rcat(1, probs)

    storeKappa <- kappas[chosen:chosen]
    model$kappa <<- storeKappa
    # nimCat("kappa", storeKappa, "\n")

    model$calculate(calcNodes)
    nimCopy(from = model, to = mvSaved, row = 1, nodes = calcNodes, logProb = TRUE)
  },
  methods = list(reset = function() {})
)

## Update theta/eta
MH_thetaeta <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control) {
    calcNodes <- model$getDependencies(target)
    p <- model$getConstants()$p
    N <- model$getConstants()$N
    paramalpha <- model$getConstants()$c

    obj_fn     <- obj_btt_theta(model)
    tempList <- nimSeq(0.04, 0.09, 0.01)
    num <- rcat(1, nimRep(1, 6))
    idxx <-tempList[num] + 0.01
    # idxx <- 0.02
    thetas <- seq(idxx, pi-0.01, 0.1)
    # thetas <- seq(0.1, 1-0.1, 0.05)*pi # orig
    # thetas <- seq(0.01, pi-0.01, 0.1)
    grid.df  <- do.call(expand.grid,
                         replicate(p-1, thetas, simplify = FALSE))
    grid.df <- as.matrix(grid.df)
    num.calc <- nimDim(grid.df)[1]

  },
  run = function() {
    # nimPrint("============rest=================")
    # nimPrint("Xliner")
    # nimPrint(model$Xlin)
    # nimPrint("linkFunction")
    # nimPrint(model$linkFunction)

    # 1) Old eta, theta, theta, alpha(beta)
    oldtheta <- model$psi
    parambeta <- model$d
    oldBeta <- model$index
    orderf <- nimOrder(model$Xlin) # index
    oldeta <- model$linkFunction; oldeta <- oldeta[orderf] # sort f
    Stheta <- diag(nimRep(0.1, N))

    # current: oldtheta
    PP1 <- nimNumeric(num.calc, init = TRUE)
    for (i in 1:num.calc){
      input <- grid.df[i, ]
      PP1[i] <- obj_fn$run(input, model$kappa[1], model$linkFunction)
    }

    # nimCat("theta probs:", PP1[1:10], "\n")

    idxTEMP <- nimC(1:num.calc)[PP1 == max(PP1)]
    # idxTEMP <- which.max(PP1)
    t <- grid.df[idxTEMP, ]/pi # theta
    # nimCat("theta: ", t*pi, "\n")
    # nimCat("Chosen theta:", grid.df[idxTEMP, ], "\n")

    # surrogate data
    g <- rmnorm_chol(1, mean = oldeta, cholesky = chol(Stheta),
                     prec_param= FALSE)
    # nimCat("Surrogate eta:", g[1:10], "\n")

    z <- nimMatrix(model$Xlin, ncol = 1)
    z <- z[orderf, ] # (n*1)
    R1 <- nimMatrix(rep(1, N), ncol = 1) %*% t(z) # (n*1) * (1*n)
    R <- (abs(R1-t(R1))) # n*n matrix
    # kappa <- model$kappa
    kappa <- nimMatrix(rep(model$kappa, N*N), nrow = N, ncol = N)
    C <- exp(-kappa * R)
    Sigmatheta <- C

    Rtheta <- Stheta - Stheta %*% inverse(Stheta+Sigmatheta) %*% Stheta
    Rtheta <- (Rtheta + t(Rtheta))/2
    LRtheta <- t(chol(Rtheta))
    mtheta_g <- Rtheta %*% inverse(Stheta) %*% g

    # latent variable
    etaLatent <- inverse(LRtheta) %*% (oldeta-mtheta_g) # (100, 1)

    # nimCat("Surrogate eta:", etaLatent[1:10,], "\n")

    # 2) New eta, theta
    newparamebeta <- numeric(p-1) # d
    for(i in 1:(p-1)){
      newparamebeta[i] <- (paramalpha[i]-1-paramalpha[i]*t[1, i]+2*t[1, i])/t[1, i]
    }
    # nimCat("newparamebeta: ", newparamebeta, "\n")

    newtheta <- nimMatrix(rep(0, p-1), ncol = 1) # New possible theta
    for (i in 1:(p-1)){
      newtheta[i,1] <- rbeta(1, paramalpha[i], newparamebeta[i])
    }
    # nimCat("newtheta: ", newtheta[,1]*pi, "\n")

    newbeta <- alphaTheta(newtheta[,1] * pi)
    z1 <- nimMatrix(0, nrow = N, ncol = 1)
    xData <- model$x
    z1 <- xData %*% nimMatrix(newbeta, ncol = 1)
    model$Xlin <<- z1[,1]
    # for (i in 1:N) {
    #   for (j in 1:length(newbeta)) {
    #     z1[i, 1] <- z1[i, 1]  + xData[i, j] * newbeta[j]
    #   }
    # }
    z1 <- z1[orderf,]

    R1 <- nimMatrix(rep(1, N), ncol = 1) %*% t(z1)
    R <- (abs(R1-t(R1)))
    C <- exp(-kappa * R)
    Sigmathetaprime <- C

    Rthetaprime <- Stheta-Stheta %*% inverse(Stheta+Sigmathetaprime) %*% Stheta
    Rthetaprime <- (Rthetaprime + t(Rthetaprime))/2
    LRthetaprime <- t(chol(Rthetaprime))
    mtheta_gprime <- Rtheta %*% inverse(Stheta) %*% g
    newetaMat <- LRthetaprime %*% etaLatent + mtheta_gprime
    neweta   <- newetaMat[ ,1]
    # nimCat("New eta:", neweta[1:10], "\n")

    ## Beta transition kernel
    righttrans <- 0.0 # (t) -> (t+1)
    reversetrans <- 0.0 # (t+1) -> (t)
    for (i in 1:(p-1)){
      reversetrans <- reversetrans +
        dbeta(model$psi[i], paramalpha[i], newparamebeta[i], log = TRUE)
      righttrans <- righttrans +
        dbeta(newtheta[i,1], paramalpha[i], parambeta[i], log = TRUE)
    }

    # nimCat("New reversetrans:", reversetrans, "\n")
    # nimCat("New righttrans:", righttrans, "\n")

    # update
    model$d <<- newparamebeta
    model$psi <<- newtheta[,1]
    storeEta <- neweta[nimOrder(orderf*1.0)]
    model$linkFunction <<- storeEta
    model$index <<- newbeta

    # model$calculate(calcNodes)
    # model$calculate(c("index", "Xlin", "cov"))
    model$calculate(c("cov"))
    # acceptance rate
    L <- (dmnorm_chol(neweta, mean=rep(0,N),
                      cholesky = chol(Sigmathetaprime), prec_param = FALSE, log=TRUE) -
            dmnorm_chol(oldeta, mean=rep(0,N),
                        cholesky = chol(Sigmatheta), prec_param = FALSE, log=TRUE) +
            dmnorm_chol(g, mean=rep(0,N),
                        cholesky = chol(Sigmathetaprime+Rthetaprime),
                        prec_param = FALSE, log=TRUE) -
            dmnorm_chol(g, mean=rep(0,N),
                        cholesky = chol(Sigmatheta+Rtheta), prec_param = FALSE, log=TRUE)) +
      reversetrans - righttrans + thetaPrior(newtheta[,1]*pi) - thetaPrior((model$psi)*pi)

    ratio <- exp(L)
    # nimCat("ratio: ", ratio, "\n")
    u <- runif(1)
    # accept/reject
    if(u < ratio) { # accept
      # nimPrint("accept")
      nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
    } else {
      # nimPrint("reject")
      nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
    }

    ###########################################################################
    # Gibbs for linkFunction
    orderf <- nimOrder(model$Xlin) # integer index
    y <- nimMatrix(model$y, ncol = 1); y <- y[orderf,] # sort y
    sigma2     <- model$sigma2
    sigma2Inv  <- 1.0 / sigma2
    kappa_inv <- model$kappa[1]
    sortXlin <- model$Xlin[orderf]
    Cinv       <- invcov(sortXlin, kappa_inv)
    A <- Cinv + diag(rep(sigma2Inv, N))
    A <- (A + t(A))/2
    Sigma <- inverse(A) # matrix
    mu    <- (Sigma %*% y) # matrix (n*n) %*% (n*1)
    mu0 <- mu[,1] # vector
    mu0 <- mu0 * rep(sigma2Inv, N)

    L1 <- chol(Sigma)
    if (any(is.nan(L1))) {
      nimPrint("Cholesky decomposition failed in linkFunction sampler.")
    }

    # nimPrint("mu0")
    # nimPrint(mu0[1:10])
    # nimPrint("Sigma")
    # nimPrint(Sigma[1:5, 1:5])


    f <- rmnorm_chol(1, mean = mu0, cholesky = L1, prec_param = FALSE)
    reorder <- orderf * rep(1.0, N)
    idx <- nimOrder(reorder)
    f <- f[idx]
    # nimPrint("f - after")
    # nimPrint(f[1:10])

    ## update
    model$linkFunction  <<- f
    # nimPrint(f)
    # model$calculate(c("linkFunction"))
    nimCopy(from = model, to = mvSaved, row = 1,
            nodes  = "linkFunction", logProb = TRUE)

  },
  methods = list( reset = function () {} )

)

#' @importFrom ggplot2 ggplot aes geom_point geom_line labs theme_minimal theme geom_ribbon
# Initial values setting for bsPolar
initFunction_bP <- function(X, Y,
                            psi, sigma2, beta,
                            df, degree, delta,
                            index_direction,
                            setSeed){

  if (setSeed != FALSE){
    set.seed(setSeed)
  }

  n <- dim(X)[1]
  p <- dim(X)[2]

  # hyper-parameter d
  init_d <- rep(6000,p-1)


  # index
  if (is.null(psi)){
    init_psi <- runif(p-1,0.01,1)
    init_index <- alphaTheta(init_psi)
  } else{
    init_psi <- psi
    init_index <- alphaTheta(init_psi * pi)
  }
  if (sum((init_index)^2) != 1){
    init_index <- init_index/sqrt(sum(init_index^2))
  }


  # sigma2
  init_sigma2 <- sigma2
  init_Xlin <- as.vector(X %*% matrix(init_index, ncol = 1))
  init_Xmat <- transX_fisher(init_Xlin, df = df, degree = degree, delta = delta)
  init_rho <- gvcCV(init_Xmat, Y)

  # beta
  if (is.null(beta)){
    init_beta <- estBeta_fisher(init_Xmat, Y, init_rho)
  } else{
    init_beta <- beta
  }
  init_linkFunction <- init_Xmat %*%  init_beta

  return(list(d = init_d, psi = init_psi,
              index = init_index, Xlin = init_Xlin,
              Xmat = init_Xmat,
              sigma2 = init_sigma2, beta = init_beta,
              linkFunction = init_linkFunction))

}

# Initial values setting for bsSpike
initFunction_bSpike <- function(X, Y,
                                pi, nu, index,
                                sigma2, beta,
                                df, degree, delta,
                                setSeed){

  if (setSeed != FALSE){
    set.seed(setSeed)
  }
  n <- dim(X)[1]
  p <- dim(X)[2]

  ## pi, nu, theta
  init_pi <- pi
  if (length(pi) >= 2 || !is.numeric(pi)){
    stop("Error: Initial value of pi should be scalar.")
  }
  if (pi < 0){
    stop("Error: Initial value of pi should be positive.")
  }

  ## nu, theta
  if (!is.null(nu) & (length(nu) != p)){
    stop("Error: Incorrect dimension on initial value of nu.")
  }

  if (!is.null(nu) & (!all(nu %in% c(0, 1)))){
    stop("Error: Delta should consist of 0 or 1.")
  }

  if (is.null(nu) & is.null(index)){
    init_nu <- c(1, rbinom(p-1, 1 ,prob = init_pi))
  } else if (is.null(nu) & !is.null(index)){
    init_nu <- as.numeric(index!=0)
  } else{
    init_nu <- nu
  }


  if (!is.null(index) & (length(index) != p)){
    stop("Error: Incorrect dimension on initial value of index")
  }

  if (is.null(index)){
    nonzero <- sum(init_nu)
    nonzeroidx <- which(init_nu == 1)
    temp <- rtruncnorm(n = sum(nonzeroidx), 0, 1, lower = 0, upper = Inf)
    init_index <- rep(0, p)
    init_index[nonzeroidx] <-temp[nonzeroidx]
  } else{
    bad_zero  <- which(init_nu == 0 & index != 0)
    bad_nonzero <- which(init_nu == 1 & index == 0)

    if (length(bad_zero) > 0) {
      warning(sprintf(
        "Theta is nonzero at positions where nu = 0: %s",
        paste(bad_zero, collapse = ", ")
      ))
    }

    if (length(bad_nonzero) > 0) {
      warning(sprintf(
        "Theta is zero at positions where nu = 1: %s",
        paste(bad_nonzero, collapse = ", ")
      ))
    }

    init_index <- index * init_nu
  }

  if (sum(init_index) < 0){
    init_index <- (-1) * init_index
  }

  init_Xlin <- as.vector(X %*% matrix(init_index, nrow = p))


  # sigma2
  init_sigma2 <- sigma2
  init_Xlin <- as.vector(X %*% matrix(init_index, ncol = 1))
  init_Xmat <- transX_fisher(init_Xlin, df = df, degree = degree, delta = delta)
  init_rho <- gvcCV(init_Xmat, Y)

  # beta
  if (is.null(beta)){
    init_beta <- estBeta_fisher(init_Xmat, Y, init_rho)
  } else{
    init_beta <- beta
  }
  init_linkFunction <- init_Xmat %*%  init_beta

  return(list(pi = init_pi, nu = init_nu,
              index_raw = init_index,
              index_temp = init_index,
              index = init_index,
              Xlin = init_Xlin,
              Xmat = init_Xmat,
              sigma2 = init_sigma2, beta = init_beta,
              linkFunction = init_linkFunction))

}

# Initial values setting for gpFisher
initFunction_gpFisher <- function(X, Y,
                                  index, lengthscale,
                                  amp, sigma2,
                                  setSeed){
  if (setSeed != FALSE){
    set.seed(setSeed)
  }

  N <- nrow(X)
  p <- ncol(X)

  # index
  if (is.null(index)){
    # init_index <- rvMFnim(1, index_direction)
    init_index <- as.vector(ppr(X, Y, nterms = p)$beta)
  } else{
    init_index <- index
  }
  if (sum((init_index)^2) != 1){
    init_index <- init_index/sqrt(sum(init_index^2))
  }

  init_Xlin <- as.vector(X %*% matrix(init_index, nrow = p))

  # lengthscale
  init_lengthscale <- lengthscale
  if (length(init_lengthscale) >= 2 || !is.numeric(init_lengthscale)){
    stop("Error: Initial value of lengthscale should be scalar.")
  }
  if (init_lengthscale < 0){
    stop("Error: Initial value of lengthscale should be positive.")
  }

  # amp (eta)
  init_amp <- amp
  if (length(init_amp) >= 2 || !is.numeric(init_amp)){
    stop("Error: Initial value of amp should be scalar.")
  }
  if (init_amp < 0){
    stop("Error: Initial value of amp should be positive.")
  }

  init_cov <- expcov_gpSphere(init_Xlin, init_lengthscale, init_amp)
  init_linkFunction <- mvtnorm::rmvnorm(1, rep(0, N), sigma = init_cov)[1,]

  # sigma2
  init_sigma2 <- sigma2
  if (length(init_sigma2) >= 2 || !is.numeric(init_sigma2)){
    stop("Error: Initial value of sigma2 should be scalar.")
  }
  if (init_sigma2 < 0){
    stop("Error: Initial value of sigma2 should be positive.")
  }
  init_Simga <- init_sigma2 * diag(1, N)

  return(list(index = init_index, index_temp = init_index,
              lengthscale = init_lengthscale, amp = init_amp,
              Xlin = init_Xlin,cov = init_cov,
              sigma2 = init_sigma2,
              linkFunction = init_linkFunction, Sigma = init_Simga))

}


param.check <- function(user, template){
  old <- options(error = NULL)
  on.exit(options(old))

  wrong <- setdiff(names(user), names(template))
  if (length(wrong) > 0) {
    stop("Invalid field(s): ", paste(wrong, collapse = ", "), call. = FALSE)
  }

  for (nm in names(user)) {
    if (is.list(user[[nm]]) && is.list(template[[nm]])) {
      param.check(user[[nm]], template[[nm]])
    }
  }

  return(TRUE)
}

# check parameters - function
validate_and_finalize_args <- function(
    sampling, niter, nburnin, thin, nchain,
    prior, init, indexprior, link
) {
  if (!is.logical(sampling)){
    stop("'sampling' argument should be logical.")
  }
  if (is.logical(sampling) & (length(sampling) > 1)){
    stop("'sampling' argument should be scalar.")
  }

  if (!is.numeric(niter) || length(niter) != 1 || is.na(niter)){
    stop("'niter' argument should be numeric scalar.")
  }
  if (!is.numeric(nburnin) || length(nburnin) != 1 || is.na(nburnin)){
    stop("'nburnin' argument should be numeric scalar.")
  }
  if (!is.numeric(thin) || length(thin) != 1 || is.na(thin)){
    stop("'thin' argument should be numeric scalar.")
  }

  if (!is.numeric(nchain) || length(nchain) != 1 || is.na(nchain)){
    stop("'nchain' argument should be numeric scalar.")
  }

  # Prior & init
  priorlist <- prior_param_default(indexprior, link)
  if (!is.null(prior) & param.check(prior, priorlist)){
    priorlist_final <- modifyList(priorlist, prior, keep.null = TRUE)
  } else if (is.null(prior)){
    priorlist_final <- priorlist
  }

  # Initial value
  initlist <- init_param_default(indexprior, link)
  if (!is.null(init) & param.check(init, initlist)){
    initlist_final <- modifyList(initlist, init, keep.null = TRUE)
  } else if (is.null(init)){
    initlist_final <- initlist
  }

  list(
    priorlist_final = priorlist_final,
    initlist_final  = initlist_final
  )
}


# prediction
pred_fitted <- nimbleFunction(
  run = function(linkFunction_samples = double(2),
                 sigma2_samples = double(1)){
    returnType(double(2))

    nsamp <- nimDim(linkFunction_samples)[1]
    N <- nimDim(linkFunction_samples)[2]
    pred <- nimMatrix(0, nrow = nsamp, ncol = N)

    for (i in 1:nsamp){
      linkPred <- linkFunction_samples[i, ]
      predsigma <- chol(diag(rep(sigma2_samples[i], N)))
      pred[i,] <- rmnorm_chol(1, mean = linkPred,
                              cholesky = predsigma, prec_param  = FALSE)
    }
    return(pred)
  }
)

# Make samples into one matrix
sampleBind <- function(mcmc.out, nchain){
  samples <- NULL
  if (nchain == 1 & !is.list(mcmc.out)){
    samples <- mcmc.out
  } else{
    for (i in 1:nchain){
      samples <- rbind(samples, mcmc.out[[i]])
    }
  }
  return(samples)
}

# Get monitorable variables - used in getVarMonitor()
expand_monitor <- function(vars) {
  out <- character(0)
  baseName <- character(0)

  for (v in vars) {

    # If there is no [], add
    if (!grepl("\\[", v)) {
      if (v == "sigma2") next
      out <- c(out, v)
      baseName <- c(baseName, v)
      next
    }

    # extract variable name
    base <- sub("\\[.*$", "", v)        # "index[1:4]" -> "index"
    if (grepl("index", base)) next
    if (base == "nu") next
    baseName <- c(baseName, base)

    # extract inside []
    inside <- sub("^.*\\[", "", v)      # "index[1:4]"     -> "1:4]"
    inside <- sub("\\]$", "", inside)   # "1:4]"           -> "1:4"

    parts <- strsplit(inside, ",")[[1]]
    parts <- trimws(parts)

    first <- parts[1]

    if (!grepl(":", first)) {
      out <- c(out, v)
      next
    }

    # a:b parsing
    ab <- as.integer(strsplit(first, ":")[[1]])
    a  <- ab[1]
    b  <- ab[2]

    rest <- ""
    if (length(parts) > 1) {
      rest <- paste0(", ", paste(parts[-1], collapse = ","))
    }

    for (k in seq(a, b)) {
      out <- c(out, sprintf("%s[%d%s]", base, k, rest))
    }
  }

  return(list(baseName = unique(baseName), out = out))
}

# Inside all the modeling functions (Extract coef, fitted, residuals etc.)
bsimFit_pointest <- function(samples, dataX, datay){
  # samples: combined samples of chain
  # relevant information
  p <- ncol(dataX)
  N <- nrow(dataX)
  namesIndex <- paste0("index[", 1:p, "]")
  namesXlin <- paste0("Xlin[", 1:N, "]")
  # link function
  colsN <- colnames(samples)
  namesLink <- grep("linkFunction", colsN, value = TRUE)
  # has_comma <- grepl(",", lf_cols)
  # namesLink <- ifelse(
  #   has_comma,
  #   paste0(lf_cols, paste0("linkFunction[", 1:N, ", 1]")),
  #   paste0(lf_cols, paste0("linkFunction[", 1:N, "]"))
  # )
  namesSigma <- "sigma2"
  namesVar <- colnames(dataX)
  indexName <- colnames(dataX)
  if (is.null(samples)){
    coefficients <- ses_coef <- se <- linear.predictors <- fitted.values <- residuals <- gof <- NULL
  } else{
    ALLsamp <- samples


    # Compute coefficients, sd, fitted.values, linear.predictors - mean
    coefficients <- apply(ALLsamp[ ,namesIndex], 2, mean)
    attr(coefficients, "names") <- indexName
    ses_coef <- apply(ALLsamp[ ,namesIndex], 2, sd)
    attr(ses_coef, "names") <- indexName
    se <- mean(ALLsamp[ ,namesSigma])
    linear.predictors <- apply(ALLsamp[ ,namesXlin], 2, mean)
    fitted.values <- apply(ALLsamp[ ,namesLink], 2, mean)
    residuals <- datay[,1] - as.vector(fitted.values)
    gof <- mean(residuals^2) # residual sum of squares
  }


  outList <- list(coefficients = coefficients,
                  ses_coef = ses_coef,
                  se = se,
                  residuals = residuals,
                  linear.predictors = as.vector(linear.predictors),
                  fitted.values = as.vector(fitted.values),
                  gof = gof)

  return(outList)

}

# Fitted plot algorithm
plot_bsim_fitted <- function(result, interval){ # result: data.frame
  idxValue <- 0; pred <- 0; LB <- 0; UB <- 0; truey <- 0

  level <- result$level
  if (is.null(result$truey)){

    if (!interval){
      if (is.vector(result$fitted)){
        pred <- result$fitted
      } else{
        pred <- result$fitted[ ,1]
      }
      results <- data.frame(pred = pred, idxValue = result$idxValue)
      df_pred_line <- results[order(results$idxValue), ]
      A2 <- ggplot(df_pred_line, aes(x = idxValue)) +
        geom_point(aes(y = pred), color = "grey20", alpha = 0.55, size = 1.6) +
        geom_line(aes(y = pred), color = "firebrick", linewidth = 1) +
        labs(x = "Index value", y = "Predicted",
             title = paste0("Fitted curve"),
             subtitle = "Points: observed, Line: posterior mean") +
        theme_minimal() +
        theme(
          panel.grid.minor = element_blank(),
          plot.title = element_text(hjust = 0.5, face = "bold"),
          axis.title = element_text(face = "bold")
        )

    } else{
      results2 <- data.frame(pred = result$fitted[,1],
                             idxValue = result$idxValue, LB = result$fitted$LB,
                             UB = result$fitted$UB)
      df_pred_line2 <- results2[order(results2$idxValue), ]
      A2 <- ggplot(df_pred_line2, aes(x = idxValue)) +
        geom_ribbon(aes(ymin = LB, ymax = UB),
                    fill = "firebrick", alpha = 0.12, colour = NA) +
        geom_point(aes(y = pred), color = "grey20", alpha = 0.55, size = 1.6) +
        geom_line(aes(y = pred), color = "firebrick", linewidth = 1) +
        labs(x = "Index value", y = "Predicted",
             title = paste0("Fitted curve with ", level, "% interval"),
             subtitle = "Points: predicted, Line: posterior mean") +
        theme_minimal() +
        theme(
          panel.grid.minor = element_blank(),
          plot.title = element_text(hjust = 0.5, face = "bold"),
          axis.title = element_text(face = "bold")
        )
    }
    return(A2)
  } else{
    if (!interval){
      if (is.vector(result$fitted)){
        pred <- result$fitted
      } else{ # se.fit = TRUE
        pred <- result$fitted[ ,1]
      }
      results <- data.frame(truey = result$truey, pred = pred,
                            idxValue = result$idxValue)
    } else{
      results <- data.frame(truey = result$truey, pred = result$fitted[,1],
                            idxValue = result$idxValue)
    }


    # 1) fitted vs. observed
    if (!is.null(result$truey)){
      rmse <- mean((results$truey-results$pred)^2)

      A1 <- ggplot(results, aes(x = truey, y = pred)) +
        geom_point(color = "black", alpha = 0.6) +
        geom_abline(slope = 1, intercept = 0, color = "firebrick", linewidth = 0.8) +
        theme_minimal() +
        labs(
          title = "Fitted Plot",
          x = "True",
          y = "Predicted",
          subtitle = paste("RMSE:", round(rmse, 4))
        ) +
        theme(
          plot.title = element_text(hjust = 0.5, face = "bold"),
          axis.title = element_text(face = "bold"),
          panel.grid.minor = element_blank()
        )
    }


    # 2) index vs. y
    if (!interval){
      df_pred_line <- results[order(results$idxValue), ]
      A2 <- ggplot(df_pred_line, aes(x = idxValue)) +
        geom_point(aes(y = truey), color = "grey20", alpha = 0.55, size = 1.6) +
        geom_line(aes(y = pred), color = "firebrick", linewidth = 1) +
        labs(x = "Index value", y = "Response",
             title = paste0("Fitted curve"),
             subtitle = "Points: observed, Line: posterior mean") +
        theme_minimal() +
        theme(
          panel.grid.minor = element_blank(),
          plot.title = element_text(hjust = 0.5, face = "bold"),
          axis.title = element_text(face = "bold")
        )

    } else{
      results2 <- data.frame(truey = result$truey, pred = result$fitted[,1],
                             idxValue = result$idxValue, LB = result$fitted$LB,
                             UB = result$fitted$UB)
      df_pred_line2 <- results2[order(results2$idxValue), ]
      A2 <- ggplot(df_pred_line2, aes(x = idxValue)) +
        geom_ribbon(aes(ymin = LB, ymax = UB),
                    fill = "firebrick", alpha = 0.12, colour = NA) +
        geom_point(aes(y = truey), color = "grey20", alpha = 0.55, size = 1.6) +
        geom_line(aes(y = pred), color = "firebrick", linewidth = 1) +
        labs(x = "Index value", y = "Response",
             title = paste0("Fitted curve with ", level, "% interval"),
             subtitle = "Points: observed, Line: posterior mean") +
        theme_minimal() +
        theme(
          panel.grid.minor = element_blank(),
          plot.title = element_text(hjust = 0.5, face = "bold"),
          axis.title = element_text(face = "bold")
        )
    }
    return(A1 + A2)
  }

}

#' Bayesian Single-Index Regression with B-Spline Link and von Mises-Fisher Prior
#' @importFrom mvtnorm rmvnorm
#' @description Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}
#' where the link \eqn{f(\cdot)} is represented by B-spline and the
#' index vector \eqn{\theta} has von Mises–Fisher prior.
#' @param formula an object of class \link{formula}. Interaction term is not allowed for single-index model.
#' @param data an data frame.
#' @param prior Optional named list of prior settings. Further descriptions are in \strong{Details} section.
#' @param init Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
#' Further descriptions are in \strong{Details} section.
#' @param monitors Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
#' then inspect the variable names stored in the model object using \link{getVarMonitor}.
#' @param niter Integer. Total MCMC iterations (default \code{10000}).
#' @param nburnin Integer. Burn-in iterations (default \code{1000}).
#' @param thin Integer. Thinning for monitors (default \code{1}).
#' @param nchain Integer. Number of MCMC chains (default \code{1}).
#' @param setSeed Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.
#'
#' @details
#' \strong{Model} The single–index model uses a \eqn{m}-order polynomial spline with \eqn{k} interior knots as follows:
#' \deqn{f(t) = \sum_{j=1}^{m+k} B_j(t)\,\beta_j} on \eqn{[a, b]} with \eqn{t_i = X_i' \theta, i = 1,\cdots, n}
#' and \eqn{\|\theta\|_2 = 1}. \eqn{\{\beta_j\}_{j=1}^{m+k}} are spline coefficients and \eqn{a_\theta}, \eqn{b_\theta} are boundary knots where \eqn{a_{\theta} = min(t_i, i = 1, \cdots, n) - \delta},
#' and \eqn{b_{\theta} = max(t_i, i = 1,\cdots, n) + \delta}.
#'
#' \strong{Priors}
#' \itemize{
#' \item von Mises–Fisher prior on the index \eqn{\theta} with direction and concentration.
#' \item Conditioned on \eqn{\theta} and \eqn{\sigma^2}, the link coefficients \eqn{\beta = (\beta_1,\ldots,\beta_{m+k})^\top} follow
#'        normal distribution with estimated mean vector \eqn{\hat{\beta}_{\theta} = (X_{\theta}'X_{\theta})^{-1}X_{\theta}'Y} and
#'        covariance \eqn{\sigma^2 (X_{\theta}^\top X_{\theta})^{-1}}, where \eqn{X_{\theta}} is the B-spline basis design matrix.
#' \item Inverse gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.
#' }
#'
#' \strong{Sampling}
#' Random walk metropolis algorithm is used for index vector \eqn{\theta}. Given \eqn{\theta}, \eqn{\sigma^2} and \eqn{\beta} are sampled from posterior distribution. Further sampling method is described in Antoniadis et al(2004).
#'
#' \strong{Prior hyper-parameters}
#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
#' \enumerate{
#'   \item Index vector: von Mises--Fisher prior for the projection vector \eqn{\theta} (\code{index}).
#'         \code{index_direction} is a unit direction vector of the von Mises--Fisher distribution.
#'         By default, the estimated vector from projection pursuit regression is assigned.
#'         \code{index_dispersion} is the positive concentration parameter. By default, \code{150} is assigned.
#'
#'   \item Link function: B-spline basis and coefficient of B-spline setup.
#'         \itemize{
#'           \item \code{basis}: For the basis of B-spline, \code{link_basis_df} is the number of basis functions (default \code{21}),
#'                 \code{link_basis_degree} is the spline degree (default \code{2}) and
#'                 \code{link_basis_delta} is a small jitter for boundary knots spacing control (default \code{0.001}).
#'           \item \code{beta}: For the coefficient of B-spline, multivariate normal prior is assigned with mean \code{link_beta_mu}, and covariance \code{link_beta_cov}.
#'                 By default, \eqn{\mathcal{N}_p\!\big(0, \mathrm{I}_p\big)} is assigned.
#'         }
#'
#'   \item Error variance (\code{sigma2}): An Inverse gamma prior is assigned to \eqn{\sigma^2}
#'         where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
#'         (default \code{sigma2_shape = 0.001, sigma2_rate = 100})
#' }
#'
#'
#' \strong{Initial values}
#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
#' \enumerate{
#'  \item Index vector: Initial unit index vector \eqn{\theta}. By default, the vector is randomly sampled from the von Mises--Fisher prior.
#'    \item Link function: Initial spline coefficients (\code{link_beta}). By default,
#'          \eqn{\big(X_{\theta}^\top X_{\theta} + \rho\, \mathrm{I}\big)^{-1} X_{\theta}^\top Y} is computed,
#'     where \eqn{X_{\theta}} is the B-spline basis design matrix.
#'
#'   \item Error variance (\code{sigma2}): Initial scalar error variance (default \code{0.01}).
#'   }
#'
#'
#' @return A \code{list} typically containing:
#' \describe{
#' \item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
#' \item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
#' \item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
#' \item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
#' \item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
#' \item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
#' \item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
#' Return list of \code{model_setup} does not include it.}
#' \item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
#' \item{\code{defModel}}{Nimble model object.}
#' \item{\code{defSampler}}{Nimble sampler object.}
#' \item{\code{modelName}}{Name of the model.}
#' }
#'
#'
#' @examples
#' \donttest{
#' set.seed(123)
#' n <- 200; d <- 4
#' theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
#' f <- function(u) u^2 * exp(u)
#' sigma <- 0.5
#' X <- matrix(runif(n * d, -1, 1), nrow = n)
#' index_vals <- as.vector(X %*% theta)
#' y <- f(index_vals) + rnorm(n, 0, sigma)
#' simdata <- data.frame(x = X, y = y)
#' colnames(simdata) <- c(paste0("X", 1:4), "y")
#'
#' # One tool version
#' fit1 <- bsFisher(y ~ ., data = simdata,
#'                  niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Split version
#' models <- bsFisher_setup(y ~ ., data = simdata)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                    nchains = 1, setSeed = TRUE, inits = initList,
#'                    summary = TRUE, samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' summary(fit2)
#'
#'
#' }
#'
#' @references
#' Antoniadis, A., Grégoire, G., & McKeague, I. W. (2004).
#' Bayesian estimation in single-index models. \emph{Statistica Sinica}, 1147-1164.
#'
#' Hornik, K., & Grün, B. (2014). movMF: an R package for fitting mixtures of von Mises-Fisher distributions.
#' \emph{Journal of Statistical Software}, 58, 1-31.
#' @name bsFisher
#' @export
bsFisher <- function(formula, data,
                     prior = NULL,
                     init = NULL,
                     monitors = NULL, niter = 10000, nburnin = 1000,
                     thin = 1, nchain = 1, setSeed = FALSE){

  return(bsFisher.default(formula = formula, data  = data,
                          prior = prior,
                          init = init,
                          sampling = TRUE,
                          monitors = monitors, niter = niter, nburnin = nburnin,
                          thin = thin, nchain = nchain, setSeed = setSeed))
}



#' @name bsFisher
#' @export
bsFisher_setup <- function(formula, data,
                           prior = NULL,
                           init = NULL,
                           monitors = NULL, niter = 10000, nburnin = 1000,
                           thin = 1, nchain = 1, setSeed = FALSE){


  return(bsFisher.default(formula = formula, data  = data,
                          prior = prior,
                          init = init,
                          sampling = FALSE,
                          monitors = monitors, niter = niter, nburnin = nburnin,
                          thin = thin, nchain = nchain, setSeed = setSeed))
}

bsFisher.default <- function(formula, data,
                             prior = NULL,
                             init = NULL,
                             sampling = TRUE, monitors = NULL,
                             niter = 10000, nburnin = 1000,
                             thin = 1, nchain = 1, setSeed = FALSE
){
  start1 <- Sys.time()
  index <- 0

  # check sampling, prior, init parameters for independent execution
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin, nchain,
    prior, init, "fisher", "bspline"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))

  # assign functions global environment
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # distribution
    "dvMFnim", "rvMFnim",

    # aa_bspline_ver3
    "SplineState", "any_duplicated", "mat_wo_col1", "update_spline_df", "update_x_index",
    "update_knot_sequence", "get_basis_simple", "simplify_knots", "get_inside_x",
    "gen_default_internal_knots", "SplineBase1", "SplineBase2", "basis", "bsNimble", "bsBasis",

    # bsFisher
    "postll_bspline_fisher","nimNorm","rW","besselI_nimble","Stheta",
    "estBeta_fisher","gvcCV", "transX_fisher",
    "pred_bsplineFisher","indexSampler_bspline_fisher","betaSampler_bspline_fisher")



  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = .GlobalEnv)
  suppressMessages(
    nimble::registerDistributions(list(
      dvMFnim = list(
        BUGSdist = "dvMFnim(theta)",
        types    = c("value = double(1)", "theta = double(1)"),
        discrete = FALSE
      )
    ), verbose = FALSE)
  )

  # --------------------------------------------------------------------------
  # Data
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  } else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
            length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  } else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  # data dimension
  N <- nrow(X)
  p <- ncol(X)

  # Model code
  Rmodel <- nimble::nimbleCode({
    # index
    index[1:p] ~ dvMFnim(theta = index_prior[1:p])

    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i, 1:p] * index[1:p])
    }

    # Design matrix - B-spline basis
    Xmat[1:N, 1:df] <- transX_fisher(Xlin[1:N], df = df, degree = degree, delta = delta)

    # likelihood
    sigma2 ~ dinvgamma(a_sig, b_sig)
    beta[1:df] ~ dmnorm(mubeta[1:df], cov = covbeta[1:df, 1:df])
    linkFunction[1:N, 1] <- Xmat[1:N, 1:df] %*% matrix(beta[1:df], ncol = 1)
    for (i in 1:N){
      Y[i, 1] ~ dnorm(linkFunction[i, 1], var = sigma2)
    }
  })

  # Prior parameters
  ## check data dimension and save
  ## index
  if (length(prior$index$dispersion) >= 2|| !is.numeric(prior$index$dispersion)){
    stop("Dispersion of index vector should be scalar.")
  }
  if (prior$index$dispersion < 0){
    stop("Dispersion of index vector should be postive")
  }

  dispersion_prior <- prior$index$dispersion

  if (!is.null(prior$index$direction) & length(prior$index$direction) != (p)){
    stop("Prior index vector has incorrect dimension.")
  }
  if (!is.null(prior$index$direction) & sum((prior$index$direction)^2) != 1){
    stop("Prior index vector should be unit vector.")
  }

  if (is.null(prior$index$direction)){
    # index_direct_input <- c(rep(0, p-1), 1)
    index_direct_input <- as.vector(ppr(X, as.matrix(Y), nterms = 1)$alpha)
    index_direct_input <- index_direct_input/sqrt(sum(index_direct_input^2))
  } else{
    index_direct_input <- prior$index$direction
  }
  index_direction <- index_direct_input * dispersion_prior
  # index_direction <- index_direct_input

  ## link - basis
  if (is.null(prior$link$basis$df)||length(prior$link$basis$df) >= 2 || prior$link$basis$df< 0){
    stop("Prior basis (df) has incorrect value.")
  } else{
    df <- prior$link$basis$df
  }

  if (is.null(prior$link$basis$degree)||length(prior$link$basis$degree) >= 2 || prior$link$basis$degree< 0){
    stop("Prior basis (degree) has incorrect value.")
  } else{
    degree <- prior$link$basis$degree
  }

  if (is.null(prior$link$basis$delta)||length(prior$link$basis$delta) >= 2 || prior$link$basis$delta< 0){
    stop("Prior basis (delta) has incorrect value.")
  } else{
    delta <- prior$link$basis$delta
  }

  ## beta
  # mu
  if (is.null(prior$link$beta$mu)){
    mubeta <- rep(0, df)
  }

  if (!is.vector(prior$link$beta$mu) & !is.null(prior$link$beta$mu)){
    stop("Prior beta (mu) should be vector.")
  }
  if (!is.null(prior$link$beta$mu) & length(prior$link$beta$mu) != (df)){
    stop("Incorrect dimension on prior beta (mu).")
  }

  if (is.null(prior$link$beta$cov)){
    covbeta <- diag(df)
  }
  if (!is.null(prior$link$beta$cov) & !is.matrix(prior$link$beta$cov)){
    stop("Prior beta (cov) should be matrix.")
  }
  # if (dim(prior$beta$cov)[1] != (df) || dim(prior$beta$cov)[2] != (df)){
  #   stop("Error: Incorrect dimention on prior beta (cov).")
  # }

  ## sigma
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (shape) has incorrect value.")
  } else{
    a_sig <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (rate) has incorrect value.")
  } else{
    b_sig <- prior$sigma2$rate
  }



  # Initialize
  ## index
  if (!is.null(init$index) & length(init$index) != p){
    stop("Incorrect dimension on initial value of index.")
  }

  ## sigma2
  init_sigma2 <- init$sigma2
  if (length(init$sigma2) >= 2 || !is.numeric(init$sigma2)){
    stop("Initial value of sigma2 should be scalar.")
  }
  if (init$sigma2 < 0){
    stop("Initial value of sigma2 should be positive.")
  }

  ## beta
  if (!is.null(init$link$beta) & length(init$link$beta) != df){
    stop("Incorrect dimention on initial value of beta.")
  }

  # seed
  seedNum <- rep(FALSE, nchain)

  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }

  inits_list <- lapply(seq_len(nchain), function(j) initFunction_bF(X = X, Y = Y,
                                                                    index = init$index,
                                                                    sigma2 = init$sigma2,
                                                                    beta = init$link$beta,
                                                                    df = df, degree = degree, delta = delta,
                                                                    index_direction = index_direction,
                                                                    setSeed = seedNum[j]))
  firstInit <- inits_list[[1]]

  # Build model
  message("== Building model ==")
  suppressMessages(
    simpleModel <- nimbleModel(Rmodel,
                               data = list(X = X, Y = Y),
                               constants = list(p = p, N = N, index_prior = index_direction,
                                                a_sig = a_sig, b_sig = b_sig,
                                                df = df, degree = degree,
                                                delta = delta,mubeta = mubeta, covbeta = covbeta),
                               inits = firstInit)
  )


  # Assign samplers
  message("== Assigning samplers ==")
  monitorsList_essential <- c("index", "sigma2")
  # Add parameters for fitting
  monitorsList <- c(monitorsList_essential, "linkFunction", "Xlin", "beta")
  suppressMessages(
    mcmcConf <- configureMCMC(simpleModel,
                              monitors = monitorsList,
                              print = FALSE)
  )


  # mcmcConf1$printSamplers(executionOrder= TRUE)
  mcmcConf$removeSamplers(c("index"))
  mcmcConf$addSampler(target = c("index"),
                      type   = indexSampler_bspline_fisher)

  mcmcConf$removeSamplers(c("beta"))
  mcmcConf$addSampler(target = c("beta"),
                      type   = betaSampler_bspline_fisher)

  mcmcConf$removeSamplers(c("sigma2"))
  mcmcConf$addSampler(target = c("sigma2"),
                      type   = sigma2Sampler_bspline_fisher)

  # mcmcConf$setSamplerExecutionOrder(c(2, 3, 1))


  mcmc1 <- buildMCMC(mcmcConf)

  samples <- NULL
  if (!sampling){} else{
    # Compile
    start2 <- Sys.time()
    message("== Compiling model ==")
    suppressMessages(CsimpleModel <- compileNimble(simpleModel))
    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1, project = simpleModel, resetFunctions = TRUE))
    end2 <- Sys.time()

    # Sampling
    message("== Running MCMC ==")
    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    mcmc.out <- NULL
    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                        thin = thin,
                        nchains = nchain, setSeed = seedNum,
                        inits = inits_list,
                        summary = FALSE, samplesAsCodaMCMC = TRUE)


    # output
    sampMCMC <- mcmc.out
    samples <- sampleBind(sampMCMC, nchain)
  }
  end1 <- Sys.time()


  # Input options
  ## Calculate execution time
  if (!sampling){
    time <- NULL
  } else{
    # total time - compile time
    time <- difftime(end1, start1, units = "secs") -
      difftime(end2, start2, units = "secs")
  }

  # Results ------------------------------------------------------------------
  # Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  # input options
  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = list(direction = index_direct_input, dispersion = dispersion_prior),
                                    link = list(basis = list(df = df, degree = degree, delta = delta),
                                                beta = list(mu = mubeta, cov = covbeta)),
                                    sigma = list(shape = a_sig, rate = b_sig)),
                       init = inits_list,
                       samplingOptions = list(sampling = sampling,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       time = time)

  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = sampMCMC, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "bsFisher")

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "bsFisher")

    class(out) <- "bsimSetup"

  }
  return(out)


}

#' @rdname gpPolar
#' @export
gpPolarHigh <- function(formula, data,
                        prior = NULL,
                        init = NULL,
                        monitors = NULL, niter = 10000, nburnin=1000,
                        thin = 1, nchain = 1, setSeed = FALSE
){
  return(gpPolarHigh.default(formula = formula, data = data,
                             prior = prior,
                             init = init,
                             sampling = TRUE, monitors = monitors,
                             niter = niter, nburnin=nburnin,
                             thin = thin, nchain = nchain, setSeed = setSeed))
}

#' @rdname gpPolar
#' @export
gpPolarHigh_setup <-function(formula, data,
                             prior = NULL,
                             init = NULL,
                             monitors = NULL, niter = 10000, nburnin=1000,
                             thin = 1, nchain = 1, setSeed = FALSE
){
  return(gpPolarHigh.default(formula = formula, data = data,
                             prior = prior,
                             init = init,
                             sampling = FALSE, monitors = monitors,
                             niter = niter, nburnin=nburnin,
                             thin = thin, nchain = nchain, setSeed = setSeed))
}


gpPolarHigh.default <- function(formula, data,
                    prior = NULL,
                    init = NULL,
                    sampling = TRUE,
                    monitors = NULL, niter = 10000, nburnin=1000,
                    thin = 1, nchain = 1, setSeed = FALSE
){
  start1 <- Sys.time()
  sigma2 <- 0
  psi <- 0

  # check sampling, prior, init parameters for independent execution
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin, nchain,
    prior, init, "polar", "gp"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))


  # aa_bspline_ver3
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # gpPolar
    "alphaTheta","Xlinear","invcov","expcov_gpPolar","expcovTest_gpPolar",
    "obj_btt_theta","thetaPrior","pred_gpPolar","gibbsSampler_sigma2",
    "gibbsSampler_kappa","MH_thetaeta",

    # utils
    "pred_fitted"
  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())



  # Data
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
           length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  }else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  # data dimension
  N <- length(Y)
  p <- ncol(X)

  # Model code
  Rmodel <- nimbleCode({
    # 1. index, psi
    for(j in 1:(p-1)) {
      d[j] ~ dunif(1e-10, 1e10)
      psi[j] ~ dbeta(c[j], d[j])
    }
    index[1:p] <- alphaTheta(psi[1:(p-1)]*pi)
    Xlin[1:N] <- Xlinear(index[1:p], x[1:N, 1:p])

    # 2. sigma2 - gibbs
    sigma2 ~ dinvgamma(a, b)

    # 2. hyperprior-kappa
    kappa ~ dunif(kappa_a, kappa_b)

    # 1. linkFunction(f)
    cov[1:N, 1:N] <- expcov_gpPolar(Xlin[1:N], kappa)
    linkFunction[1:N] ~ dmnorm(mu0[1:N], cov = cov[1:N, 1:N])

    # 0. Likelihood(y)
    Sigma[1:N,1:N] <-  diag(rep(sigma2, N))
    y[1:N] ~ dmnorm(linkFunction[1:N], cov = Sigma[1:N,1:N])


  })


  # Prior parameters
  ## psi
  if (!is.null(prior$index$psi$alpha) & length(prior$index$psi$alpha) != (p-1))
  {stop("Prior psi has incorrect dimension")}
  if (is.null(prior$index$psi$alpha)){
    psi_c <- rep(5000,(p-1))
  } else{
    psi_c <- prior$index$psi$alpha
  }

  ## sigma2 - scalar, > 0
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (a) has incorrect value.")
  } else{
    sigma2_shape <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (b) has incorrect value.")
  } else{
    sigma2_rate <- prior$sigma2$rate
  }

  # kappa
  if (is.null(prior$link$kappa$min)||length(prior$link$kappa$min) >= 2 || prior$link$kappa$min < 0){
    stop("Prior kappa (min) has incorrect value.")
  } else{
    kappa_min <- prior$link$kappa$min
  }

  if (is.null(prior$link$kappa$max)||length(prior$link$kappa$max) >= 2 || prior$link$kappa$max < 0){
    stop("Prior kappa (max) has incorrect value.")
  } else{
    kappa_max <- prior$link$kappa$max
  }

  if (is.null(prior$link$kappa$grid_width)||length(prior$link$kappa$grid_width) >= 2 || prior$link$kappa$grid_width < 0){
    stop("Prior kappa (grid_width) has incorrect value.")
  } else{
    kappa_grid_width <- prior$link$kappa$grid_width
  }

  # Initialize
  init_psi <- init$link$psi
  if (!is.null(init_psi) & length(init_psi) != (p-1)){
    stop("Initial psi has incorrect dimension")
  }

  if (is.null(init$link$kappa)||length(init$link$kappa) >= 2 || init$link$kappa < 0){
    stop("Initial kappa has incorrect value.")
  } else{
    init_kappa <- init$link$kappa
  }

  if (is.null(init$sigma2)||length(init$sigma2) >= 2 || init$sigma2 < 0){
    stop("Initial sigma2 has incorrect value.")
  } else{
    init_sigma2 <- init$sigma2
  }

  # seed
  seedNum <- rep(FALSE, nchain)

  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }

  inits_list <- lapply(seq_len(nchain),
                       function(j) initfunction_gpPolar2(X = X, kappa = init_kappa,
                                                         sigma2 = init_sigma2, psi = init_psi,
                                                         setSeed = seedNum[j]))
  firstInit <- inits_list[[1]]




  message("== Building model ==")
  suppressMessages(simpleModel <- nimbleModel(Rmodel,
                             data = list(x = X,
                                         y = as.vector(Y)),
                             constants = list(c = psi_c,
                                              kappa_a = kappa_min, kappa_b = kappa_max,
                                              a = sigma2_shape, b = sigma2_rate,
                                              p = p, N = N, mu0 = rep(0, N), pi = pi),
                             inits = firstInit))

  # Assign samplers
  message("== Assigning samplers ==")
  # monitorsList <-  c("linkFunction","index", "psi", "kappa", "sigma", "d")
  monitorsList <- c("index", "sigma2", "linkFunction", "kappa", "Xlin", "d", "psi")
  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
                                             monitors = monitorsList,
                                             print = FALSE))



  mcmcConf$removeSamplers(c("sigma2"))
  mcmcConf$addSampler(target = c("sigma2"),
                      type   = gibbsSampler_sigma2)


  mcmc1 <- buildMCMC(mcmcConf)
  end1 <- Sys.time()

  if (!sampling){
    mcmc.out <- NULL
    samples <- NULL

  } else{
    start2 <- Sys.time()
    # Compile
    message("== Compiling model ==")
    suppressMessages(CsimpleModel <- compileNimble(simpleModel))
    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1,
                                            project = simpleModel,
                                            resetFunctions = TRUE))
    end2 <- Sys.time()
    # Sampling
    message("== Running MCMC ==")
    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                        thin = thin,
                        nchains = nchain, setSeed = seedNum, inits = inits_list,
                        summary = FALSE, samplesAsCodaMCMC = TRUE)

    # output
    samples <- NULL

    if (nchain > 1){
      for (i in 1:nchain){
        samples <- rbind(samples, mcmc.out[[i]])
      }
    } else if (nchain == 1){
      samples <- mcmc.out
    }
  }
  end1 <- 0
  ## Input options
  if (!sampling){
    time <- NULL
  } else{
    time <- difftime(end1, start1, units = "secs") - difftime(end2, start2, units = "secs")
  }


  # Results ------------------------------------------------------------------
  # Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  # input options
  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = list(psi = list(alpha = psi_c)),
                                    link = list(kappa = list(min = kappa_min, max = kappa_max,
                                                             grid_width = kappa_grid_width)),
                                    sigma2 = list(shape = sigma2_shape, rate = sigma2_rate)),
                       init = inits_list,
                       samplingOptions = list(sampling = sampling,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       time = time)

  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = mcmc.out, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "gpPolar")

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "gpPolar")

    class(out) <- "bsimSetup"

  }
  return(out)
}

#' Bayesian Single-Index Regression with Gaussian Process Link and One-to-One Polar Transformation
#'
#' @description
#' Fits a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}, where
#' the index \eqn{\theta} is specified and computed via a one-to-one polar
#' transformation, and the link \eqn{f(\cdot)} is represented with a Gaussian process.
#' @inheritParams bsFisher
#'
#' @details
#' \strong{Model} The single–index model is specified as \eqn{Y_i = f(X_i'{\theta}) + \epsilon_i},
#' where the index vector \eqn{\theta} lies on the unit sphere with (\eqn{\|\theta\|_2=1}) with non-zero first component
#' to ensure identifiability and is parameterized via a one-to-one polar transformation with angle \eqn{\psi_1,...,\psi_{p-1}}.
#'
#' The mapping is
#' \deqn{
#' \begin{aligned}
#' \theta_1 &= \sin(\psi_1),\\
#' \theta_i &= \Big(\prod_{j=1}^{i-1}\cos(\psi_j)\Big)\sin(\psi_i), \quad i=2,\dots,p-1,\\
#' \theta_p &= \prod_{j=1}^{p-1}\cos(\psi_j).
#' \end{aligned}
#' }
#' The vector is then scaled to unit length.
#'
#' Sampling is  performed on the angular parameters \eqn{\theta} defining
#' the index vector. The link function \eqn{f(\cdot)} is modeled by a Gaussian process
#' prior with zero mean and an Ornstein–Uhlenbeck (OU) covariance kernel
#' \eqn{\exp(-\kappa \cdot |t_i - t_j|), i, j = 1,\ldots, n}, where \eqn{\kappa} is a bandwidth (smoothness)
#' parameter and \eqn{t_i, t_j} is index value (\eqn{t_i = X_i'\theta}).

#'
#' \strong{Priors}
#' \itemize{
#'   \item \eqn{\psi} is \eqn{p-1} dimension of polar angle of index vector and re-scaled Beta distribution on \eqn{[0, \pi]} is assigned for prior.
#'   \item Prior for \eqn{\kappa} (bandwidth parameter) is discrete uniform of equally spaced grid points in \eqn{[\kappa_{\text{min}}, \kappa_{\text{max}}}].
#'   \item Inverse gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.
#'
#' }
#'
#' \strong{Sampling} For \code{gpPolar}, \eqn{\theta} is sampled by Metropolis-Hastings and updated with \eqn{f},
#' \eqn{\kappa} is chosen by grid search method that maximizes likelihood,
#' \eqn{\sigma^2} are sampled from their full conditional
#' distributions using Gibbs sampling.
#' Since \eqn{\kappa} is sampled by grid search, more than 5 dimension of variables \code{gpPolarHigh} is recommended.
#' For \code{gpPolarHigh}, all sampling parameters' samplers are assigned by nimble.
#'
#' \strong{Prior hyper-parameters}
#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
#' \enumerate{
#' \item Index vector: Only shape parameter \code{index_psi_alpha} of \eqn{p-1} dimension vector is needed since rate parameters is computed to satisfy \eqn{\theta_{j, \text{MAP}}}.
#'     By default, the shape parameter for each element of the polar vector is set to \code{5000}.
#'
#' \item Link function:
#'      \code{link_kappa_min} is minimum value of kappa (default \code{0.5}), \code{link_kappa_max} is maximum value of kappa (default \code{4}),
#'      and \code{link_kappa_grid_width} is space (default \code{0.1}).
#' \item Error variance (\code{sigma2}): An Inverse gamma prior is assigned to \eqn{\sigma^2}
#'         where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
#'         (default \code{sigma2_shape = 2, sigma2_rate = 0.01})
#' }
#'
#' \strong{Initial values}
#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
#' \enumerate{
#'
#' \item Index vector: Initial vector of polar angle \code{index_psi} with \eqn{p-1} dimension. Each element of angle is between 0 and \eqn{\pi}.
#'      \item Link function: Initial scalar scale parameter of covariance kernel \code{link_kappa}. (default: \code{2})
#'      \item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{0.01})
#' }
#'
#'
#'
#' @inherit bsFisher return
#'
#' @examples
#' \donttest{
#' library(MASS)
#' N <- 100    # Sample Size
#' p <- 3
#' mu <- c(0,0,0)
#' rho <- 0
#' Cx <- rbind(c(1,rho,rho), c(rho,1,rho), c(rho, rho,1))
#' X <- mvrnorm(n = N, mu=mu, Sigma=Cx, tol=1e-8)
#' alpha <- c(1,1,1)
#' alpha <- alpha/sqrt(sum(alpha^2))
#' z <- matrix(0,N)
#' z <- X %*% alpha
#' z <- z[,1]
#' sigma <- 0.3
#' f <- exp(z)
#' y <- f + rnorm(N, 0, sd=sigma) # adding noise
#' y <- y-mean(y)
#' f_all <- f
#' x_all <- X
#' y_all <- y
#' simdata <- cbind(x_all, y, f)
#' simdata <- as.data.frame(simdata)
#' colnames(simdata) = c('x1', 'x2', 'x3', 'y','f')
#'
#' prior_gpPolar <- prior_param(indexprior = "polar", link = "gp",
#'                             link_kappa_max = 2, link_kappa_grid_width = 0.05)
#'
#' # One tool version
#' fit1 <- gpPolar(y ~ x1 + x2 + x3, data = simdata,
#'                 prior = prior_gpPolar,
#'                 niter = 3000, nburnin = 2000, nchain = 1)
#' fit2 <- gpPolarHigh(y ~ x1 + x2 + x3, data = simdata,
#'                     niter = 3000, nburnin = 2000, nchain = 1)
#'
#' # Split version
#' models1 <- gpPolar_setup(y ~ x1 + x2 + x3, data = simdata,
#'                          prior = prior_gpPolar)
#' models2 <- gpPolarHigh_setup(y ~ x1 + x2 + x3, data = simdata,
#'                          prior = prior_gpPolar)
#' Ccompile1 <- compileModelAndMCMC(models1)
#' Ccompile2 <- compileModelAndMCMC(models2)
#' sampler1 <- getSampler(Ccompile1)
#' sampler2 <- getSampler(Ccompile2)
#' initList1 <- getInit(models1)
#' initList2 <- getInit(models2)
#' mcmc.out1 <- runMCMC(sampler1, niter = 3000, nburnin = 2000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, init = initList1,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#' mcmc.out2 <- runMCMC(sampler2, niter = 3000, nburnin = 2000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, init = initList2,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#' fit1_split <- as_bsim(models1, mcmc.out1)
#' fit2_split <- as_bsim(models2, mcmc.out2)
#' }
#'
#' @references
#' Dhara, K., Lipsitz, S., Pati, D., & Sinha, D. (2019). A new Bayesian single index model with or without covariates missing at random.
#' \emph{Bayesian analysis}, 15(3), 759.
#'
#' @name gpPolar
#' @export
gpPolar <- function(formula, data,
                   prior = NULL,
                   init = NULL,
                   monitors = NULL, niter = 10000, nburnin=1000,
                   thin = 1, nchain = 1, setSeed = FALSE){

  return(gpPolar.default(formula = formula, data = data,
                          prior = prior,
                          init = init,
                          sampling = TRUE, monitors = monitors,
                         niter = niter, nburnin=nburnin,
                         thin = thin, nchain = nchain, setSeed = setSeed))
}

#' @rdname gpPolar
#' @export
gpPolar_setup <- function(formula, data,
                          prior = NULL,
                          init = NULL,
                          monitors = NULL, niter = 10000, nburnin=1000,
                          thin = 1, nchain = 1, setSeed = FALSE){
  return(gpPolar.default(formula = formula, data = data,
                         prior = prior,
                         init = init,
                         sampling = FALSE, monitors = monitors,
                         niter = niter, nburnin=nburnin,
                         thin = thin, nchain = nchain, setSeed = setSeed))
}

gpPolar.default <- function(formula, data,
                    prior = NULL,
                    init = NULL,
                    sampling = TRUE,
                    monitors = NULL, niter = 10000, nburnin=1000,
                    thin = 1, nchain = 1, setSeed = FALSE){

  start1 <- Sys.time()
  sigma2 <- 0; psi <- 0

  # check sampling, prior, init parameters for independent execution
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin, nchain,
    prior, init, "polar", "gp"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  # environment
  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))


  # aa_bspline_ver3
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # gpPolar
    "alphaTheta","Xlinear","invcov","expcov_gpPolar","expcovTest_gpPolar",
    "obj_btt_theta","thetaPrior","pred_gpPolar","gibbsSampler_sigma2",
    "gibbsSampler_kappa","MH_thetaeta",


    # utils
    "pred_fitted"
  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())



  # data
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
           length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  }else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  # data dimension
  N <- length(Y)
  p <- ncol(X)

  # Model code
  Rmodel <- nimbleCode({
    # 0. Likelihood(y)
    Sigma[1:N,1:N] <-  diag(rep(sigma2, N))
    y[1:N] ~ dmnorm(linkFunction[1:N], cov = Sigma[1:N,1:N])

    # 1. linkFunction(f) - gibbs
    cov[1:N, 1:N] <- expcov_gpPolar(Xlin[1:N], kappa)
    linkFunction[1:N] ~ dmnorm(mu0[1:N], cov = cov[1:N, 1:N])

    # 2. sigma2 - gibbs
    sigma2 ~ dinvgamma(a, b)

    # 3. hyper-prior-kappa(MAP)
    kappa ~ dunif(kappa_a, kappa_b)

    # 4. index, psi
    for(j in 1:(p-1)) {
      d[j] ~ dunif(1e-10, 1e10)
      psi[j] ~ dbeta(c[j], d[j])
    }
    index[1:p] <- alphaTheta(psi[1:(p-1)]*pi)
    Xlin[1:N] <- Xlinear(index[1:p], x[1:N, 1:p])
  })


  # Prior parameters
  ## psi
  if (!is.null(prior$index$psi$alpha) & length(prior$index$psi$alpha) != (p-1))
  {stop("Prior psi has incorrect dimension")}
  if (is.null(prior$index$psi$alpha)){
    psi_c <- c(rep(5000,(p-1)))
  } else{
    psi_c <- prior$index$psi$alpha
  }

  ## sigma2 - scalar
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (a) has incorrect value.")
  } else{
    sigma2_shape <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (b) has incorrect value.")
  } else{
    sigma2_rate <- prior$sigma2$rate
  }

  # kappa
  if (is.null(prior$link$kappa$min)||length(prior$link$kappa$min) >= 2 || prior$link$kappa$min < 0){
    stop("Prior kappa (min) has incorrect value.")
  } else{
    kappa_min <- prior$link$kappa$min
  }

  if (is.null(prior$link$kappa$max)||length(prior$link$kappa$max) >= 2 || prior$link$kappa$max < 0){
    stop("Prior kappa (max) has incorrect value.")
  } else{
    kappa_max <- prior$link$kappa$max
  }

  if (is.null(prior$link$kappa$grid_width)||length(prior$link$kappa$grid_width) >= 2 || prior$link$kappa$grid_width < 0){
    stop("Prior kappa (grid_width) has incorrect value.")
  } else{
    kappa_grid_width <- prior$link$kappa$grid_width
  }

  # Initialize
  init_psi <- init$index$psi
  if (!is.null(init_psi) & length(init_psi) != (p-1)){
    stop("Initial psi has incorrect dimension")
  }

  if (is.null(init$link$kappa)||length(init$link$kappa) >= 2 || init$link$kappa < 0){
    stop("Initial kappa has incorrect value.")
  } else{
    init_kappa <- init$link$kappa
  }

  if (is.null(init$sigma2)||length(init$sigma2) >= 2 || init$sigma2 < 0){
    stop("Initial sigma2 has incorrect value.")
  } else{
    init_sigma2 <- init$sigma2
  }

  # seed
  seedNum <- rep(FALSE, nchain)
  seedNum <- rep(FALSE, nchain)

  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }


  inits_list <- lapply(seq_len(nchain), function(j) initfunction_gpPolar(xobs = X, yobs = as.vector(Y), kappa_init = init_kappa,
                                                                         sigma2_init = init_sigma2, psi_init = init_psi,
                                                                         grid.with = 0.1,
                                                                         sig_a = sigma2_shape, sig_b = sigma2_rate,
                                                                         setSeed = seedNum[j]))
  firstInit <- inits_list[[1]]




  message("== Building model ==")
  suppressMessages(simpleModel <- nimbleModel(Rmodel,
                             data = list(x = X,
                                         y = as.vector(Y)),
                             constants = list(c = psi_c,
                                              kappa_a = kappa_min, kappa_b = kappa_max,
                                              a = sigma2_shape, b = sigma2_rate,
                                              p = p, N = N, mu0 = rep(0, N), pi = pi),
                             inits = firstInit))

  # Assign samplers
  message("== Assigning samplers ==")
  # monitorsList <-  c("linkFunction","index", "psi", "kappa", "sigma", "d")
  monitorsList <- c("index", "sigma2", "linkFunction", "kappa", "Xlin", "d", "psi")
  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
                                             monitors = monitorsList,
                                             print = FALSE))


  mcmcConf$removeSamplers(c("sigma2"))
  mcmcConf$addSampler(target = c("sigma2"),
                      type   = gibbsSampler_sigma2)

  mcmcConf$removeSamplers(c("kappa"))
  mcmcConf$addSampler(target = c("kappa"),
                      type   = gibbsSampler_kappa,
                      control = list(grid.width = kappa_grid_width))

  mcmcConf$removeSamplers(c("psi", "d", "linkFunction"))
  mcmcConf$addSampler(target = c("psi", "d", "linkFunction"),
                      type   = MH_thetaeta)


  mcmc1 <- buildMCMC(mcmcConf)
  end1 <- Sys.time()


  if (!sampling){
    samples <- NULL
    mcmc.out <- NULL

  } else{
    # Compile
    start2 <- Sys.time()
    message("== Compiling model ==")
    suppressMessages(CsimpleModel <- compileNimble(simpleModel))
    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1,
                                            project = simpleModel,
                                            resetFunctions = TRUE))
    end2 <- Sys.time()

    # Sampling
    message("== Running MCMC ==")
    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                        thin = thin,
                        nchains = nchain, setSeed = seedNum, inits = inits_list,
                        summary = FALSE, samplesAsCodaMCMC = TRUE)
    # output
    if (nchain > 1){
      for (i in 1:nchain){
        samples <- rbind(samples, mcmc.out[[i]])
      }
    } else{
      samples <- mcmc.out
    }
  }

  ## Input options
  if (!sampling){
    time <- NULL
  } else{
    time <- difftime(end1, start1, units = "secs") - difftime(end2, start2, units = "secs")
  }

  # results ---------------------------------------------------------------------
  #Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = list(psi = list(alpha = psi_c)),
                                    link = list(kappa = list(min = kappa_min, max = kappa_max,
                                                             grid_width = kappa_grid_width)),
                                    sigma2 = list(shape = sigma2_shape, rate = sigma2_rate)),
                       samplingOptions = list(sampling = sampling,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       init = inits_list, # initial value for MCMC
                       time = time)


  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = mcmc.out, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "gpPolar")

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "gpPolar")

    class(out) <- "bsimSetup"

  }
  return(out)

}

SplineState <- nimble::nimbleList(
  x                       = double(1),
  internal_knots          = double(1),
  boundary_knots          = double(1),
  degree                  = integer(0, default = 3),
  order                   = integer(0, default = 4),
  spline_df               = integer(0, default = 4),

  knot_sequence           = double(1),
  has_internal_multiplicity = logical(0, default = 0),
  is_knot_sequence_latest   = logical(0, default = 0),
  is_extended_knot_sequence = logical(0, default = 0),

  x_index                 = integer(1),
  is_x_index_latest       = logical(0),

  basis = double(2)
)


any_duplicated <- nimble::nimbleFunction(
  run = function(x = double(1)) {
    returnType(logical(0))
    n <- length(x)
    if(n <= 1)
      return(FALSE)

    for(i in 1:n) {
      if(i > 1) {
        for(j in 1:(i-1)) {
          if(x[i] == x[j])
            return(TRUE)            # at least one duplicated element
        }
      }
    }
    return(FALSE)                   # no duplicated
  }
)






mat_wo_col1 <- nimble::nimbleFunction(
  run = function(mat = double(2)){
    returnType(double(2))
    x_ncol <- nimDim(mat)[2]
    if (x_ncol > 1){
      idxCol <- 2:x_ncol
      return(mat[, idxCol])
    } else{
      nimStop("No column left in the matrix.")
    }
  }
)


update_spline_df <- nimble::nimbleFunction(
  run = function(temp = SplineState()){
    temp$spline_df <- nimDim(temp$internal_knots)[1]+ temp$order
  }
)


update_x_index <- nimble::nimbleFunction(
  run = function(temp = SplineState()){
    if (temp$is_x_index_latest & nimDim(temp$x_index)[1] > 0){
      return()
    }

    n <- nimDim(temp$x)[1]
    x_index <- numeric(n)
    m <- nimDim(temp$internal_knots)[1]

    if (m == 0) {
      x_index <- nimRep(1, nimDim(temp$x)[1])
    } else{
      idx <- nimOrder(temp$x)
      sortedX <- nimSort(temp$x)

      j <- 0
      for(i in 1:n) {
        xi <- sortedX[i]
        while((j < m) & (xi >= temp$internal_knots[j+1]))
          j <- j + 1L
        x_index[i] <- (j)
      }

      x_index <- x_index[nimOrder(idx*1.0)]
    }
    # print(x_index)
    temp$x_index <- x_index
    temp$is_x_index_latest <- TRUE

  })


update_knot_sequence <- nimble::nimbleFunction(
  run = function(temp = SplineState()){
    n <- temp$order
    out <- nimC(nimRep(temp$boundary_knots[1], n), temp$internal_knots,
                nimRep(temp$boundary_knots[2], n))
    temp$knot_sequence <- out
    temp$is_knot_sequence_latest <- TRUE
  })



get_basis_simple <- nimble::nimbleFunction(
  run = function(temp = SplineState()){
    returnType(double(2))

    update_spline_df(temp)
    update_x_index(temp)

    bmat <- nimMatrix(0, nimDim(temp$x)[1], temp$spline_df)

    for(i in 1:nimDim(temp$x)[1]){
      bmat[i, (temp$x_index[i]+1)] <- 1
    }

    if (temp$degree > 0){
      update_knot_sequence(temp)
    }

    for (k in 1:temp$degree){
      k_offset <- temp$degree - k
      for (i in 1:nimDim(temp$x)[1]){
        saved <- 0.0
        for (j in 1:k){
          j_index <- (temp$x_index[i]) + j
          i1 <- j_index + k_offset + 1
          i2 <- j_index + temp$order
          den <- temp$knot_sequence[i2] - temp$knot_sequence[i1]
          term <- bmat[i, j_index]/den
          bmat[i, j_index] <- saved + (temp$knot_sequence[i2] - temp$x[i])*term
          saved <- (temp$x[i] - temp$knot_sequence[i1])*term
        }
        bmat[i, (temp$x_index[i]+k+1)] <- saved
      }
    }

    return(bmat)

  }
)



simplify_knots <- nimble::nimbleFunction(
  run = function(temp = SplineState(),
                 internal_knots = double(1),
                 boundary_knots = double(1)){

    # boundary knots
    if (nimDim(boundary_knots)[1] == 0){
      if(nimDim(boundary_knots)[1] != 2 & nimDim(temp$x)[1] > 0){
        left <- min(temp$x)
        right <- max(temp$x)
        if (left == right){
          nimStop("Cannot set boundary knots from x.")
        }
        temp$boundary_knots <- nimC(left, right)

      }
    }
    else{
      if (sum(is.na(boundary_knots)) > 0){
        nimStop("Cannot set boundary knots from x.")
      }

      left <- boundary_knots[1]
      right <- boundary_knots[2]

      if (left == right){
        nimStop("Need two distinct boundary knots.")
      }
      temp$boundary_knots <- nimC(left, right)
    }

    # Inner knots
    if (sum(is.na(internal_knots)) > 0){
      nimStop("Internal knots cannot contain NA.")
    }

    if (nimDim(internal_knots)[1] > 0){
      # sort knots
      sortedKnots <- nimSort(internal_knots)
      minKnots <- sortedKnots[1]
      maxKnots <- sortedKnots[nimDim(internal_knots)[1]]

      if (nimDim(temp$boundary_knots)[1] == 2 &
          (temp$boundary_knots[1] >= minKnots | temp$boundary_knots[2] <= maxKnots)){
        nimStop("Internal knots must be set inside boundary.")
      }
      allKnots <- nimC(internal_knots, temp$boundary_knots)
      temp$has_internal_multiplicity <- any_duplicated(allKnots)
      temp$internal_knots <- sortedKnots
    }
    else{
      temp$has_internal_multiplicity <- FALSE
      temp$internal_knots <- numeric(0)
    }
  }
)


get_inside_x <- nimble::nimbleFunction(
  run = function(x = double(1), boundary_knots = double(1)){
    returnType(double(1))
    minKnots <- min(boundary_knots)
    maxKnots <- max(boundary_knots)
    elements <- (x >= minKnots & x <= maxKnots)
    res <- x[which(elements)]

    return(res)
  })


gen_default_internal_knots <- nimble::nimbleFunction(
  run = function(inside_x = double(1),
                 boundary_knots = double(1),
                 n_internal_knots = integer(0)){
    returnType(double(1))

    prob_vec <- nimSeq(0, 1, length.out = n_internal_knots+2)[2:(n_internal_knots+1)]
    internal_knots <- quantile_nimble(inside_x, prob_vec)
    any_dup  <- any_duplicated(internal_knots)

    if (any_dup){
      nimPrint("Set equidistant internal knots \n")
      nimPrint("(found duplicated knots from quantiles).")
      return(nimSeq(boundary_knots[1], boundary_knots[2],
                    length.out = n_internal_knots+2)[2:(n_internal_knots+1)])
    }

    min_int_knots <- internal_knots[1]
    max_int_knots <- internal_knots[n_internal_knots]
    if (boundary_knots[1] >= min_int_knots |
        boundary_knots[2] <= max_int_knots) {
      return(nimSeq(boundary_knots[1], boundary_knots[2],
                    length.out = n_internal_knots+2)[2:(n_internal_knots+1)])
    }
    return(internal_knots)
  }
)




SplineBase1 <- nimble::nimbleFunction(
  run = function(x = double(1), spline_df = integer(0), degree = integer(0, default = 3),
                 intercept = integer(0), Boundary.knots =  double(1, default = numeric(0))){
    returnType(SplineState())

    temp <- SplineState$new()

    temp$x <- x
    temp$degree <- degree
    temp$order <- degree + 1
    temp$spline_df <- spline_df

    # determine internal knots
    num_knots <- spline_df - temp$order
    simplify_knots(temp, numeric(0), Boundary.knots)

    # Internal knots
    if(num_knots > 0){
      inside_x <- get_inside_x(temp$x, temp$boundary_knots)
      temp$internal_knots <- gen_default_internal_knots(inside_x, temp$boundary_knots,num_knots)
    }

    return(temp)

  }
)


SplineBase2 <- nimble::nimbleFunction(
  run = function(x = double(1), degree = integer(0, default = 3),
                 internal_knots = double(1),
                 Boundary.knots =  double(1, default = numeric(0))){
    returnType(SplineState())

    temp <- SplineState$new()
    temp$x <- x
    temp$degree <- degree
    simplify_knots(temp, internal_knots, Boundary.knots)
    temp$order <- degree + 1

    return(temp)

  }
)


basis <- nimble::nimbleFunction(
  run = function(temp = SplineState(), complete_basis = logical(0)){
    returnType(SplineState())

    b_mat <- get_basis_simple(temp)

    if (complete_basis){
      temp$basis <- b_mat
    } else{
      b_mat_fix <- mat_wo_col1(b_mat)
      temp$basis <- b_mat_fix
    }

    return(temp)

  }
)


bsNimble <- nimble::nimbleFunction(
  run = function(x = double(1), df = integer(0, default = 0),
                 knots = double(1, default = numeric(0)),
                 degree = integer(0, default = 3),
                 intercept = logical(0, default = FALSE),
                 boundary_knots = double(1, default = numeric(0))){
    # returnType(double(2))
    returnType(SplineState())

    # Decide knots
    if (df > 0 & length(knots) == 0){
      wo_intercept <- !intercept
      spline_df <- df + wo_intercept

      B <- SplineBase1(x              = x,
                       spline_df      = spline_df,
                       degree         = degree,
                       intercept      = wo_intercept,
                       Boundary.knots = boundary_knots)
    } else{
      B <- SplineBase2(x              = x,
                            internal_knots = knots,
                            degree         = degree,
                            Boundary.knots = boundary_knots)
    }

    # Calculate basis
    basisT <- basis(B, complete_basis = intercept)


    return(basisT)

  }
)


bsBasis <- nimble::nimbleFunction(
  run = function(x = double(1), df = integer(0, default = 0),
                 knots = double(1),
                 degree = integer(0, default = 3),
                 intercept = logical(0, default = FALSE),
                 boundary_knots = double(1)){
    returnType(double(2))
    A <- bsNimble(x = x, df = df,
                  knots = knots, degree = degree,
                  intercept = intercept,
                  boundary_knots = boundary_knots)
    return(A$basis)

  }
)

nimNorm <- nimble::nimbleFunction(
  run = function(vec = double(1)){
    returnType(double(1))
    sumALL <- sqrt(sum(vec^2))
    result <- vec/sumALL

    return(result)
  }
)


rW <- nimble::nimbleFunction(
  run = function(size = integer(0), lambda = double(0), d = integer(0)){
    returnType(double(1))
    resultW <- numeric(size)
    b <- d / (sqrt(4 * lambda * lambda + d * d) + 2 * lambda)
    x0 <- (1 - b) / (1 + b)
    c <- lambda * x0 + d * log(1 - x0^2)

    for(i in 1:size){
      continue_sampling <- TRUE
      while (continue_sampling) {
        Z <- rbeta(1, d/2, d/2)
        w <- (1 - (1 + b) * Z) / (1 - (1 - b) * Z)
        U <- runif(1, 0, 1)

        if (lambda * w + d * log(1 - x0 * w) - c < log(U)) {
          resultW[i] <- w
          continue_sampling <- FALSE
        }
      }
    }

    return(resultW)
  }
)


# modified Bessel function of the first kind
besselI_nimble <- nimble::nimbleFunction(
  run = function(x = double(0), nu = double(0), tol = double(0, default = 1e-10), max_iter = integer(0, default = 1e4)) {
    returnType(double(0))

    sum <- 0.0
    m <- 0
    term <- 1.0

    while (term > tol & m < max_iter) {
      logpow_val <- (2 * m + nu) * log(x / 2)
      logdenom <- lgamma(m + 1.0) + lgamma(m + nu + 1.0)
      logterm <- logpow_val - logdenom
      term <- exp(logterm)
      sum <- sum + term
      m <- m + 1
    }

    return(sum)
  }
)


dvMFnim <- nimble::nimbleFunction(
  run = function(x = double(1), theta = double(1), log = integer(0, default = 0)) {
    returnType(double(0))
    d <- nimDim(x)[1]
    nu_1 <- (d / 2)
    kappa <- sqrt(sum(theta^2))

    ## log density
    part1 <- inprod(theta, x)
    part2 <- log(besselI_nimble(x = kappa, nu = nu_1))
    part3 <- lgamma(nu_1)
    part4 <- (nu_1 - 1) * log(kappa / 2)

    logden <- part1 - (part2 + part3 - part4)

    if (log)
      return(logden)
    else
      return(exp(logden))
  }
)


rvMFnim <- nimble::nimbleFunction(
  run = function(n = integer(0), theta = double(1)) {
    returnType(double(1))
    p <- nimDim(theta)[1]
    kappa <- sqrt(sum(theta^2))

    if (kappa == 0) {
      X <- nimMatrix(rnorm(n * p), n, p)
      for (j in 1:n)
        X[j, 1:p] <- nimNorm(X[j, 1:p])
    } else {
      d <- p - 1
      W <- rW(n, kappa, d)

      mu <- nimMatrix(nimNorm(theta), ncol = 1)
      Z <- nimMatrix(rnorm(n * p), nrow = n, ncol = p)
      Vtemp <- Z - Z %*% mu %*% t(mu)
      X <- matrix(0, nrow = n, ncol = p)
      for (i in 1:n) {
        Vtemp[i, 1:p] <- nimNorm(Vtemp[i, 1:p])
        sqrt_term <- sqrt(1 - W[i]^2)
        for (j in 1:p) {
          X[i, j] <- W[i] * mu[j, 1] +
            sqrt_term * Vtemp[i, j]
        }
      }
    }
    return(X[1, ])
  }
)


Stheta <- nimble::nimbleFunction(
  run = function(Xmat = double(2), Y = double(2), rho = double(0)){
    returnType(double(0))
    p <- nimDim(Xmat)[2]
    Y_mat <- matrix(Y, ncol = 1)

    temp_xmat <- t(Xmat) %*% Xmat
    temp_rho <- diag(nimRep(rho, p))
    Sigma_0 <- inverse(temp_xmat)
    temp_sum <- temp_xmat + temp_rho
    Sigma_rho <- inverse(temp_sum)

    part1 <- t(Y_mat) %*% Y_mat
    part2 <- t(Y_mat) %*% Xmat
    part3 <- Sigma_0 + 0.5 * (Sigma_rho %*% (diag(p) - inverse(Sigma_0) %*% Sigma_rho))

    first <- part1[1,1]
    second <- part2 %*% part3 %*% t(part2)
    result <- first - second[1,1]
    return(result)

  }
)

# marginalized posterior(D)
postll_bspline_fisher <- nimble::nimbleFunction(
  setup = function(model){
    A <- model$getConstants()$a_sig
    B <- model$getConstants()$b_sig
    n <- model$getConstants()$N
    Y <- model$Y
    index_prior <- model$getConstants()$index_prior
    consprior <- sqrt(sum(index_prior^2))
  },
  run = function(rho = double(0)){
    returnType(double(0))

    index <- model$index
    Xmat <- model$Xmat

    Stheta0 <- Stheta(Xmat, Y, rho)
    part1 <- (-A + n/2)*log(Stheta0 +(2/B))

    angle <- inprod(index, index_prior)
    part2 <- consprior * angle

    logresult <- part1 + part2
    return(logresult)

  }
)

# estBeta
estBeta_fisher <- nimble::nimbleFunction(
  run = function(Xmat = double(2), Y = double(2), rho = double(0)){
    returnType(double(1))
    p <- nimDim(Xmat)[2]
    part1 <- inverse(t(Xmat) %*% Xmat + diag(nimRep(rho, p)))
    part2 <- t(Xmat) %*% Y
    result <- part1 %*% part2
    return (result[,1])
  }
)



gvcCV <- nimble::nimbleFunction(
  run = function(Xmat = double(2), Y = double(2)){
    returnType(double(0))
    grid <- nimSeq(from = 0.1, to = 4, length.out = 20)
    n <- nimDim(Xmat)[1]
    p <- nimDim(Xmat)[2]

    for (i in 1:20){
      rho <- grid[i]
      # print(paste("time: ", i))
      beta <- estBeta_fisher(Xmat, Y, rho) ## beta
      norm <- (Y - Xmat %*% beta)^2
      RSS <- mean(norm[,1])

      Sigma_rho <- inverse(t(Xmat) %*% Xmat + diag(nimRep(rho, p)))
      Ls <- Xmat %*% Sigma_rho %*% t(Xmat) # N*N
      part1 <- (diag(n) - Ls) %*% Xmat # n*p
      part2 <- inverse(t(Xmat) %*% (diag(n) - Ls) %*% Xmat)
      part3 <- t(Xmat) %*% (diag(n) - Ls)
      L_rho <- Ls + part1 %*% part2 %*% part3 # N*N
      df <- (1-(sum(diag(L_rho))/n))^2
      currgcv <- RSS/df
      # print(currgcv)

      if (i == 1){
        mingcv <- currgcv
        optRho <- rho
      } else{
        if (currgcv < mingcv){
          # Find min
          mingcv <- currgcv
          optRho <- rho
        }
      }
    }

    return(optRho)

  }
)



# Design Matrix of bsFisher(), bsPolar(), bsSpike()
transX_fisher <- nimble::nimbleFunction(
  run = function(Xlin = double(1), df = integer(0), degree = integer(0), delta = double(0)){
    returnType(double(2))
    n_internal_knots <- df - degree
    idxmax <- n_internal_knots + 1
    # prob_vec <- nimSeq(0, 1, length.out = n_internal_knots+2)[2:idxmax]

    seq_full <- nimSeq(0, 1, length.out = n_internal_knots + 2)
    prob_vec <- seq_full[2:idxmax]
    internal_knots <- quantile_nimble(Xlin, prob_vec)

    X <- bsBasis(Xlin, df = df, degree = degree,
                 knots = internal_knots,
                 boundary_knots = c(min(Xlin)-delta, max(Xlin)+delta))
    #
    # N <- nimDim(X)[1]
    # result <- matrix(X, nrow = N, ncol = df)
    return(X)

  }
)

## initial value
initFunction_bF <- function(X, Y,
                            index, sigma2, beta,
                            df, degree, delta,
                            index_direction, setSeed){
  if (setSeed != FALSE){
    set.seed(setSeed)
  }

  # index
  if (is.null(index)){
    init_index <- rvMFnim(1, index_direction)
  } else{
    init_index <- index
  }
  if (sum((init_index)^2) != 1){
    init_index <- init_index/sqrt(sum(init_index^2))
  }

  # sigma2
  init_sigma2 <- sigma2
  init_Xlin <- as.vector(X %*% matrix(init_index, ncol = 1))
  init_Xmat <- transX_fisher(init_Xlin, df = df, degree = degree, delta = delta)
  init_rho <- gvcCV(init_Xmat, Y)

  # beta
  if (is.null(beta)){
    init_beta <- estBeta_fisher(init_Xmat, Y, init_rho)
  } else{
    init_beta <- beta
  }
  init_linkFunction <- init_Xmat %*%  init_beta

  return(list(index = init_index, Xlin = init_Xlin,
              Xmat = init_Xmat,
              sigma2 = init_sigma2, beta = init_beta,
              linkFunction = init_linkFunction))

}


pred_bsplineFisher <- nimbleFunction(
  run = function(newdata = double(2), indexSample = double(2),
                 knotsSample = double(2), XlinSample = double(2),
                 betaSample = double(2), sigma2_samples = double(1), nsamp = double(0),
                 df = double(0), degree = double(0), delta = double(0),
                 prediction = integer(0)){
    # prediction = 1: latent, prediction = 2: response
    returnType(double(2))
    new_ncol <- nimDim(newdata)[1]
    testPred <- nimMatrix(0, nrow = nsamp, ncol = new_ncol)

    for (i in 1:nsamp){
      sampleZ <- newdata %*% matrix(indexSample[i, ], ncol = 1) # linear pred
      Xmat <- bsBasis(sampleZ[,1], df = df,
                      degree = degree, knots = knotsSample[i,],
                      boundary_knots = c(min(XlinSample[i,])-delta,
                                         max(XlinSample[i,])+delta))
      linkPred <- Xmat %*% betaSample[i,]

      if (prediction == 1){ # latent
        testPred[i,] <- linkPred[,1]

      } else if (prediction == 2){
        predsigma <- chol(diag(rep(sigma2_samples[i], new_ncol)))
        pred <- t(rmnorm_chol(1, mean = linkPred[,1], cholesky = predsigma, prec_param  = FALSE))

        # computation for data over boundaries: substitute to last values
        idxmin <- (sampleZ[,1] < (min(XlinSample[i,]) - delta))
        idxmax <- (sampleZ[,1] > (max(XlinSample[i,]) + delta))

        if (sum(idxmin) != 0){
          minVal_idx <- which(XlinSample[i,] == min(XlinSample[i,]))
          predmin <- min(pred[1, minVal_idx])
          pred[1, idxmin] <- nimRep(predmin, sum(idxmin))
        }

        if (sum(idxmax) != 0){
          maxVal_idx <- which(XlinSample[i,] == max(XlinSample[i,]))
          predmax <- max(pred[1, maxVal_idx])
          pred[1, idxmax] <- nimRep(predmax, sum(idxmax))
        }

        testPred[i,] <- pred[1,]
      }

    }
    return(testPred)
  }
)

# Sampler
## 1. nuSampler
nuSampler_bspline_sphere <- nimble::nimbleFunction(
  contains = nimble::sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(c(target, "index", "knots"))
    p <- model$getConstants()$p
    r1 <- model$getConstants()$r1
    r2 <- model$getConstants()$r2
    r3 <- 0.5
    Rpostll <- postll_bspline_sphere(model)
  },
  run = function(){
    # nimPrint("nu")
    k <- 0; olda <- 0.0; oldb <- 0.0
    k <- model$k[1]
    currnu <- model$nu # vector[1:p]
    currTheta <- model$index # vector[1:p]
    oldknots <- model$knots # vector[1:maxknots]
    olda <- model$a_alpha[1]; oldb <- model$b_alpha[1]
    # nimPrint("################# Start iteration #################")
    beforell <- Rpostll$run() # compute
    # nimCat("beforell:", beforell, "\n")

    # proposalidx <- integer(1) # initialize
    proposalidx <- rcat(1, prob = rep(1,p)/p)
    # nimCat("Propsed idx: ", proposalidx, "\n")
    currDeltIdx <- 0; curralpha <- 0; nnu <- 0
    currDeltIdx <- currnu[proposalidx] ## current nu value
    curralpha <- currTheta[proposalidx]
    nnu <- sum(currnu)
    propnu <- currnu
    propnu[proposalidx] <- 1-currnu[proposalidx]

    if (proposalidx == 1){
      # nimPrint("Reject assumption")
    } else{
      model$nu <<- propnu

      if (currDeltIdx == 1){
        # nimPrint("change to 0")
        changedTheta <- currTheta
        changedTheta[proposalidx] <- 0
        changedTheta <- changedTheta/sqrt(sum(changedTheta^2))
        model$index <<- changedTheta
        r4 <- 0.0; r4 <- (nnu-1)/2

        # computeA1
        logA1 <- computeA1_nu1(r1, r2, nnu, p, r3, r4, curralpha)

      } else{
        # nimPrint("change to 1")
        r4 <- 0.0; eta <- 0.0; signTheta <- 0.0
        r4 <- nnu/2
        eta <- rbeta(1, r3, r4)
        changedTheta <- currTheta * sqrt(1-eta)
        signTheta <- 2*rbinom(1, 1, 1/2) -1
        changedTheta[proposalidx] <- signTheta * sqrt(eta)
        model$index <<- changedTheta

        # computeA1
        logA1 <- computeA1_nu0(r1, r2, nnu, p, r3, r4, eta)
      }

      model$calculate(c("Xlin", "a_alpha", "b_alpha")) ##
      newa <- 0.0; newb <- 0.0
      newa <- model$a_alpha[1]; newb <- model$b_alpha[1]
      changesKnots <- newKnots(oldknots, olda, oldb, newa, newb, k)
      model$knots <<- changesKnots
      model$calculate(calcNodes) ## Change all

      # nimCat("logA1:", logA1, "\n")
      afterll <- Rpostll$run()
      # nimCat("afterll:", afterll, "\n")

      L <- (afterll - beforell) + logA1
      # nimCat("logratio:", L, "\n")
      # nimPrint(L)
      ratio <- min(c(1, exp(L)))
      # nimCat("ratio:", ratio, "\n")
      u <- runif(1)

      # accept/reject
      if(u < ratio) { # accept
        # nimPrint("accept")
        # calculate knots
        tempTheta <- 0.0; tempTheta <- changedTheta[1]
        if (tempTheta < 0 ){
          # nimPrint("change sign")
          newa <- 0.0; newb <- 0.0
          newa <- model$a_alpha[1]; newb <- model$b_alpha[1]
          model$index <<- (model$index) * (-1)
          temp <- (model$knots) * (-1)
          tempSort <- nimSort(temp)
          model$nu <<- tempSort
          model$a_alpha <<- nimC((newa) * (-1))
          model$b_alpha <<- nimC((newb) * (-1))
          model$calculate(calcNodes)
        }

        nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
      }
      if(u > ratio | changedTheta[1] == 0){
        # nimPrint("reject")
        nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
      }
    }

  },
  methods = list(reset = function(){})
)

## 2. indexSampler
indexSampler_bspline_sphere <-  nimble::nimbleFunction(
  contains = nimble::sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(c(target, "knots"))
    p <- model$getConstants()$p
    Rpostll <- postll_bspline_sphere(model)
    sig1_tune <- 0.15
  },
  run = function(){
    # nimPrint("################# Sampling index #################")
    # nimPrint("index")
    k <- 0; olda <- 0.0; oldb <- 0.0
    oldknots <- model$knots
    k <- model$k[1]
    olda <- model$a_alpha[1]
    oldb <- model$b_alpha[1]
    currTheta <- model$index
    # nimCat("currTheta:", currTheta, "\n")

    beforell <- Rpostll$run() # compute

    # nimCat("beforell:", beforell, "\n")

    totalP <- which(currTheta != 0)
    numtotalP <- 0; numtotalP <- length(totalP)
    # nimCat("totalP:", totalP, "\n")
    if (numtotalP < 2){
      # nimCat("no variable selected \n")
    } else{
      idxs <- integer(2)
      idxs[1] <- rcat(1, nimRep(1 / numtotalP, numtotalP))
      idxs[2] <- rcat(1, nimRep(1 / numtotalP, numtotalP))

      while (idxs[1] == idxs[2]) {
        idxs[1] <- rcat(1, nimRep(1 / numtotalP, numtotalP))
        idxs[2] <- rcat(1, nimRep(1 / numtotalP, numtotalP))
      }

      eta <- 0.0
      idx_i <- totalP[idxs[1]]
      idx_j <- totalP[idxs[2]]
      eta <- rnormTrun(0, sig1_tune)
      # nimCat("idxs: ", idxs, "\n")
      # nimCat("idx_i: ", idx_i, "\n")
      # nimCat("idx_j: ", idx_j, "\n")
      # nimCat("eta: ", eta, "\n")

      proposedTheta <- currTheta
      proposedTheta[idx_i] <- currTheta[idx_i] * cos(eta) + currTheta[idx_j] * sin(eta)
      proposedTheta[idx_j] <- -currTheta[idx_i] * sin(eta) + currTheta[idx_j] * cos(eta)
      # nimCat("proposedTheta:", proposedTheta, "\n")

      model$index <<- proposedTheta
      model$calculate(c("Xlin", "a_alpha", "b_alpha")) ##
      newa <- 0.0; newb <- 0.0
      newa <- model$a_alpha[1]; newb <- model$b_alpha[1]
      changesKnots <- newKnots(oldknots, olda, oldb, newa, newb, k)
      model$knots <<- changesKnots

      model$calculate(calcNodes) ##
      afterll <- Rpostll$run()
      # nimCat("afterll:", afterll, "\n")

      L <- (afterll - beforell)
      # nimCat("logratio:", L, "\n")
      # nimPrint(L)
      ratio <- min(c(1, exp(L)))
      # nimPrint(ratio)
      u <- runif(1)

      # accept/reject
      if(u < ratio) { # accept
        # nimPrint("accept")
        # calculate knots
        tempTheta <- 0.0; tempTheta <- proposedTheta[1]
        if (tempTheta < 0 ){
          # nimPrint("change sign")
          newa <- 0.0; newb <- 0.0
          newa <- model$a_alpha[1]; newb <- model$b_alpha[1]
          model$index <<- (model$index) * (-1)
          temp <- (model$knots) * (-1)
          tempSort <- nimSort(temp)
          model$nu <<- tempSort
          model$a_alpha <<- nimC((newa) * (-1))
          model$b_alpha <<- nimC((newb) * (-1))
          model$calculate(calcNodes)
        }
        ## change knots
        nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
      }
      if(u > ratio | proposedTheta[1] == 0){
        # nimPrint("reject")
        nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
      }
    }



  },
  methods = list(reset = function(){})
)

## 3. sampling knots
knotsSampler_bspline_sphere <-  nimble::nimbleFunction(
  contains = nimble::sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(c(target, "k"))
    p <- model$getConstants()$p[1]
    Rpostll <- postll_knots(model)
    lambda <- model$getConstants()$lambda_k[1]
    tau <- model$getConstants()$tau[1]
    r5 <- 1
    r6 <- 1
    maxknots <- model$getConstants()$maxknots[1]
    sig2_tune <- 0.4
  },
  run = function(){
    # nimPrint("################# Sampling knots #################")
    # Calculate probabilities: bk, dk, mk
    # nimCopy(from = model, to = mvSaved, row = 1, nodes = saveNodes, logProb = TRUE)
    currk <- 0; a_alpha <- 0.0; b_alpha <- 0.0
    currk <- model$k[1]
    a_alpha <- model$a_alpha[1]
    b_alpha <- model$b_alpha[1]
    bk <- prop_add(lambda, currk, tau)
    dk <- prop_delete(lambda, currk, tau)
    mk <- 1 - bk - dk
    currknots <- model$knots
    # nimCat("curr knots:", currknots, "\n")
    beforell <- Rpostll$run() # compute
    # nimCat("beforell:", beforell, "\n")
    # nimCat("probabilities:, ",  nimC(bk, dk, mk), "\n")

    choseu <- rcat(1, prob = nimC(bk, dk, mk))
    newknots <- nimNumeric(maxknots)
    # nimCat("choseu:, ", choseu, "\n")
    if (choseu == 1){
      # nimPrint("Add knots")
      # add knots
      idx <- rcat(1, prob = nimRep(1/(currk+1), (currk+1))) # interval index
      # nimCat("Location:", idx, "\n")
      minIdx <- 0.0; maxIdx <- 0.0

      if (idx == 1){
        minIdx <- a_alpha; maxIdx <- currknots[idx]
      } else if (idx == (currk + 1)){
        minIdx <- currknots[(idx-1)]; maxIdx <- b_alpha
      } else {
        minIdx <- currknots[(idx-1)]; maxIdx <- currknots[idx]
      }
      # nimCat("minIdx:", minIdx, ", maxIdx:", maxIdx, "\n")

      augEta <- 0.0
      augEta <- rbeta(1, r5, r6) * (maxIdx - minIdx) + minIdx
      # nimCat("augEta:", augEta, "\n")

      for (i in 1:(maxknots)){
        if (i < idx){
          newknots[i] <- currknots[i]
        } else if (i == idx){
          newknots[i] <- augEta
        } else{
          newknots[i] <- currknots[(i-1)] ## index
        }
      }
      # nimCat("newknots:", newknots, "\n")
      newk <- currk + 1
      logA2 <- computeA2_add(currk, minIdx, maxIdx, a_alpha, b_alpha)
      # nimCat("logA2", logA2, "\n")
    } else if (choseu == 2){
      # nimPrint("Delete knots")
      # delete
      idx <- rcat(1, prob = nimRep(1/(currk), currk))
      # nimCat("Location:", idx, "\n")
      minIdx <- 0.0; maxIdx <- 0.0

      if (currk == 1){
        minIdx <- a_alpha; maxIdx <- b_alpha
      } else{
        if (idx == 1){
          minIdx <- a_alpha; maxIdx <- currknots[(idx+1)]
        } else if (idx == (currk)){
          minIdx <- currknots[(idx-1)]; maxIdx <- b_alpha
        } else {
          minIdx <- currknots[(idx-1)]; maxIdx <- currknots[(idx+1)]
        }
      }

      # nimCat("minIdx:", minIdx, ", maxIdx:", maxIdx, "\n")

      for (i in 1:(maxknots-1)){ ## (maxknots)
        if (i < idx){
          newknots[i] <- currknots[i]
        } else if (i >= idx & i < maxknots){
          newknots[i] <- currknots[i+1]
        } else if (i == maxknots){
          newknots[i] <- 0
        }
      }
      # nimCat("newknots:", newknots, "\n")
      newk <- currk - 1
      logA2 <- computeA2_delete(currk, minIdx, maxIdx, a_alpha, b_alpha)
      # nimCat("logA2", logA2, "\n")
    } else{
      # nimPrint("Relocate knots")
      # relocate
      idx <- rcat(1, prob = nimRep(1/(currk), currk))
      # nimCat("Location:", idx, "\n")
      minIdx <- 0.0; maxIdx <- 0.0

      if (currk == 1){
        minIdx <- a_alpha; maxIdx <- b_alpha
      } else{
        if (idx == 1){
          minIdx <- a_alpha; maxIdx <- currknots[(idx+1)]
        } else if (idx == (currk)){
          minIdx <- currknots[(idx-1)]; maxIdx <- b_alpha
        } else {
          minIdx <- currknots[(idx-1)]; maxIdx <- currknots[(idx+1)]
        }
      }

      # nimCat("minIdx:", minIdx, ", maxIdx:", maxIdx, "\n")

      newLoc <- 0.0
      newLoc <- rtruncnorm(1, mean = currknots[idx],
                           sigma = (maxIdx - minIdx)*sig2_tune,
                           lower = minIdx, upper = maxIdx)[1]
      # nimCat("newLoc:", newLoc, "\n")
      newknots <- currknots
      newknots[idx] <- newLoc
      # nimCat("newknots:", newknots, "\n")
      newk <- currk
      logA2 <- 0
    }

    if (newk == 0){
      # nimPrint("no knots!")
    } else{
      model$k <<- nimC(newk)
      model$knots <<- newknots
      model$calculate(calcNodes)

      afterll <- Rpostll$run()
      # nimCat("afterll:", afterll, "\n")

      L <- (afterll - beforell) + logA2
      # nimCat("logratio:", L, "\n")
      # nimPrint(L)
      ratio <- min(c(1, exp(L)))
      # nimCat("ratio:", ratio, "\n")
      u <- runif(1)

      # accept/reject
      if(u < ratio) { # accept
        # nimPrint("accept")
        ## change knots
        nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
      }
      if(u > ratio){
        # nimPrint("reject")
        nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
      }
    }


  },
  methods = list(reset = function(){})
)


## 4. beta Sampler
betaSampler_bspline_sphere <- nimble::nimbleFunction(
    contains = nimble::sampler_BASE,
    setup = function(model, mvSaved, target, control) {
      calcNodes <- model$getDependencies(target)
      N <- model$getConstants()$N
      y <- model$Y
      tau2 <- model$getConstants()$tau
      Sigma0 <- model$getConstants()$Sigma0
      maxBasis <- model$getConstants()$maxBasis
    },
    run = function(){
      # nimPrint("beta")
      numBasis <- 0; sigma2 <- 0.0; alpha <- 0.0
      numBasis <- model$numBasis[1]
      xmat <- model$Xmat
      sigma2 <- model$sigma2[1]
      alpha <- N/2
      numBasisI <- as.integer(numBasis)
      Sigma <- computeSig(tau2, Sigma0, xmat, numBasisI)
      realxmat <- xmat[,1:numBasis]

      meanBeta <- Sigma %*% t(realxmat) %*% y
      covBeta <- sigma2 * Sigma
      cholBeta <- chol(covBeta)
      newBeta <- rmnorm_chol(1, mean = meanBeta[,1], cholesky  = cholBeta,
                             prec_param = FALSE)
      betaMat <- nimMatrix(0, nrow = maxBasis, ncol = 1)
      for (i in 1:numBasis){
        betaMat[i, 1] <- newBeta[i]
      }

      model$beta <<- betaMat
      model$calculate(calcNodes)
      nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)

    },
    methods = list( reset = function () {} )

  )


## 5. sigma sampler
sigma2Sampler_bspline_sphere <-  nimble::nimbleFunction(
    contains = nimble::sampler_BASE,
    setup = function(model, mvSaved, target, control) {
      calcNodes <- model$getDependencies(target)
      N <- model$getConstants()$N
      y <- model$Y
      tau2 <- model$getConstants()$tau
      Sigma0 <- model$getConstants()$Sigma0
    },
    run = function(){
      # nimPrint("sigma2")
      numBasis <- model$numBasis
      xmat <- model$Xmat
      alpha <- N/2
      numBasisI <- as.integer(numBasis[1])
      Sigma <- computeSig(tau2, Sigma0, xmat, numBasisI)
      SS <- ComputeS(y, xmat, Sigma, numBasisI)
      beta <- SS/2
      newSigma <- rinvgamma(1, alpha, beta)
      # nimPrint(newSigma)

      model$sigma2 <<- nimC(newSigma)
      model$calculate(calcNodes)
      nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)

    },
    methods = list( reset = function () {} )

  )

expcov_gpSphere <- nimbleFunction(
  run = function(vec = double(1), l = double(0), amp = double(0)){
    returnType(double(2))
    n <- length(vec)
    result <- matrix(nrow = n, ncol = n, init = FALSE)
    for(i in 1:n){
      for(j in 1:n){
        result[i, j] <- amp * exp(-(vec[i] - vec[j])^2/l)
      }
    }
    result <- (result + t(result))/2 + diag(rep(1e-4, n))

    return(result)

  }
)

# test expcov (n1 * n2)
expcovTest_gpSphere <- nimbleFunction(
  run = function(vec1 = double(1), vec2 = double(1), l = double(0), amp = double(0)){
    returnType(double(2))
    n1 <- length(vec1)
    n2 <- length(vec2)
    result <- matrix(nrow = n1, ncol = n2, init = FALSE)
    for(i in 1:n1){
      for(j in 1:n2){
        result[i, j] <- amp * exp(-(vec1[i] - vec2[j])^2/l)
      }
    }

    return(result)

  }
)

# beta
conBeta <- nimbleFunction(
  run = function(beta = double(1)){
    returnType(double(1))
    beta <- beta/sqrt(sum(beta^2))
    if (beta[1] < 0) beta <- beta*(-1)
    return (beta)
  }
)


# MAP
obj_btt <- nimbleFunction(
  setup = function(model, p, shape, rate,
                   a_amp, b_amp, a_sig, b_sig) {
    X <- model$X
    Y <- model$Y
  },
  run = function(par = double(1)) {
    returnType(double(0))
    # nimCat("params: ",par)
    # nimPrint("")
    index <- par[1:p]
    lengthscale  <- exp(par[(p+1)])
    amp    <- exp(par[(p+2)])
    # if (sigmaTrue) sigma2  <- exp(par[p+3])
    index <- index/sqrt(sum(index^2))
    # nimCat("index: ",index)

    if (abs(lengthscale) <= 1e-10){
      lengthscale <- lengthscale + 1e-6
    }

    n <- nimDim(X)[1]
    Xlin <- numeric(n)
    for (i in 1:n){
      Xlin[i] <- sum(X[i,] * index)
    }

    # nimCat("sigma2:", model$sigma2)
    An <- expcov_gpSphere(Xlin, lengthscale, amp) +
      diag(rep(model$sigma2 + 1e-5, n))


    # if (!sigmaTrue){
    #
    # } else{
    #   An <- expcov_gpSphere(Xlin, lengthscale, amp) +
    #     diag(rep(sigma2 + 1e-10, n))
    # }
    y <- matrix(Y, ncol = 1)
    obj1 <- - (1/2) * t(y) %*% solve(An, y)
    # nimCat("obj1:",obj1)
    # nimPrint("")
    R <- chol(An)
    logdet_An <- 2 * sum(log(diag(R)))
    obj2 <- - logdet_An/2
    # nimCat("obj2:",obj2)
    # nimPrint("")
    objs <- obj1[1,1] + obj2
    # nimCat("lik:",objs)
    # nimPrint("")
    lp <- dunitSphere(index, dim = p, log = TRUE) +
      dgamma(lengthscale,shape, rate, log = TRUE) +
      dlnorm(amp, a_amp ,b_amp, log = TRUE)

    # if (sigmaTrue){
    #   lp <- lp + dinvgamma(sigma2, a_sig, b_sig, log = TRUE)
    # }

    # nimCat("prior:",lp)
    # nimPrint("")
    # nimPrint(-objs-lp)

    return(-objs-lp)
  }
)

# MAP
obj_btt_EB <- nimbleFunction(
  setup = function(X, Y, p, shape, rate,
                   a_amp, b_amp, a_sig, b_sig) {
  },
  run = function(par = double(1)) {
    returnType(double(0))
    # nimCat("params: ",par)
    # nimPrint("")
    index <- par[1:p]
    lengthscale  <- exp(par[(p+1)])
    amp    <- exp(par[(p+2)])
    sigma2  <- exp(par[p+3])
    # if (sigmaTrue) sigma2  <- exp(par[p+3])
    index <- index/sqrt(sum(index^2))
    # nimPrint(index)

    n <- nimDim(X)[1]
    Xlin <- numeric(n)
    for (i in 1:n){
      Xlin[i] <- sum(X[i,] * index)
    }

    An <- expcov_gpSphere(Xlin, lengthscale, amp) +
      diag(rep(sigma2 + 1e-3, n))

    # if (!sigmaTrue){
    #
    # } else{
    #   An <- expcov_gpSphere(Xlin, lengthscale, amp) +
    #     diag(rep(sigma2 + 1e-10, n))
    # }
    y <- matrix(Y, ncol = 1)
    obj1 <- - (1/2) * t(y) %*% solve(An, y)
    # nimCat("obj1:",obj1)
    # nimPrint("")
    R <- chol(An)
    logdet_An <- 2 * sum(log(diag(R)))
    obj2 <- - logdet_An/2
    # nimCat("obj2:",obj2)
    # nimPrint("")
    objs <- obj1[1,1] + obj2
    # nimCat("lik:",objs)
    # nimPrint("")
    lp <- dunitSphere(index, dim = p, log = TRUE) +
      dgamma(lengthscale,shape, rate, log = TRUE) +
      dlnorm(amp, a_amp ,b_amp, log = TRUE)

    # if (sigmaTrue){
    #   lp <- lp + dinvgamma(sigma2, a_sig, b_sig, log = TRUE)
    # }

    # nimCat("prior:",lp)
    # nimPrint("")
    # nimPrint(-objs-lp)

    return(-objs-lp)
  }
)

## initfunction
initFunction_gpSphere <- function(N, p, X, Y,
                                index, lengthscale,
                                amp, sigma2, method,
                                setSeed){
  # init$index, init$lengthscale, init$amp, init$sigma2
  # N, p

  if (setSeed != FALSE){
    set.seed(setSeed)
  }

  # index
  init_index <- numeric(p)
  if (is.null(index)) {
    init_index <- rnorm(p)
  } else {
    stopifnot(length(index) == p)
    init_index <- index
  }
  init_index <- conBeta(init_index)
  init_Xlin <- as.vector(X %*% matrix(init_index, nrow = p))

  # lengthscale
  init_lengthscale <- lengthscale
  if (length(init_lengthscale) >= 2 || !is.numeric(init_lengthscale)){
    stop("Error: Initial value of lengthscale should be scalar.")
  }
  if (init_lengthscale < 0){
    stop("Error: Initial value of lengthscale should be positive.")
  }

  # amp (eta)
  init_amp <- amp
  if (length(init_amp) >= 2 || !is.numeric(init_amp)){
    stop("Error: Initial value of amp should be scalar.")
  }
  if (init_amp < 0){
    stop("Error: Initial value of amp should be positive.")
  }
  init_log_amp <- log(amp)

  init_cov <- expcov_gpSphere(init_Xlin, init_lengthscale, init_amp)
  init_linkFunction <- mvtnorm::rmvnorm(1, rep(0, N), sigma = init_cov)[1,]

  # sigma2
  init_sigma2 <- sigma2
  if (length(init_sigma2) >= 2 || !is.numeric(init_sigma2)){
    stop("Error: Initial value of sigma2 should be scalar.")
  }
  if (init_sigma2 < 0){
    stop("Error: Initial value of sigma2 should be positive.")
  }
  init_Simga <- init_sigma2 * diag(1, N)

  # EB : MAP


  if (method == "FB"){
    return(list(index0 = init_index, index = init_index,
                lengthscale = init_lengthscale, amp = init_amp,
                Xlin = init_Xlin,cov = init_cov,
                sigma2 = init_sigma2,
                linkFunction = init_linkFunction, Sigma = init_Simga))
  } else{
    return(list(index = init_index,
                lengthscale = init_lengthscale, amp = init_amp,
                Xlin = init_Xlin,cov = init_cov,
                sigma2 = init_sigma2,
                linkFunction = init_linkFunction, Sigma = init_Simga))
  }



}

## predict test function
pred_gpSphere <- nimbleFunction(
  run = function(newdata = double(2), nsamp = integer(0), y = double(1),
                 indexSample = double(2),
                 XlinSample = double(2), sigma2_samples = double(1),
                 lengthSample = double(1), ampSample = double(1),
                 prediction = integer(0)){
    returnType(double(2))

    new_ncol <- nimDim(newdata)[1]
    orig_ncol <- length(y)

    testPred <- matrix(0, nrow = nsamp, ncol = new_ncol)
    for (i in 1:nsamp){
      currLength <- lengthSample[i]
      currAmp <- ampSample[i]
      currSigma <- sigma2_samples[i]
      # 1. compute index
      sampleZ <- newdata %*% matrix(indexSample[i, ], ncol = 1)

      # 2. compute covariance
      ## 1) C(ori, ori)
      cov_ori_ori <- expcov_gpSphere(XlinSample[i,], currLength, currAmp)
      ## 2) C(new, ori)
      cov_new_ori <- expcovTest_gpSphere(sampleZ[,1], XlinSample[i,], currLength, currAmp)
      ## 3) c(new, new)
      cov_new_new <- expcov_gpSphere(sampleZ[,1], currLength, currAmp)

      # 3. compute mu, Sigma
      midMatrix <- inverse(cov_ori_ori + diag(rep(currSigma, orig_ncol)))
      mu <- cov_new_ori %*% midMatrix %*% matrix(y, ncol = 1)
      Sigcov <- cov_new_new - cov_new_ori %*% midMatrix %*%  t(cov_new_ori)

      # 4. Sampling
      if (prediction == 1){ # latent
        cholpredcov <- chol(Sigcov)
        testPred[i,] <- t(rmnorm_chol(1, mean = mu[,1], cholesky = cholpredcov,
                                      prec_param  = FALSE))
      } else{ # response
        predcov <- Sigcov + diag(rep(currSigma, new_ncol))
        cholpredcov <- chol(predcov)
        testPred[i,] <- t(rmnorm_chol(1, mean = mu[,1], cholesky = cholpredcov,
                                      prec_param  = FALSE))
      }



    }
    return(testPred)

  }
)

# Additional functions for gpPolar
initfunction_gpPolar <- function(xobs, yobs, kappa_init, sigma2_init, psi_init = NULL, grid.with = 0.1,
                         sig_a = 1, sig_b = 0.01, setSeed){

  if (setSeed != FALSE){
    set.seed(setSeed)
  }

  n <- dim(xobs)[1]
  p <- dim(xobs)[2]
  theta <- matrix(0,p-1,1)
  if (is.null(psi_init)){
    theta[,1] <- runif(p-1, 0, pi)
  } else{
    theta[,1] <- psi_init
  }
  # theta[,1] <- ifelse(is.null(psi_init), runif(p-1, 0, pi), psi_init)
  thetat <- theta
  parabeta <- betaalpha1(thetat)

  #Converting theta to alpha
  alphahat<-matrix(0,p,1) # p*1 matrix
  alphahat[,1] <- alphaTheta(theta)

  Xlin <-xobs %*% alphahat
  storeorder <- order(Xlin)

  #Ordering x and y for fitting OU process
  y <- yobs[storeorder]
  Xlin <- Xlin[storeorder]

  #Finding inverse of C
  Cinv <- invcov(Xlin, kappa_init)

  #generate from f
  sigma2 <-sigma2_init #Initial sigma^2
  Sigma <- solve(Cinv+diag(n)/sigma2)
  mu <- (Sigma %*% y)/sigma2
  f <- mvrnorm(1,mu=mu,Sigma=Sigma,tol=1e-8)

  #Generate from sigma2
  a <- sig_a
  b <- sig_b
  err <- sum((y-f)^2)/2
  sigma2 <-1/rgamma(1,shape=(a+n/2), rate=(b+err))


  thetaNum <- seq(0.01, pi-0.01, grid.with)
  thetas  <- do.call(expand.grid,
                     replicate(p-1, thetaNum, simplify = FALSE))
  thetas<-as.matrix(thetas)
  num.calc<-dim(thetas)[1]
  g <- f[order(storeorder)]
  # cat("second g": g[1:10], "\n")

  PP1 <- apply(thetas,1,function(x) optimized(g,xobs,kappa_init,x,p))
  theta <- thetas[which.max(PP1), ]

  thetat <- theta
  parabeta <- betaalpha1(thetat)

  #Converting theta to alpha
  alphahat<-matrix(0,p,1)
  alphahat[,1] <- alphaTheta(theta)

  Xlin <- xobs %*% alphahat
  storeorder <- order(Xlin)
  y<-yobs[storeorder]
  Xlin<-Xlin[storeorder]

  #Finding inverse of C
  Cinv <- invcov(Xlin, kappa_init)


  #generate from f
  sigma2<-sigma2_init
  Sigma<-solve(Cinv+diag(n)/sigma2)
  mu<-(Sigma %*% y)/sigma2
  f <- mvrnorm(1,mu=mu,Sigma=Sigma,tol=1e-8)


  #Generate from sigma2
  err<-sum((y-f)^2)/2
  sigma2<-1/rgamma(1,shape=(a+n/2),rate=(b+err))



  Sigma <- solve(Cinv+diag(n)/sigma2)
  mu <- (Sigma %*% y)/sigma2
  f <- mvrnorm(1,mu=mu,Sigma=Sigma,tol=1e-8)

  # Xlin, f is ordered
  Xlin <- Xlin[order(storeorder)]
  f <- f[order(storeorder)]

  TEMPlist <- list(psi_init = theta, index_init = alphahat[,1],
            kappa_init = kappa_init, Xlin_init = Xlin, d_init = parabeta[,2],
            sigma2_init = sigma2, linkFunction_init = f)

  init_psi <- TEMPlist$psi_init/pi
  init_index <- TEMPlist$index_init
  init_kappa <- TEMPlist$kappa_init
  init_Xlin <- TEMPlist$Xlin_init
  init_cov <- expcov_gpPolar(init_Xlin, init_kappa)
  init_sigma2 <- TEMPlist$sigma2_init
  init_linkFunction <- TEMPlist$linkFunction_init
  init_d <- TEMPlist$d_init

  return(list(psi = init_psi, index = init_index,
              kappa = init_kappa, d = init_d,
              Xlin = init_Xlin, cov = init_cov,
              sigma2 = init_sigma2, linkFunction = init_linkFunction))

}


optimized<-function(f,xobs,kappa,thetahat,p)
{
  n <- length(f)
  thetahat<-as.numeric(as.matrix(thetahat))
  alphahat<-matrix(0,p,1)
  alphahat[,1] <- alphaTheta(thetahat)
  #alphahat<-as.matrix(alphahat)
  x<-xobs %*% alphahat
  f<-f[order(x)]
  x<-x[order(x)]

  r <- matrix(0,n+1,1)
  r[2:n]<-exp(-kappa*(x[2:n]-x[1:(n-1)]))
  r[1]<-0
  r[n+1]<-0

  e<-matrix(0,n+1,1)
  e<-r/(1-(r^2))
  d<-matrix(0,n,1)
  d[1:n]<-1+r[1:n]*e[1:n]+r[2:(n+1)]*e[2:(n+1)]

  Cinv<-matrix(0,n,n)
  diag(Cinv) <- d

  for(i in 1:(n-1))
  {
    Cinv[i,i+1] <- -e[i+1]
    Cinv[i+1,i] <- -e[i+1]
  }


  return(0.5*log((det(Cinv)))+(-0.5*f%*%Cinv%*%f))


}

betaalpha1<-function(theta)
{
  p <- length(theta)
  t <- matrix(0,p,1)
  t[,1] <- theta/pi
  abeta <- matrix(0,p,1)
  abeta[,1] <- 5000
  bbeta <- matrix(0,p,1)
  for(i in 1:p){
    bbeta[i]<-(abeta[i]-1-abeta[i]*t[i]+2*t[i])/t[i]}
  return(cbind(abeta,bbeta))
}


alphaTheta <- nimbleFunction(
  run = function(theta = double(1)){
    returnType(double(1))
    p <- length(theta) + 1
    alpha <- nimNumeric(p, init = TRUE)
    for (i in 1:p){
      if (i == 1){
        alpha[i] <- sin(theta[i])
      } else if (i == p){
        alpha[i] <- prod(cos(theta[1:(i-1)]))
      } else {
        alpha[i] <- prod(cos(theta[1:(i-1)]))*sin(theta[i])
      }
    }

    alpha <- alpha/sqrt(sum(alpha^2))

    if (alpha[1] < 0) {
      alpha <- -alpha
    }

    return(alpha)
  }
)

############################ Linear predictor ####################################
Xlinear <- nimbleFunction(
  run = function(betavec = double(1), dataX = double(2)){
    returnType(double(1))
    n <- nimDim(dataX)[1]
    p <- nimDim(dataX)[2]

    Xlin <- numeric(n)
    for (i in 1:n) {
      Xlin[i] <- 0
      for (j in 1:p) {
        Xlin[i] <- Xlin[i] + dataX[i, j] * betavec[j]
      }
    }

    return(Xlin)

  }
)


invcov <- nimbleFunction(
  run = function(xlin = double(1), kappa = double(0)){
    returnType(double(2))
    # sort Xlin
    # sortedXlin <- nimSort(xlin) ## needs to be coded
    n <- length(xlin)

    r <- nimNumeric(n+1, init = TRUE)
    r[2:n] <- exp(-kappa*(xlin[2:n]-xlin[1:(n-1)]))
    r[1] <- 0; r[n+1] <- 0
    # nimCat("r: ", r[1:5], "\n")


    e <- nimNumeric(n+1, init = TRUE)
    denom <- 1 - (r^2)
    # for (i in 1:(n+1)) {
    #   if (abs(denom[i]) < 1e-10) {
    #     denom[i] <- 1e-10
    #   }
    # }
    e <- r / denom
    # nimCat("e: ", e[1:5], "\n")

    d <- nimNumeric(n, init = TRUE)
    d <- 1 + r[1:n]*e[1:n] + r[2:(n+1)]*e[2:(n+1)]
    # nimCat("d: ", d[1:5], "\n")

    # tri-diagonal
    Cinv <- nimMatrix(0, nrow = n, ncol = n)
    for(i in 1:(n-1)){
      Cinv[i, i] <- d[i]
      Cinv[i,i+1] <- -e[i+1]
      Cinv[i+1,i] <- -e[i+1]
    }
    Cinv[n, n] <- d[n]


    return(Cinv)

  }
)


expcov_gpPolar <- nimbleFunction(
  run = function(vec = double(1), kappa = double(0)){
    returnType(double(2))
    n <- length(vec)
    result <- matrix(nrow = n, ncol = n, init = FALSE)
    for(i in 1:n){
      for(j in 1:n){
        result[i, j] <- exp(-kappa * abs(vec[i] - vec[j]))
      }
    }
    result <- (result + t(result))/2 #  + diag(rep(1e-10, n))

    return(result)

  }
)


expcovTest_gpPolar <- nimbleFunction(
  run = function(vec1 = double(1), vec2 = double(1), kappa = double(0)){
    returnType(double(2))
    n1 <- length(vec1)
    n2 <- length(vec2)
    result <- matrix(nrow = n1, ncol = n2, init = FALSE)
    for(i in 1:n1){
      for(j in 1:n2){
        result[i, j] <- exp(-kappa * abs(vec1[i] - vec2[j]))
      }
    }
    return(result)

  }
)


obj_btt_theta <- nimbleFunction(
  setup = function(model) {
  },
  run = function(par = double(1), kappa = double(0),
                 linkFunction = double(1)) {
    returnType(double(0))
    # theta <- par

    # beta <- alphaTheta(theta*pi)
    beta <- alphaTheta(par)
    # nimPrint(beta)
    X <- model$x
    n <- nimDim(X)[1]
    Xlin <- X %*% nimMatrix(beta, ncol = 1)
    XlinVec <- Xlin[,1]
    orderf <- nimOrder(XlinVec)

    sortXlin <- XlinVec[orderf]
    f <- linkFunction[orderf]
    # nimCat("sortXlin: ", sortXlin[1:10], "\n")
    # nimCat("sortf: ", f[1:10], "\n")

    # kappa <- model$kappa[1]
    inv_cov <- invcov(sortXlin, kappa)
    matF <- nimMatrix(f, ncol = 1)
    # lp <- dmnorm_chol(f, mean = rep(0, n),
    #                   cholesky = chol(inv_cov),
    #                   prec_param = TRUE, log=TRUE)
    lp1 <- 0.5 * log(det(inv_cov))
    # nimCat("lp1: ", lp1, "\n")
    lp2 <- - 0.5 * t(matF) %*% inv_cov %*% matF
    # nimCat("lp2: ", lp2[1,1], "\n")
    lp <-  lp1 + lp2[1,1]
    return(lp)
  }
)


thetaPrior <- nimbleFunction(
  run = function(theta = double(1)) {
    returnType(double(0))
    logP <- 0.0
    p_1    <- nimDim(theta)[1] # 2
    p    <- p_1 + 1 # 3
    for(j in 1:p_1) {
      logP <- logP + (p - j) * log(abs(cos(theta[j])))
    }
    return(logP)
  }
)

initfunction_gpPolar2 <- function(X, kappa, sigma2, psi , setSeed){

  if (setSeed != FALSE){
    set.seed(setSeed)
  }
  n <- dim(X)[1]
  p <- dim(X)[2]

  # hyper-parameter d
  init_d <- rep(6000,p-1)


  # index
  if (is.null(psi)){
    init_psi <- runif(p-1,0.01,1)
    init_index <- alphaTheta(init_psi)
  } else{
    init_psi <- psi
    init_index <- alphaTheta(init_psi * pi)
  }
  if (sum((init_index)^2) != 1){
    init_index <- init_index/sqrt(sum(init_index^2))
  }

  # sigma2
  init_sigma2 <- sigma2
  init_kappa <- kappa
  init_Xlin <- as.vector(X %*% matrix(init_index, ncol = 1))
  init_cov <-  expcov_gpPolar(init_Xlin, init_kappa)
  init_linkFunction <- mvtnorm::rmvnorm(1, rep(0, n), sigma = init_cov)[1,]

  return(list(psi = init_psi, index = init_index,
              kappa = init_kappa, d = init_d,
              Xlin = init_Xlin, cov = init_cov,
              sigma2 = init_sigma2, linkFunction = init_linkFunction))

}


pred_gpPolar <- nimbleFunction(
  run = function(newdata = double(2), nsamp = integer(0), y = double(1),
                 indexSample = double(2),
                 XlinSample = double(2), sigma2_samples = double(1),
                 kappaSample = double(1), prediction = integer(0)){
    returnType(double(2))

    new_ncol <- nimDim(newdata)[1]
    orig_ncol <- length(y)

    testPred <- matrix(0, nrow = nsamp, ncol = new_ncol)
    for (i in 1:nsamp){
      currKappa <- kappaSample[i]
      currSigma <- sigma2_samples[i]
      # 1. compute index
      sampleZ <- newdata %*% matrix(indexSample[i, ], ncol = 1)

      # 2. compute covariance
      ## 1) C(ori, ori)
      cov_ori_ori <- expcov_gpPolar(XlinSample[i,], currKappa)
      ## 2) C(new, ori)
      cov_new_ori <- expcovTest_gpPolar(sampleZ[,1], XlinSample[i,], currKappa)
      ## 3) c(new, new)
      cov_new_new <- expcov_gpPolar(sampleZ[,1], currKappa)

      # 3. compute mu, Sigma
      midMatrix <- inverse(cov_ori_ori + diag(rep(currSigma, orig_ncol)))
      mu <- cov_new_ori %*% midMatrix %*% matrix(y, ncol = 1)
      Sigcov <- cov_new_new - cov_new_ori %*% midMatrix %*%  t(cov_new_ori)

      # 4. Sampling
      if (prediction == 1){ # latent
        cholpredcov <- chol(Sigcov)
        testPred[i,] <- t(rmnorm_chol(1, mean = mu[,1], cholesky = cholpredcov,
                                      prec_param  = FALSE))
      } else{ # response
        predcov <- Sigcov + diag(rep(currSigma, new_ncol))
        cholpredcov <- chol(predcov)
        testPred[i,] <- t(rmnorm_chol(1, mean = mu[,1], cholesky = cholpredcov,
                                      prec_param  = FALSE))
      }

    }
    return(testPred)
  }
)

# index - MH
indexSampler_gpSphere <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(target)
    p <- model$getConstants()$p
    N <- model$getConstants()$N
  },
  run = function(){
    oldIndex <- model$index
    beforell <- model$calculate(calcNodes)
    covMat <- chol(diag(nimRep(0.25, p)))
    proposal <- rmnorm_chol(n = 1, oldIndex,
                            cholesky = covMat,
                            prec_param = FALSE)
    proposal <- proposal/sqrt(sum(proposal^2))
    if (proposal[1] < 0){
      proposal <- proposal * (-1)
    }
    model$index <<- nimC(proposal)
    model$calculate(calcNodes)
    afterll <- model$calculate(calcNodes)

    L <- (afterll - beforell)
    ratio <- exp(L)
    u <- runif(1)

    # accept/reject
    if(u < ratio) { # accept
      nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
    }
    if(u > ratio){
      nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
    }
  },
  methods = list(reset = function(){})
)

# sigma2 - gibbs
sigma2Sampler_gpSphere <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control) {
    calcNodes  <-  model$getDependencies(target)
    a_sig <- model$getConstants()$a_sig
    b_sig <- model$getConstants()$b_sig
    n <- model$getConstants()$N
    Y <- model$Y # not matrix
  },
  run = function() {
    eta <- model$linkFunction # not matrix
    # inverse gamma
    post_alpha <- a_sig  + (n/2)
    post_beta <- (1/b_sig) + 0.5 * sum((Y - eta)^2)
    proposal <- rinvgamma(n = 1, post_alpha, post_beta)
    model$sigma2 <<- nimC(proposal)
    model$calculate(calcNodes)
    nimCopy(from = model, to = mvSaved, row = 1, nodes  = target, logProb = TRUE)
  },
  methods = list( reset = function () {} )

)

# opt - MAP
optSampler <- nimbleFunction(
  contains = sampler_BASE,
  setup = function(model, mvSaved, target, control) {
    calcNodes  <-  model$getDependencies(target)
    shape <- model$getConstants()$shape
    rate <- model$getConstants()$rate
    a_amp <- model$getConstants()$a_amp
    b_amp <- model$getConstants()$b_amp
    a_sig <- model$getConstants()$a_sig
    b_sig <- model$getConstants()$b_sig
    obj_fn <- obj_btt(model, length(model$index),
                      shape, rate, a_amp, b_amp,
                      a_sig, b_sig)
    lowerB <- control$lowerB
    upperB <- control$upperB
    # lowerB <- c(rep(-1, length(model$index)),-1e2, -1e2)
    # upperB <- c(rep(1, length(model$index)),1e2, 1e2)

  },
  run = function() {
    current <- c(model$index, log(model$lengthscale), log(model$amp))
    # if (sigmaTrue){
    #   current <- c(current, log(model$sigma2))
    # }
    # controlOpt <- optimDefaultControl()
    # controlOpt$factr <- 1e11
    # controlOpt$pgtol <- 1e-4
    # controlOpt$maxit <- 80


    optRes  <- nimOptim(par    = current,
                        fn     = obj_fn$run,
                        method = "L-BFGS-B",
                        lower  = lowerB,
                        upper  = upperB)

    ## update
    p1 <- length(model$index)
    proposedIndex <- optRes$par[1:p1]
    if (proposedIndex[1] < 0){
      proposedIndex <- (proposedIndex) * (-1)
    }
    model$index  <<- proposedIndex/sqrt(sum((proposedIndex)^2))
    lengthscale <- exp(optRes$par[(p1+1):(p1+1)])
    if (lengthscale[1] < 1e-10){
      lengthscale <- lengthscale + 1e-6
    }
    model$lengthscale <<- lengthscale
    model$amp   <<- exp(optRes$par[(p1+2):(p1+2)])
    # if (sigmaTrue) model$sigma2   <<- exp(optRes$par[(p1+3):(p1+3)])

    model$calculate(calcNodes)

    ## deterministic move → always accept
    nimCopy(from = model, to = mvSaved, row = 1, nodes  = target, logProb = TRUE)
  },
  methods = list( reset = function () {} )

)

#' Integrated Function for Bayesian Single-Index Regression
#' @importFrom utils modifyList
#' @import nimble
#' @description
#' Fitting a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n} in single integrated function.
#' @inheritParams bsFisher
#' @param indexprior Index vector prior among \code{"fisher"} (default), \code{"sphere"}, \code{"polar"}, \code{"spike"}.
#' @param link Link function among \code{"bspline"} (default), \code{"gp"}
#' @param prior Optional named list of prior settings. Further descriptions are in every specific model's \strong{Details} section.
#' @param init Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
#' Further descriptions are in every specific model's \strong{Details} section.
#' @param method Character, `gpSphere` model has 3 different types of sampling method, fully Bayesian method (\code{"FB"}), empirical Bayes approach (\code{"EB"}), and empirical Gibbs sampler (\code{"EG"}).
#' Assign one sampler method. Empirical sampling approach is recommended in high-dimensional data. By default, fully Bayesian approach is assigned.
#' @param lowerB This parameter is only for `gpSphere` model. Numeric vector of element-wise lower bounds for the \code{"L-BFGS-B"} method.
#' When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
#' The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
#' By default, the lower bounds are \code{-1} for the index vector and \code{-1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.
#' @param upperB This parameter is only for `gpSphere` model. Numeric vector of element-wise upper bounds for the \code{"L-BFGS-B"} method.
#' When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
#' The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
#' By default, the upper bounds are \code{1} for the index vector and \code{1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.
#' @param x A fitted `BayesSIM` object.
#' @param digits Number of digits to display.
#' @param ... Additional arguments.
#'
#'
#' @inherit bsFisher return
#'
#' @details
#' Integrated function for Bayesian single-index model. Default model is von-Mises Fisher distribution for index vector with B-spline link function.
#'
#' @seealso [bsFisher()], [bsSphere()], [bsPolar()], [bsSpike()],
#' [gpFisher()], [gpSphere()], [gpPolar()], [gpPolarHigh()], [gpSpike()]
#'
#' @examples
#' \donttest{
#' set.seed(123)
#' n <- 200; d <- 4
#' theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
#' f <- function(u) u^2 * exp(u)
#' sigma <- 0.5
#' X <- matrix(runif(n * d, -1, 1), nrow = n)
#' index_vals <- as.vector(X %*% theta)
#' y <- f(index_vals) + rnorm(n, 0, sigma)
#' simdata <- data.frame(x = X, y = y)
#' colnames(simdata) <- c(paste0("X", 1:4), "y")
#'
#' # One tool version - bsFisher
#' fit1 <- BayesSIM(y ~ ., data = simdata,
#'                  niter = 5000, nburnin = 1000,
#'                  nchain = 1)
#'
#' # Split version- bsFisher
#' models <- BayesSIM_setup(y ~ ., data = simdata)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                    nchains = 1, setSeed = TRUE, inits = initList,
#'                    summary = TRUE, samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' summary(fit2)
#' }
#' @name BayesSIM
#' @export
BayesSIM <- function(formula, data,
                     indexprior = "fisher", link = "bspline",
                     prior = NULL, init = NULL,
                     method = "FB",
                     lowerB = NULL, upperB = NULL,
                     monitors = NULL, niter = 10000, nburnin=1000,
                     thin = 1, nchain = 1, setSeed = FALSE){
  # prior_list <- 0; init_list <- 0



  if (link == "bspline"){
    if (indexprior == "fisher"){

      fit <- bsFisher(formula = formula, data = data,
                      prior = prior, init = init,
                      monitors = monitors, niter = niter, nburnin = nburnin,
                      thin = thin, nchain = nchain, setSeed = setSeed)

    } else if (indexprior == "sphere"){
      fit <- bsSphere(formula = formula, data = data,
                      prior = prior, init = init,
                      monitors = monitors, niter = niter, nburnin = nburnin,
                      thin = thin, nchain = nchain, setSeed = setSeed)

    } else if (indexprior == "polar"){
      fit <- bsPolar(formula = formula, data = data,
                     prior = prior, init = init,
                     monitors = monitors, niter = niter, nburnin = nburnin,
                     thin = thin, nchain = nchain, setSeed = setSeed)

    } else if (indexprior == "spike"){
      fit <- bsSpike(formula = formula, data = data,
                     prior = prior, init = init,
                     monitors = monitors, niter = niter, nburnin = nburnin,
                     thin = thin, nchain = nchain, setSeed = setSeed)

    } else{
      stop("Wrong index prior name!")
    }

  } else if (link == "gp"){
    if (indexprior == "fisher"){
      fit <- gpFisher(formula = formula, data = data,
                      prior = prior, init = init,
                      monitors = monitors, niter = niter, nburnin = nburnin,
                      thin = thin, nchain = nchain, setSeed = setSeed)

    } else if (indexprior == "sphere"){
      fit <- gpSphere(formula = formula, data = data,
                      prior = prior, init = init,
                      method = method, lowerB = lowerB, upperB = upperB,
                      monitors = monitors, niter = niter, nburnin = nburnin,
                      thin = thin, nchain = nchain, setSeed = setSeed)

    } else if (indexprior == "polar"){
      if (!is.data.frame(data)){
        stop("data should be an data.frame.")
      }

      Call <- match.call()
      indx <- match(c("formula","data"), names(Call), nomatch = 0L)
      if (indx[1] == 0L)
        stop("a 'formula' argument is required")
      temp <- Call[c(1L,indx)]
      temp[[1L]] <- quote(stats::model.frame)
      m <- eval.parent(temp)
      Terms <- attr(m,"terms")
      formula_temp <- as.character(formula)
      response.name <- formula_temp[2]
      data.name <- strsplit(formula_temp[3]," \\+ ")[[1]]
      int.flag <- any(strsplit(formula_temp[3]," \\* ")[[1]] == formula_temp[3])
      if(data.name[1]=="."){
        tot.name <- response.name
      } else{
        tot.name <- c(response.name ,data.name)
      }
      if(!int.flag){
        stop("BayesSIM cannot treat interaction terms.")
      }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
               length(tot.name)){
        stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                               tot.name))[-c(1:ncol(data))]],collapse=","),
                   " is/are not in your data."))
      }else{
        origY <- data[ ,response.name]
        if(data.name[1]=="."){
          origX <- data[,colnames(data) != response.name]
        }else {
          origX <- data[ ,data.name,drop=FALSE]
        }
      }

      # X = origX, Y = origY
      X <- as.matrix(origX)
      if (ncol(X) >= 5){
        fit <- gpPolarHigh(formula = formula, data = data,
                           prior = prior, init = init,
                           monitors = monitors, niter = niter, nburnin = nburnin,
                           thin = thin, nchain = nchain, setSeed = setSeed)
      } else{
        fit <- gpPolar(formula = formula, data = data,
                       prior = prior, init = init,
                       monitors = monitors, niter = niter, nburnin = nburnin,
                       thin = thin, nchain = nchain, setSeed = setSeed)
      }
    } else if (indexprior == "spike"){
      fit <- gpSpike(formula = formula, data = data,
                     prior = prior, init = init,
                     monitors = monitors, niter = niter, nburnin = nburnin,
                     thin = thin, nchain = nchain, setSeed = setSeed)

    } else{
      stop("Wrong index prior name!")
    }

  } else{
    stop("Wrong link function name!")
  }

  return(fit)

}
#' @rdname BayesSIM
#' @export
BayesSIM_setup <- function(formula, data,
                     indexprior = "fisher", link = "bspline",
                     prior = NULL, init = NULL,
                     method = "FB",
                     lowerB = NULL, upperB = NULL,
                     monitors = NULL, niter = 10000, nburnin=1000,
                     thin = 1, nchain = 1, setSeed = FALSE){
  # modifyList <- 0
  # prior_list <- 0; init_list <- 0

  if (link == "bspline"){
    if (indexprior == "fisher"){

      fit <- bsFisher_setup(formula = formula, data = data,
                            prior = prior, init = init,
                            monitors, niter, nburnin,
                            thin, nchain, setSeed)

    } else if (indexprior == "sphere"){
      fit <- bsSphere_setup(formula = formula, data = data,
                            prior = prior, init = init,
                            monitors, niter, nburnin,
                            thin, nchain, setSeed)

    } else if (indexprior == "polar"){
      fit <- bsPolar_setup(formula = formula, data = data,
                           prior = prior, init = init,
                           monitors, niter, nburnin,
                           thin, nchain, setSeed)

    } else if (indexprior == "spike"){
      fit <- bsSpike_setup(formula = formula, data = data,
                           prior = prior, init = init,
                           monitors, niter, nburnin,
                           thin, nchain, setSeed)

    } else{
      stop("Wrong index prior name!")
    }

  } else if (link == "gp"){
    if (indexprior == "fisher"){
      fit <- gpFisher_setup(formula = formula, data = data,
                            prior = prior, init = init,
                            monitors, niter, nburnin,
                            thin, nchain, setSeed)

    } else if (indexprior == "sphere"){
      fit <- gpSphere_setup(formula = formula, data = data,
                            prior = prior, init = init,
                            method, lowerB, upperB,
                            monitors, niter, nburnin,
                            thin, nchain, setSeed)

    } else if (indexprior == "polar"){
      if (!is.data.frame(data)){
        stop("data should be an data.frame.")
      }

      Call <- match.call()
      indx <- match(c("formula","data"), names(Call), nomatch = 0L)
      if (indx[1] == 0L)
        stop("a 'formula' argument is required")
      temp <- Call[c(1L,indx)]
      temp[[1L]] <- quote(stats::model.frame)
      m <- eval.parent(temp)
      Terms <- attr(m,"terms")
      formula_temp <- as.character(formula)
      response.name <- formula_temp[2]
      data.name <- strsplit(formula_temp[3]," \\+ ")[[1]]
      int.flag <- any(strsplit(formula_temp[3]," \\* ")[[1]] == formula_temp[3])
      if(data.name[1]=="."){
        tot.name <- response.name
      } else{
        tot.name <- c(response.name ,data.name)
      }
      if(!int.flag){
        stop("BayesSIM cannot treat interaction terms")
      }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
               length(tot.name)){
        stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                               tot.name))[-c(1:ncol(data))]],collapse=","),
                   " is/are not in your data"))
      }else{
        origY <- data[ ,response.name]
        if(data.name[1]=="."){
          origX <- data[,colnames(data) != response.name]
        }else {
          origX <- data[ ,data.name,drop=FALSE]
        }
      }

      # X = origX, Y = origY
      X <- as.matrix(origX)

      if (ncol(X) >= 5){
        fit <- gpPolarHigh_setup(formula = formula, data = data,
                                 prior = prior, init = init,
                                 monitors, niter, nburnin,
                                 thin, nchain, setSeed)
      } else{
        fit <- gpPolar_setup(formula = formula, data = data,
                             prior = prior, init = init,
                             monitors, niter, nburnin,
                             thin, nchain, setSeed)
      }
    } else if (indexprior == "spike"){
      fit <- gpSpike_setup(formula = formula, data = data,
                           prior = prior, init = init,
                           monitors, niter, nburnin,
                           thin, nchain, setSeed)

    } else{
      stop("Wrong index prior name!")
    }

  } else{
    stop("Wrong link function name!")
  }

  return(fit)

}

# BayesSIM
# Test for input
# tests/testthat/test-input-args.R
test_that("check input", {
  dataset <- data.frame(matrix(rnorm(300), 100, 3), y = rnorm(100))
  base_args <- list(
    formula = y ~ .,
    data = dataset,
    niter    = 10,
    nburnin  = 5,
    thin     = 1,
    nchain   = 2
  )

  run_call <- function(overrides = list()) {
    args <- base_args
    for (nm in names(overrides)) args[[nm]] <- overrides[[nm]]
    do.call(BayesSIM, args)
    do.call(BayesSIM.setup, args)
  }

  # expect_no_error(run_call())

  # --- niter: numeric scalar & not NA ---
  expect_error(run_call(list(niter = "10")),    "'niter' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(niter = c(10, 20))),"'niter' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(niter = NA_real_)),"'niter' argument should be numeric scalar.", fixed = TRUE)

  # --- nburnin ---
  expect_error(run_call(list(nburnin = "5")),      "'nburnin' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(nburnin = c(1, 2))),  "'nburnin' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(nburnin = NA_real_)), "'nburnin' argument should be numeric scalar.", fixed = TRUE)

  # --- thin ---
  expect_error(run_call(list(thin = "1")),       "'thin' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(thin = c(1, 2))),   "'thin' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(thin = NA_real_)),  "'thin' argument should be numeric scalar.", fixed = TRUE)

  # --- nchain ---
  expect_error(run_call(list(nchain = "2")),      "'nchain' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(nchain = c(1, 2))),  "'nchain' argument should be numeric scalar.", fixed = TRUE)
  expect_error(run_call(list(nchain = NA_real_)), "'nchain' argument should be numeric scalar.", fixed = TRUE)
})


test_that("BayesSIM errors on wrong indexprior", {
  data <- data.frame(y = rnorm(10), x1 = rnorm(10))
  formula <- y ~ x1

  expect_error(
    BayesSIM(formula, data, indexprior = "wrong", link = "bspline"),
    "Wrong index prior name!"
  )
})

test_that("BayesSIM errors on wrong link", {
  data <- data.frame(y = rnorm(10), x1 = rnorm(10))
  formula <- y ~ x1

  expect_error(
    BayesSIM(formula, data, indexprior = "fisher", link = "wrong"),
    "Wrong link function name!"
  )

  expect_error(
    BayesSIM_setup(formula, data, indexprior = "fisher", link = "wrong"),
    "Wrong link function name!"
  )
})

test_that("BayesSIM have wrong output structure", {
  data <- data.frame(y = rnorm(10), x1 = rnorm(10))
  formula <- y ~ x1

  expect_error(
    BayesSIM(formula, data, indexprior = "fisher", link = "wrong"),
    "Wrong link function name!"
  )

  expect_error(
    BayesSIM_setup(formula, data, indexprior = "fisher", link = "wrong"),
    "Wrong link function name!"
  )
})

# Specific model
test_that("bsFisher_setup runs and returns bsimSetup", {
  skip_if_not_installed("nimble")
  skip_on_cran()

  set.seed(123)
  n <- 50
  dat <- data.frame(
    y  = rnorm(n),
    x1 = rnorm(n),
    x2 = rnorm(n),
    x3 = rnorm(n)
  )

  fit <- bsFisher_setup(
    formula  = y ~ .,
    data     = dat,
    prior    = NULL,
    init     = NULL,
    niter    = 50,
    nburnin  = 10,
    thin     = 2,
    nchain   = 1,
    setSeed  = FALSE
  )

  # check class
  expect_equal(class(fit), "bsimSetup")

  # check structure
  expect_true(is.list(fit$input))
  expect_equal(fit$modelName, "bsFisher")

  # save original data
  expect_true(is.matrix(fit$input$origdata$x))
  expect_true(is.matrix(fit$input$origdata$y))
  expect_equal(nrow(fit$input$origdata$x), n)
  expect_equal(nrow(fit$input$origdata$y), n)

  # check sampling options
  expect_false(fit$input$samplingOptions$sampling)
  expect_equal(fit$input$samplingOptions$niter, 50)
  expect_equal(fit$input$samplingOptions$nburnin, 10)
  expect_equal(fit$input$samplingOptions$thin, 2)
  expect_equal(fit$input$samplingOptions$nchain, 1)

  expect_error(
    bsFisher_setup(
      formula  = y ~ x1 + x2,
      data     = dat,
      prior    = NULL,
      init     = NULL,
      nchain   = 2,
      setSeed  = "wrong_type"
    ),
    "'setSeed' must be logical or numeric."
  )

  y  <- rnorm(10)
  x1 <- rnorm(10)
  x2 <- rnorm(10)
  mat_data <- cbind(y, x1, x2)
  expect_error(
    bsFisher_setup(
      formula  = y ~ x1 + x2,
      data     = mat_data,
      prior    = NULL,
      init     = NULL
    ),
    "data should be an data.frame."
  )
})

# prior/initial parameter
test_that("prior/initial parameter", {
  skip_if_not_installed("nimble")
  skip_on_cran()

  set.seed(123)
  n <- 50
  dat <- data.frame(
    y  = rnorm(n),
    x1 = rnorm(n),
    x2 = rnorm(n),
    x3 = rnorm(n)
  )

  prior <- prior_param(indexprior = "sphere", link = "bspline")
  init <- init_param(indexprior = "fisher", link = "bspline")



  expect_error(BayesSIM_setup(
    formula  = y ~ ., data = dat,
    prior    = prior, init     = init,
    niter    = 50, nburnin  = 10,
    thin     = 2, nchain   = 1, setSeed  = FALSE))

})

# Sampler
## 1. theta random walk metropolis algorithm
indexSampler_bspline_fisher <- nimble::nimbleFunction(
  contains = nimble::sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(target)
    # vMFnimsample <- vMFnim()
    Rpostll <- postll_bspline_fisher(model)
    propDens <- 1000
    Y <- model$Y
    rho <- gvcCV(model$Xmat, Y)
  },
  run = function(){
    currTheta <- model$index
    beforell <- Rpostll$run(rho)
    proposal <- rvMFnim(1, theta = currTheta * propDens)
    model$index <<- nimC(proposal)
    model$calculate(calcNodes)
    afterll <- Rpostll$run(rho)

    L <- (afterll - beforell)
    ratio <- min(c(1, exp(L)))
    u <- runif(1)

    # accept/reject
    if(u < ratio) {
      nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)
    }
    if(u > ratio){
      nimCopy(from = mvSaved, to = model, row = 1, nodes  = calcNodes, logProb = TRUE)
    }
  },
  methods = list(reset = function(){})
)

## 2. sampling beta - gibbs
betaSampler_bspline_fisher <- nimble::nimbleFunction(
  contains = nimble::sampler_BASE,
  setup = function(model, mvSaved, target, control){
    calcNodes <- model$getDependencies(target)
    Y <- model$Y
    N <- model$getConstants()$N
    rho <- gvcCV(model$Xmat, Y)
  },
  run = function(){
    Xmat <- model$Xmat
    temp <- nimDim(Xmat)[2]
    sigma2 <- model$sigma2
    cov <- chol(inverse(t(Xmat) %*% Xmat) *
                  nimMatrix(sigma2, ncol = temp, nrow = temp))
    betasamp <- rmnorm_chol(n = 1, mean = estBeta_fisher(Xmat, Y, rho),
                           cholesky = cov, prec_param = FALSE)
    model$beta <<- betasamp

    model$calculate(calcNodes)
    nimCopy(from = model, to = mvSaved, row = 1,
            nodes  = calcNodes, logProb = TRUE)

  },
  methods = list(reset = function(){})
)

# 3. sigma2 sampler
sigma2Sampler_bspline_fisher <-  nimble::nimbleFunction(
  contains = nimble::sampler_BASE,
  setup = function(model, mvSaved, target, control) {
    calcNodes <- model$getDependencies(target)
    N <- model$getConstants()$N
    Y <- model$Y[,1]
    a_sig <- model$getConstants()$a_sig
    b_sig <- model$getConstants()$b_sig
    # rho <- gvcCV(model$Xmat, Y)
  },
  run = function(){
    # nimPrint("sigma2")
    xmat <- model$Xmat
    alpha <- N/2 + a_sig
    # SS <- Stheta(Xmat = xmat, Y = Y, rho = rho)
    mu <- model$linkFunction[,1]
    SS <- sum((Y-mu)^2)
    beta <- SS/2 + (1/b_sig)
    newSigma <- rinvgamma(1, alpha, beta)
    # nimPrint(newSigma)

    model$sigma2 <<- nimC(newSigma)
    model$calculate(calcNodes)
    nimCopy(from = model, to = mvSaved, row = 1, nodes  = calcNodes, logProb = TRUE)

  },
  methods = list( reset = function () {} )

)

#' Bayesian Single-Index Regression with B-Spline Link and One-to-One Polar Transformation
#'
#' @description Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}
#' where the link \eqn{f(\cdot)} is represented by B-spline link function and the
#' index vector \eqn{\theta} is parameterized on the unit sphere via a one-to-one polar transformation.
#'
#' @inheritParams bsFisher
#' @details
#' \strong{Model} The single–index model uses a \eqn{m}-order polynomial spline with \eqn{k} interior knots as follows:
#' \deqn{f(t) = \sum_{j=1}^{m+k} B_j(t)\,\beta_j} on \eqn{[a, b]} with \eqn{t_i = X_i' \theta, i = 1,\cdots, n}
#' and \eqn{\|\theta\|_2 = 1}. \eqn{\{\beta_j\}_{j=1}^{m+k}} are spline coefficient and \eqn{a_\theta}, \eqn{ b_\theta} are boundary knots where \eqn{a_\theta = min(t_i, i = 1, \cdots, n) - \delta},
#' and \eqn{b_\theta = max(t_i, i = 1,\cdots, n) + \delta}. \eqn{\theta} lies on the unit sphere (\eqn{\|\theta\|_2=1})
#' to ensure identifiability and is parameterized via a one-to-one polar transformation with angle \eqn{\psi_1,...,\psi_{p-1}}, where \eqn{p} is the dimension of predictor.
#' Sampling is  performed on the angular parameters \eqn{\\psi} defining
#' the index vector.
#'
#' The mapping is
#' \deqn{
#' \begin{aligned}
#' \theta_1 &= \sin(\psi_1),\\
#' \theta_i &= \Big(\prod_{j=1}^{i-1}\cos(\psi_j)\Big)\sin(\psi_i), \quad i=2,\dots,p-1,\\
#' \theta_p &= \prod_{j=1}^{p-1}\cos(\psi_j).
#' \end{aligned}
#' }
#' The vector is then scaled to unit length.
#'
#' \strong{Priors}
#' \itemize{
#' \item \eqn{\psi} is \eqn{p-1} dimension of polar angle of index vector and re-scaled Beta distribution on \eqn{[0, \pi]} is assigned.
#' \item Conditioned on \eqn{\theta} and \eqn{\sigma^2}, the link coefficients \eqn{\beta = (\beta_1,\ldots,\beta_{m+k})^\top} follow
#'        normal distribution with estimated mean vector \eqn{\hat{\beta}_{\theta} = (X_{\theta}'X_{\theta})^{-1}X_{\theta}'Y} and
#'        covariance \eqn{\sigma^2 (X_{\theta}^\top X_{\theta})^{-1}}, where \eqn{X_{\theta}} is the B-spline basis design matrix.
#' \item Inverse gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.
#' }
#'
#' \strong{Sampling}
#' Samplers are automatically assigned by nimble.
#'
#'
#' \strong{Prior hyper-parameters}
#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
#' \enumerate{
#' \item Index vector:
#'     Only shape parameter \code{index_psi_alpha} of \eqn{p-1} dimension vector is needed since rate parameters is computed to satisfy \eqn{\theta_{j, \text{MAP}}}.
#'     By default, the shape parameter for each element of the polar vector is set to \code{5000}.
#'     \item Link function: B-spline basis and coefficient of B-spline setup.
#'          \itemize{
#'          \item{\code{basis}: For the basis of B-spline, \code{link_basis_df} is the number of basis functions (default \code{21}), \code{link_basis_degree} is the spline degree (default \code{2}) and
#'          \code{link_basis_delta} is a small jitter for boundary-knot spacing control (default \code{0.001}).}
#'          \item{\code{beta}: For the coefficient of B-spline, multivariate normal prior is assigned with mean \code{link_beta_mu}, and covariance \code{link_beta_cov}.
#'           By default, \eqn{\mathcal{N}_p\!\big(0, \mathrm{I}_p\big)}} is assigned.
#'           }
#'
#'   \item Error variance (\code{sigma2}): An Inverse gamma prior is assigned to \eqn{\sigma^2}
#'         where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
#'         (default \code{sigma2_shape = 0.001, sigma2_rate = 100})
#'}
#'
#' \strong{Initial values}
#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
#' \enumerate{
#' \item Index vector: Initial vector of polar angle \code{index_psi} (\eqn{p-1} dimension). Each element of angle is between 0 and \eqn{\pi}.
#' By default, it is randomly draw from uniform distribution.
#'      \item Link function: Initial spline coefficients(\code{link_beta}). By default,
#'          \eqn{\big(X_{\theta}^\top X_{\theta} + \rho\, \mathrm{I}\big)^{-1} X_{\theta}^\top Y} is computed,
#'     where \eqn{X_{\theta}} is the B-spline basis design matrix.
#'   \item Error variance (\code{sigma2}): Initial scalar error variance (default \code{0.01}).
#' }
#'
#' @inherit bsFisher return
#'
#' @examples
#' \donttest{
#' set.seed(123)
#' n <- 200; d <- 4
#' theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
#' f <- function(u) u^2 * exp(u)
#' sigma <- 0.5
#' X <- matrix(runif(n * d, -1, 1), nrow = n)
#' index_vals <- as.vector(X %*% theta)
#' y <- f(index_vals) + rnorm(n, 0, sigma)
#' simdata <- data.frame(x = X, y = y)
#' colnames(simdata) <- c(paste0("X", 1:4), "y")
#'
#' # One tool version
#' fit1 <- bsPolar(y ~ ., data = simdata,
#'                 niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Split version
#' models <- bsPolar_setup(y ~ ., data = simdata)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                    nchains = 1, setSeed = TRUE, inits = initList,
#'                    samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' summary(fit2)
#' }
#'
#' @references
#' Antoniadis, A., Grégoire, G., & McKeague, I. W. (2004).
#' Bayesian estimation in single-index models. \emph{Statistica Sinica}, 1147-1164.
#'
#' Hornik, K., & Grün, B. (2014). movMF: an R package for fitting mixtures of von Mises-Fisher distributions.
#' \emph{Journal of Statistical Software}, 58, 1-31.
#'
#' Dhara, K., Lipsitz, S., Pati, D., & Sinha, D. (2019). A new Bayesian single index model with or without covariates missing at random.
#' \emph{Bayesian analysis}, 15(3), 759.
#'
#' @name bsPolar
#' @export
bsPolar <- function(formula, data,
                    prior = NULL,
                    init = NULL,
                    monitors = NULL, niter = 10000, nburnin=1000,
                    thin = 1, nchain = 1, setSeed = FALSE
){
  return(
    bsPolar.default(formula = formula, data  = data,
                    prior = prior,
                    init = init,
                    sampling = TRUE,
                    monitors = monitors, niter = niter, nburnin = nburnin,
                    thin = thin, nchain = nchain, setSeed = setSeed)
  )

}

#' @rdname bsPolar
#' @export
bsPolar_setup <- function(formula, data,
                          prior = NULL,
                          init = NULL,
                          monitors = NULL, niter = 10000, nburnin=1000,
                          thin = 1, nchain = 1, setSeed = FALSE
){
  return(
    bsPolar.default(formula = formula, data  = data,
                    prior = prior,
                    init = init,
                    sampling = FALSE,
                    monitors = monitors, niter = niter, nburnin = nburnin,
                    thin = thin, nchain = nchain, setSeed = setSeed)
  )
}

bsPolar.default <- function(formula, data,
                    prior = NULL,
                    init = NULL,
                    sampling = TRUE,
                    monitors = NULL, niter = 10000, nburnin=1000,
                    thin = 1, nchain = 1, setSeed = FALSE
){
  start1 <- Sys.time()
  psi <- 0

  # check sampling, prior, init parameters for independent execution
  checkOutput <- validate_and_finalize_args(
    sampling, niter, nburnin, thin,  nchain,
    prior, init,"polar", "bspline"
  )
  prior <- checkOutput$priorlist_final
  init <- checkOutput$initlist_final

  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))

  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # aa_bspline_ver3
    "SplineState", "any_duplicated", "mat_wo_col1", "update_spline_df", "update_x_index",
    "update_knot_sequence", "get_basis_simple", "simplify_knots", "get_inside_x",
    "gen_default_internal_knots", "SplineBase1", "SplineBase2", "basis", "bsNimble", "bsBasis",

    # bsFisher
    "postll_bspline_fisher","nimNorm","rW","besselI_nimble","Stheta",
    "estBeta_fisher","gvcCV","transX_fisher",
    "pred_bsplineFisher","indexSampler_bspline_fisher","betaSampler_bspline_fisher",

    "pred_fitted", "Xlinear", "alphaTheta", "transX_fisher"
  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())


  # check data dimension
  # Data
  if (!is.data.frame(data)){
    stop("data should be an data.frame.")
  }

  Call <- match.call()
  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
  if (indx[1] == 0L)
    stop("a 'formula' argument is required")
  temp <- Call[c(1L,indx)]
  temp[[1L]] <- quote(stats::model.frame)
  m <- eval.parent(temp)
  Terms <- attr(m,"terms")
  formula <- as.character(formula)
  response.name <- formula[2]
  data.name <- strsplit(formula[3]," \\+ ")[[1]]
  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
  if(data.name[1]=="."){
    tot.name <- response.name
  } else{
    tot.name <- c(response.name ,data.name)
  }
  if(!int.flag){
    stop("BayesSIM cannot treat interaction terms")
  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
           length(tot.name)){
    stop(paste(paste(tot.name[duplicated(c(colnames(data),
                                           tot.name))[-c(1:ncol(data))]],collapse=","),
               " is/are not in your data"))
  }else{
    origY <- data[ ,response.name]
    if(data.name[1]=="."){
      origX <- data[,colnames(data) != response.name]
    }else {
      origX <- data[ ,data.name,drop=FALSE]
    }
  }

  # X = origX, Y = origY
  X <- as.matrix(origX)
  Y <- as.matrix(origY)

  # data dimension
  N <- length(Y)
  p <- ncol(X)


  # Model code
  Rmodel <- nimbleCode({
    # c is constant
    for(j in 1:(p-1)) {
      d[j] ~ dunif(1e-10, 1e10)
      psi[j] ~ dbeta(c[j], d[j])
    }

    index[1:p] <- alphaTheta(psi[1:(p-1)]*pi)

    # Linear predictor
    for (i in 1:N){
      Xlin[i] <- sum(X[i, 1:p] * index[1:p])
    }

    # Design matrix - b spline basis
    Xmat[1:N, 1:df] <- transX_fisher(Xlin[1:N], df = df, degree = degree, delta = delta)

    # likelihood
    sigma2 ~ dinvgamma(a_sig, b_sig)
    beta[1:df] ~ dmnorm(mubeta[1:df], cov = covbeta[1:df, 1:df])
    linkFunction[1:N, 1] <- Xmat[1:N, 1:df] %*% matrix(beta[1:df], ncol = 1)
    for (i in 1:N){
      Y[i, 1] ~ dnorm(linkFunction[i, 1], var = sigma2)
    }

  })


  # Prior parameters
  ## check data dimension and save
  ## index
  ## psi
  if (!is.null(prior$index$psi$alpha) & length(prior$index$psi$alpha) != (p-1))
  {stop("Prior psi has incorrect dimension")}
  if (is.null(prior$index$psi$alpha)){
    psi_c <- rep(5000,(p-1))
  } else{
    psi_c <- prior$index$psi$alpha
  }

  ## link-basis
  if (is.null(prior$link$basis$df)||length(prior$link$basis$df) >= 2 || prior$link$basis$df< 0){
    stop("Prior basis (df) has incorrect value.")
  } else{
    df <- prior$link$basis$df
  }

  if (is.null(prior$link$basis$degree)||length(prior$link$basis$degree) >= 2 || prior$link$basis$degree< 0){
    stop("Prior basis (degree) has incorrect value.")
  } else{
    degree <- prior$link$basis$degree
  }

  if (is.null(prior$link$basis$delta)||length(prior$link$basis$delta) >= 2 || prior$link$basis$delta< 0){
    stop("Prior basis (delta) has incorrect value.")
  } else{
    delta <- prior$link$basis$delta
  }

  ## beta
  if (is.null(prior$link$beta$mu)){
    mubeta <- rep(0, df)
  }

  if (!is.vector(prior$link$beta$mu) & !is.null(prior$link$beta$mu)){
    stop("Prior beta (mu) should be vector.")
  }
  if (!is.null(prior$link$beta$mu) & length(prior$link$beta$mu) != (df)){
    stop("Incorrect dimension on prior beta (mu).")
  }

  if (is.null(prior$link$beta$cov)){
    covbeta <- diag(df)
  }
  if (!is.null(prior$link$beta$cov) & !is.matrix(prior$link$beta$cov)){
    stop("Prior beta (cov) should be matrix.")
  }


  ## sigma
  if (is.null(prior$sigma2$shape)||length(prior$sigma2$shape) >= 2 || prior$sigma2$shape < 0){
    stop("Prior sigma2 (shape) has incorrect value.")
  } else{
    a_sig <- prior$sigma2$shape
  }

  if (is.null(prior$sigma2$rate)||length(prior$sigma2$rate) >= 2 || prior$sigma2$rate < 0){
    stop("Prior sigma2 (rate) has incorrect value.")
  } else{
    b_sig <- prior$sigma2$rate
  }



  # Initialize
  ## index
  init_psi <- init$index$psi
  if (!is.null(init_psi) & length(init_psi) != (p-1)){
    stop("Initial psi has incorrect dimension")
  }

  ## sigma2
  init_sigma2 <- init$sigma2
  if (length(init$sigma2) >= 2 || !is.numeric(init$sigma2)){
    stop("Initial value of sigma2 should be scalar.")
  }
  if (init$sigma2 < 0){
    stop("Initial value of sigma2 should be positive.")
  }

  ## beta
  if (!is.null(init$link$beta) & length(init$link$beta) != df){
    stop("Incorrect dimention on initial value of beta.")
  }

  # seed
  seedNum <- rep(FALSE, nchain)
  seedNum <- rep(FALSE, nchain)

  if (!is.logical(setSeed) && !is.numeric(setSeed)) {
    stop("'setSeed' must be logical or numeric.")
  }

  if (is.logical(setSeed)) {

    if (length(setSeed) != 1) {
      stop("If 'setSeed' is logical, it must be length 1.")
    }

    if (setSeed) {
      seedNum <- seq_len(nchain)
    } else {
      seedNum <- rep(FALSE, nchain)
    }
  }

  if (is.numeric(setSeed)) {

    if (length(setSeed) == 1) {
      seedNum <- rep(setSeed, nchain)

    } else if (length(setSeed) == nchain) {
      seedNum <- setSeed

    } else {
      stop("Numeric 'setSeed' must be length 1 or equal to 'nchain'.")
    }
  }

  inits_list <- lapply(seq_len(nchain), function(j) initFunction_bP(X = X, Y = Y,
                                                                    psi = init_psi,
                                                                    sigma2 = init_sigma2,
                                                                    beta = init$link$beta,
                                                                    df = df, degree = degree, delta = delta,
                                                                    setSeed = seedNum[j]))

  firstInit <- inits_list[[1]]

  # Build model
  message("== Building model ==")
  suppressMessages(simpleModel <- nimbleModel(Rmodel,
                             data = list(X = X, Y = Y),
                             constants = list(p = p, N = N, c = psi_c,
                                              a_sig = a_sig, b_sig = b_sig,
                                              df = df, degree = degree, pi = pi,
                                              delta = delta,mubeta = mubeta, covbeta = covbeta),
                             inits = firstInit))

  # Assign samplers
  message("== Assigning samplers ==")
  monitorsList <- c("index", "sigma2", "linkFunction", "Xlin", "beta", "d", "psi")
  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
                                             monitors = monitorsList,
                                             print = FALSE))


  mcmcConf$removeSamplers(c("beta"))
  mcmcConf$addSampler(target = c("beta"),
                      type   = "betaSampler_bspline_fisher")

  # mcmcConf$setSamplerExecutionOrder(c(1, 2, 3, 5, 6, 7, 8, 4))

  mcmc1 <- buildMCMC(mcmcConf)


  mcmc.out <- NULL
  if (!sampling){
    sampMCMC <- NULL
    samples <- NULL
  } else{
    # Compile
    start2 <- Sys.time()
    message("== Compiling model ==")
    suppressMessages(CsimpleModel <- compileNimble(simpleModel))
    message("== Compiling samplers ==")
    suppressMessages(Cmcmc <- compileNimble(mcmc1, project = simpleModel, resetFunctions = TRUE))
    end2 <- Sys.time()

    # Sampling
    message("== Running MCMC ==")

    if (is.logical(setSeed)) {
      seedNum <- setSeed
    }
    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
                        thin = thin,
                        nchains = nchain, setSeed = seedNum,
                        inits = inits_list,
                        summary = FALSE, samplesAsCodaMCMC = TRUE)



    # output
    ## combine all chains
    sampMCMC <- mcmc.out
    samples <- sampleBind(sampMCMC, nchain)
  }
  end1 <- Sys.time()

  ## Input options
  if (!sampling){
    time <- NULL
  } else{
    # total time - compile time
    time_wo_compile <- difftime(end1, start1, units = "secs") -
      difftime(end2, start2, units = "secs")

    time <- list(time_wo_compile = time_wo_compile)
  }
  # Results ------------------------------------------------------------------
  #Model point estimation
  modelESTLIST <- bsimFit_pointest(samples, X, Y)

  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                       prior = list(index = list(psi = list(alpha = psi_c)),
                                    link = list(basis = list(df = df, degree = degree, delta = delta),
                                                beta = list(mu = mubeta, cov = covbeta)),
                                    sigma2 = list(shape = a_sig, rate = b_sig)),
                       init = inits_list,
                       samplingOptions = list(sampling = sampling,
                                              monitors = monitors, niter = niter,
                                              nburnin = nburnin, thin = thin,
                                              nchain = nchain, setSeed = setSeed),
                       time = time)


  if (sampling){
    out <- list(coefficients = modelESTLIST$coefficients,
                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
                residuals = modelESTLIST$residuals,
                fitted.values = modelESTLIST$fitted.values,
                linear.predictors = modelESTLIST$linear.predictors,
                gof = modelESTLIST$gof,
                samples = sampMCMC, # return of runMCMC
                input = inputOptions,
                defModel = simpleModel, defSampler = mcmc1,
                modelName = "bsPolar")

    class(out) <- "bsim"


  } else{
    out <- list(input = inputOptions,
                defModel = simpleModel,
                defSampler = mcmc1,
                modelName = "bsPolar")

    class(out) <- "bsimSetup"

  }
  return(out)

}

#' @import methods
#' @import patchwork
#' @importFrom magrittr %>%
#' @import ggplot2
#' @importFrom tidyr pivot_longer all_of
#' @importFrom dplyr select
#' @importFrom stats mad
#' @importFrom utils str

# Common functions and S3 class for the BayesSIM
## bsim: Total results for fitted model
## bsimPred: object for prediction -> used in fitted plots
bsim <- function(coefficients, ses, residuals,
                 fitted.values, linear.predictors, gof,
                 input, samples, model, sampler, modelName) {
  structure(
    list(
      coefficients = coefficients,
      ses = ses, residuals = residuals,
      fitted.values = fitted.values,
      linear.predictors = linear.predictors,
      gof = gof,
      input = input,
      samples = samples,
      defModel = model, defSampler = sampler,
      modelName = modelName
    ),
    class = "bsim"
  )
}

bsimSetup <- function(coefficients, ses, residuals,
                      fitted.values, linear.predictors, gof,
                      input, samples, model, sampler, modelName) {
  structure(
    list(
      input = input,
      defModel = model,
      defSampler = sampler,
      modelName = modelName
    ),
    class = "bsimSetup"
  )
}

bsim.predict <- function(fitted, truey, idxValue, level, type){
  structure(
    list(
      fitted = fitted,
      truey = truey,
      idxValue = idxValue,
      level = level,
      type = type
    ),
    class = "bsimPred"
  )
}


#' Construct a Fitted Model Object from Model Setup and MCMC Output
#'
#' @param object A \code{bsimSetup} object.
#' @param ... Additional arguments passed to other methods.
#'
#' @rdname as_bsim.bsimSetup
#' @aliases as_bsim.bsimSetup
#' @export
as_bsim <- function(object, mcmc.out,...) {
  UseMethod("as_bsim")
}


#' Construct a Fitted Model Object from Model Setup and MCMC Output
#'
#' Create a fitted \code{bsim} object by combining a `BayesSIM`
#' setup object with MCMC samples returned by \code{runMCMC()}.
#'
#' @param object A `BayesSIM` setup object, typically the output of a
#'   \code{_setup} function.
#' @param mcmc.out MCMC output corresponding to the result of a call to \code{runMCMC()}.
#' @param ... Additional arguments passed to other methods.
#'
#' @return
#' An object of class \code{"bsim"} containing posterior samples,
#' point estimates, fitted values, and related model information.
#'
#' @details
#' This function is mainly intended for workflows where the model structure
#' and the MCMC sampling are performed separately. It collects the MCMC draws across chains, and returns an object of class \code{"bsim"}
#' that can be used with generic functions such as \code{summary()}, \code{plot()}, and \code{predict()}.
#'
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                    nchains = 1, setSeed = TRUE, inits = initList,
#'                    summary = TRUE, samplesAsCodaMCMC = TRUE)
#' fit2 <- as_bsim(models, mcmc.out)
#' summary(fit2)
#' }
#' @method as_bsim bsimSetup
#' @export
as_bsim.bsimSetup <- function(object, mcmc.out, ...){
  nchain <- object$input$samplingOptions$nchain
  # samples <- sampleBind(mcmc.out, nchain)
  if ("samples" %in% names(mcmc.out)){
    temp <- mcmc.out[["samples"]]
  } else{
    temp <- mcmc.out
  }

  # gpSphere - EB
  if (object$modelName == "gpSphere"){
    if (object$input$samplingOptions$method == "EB"){
      map_index <- object$defModel$getConstants()$index
      map_amp <- object$defModel$getConstants()$amp
      map_lengthscale <- object$defModel$getConstants()$lengthscale
      map_sigma2 <- object$defModel$getConstants()$sigma2
      p <- length(map_index)

      sampMCMC <- as.matrix(temp)
      nmcmcsamp <- nrow(sampMCMC)
      index_mat <- matrix(rep(map_index, nmcmcsamp), ncol = p, byrow = TRUE)
      colnames(index_mat) <- paste0("index[",1:p, "]")
      hype_mat <- matrix(rep(c(map_lengthscale, map_amp, map_sigma2), nmcmcsamp),
                         ncol = 3, byrow = TRUE)
      colnames(hype_mat) <- c("lengthscale", "amp", "sigma2")
      temp <- cbind(sampMCMC, index_mat, hype_mat)
    }
  }

  # gpSpike
  samples <- sampleBind(temp, nchain)


  X <- object$input$origdata$x
  Y <- object$input$origdata$y
  modelESTLIST <- bsimFit_pointest(samples, X, Y)
  out <- list(
    coefficients = modelESTLIST$coefficients,
    ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
    residuals = modelESTLIST$residuals,
    fitted.values = modelESTLIST$fitted.values,
    linear.predictors = modelESTLIST$linear.predictors,
    gof = modelESTLIST$gof,
    input = object$input,
    samples = temp,
    defModel = object$defModel, defSampler = object$defSampler,
    modelName = object$modelName
  )

  class(out) <- "bsim"
  return(out)

}

#' @rdname BayesSIM
#' @method print bsim
#' @export
print.bsim <- function(x, digits = 3, ...){
  # 1) Header
  cat("BayesSIM model\n")

  # 2) Model info
  cat("formula:     ", x$input$formula[c(2, 1, 3)], "\n")
  cat("observations:", nrow(x$input$origdata$x), "\n")
  cat("predictors:  ", ncol(x$input$origdata$x), "\n")
  cat("------\n")

  # estimated coefficient
  cat("Single-index estimates: \n")
  # cat(x$coefficients)
  A <- data.frame(mean = x$coefficients,
                  empirical.se = x$ses_coef)

  rownames(A) <- names(x$coefficients)
  colnames(A) <- c("Estimate (mean)", "SE")

  print(round(A, digits))

  cat("\n")

  # Goodness of fit: Residual sum of square
  cat("Goodness of fit (Mean residual sum of square): ", round(x$gof, digits))
  cat("\n")

}

#' @rdname BayesSIM
#' @method print bsimSetup
#' @export
print.bsimSetup <- function(x, digits = 3, ...){
  # 1) Header
  cat("BayesSIM model\n")

  # 2) Model info
  cat("formula:     ", x$input$formula[c(2, 1, 3)], "\n")
  cat("observations:", nrow(x$input$origdata$x), "\n")
  cat("predictors:  ", ncol(x$input$origdata$x), "\n")
  cat("------\n")

  cat("Try compiling the object with compileModelAndMCMC() and drawing posterior samples from runMCMC()")

}

#' Extract Index Vector Coefficients from `BayesSIM`
#'
#' Computes posterior summaries of the single-index model index vector
#' from a fitted `BayesSIM`. Users may choose either the
#' posterior mean or median as the point estimate.
#'
#' @param object A fitted object of \code{BayesSIM} or individual model.
#' @param method Character string indicating the summary statistic to compute.
#'   Options are \code{"mean"} or \code{"median"}. Default is \code{"mean"}.
#' @param se Logical value whether computing standard error for index estimates.
#' If `method` is \code{"mean"}, standard deviation of index vector MCMC samples is gained.
#' If  `method` is \code{"median"}, median absolute deviation of index vector MCMC samples is gained.
#' `FALSE` is default.
#' @param ... Additional arguments passed to other methods.
#'
#' @return A numeric vector or data.frame of estimated coefficient and standard error of the index vector.
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#'
#' @method coef bsim
#' @export
coef.bsim <- function(object, method = c("mean", "median"), se = FALSE, ...){


  method <- match.arg(method)

  if (sum(method == c("mean", "median")) == 2){
    method <- NULL
  }

  dataX <- object$input$origdata$x
  p <- ncol(dataX)
  namesIndex <- paste0("index[", 1:p, "]")
  indexName <- colnames(dataX)

  coefVector <- NULL

  if (is.null(method) || method == "mean"){
    coefVector <- object$coefficients

  } else if (method == "median"){
    nchain <- object$input$samplingOptions$nchain
    ALLsamp <- sampleBind(object$samples, nchain)

    coefficients_median <- apply(ALLsamp[ ,namesIndex], 2, median)
    # names(coefficients_median) <- indexName
    coefVector <- coefficients_median

  } else{
    stop("'method' argument could be either mean or median.")
  }

  # attr(coefVector, "names") <- indexName
  names(coefVector) <- indexName

  if (se){
    if (is.null(method) || method == "mean"){
      se.est <- object$ses_coef
    } else{
      se.est <- apply(ALLsamp[ ,namesIndex], 2, function(x) mad(x))
    }
    matResult <- rbind(coefVector, se.est)
    rownames(matResult) <- c(paste0("Estimate (", method, ")"), "SE")
    return(matResult)
  } else{
    return(coefVector)
  }

}

#' @rdname GOF.bsim
#' @export
GOF <- function(object){
  UseMethod("GOF")
}

#' Goodness of Fit for `BayesSIM`
#'
#' Generic function applied to \code{BayesSIM}.
#' It extracts goodness of fit of the `BayesSIM`.
#'
#' @param object A fitted object of \code{BayesSIM} or individual model.
#' @param ... Additional arguments passed to other methods.
#'
#' @return Mean squared error of model with mean of MCMC sample is applied.
#' @method GOF bsim
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#' @name GOF.bsim
#' @export
GOF.bsim <- function(object, ...){
  object$gof
}

#' Extract Residuals from `BayesSIM`
#'
#' Returns the model residuals based on the posterior fitted values of a
#' `BayesSIM`. Residuals can be computed using either the posterior mean or median fitted values.
#'
#' @param object A fitted object of \code{BayesSIM} or individual model.
#' @param method Character string specifying the summary statistic used to
#'   compute the fitted values. Options are \code{"mean"} or \code{"median"}.
#'   Default is \code{"mean"}.
#' @param ... Additional arguments passed to other methods.
#'
#' @return A numeric vector of residuals. (\eqn{\mathbf{r} = \mathbf{Y} - \hat{\mathbf{Y}}})
#' \eqn{\hat{\mathbf{Y}}} can be mean or median of MCMC samples.
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#' @name residuals.bsim
#' @method residuals bsim
#' @export
residuals.bsim <- function(object, method = c("mean", "median"), ...){
  method <- match.arg(method)
  # if (class(object) != "bsim"){
  #   stop("object should be 'bsim'.")
  # }

  if (sum(method == c("mean", "median")) == 2){
    method <- NULL
  }


  if (is.null(method) || method == "mean"){
    return(object$residuals)

  } else if (method == "median"){
    N <- nrow(object$input$origdata$x)
    nchain <- object$input$samplingOptions$nchain
    ALLsamp <- sampleBind(object$samples, nchain)
    colsN <- colnames(ALLsamp)
    namesLink <- grep("linkFunction", colsN, value = TRUE)
    datay <- object$input$origdata$y


    fitted.median <- apply(ALLsamp[ ,namesLink], 2, median)
    residuals.median <- datay[,1] - as.vector(fitted.median)
    return(residuals.median)

  } else{
    stop("'method' argument could be either mean or median.")
  }

}

#' Extract Fitted Values from `BayesSIM`
#'
#' Computes fitted values from a `BayesSIM`, using either the
#' posterior mean or median of the estimated link function with index values.
#' Fitted values can be returned on the latent scale or on the linear
#' predictor scale.
#'
#' @inheritParams residuals.bsim
#' @param type Character string indicating the scale on which fitted values
#'   are returned. Default is \code{"latent"}.
#'   \itemize{
#'     \item \code{"latent"}: fitted response values \eqn{\hat{y} = E(\mathbf{Y}|\mathbf{X})}.
#'     \item \code{"linpred"}: linear predictor \eqn{X'\theta}.
#'   }
#' @return A numeric vector of fitted values.
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#'
#' @method fitted bsim
#' @export
fitted.bsim <- function(object,
                        type = c("latent", "linpred"),
                        method = c("mean", "median"),...){ # point estimate
  type <- match.arg(type)
  method <- match.arg(method)

  # if (class(object) != "bsim"){
  #   stop("object should be 'bsim'.")
  # }
  if (sum(type == c("latent", "linpred")) == 2){
    type <- NULL
  }

  if (sum(method == c("mean", "median")) == 2){
    method <- NULL
  }


  if (is.null(type) || type == "latent"){
    if (is.null(method) || method == "mean"){
      return(object$fitted.values)

    } else if (method == "median"){
      N <- nrow(object$input$origdata$x)
      nchain <- object$input$samplingOptions$nchain
      ALLsamp <- sampleBind(object$samples, nchain)
      colsN <- colnames(ALLsamp)
      namesLink <- grep("linkFunction", colsN, value = TRUE)

      fitted.median <- apply(ALLsamp[ ,namesLink], 2, median)
      return(as.vector(fitted.median))

    } else{ # method error
      stop("'method' argument could be either mean or median.")
    }
  } else if (type == "linpred"){
    N <- nrow(object$input$origdata$x)
    namesXlin <- paste0("Xlin[", 1:N, "]")
    nchain <- object$input$samplingOptions$nchain
    ALLsamp <- sampleBind(object$samples, nchain)

    if (is.null(method) || method == "mean"){
      linpred.mean <- apply(ALLsamp[ ,namesXlin], 2, mean)
      return(as.vector(linpred.mean))

    } else if (method == "median"){
      linpred.median <- apply(ALLsamp[ ,namesXlin], 2, median)
      return(as.vector(linpred.median))

    } else{ # method error
      stop("'method' argument could be either mean or median.")
    }

  } else{ # type error
    stop("'type' argument could be either latent or linpred.")
  }
}

#' Summarize `BayesSIM`
#' @name summary.bsim
#' @description
#' Provides a \code{summary} for `BayesSIM`.
#'
#' @param object A fitted object of \code{BayesSIM} or individual model.
#' @param x A summary output of \code{BayesSIM} or individual model.
#' @param digits The minimum number of significant digits to be printed.
#' @param ... Further arguments passed.
#'
#' @details
#' A \code{list} of summary statistics for MCMC samples, including \code{data.frame} table for the results.
#' Each row corresponds to a model parameter, and columns report the statistics.
#'
#' @return
#' The function summarizes posterior MCMC samples by reporting key statistics, including:
#' \itemize{
#'   \item Posterior mean and median
#'   \item Empirical standard deviation
#'   \item 95% credible interval (lower and upper quantiles)
#'   \item Potential scale reduction factor (\code{Rhat}) for multiple chains
#'   \item Effective sample size (\code{ESS})
#' }
#'
#' By default, the index vector and error variance are only included in the summary.
#' If variable selection methods are used, such as uniform sphere and spike-and-slab prior,
#' the indicator vector (\code{nu}) is also included.
#' Note that the potential scale reduction factor for \code{nu} can be reported as
#' \code{NaN} or \code{Inf}, since the indicator rarely changes during the MCMC run.
#'
#' If the model is fitted with single chain, both \code{all.chain} and \code{chain} have identical information.
#'
#' @method summary bsim
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#'
#' @seealso \code{\link[coda]{gelman.diag}}, \code{\link[coda]{effectiveSize}}
#' @export
summary.bsim <- function(object, ...){
  p <- ncol(object$input$origdata$x)
  nchain <- object$input$samplingOptions$nchain
  namesVar <- colnames(object$input$origdata$x)
  printIndex <- namesVar
  namesIndex <- paste0("index[", 1:p, "]")
  namesSigma <- "sigma2"
  namesExtra <- character(0)
  temp <- object$input$samplingOptions$monitors
  if (!is.null(temp)){
    varList <- getVarMonitor(object, type = "list")
    for (v in temp){
      if (!grepl("\\[", v)){ # do not include "["
        # list up all name relevant to monitors variable
        for (j in varList){
          base <- sub("\\[.*$", "", j)
          if (v == base){
            namesExtra <- c(namesExtra, j)
          }
        }
      } else{
        namesExtra <- c(namesExtra, v)
      }
    }
  }


  # Names
  if (object$modelName %in% c("bsSphere", "bsSpike", "gpSpike")){ # Variable selection methods
    namesIndicator <- paste0("nu[", 1:p, "]")
    namesIndex_Var <- paste0("nu_",namesVar)

    if (!is.null(temp)){
      # param_names <- c(namesSigma, namesIndicator, namesIndex, namesExtra)
      param_names_order <- c(namesIndicator,  namesIndex, namesSigma, namesExtra)
      param_names_list <- c(namesIndex_Var, printIndex, namesSigma, namesExtra)
    } else{
      # param_names <- c(namesSigma, namesIndicator, namesIndex)
      param_names_order <- c(namesIndicator,  namesIndex, namesSigma)
      param_names_list <- c(namesIndex_Var, printIndex, namesSigma)
    }
  } else{
    if (!is.null(temp)){
      param_names_order <- c(namesIndex, namesSigma, namesExtra)
      param_names_list <- c(printIndex, namesSigma, namesExtra)
    } else{
      param_names_order <- c(namesIndex, namesSigma)
      param_names_list <- c(printIndex, namesSigma)
    }
  }

  calc_stat <- function(nchain, sampObj, namesIndex,
                       param_names_order, param_names_list){
    summarize_one_chain <- function(sampMat) {
      # sampMat: iter × param (matrix)
      # index standardized
      realTheta <- t(apply(sampMat[, namesIndex, drop = FALSE], 1, function(x) {
        x / sqrt(sum(x^2))
      }))
      colnames(realTheta) <- namesIndex

      # Rest of the parameters
      restNames <- setdiff(param_names_order, namesIndex)
      subTemp   <- cbind(sampMat[, restNames, drop = FALSE], realTheta)
      subTemp   <- subTemp[, param_names_order, drop = FALSE]

      # Compute mean, median, sd, CI
      stat_mat <- apply(subTemp, 2, function(x) {
        c(
          mean           = mean(x, na.rm = TRUE),
          median         = median(x, na.rm = TRUE),
          Standard.error = sd(x, na.rm = TRUE),
          LB             = quantile(x, probs = 0.025, na.rm = TRUE),
          UB             = quantile(x, probs = 0.975, na.rm = TRUE)
        )
      })

      stat_df <- as.data.frame(t(stat_mat))
      stat_df$ess <- coda::effectiveSize(subTemp)
      colnames(stat_df) <- c("Mean", "Median", "SD", "CI_lower", "CI_upper", "ESS")
      rownames(stat_df) <- param_names_list

      return(stat_df)
    }

    finalSummaryList <- list(
      chain     = vector("list", nchain),
      all.chain = NULL
    )

    if (nchain == 1){
      tmpResult <- summarize_one_chain(sampObj)
      finalSummaryList$chain <- tmpResult
      finalSummaryList$all.chain <- tmpResult
    } else{
      for (i in seq_len(nchain)) {
        sampTemp <- object$samples[[i]]
        tmpResult <- summarize_one_chain(sampTemp)
        finalSummaryList$chain[[i]] <- tmpResult
      }
      all_sub <- sampleBind(sampObj, nchain) # sampObj is MCMC
      tempALL <- summarize_one_chain(all_sub)
      gel     <- as.vector(coda::gelman.diag(sampObj[,param_names_order],
                                             multivariate = FALSE)$psrf[, 1])
      tempALL$Rhat <- gel
      finalSummaryList$all.chain <- tempALL
    }
    return(finalSummaryList)

  }

  totalTable <- calc_stat(nchain, object$samples, namesIndex,
                          param_names_order, param_names_list)

  class(totalTable) <- "summary.bsim"
  return(totalTable)

}

#' @rdname summary.bsim
#' @export
print.summary.bsim <- function(x, digits = 3, ...){
  cat("Summary statistics of all chains\n")
  tempTable <- x$all.chain
  if ("Rhat" %in% colnames(tempTable)){
    tempTable$Rhat[is.nan(tempTable$Rhat)] <- NA
    tempTable$Rhat <- round(tempTable$Rhat, digits = digits)
    tempTable$Rhat <- as.character(tempTable$Rhat)
    tempTable$Rhat[is.na(tempTable$Rhat)] <- "-"
  }
  print(tempTable, digits = digits)
}

#' Summarize `BayesSIM_setup`
#' @name summary.bsimSetup
#' @description
#' Provides a \code{summary} for `BayesSIM_setup`.
#'
#' @param object A fitted object of \code{BayesSIM_setup}.
#' @param x A summary output of \code{BayesSIM_setup}.
#' @param digits The minimum number of significant digits to be printed.
#' @param ... Further arguments passed.
#'
#' @details Summarize the model setup information of an object created by
#'   a setup function (e.g., \code{\link{BayesSIM_setup}}),
#'   including the model name, data dimensions, prior settings, and sampling options.
#'
#' @return
#' An object of class \code{summary.bsimSetup}, which is a list containing:
#'   \item{modelName}{The name of the fitted model.}
#'   \item{n}{The number of observations.}
#'   \item{p}{The number of predictors.}
#'   \item{prior}{A list of prior settings used in the model.}
#'   \item{samplingOptions}{A list of MCMC sampling options, including \code{niter}, \code{nburnin}, \code{thin}, and \code{nchain}.}
#'   \item{model}{The compiled nimble model object.}
#'
#' @method summary bsimSetup
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' summary(models)
#'
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes the class of "bsim".
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#'
#' @export
summary.bsimSetup <- function(object, ...) {
  out <- list(
    modelName = object$modelName,
    n = nrow(object$input$origdata$x),
    p = ncol(object$input$origdata$x),
    prior = object$input$prior,
    samplingOptions = object$input$samplingOptions,
    model = object$defModel
  )
  class(out) <- "summary.bsimSetup"
  out
}

#' @rdname summary.bsimSetup
#' @export
print.summary.bsimSetup <- function(x, digits = 1, ...) {
  cat("=== BayesSIM Setup Summary ===\n\n")
  cat("Model:", x$modelName, "\n")
  cat("Data: n =", x$n, ", p =", x$p, "\n\n")

  cat("--- Prior ---\n")
  str(x$prior, max.level = 2, give.attr = FALSE)
  cat("\n")

  cat("--- Sampling Options ---\n")
  cat("  niter =", x$samplingOptions$niter, "\n")
  cat("  nburnin =", x$samplingOptions$nburnin, "\n")
  cat("  thin =", x$samplingOptions$thin, "\n")
  cat("  nchain =", x$samplingOptions$nchain, "\n")

  invisible(x)

}


#' Traceplot for `BayesSIM`
#'
#' @description
#' Provides traceplot for objects of `BayesSIM`.
#'
#' @param x A fitted object of \code{BayesSIM} or individual model.
#' @param ... Further arguments passed to \code{\link{plot}}.
#' @return
#' Traceplots for MCMC samples are displayed.
#' By default, the index vector and error variance are only included in the summary.
#' Extra monitor variables are included, if specified.
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#' @export
nimTraceplot <- function(x, ...){
  Iteration <- 0; Value <- 0; num.chain <- 0; n <- 0
  p <- ncol(x$input$origdata$x)

  nchain <- x$input$samplingOptions$nchain
  varnames <- colnames(x$input$origdata$x)
  index_var <- paste0("index_", varnames)

  if (nchain > 1){
    samples <- NULL
    nchain <- length(x$samples)
    for (i in 1:nchain){
      n <- nrow(x$samples[[i]])
      temp <- cbind(chain.num = rep(i, n), x$samples[[i]])
      samples <- rbind(samples, temp)
      num.chain <- samples[,1]
    }
  } else{
    n <- nrow(x$samples)
    samples <- x$samples
    num.chain <- rep(1,n)
  }
  ##
  # index
  namesTheta <- paste0("index[", 1:p, "]")
  theta_samples <- samples[, namesTheta]

  # sigma2
  namesSigma <- "sigma2"
  sigma2_samples <- samples[, namesSigma]
  sigma2_df <- data.frame(sigma2 = sigma2_samples)

  # monitor variables
  temp <- x$input$samplingOptions$monitors
  if (!is.null(temp)){
    namesExtra <- character(0)
    temp <- x$input$samplingOptions$monitors
    varList <- getVarMonitor(x, type = "list")

     for (v in temp){
       if (!grepl("\\[", v)){ # do not include "["
         # list up all name relevant to monitors variable
         for (j in varList){
           base <- sub("\\[.*$", "", j)
           if (v == base){
             namesExtra <- c(namesExtra, j)
           }
           }
         } else{
           namesExtra <- c(namesExtra, v)
           }
      }

    extra_samples <- samples[, namesExtra]
  }

  if (x$modelName %in% c("bsSphere", "bsSpike", "gpSpike")){
    namesNu <- paste0("nu[", 1:p, "]")
    varNu <- paste0("nu_", varnames)
    nu_samples <- samples[, namesNu]
    if (!is.null(temp)){
      df <- data.frame(Iteration = rep(1:n, nchain),
                       num.chain =  num.chain,
                       nu_samples,
                       theta_samples,
                       sigma2_df, extra_samples)
      totalName <- c(varNu, index_var, namesSigma, namesExtra)
    } else{
      df <- data.frame(Iteration = rep(1:n, nchain),
                       num.chain =  num.chain,
                       nu_samples,
                       theta_samples,
                       sigma2_df)
      totalName <- c(varNu, index_var, namesSigma)
    }
  } else{
    if (!is.null(temp)){
      df <- data.frame(Iteration = rep(1:n, nchain),
                       num.chain =  num.chain,
                       theta_samples,
                       sigma2_df, extra_samples)
      totalName <- c(index_var, namesSigma, namesExtra)
    } else{
      df <- data.frame(Iteration = rep(1:n, nchain),
                       num.chain =  num.chain,
                       theta_samples,
                       sigma2_df)
      totalName <- c(index_var, namesSigma)
    }
  }

  colnames(df)[-c(1,2)] <- totalName

  if (nchain == 1){

    df_long <- df %>% dplyr::select(-num.chain) %>%
      pivot_longer(
        cols = -Iteration,
        names_to = "Parameter",
        values_to = "Value"
      )
    df_long$Parameter <- factor(df_long$Parameter,
                                levels = totalName)
    ggplot2::ggplot(df_long, aes(x = Iteration, y = Value)) +
      geom_line() +
      facet_wrap(~Parameter, scales = "free_y") +
      theme_bw() +
      labs(title = "Traceplots for parameters",
           x = "Iteration", y = "Value")

  } else{
    df_long <- df %>%
      pivot_longer(
        cols = all_of(totalName),
        names_to = "Parameter",
        values_to = "Value"
      )
    df_long$Parameter <- factor(df_long$Parameter,
                                levels = totalName)
    df_long$num.chain <- factor(df_long$num.chain,
                                levels = 1:nchain)
    # traceplot
    ggplot2::ggplot(df_long, aes(x = Iteration, y = Value, col = num.chain)) +
      geom_line() +
      facet_wrap(~Parameter, scales = "free_y") +
      theme_bw() +
      labs(title = "Traceplots for parameters",
           x = "Iteration", y = "Value")
  }

}

#' Plot Method for `BayesSIM`
#'
#' Produce diagnostic plots for a fitted Bayesian single-index model.
#'
#' @param x A fitted object of \code{BayesSIM} or individual model.
#' @param method Character string specifying the summary used for the
#'   posterior fitted values. Options are `"mean"` or `"median"`.
#'   Default is `"mean"`.
#' @param interval A logical value indicating whether a credible interval is included in the fitted plot. Default is `TRUE`.
#' @param alpha Numeric value between 0 and 1 specifying the credible level.
#'   By default, `alpha = 0.95` produces a 95% credible interval.
#' @param ... Additional arguments passed to underlying plotting functions.
#'
#'
#' @details
#' The function internally calls [predict()] on the fitted model object
#' to obtain posterior summaries of \eqn{\hat{y}}. Predicted value of \eqn{y} is \eqn{\hat{f}(X'\hat{\theta})}.
#'
#' - If `interval = TRUE`, the function requests posterior credible intervals
#'   and overlays them on the fitted curve.
#' - If `interval = FALSE`, only the posterior mean or median curve is drawn.
#' @return The output consists of two plots:
#' \enumerate{
#'   \item **Observed vs Predicted plot**: a diagnostic scatter plot comparing
#'   actual outcomes with posterior fitted values to visually assess model fit.
#'   \item **Fitted curve plot**: posterior \eqn{\hat{y}} as a function of the
#'   estimated single index, optionally with \eqn{100 \times \alpha} % credible intervals.
#' }
#'
#' @seealso [predict.bsim()], [summary.bsim()]
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#' @method plot bsim
#' @name plot.bsim
#' @export
plot.bsim <- function(x, method = c("mean", "median"),
                            interval = TRUE, alpha = 0.95, ...){

  method <- match.arg(method)

  if (!is.logical(interval)){
    stop("'interval' argument should be logical value.")
  }

  if (interval == TRUE){
    pred <- predict.bsim(x, method = method,
                    interval = "credible",
                    alpha = alpha)
  } else{
    pred <- predict.bsim(x, method = method,
                    alpha = alpha)
  }


  plot_bsim_fitted(pred, interval)
}

#' Plot Method for `BayesSIM_setup`
#'
#' Produce diagnostic plots for a raw data.
#'
#' @param x A fitted object of \code{BayesSIM} or individual model.
#' @param select A vector of index or name of variables in data.
#' @param ... Additional arguments passed to underlying plotting functions.
#'
#'
#' @details The function displays scatter plots of the response variable against
#'   each predictor variable. Each panel includes a smoothed trend line
#'   to help assess the marginal relationship before fitting the model.
#'
#' @return A \code{ggplot} object is returned invisibly, allowing further modification if needed.
#'
#' @seealso [summary.bsimSetup()]
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#'
#' plot(models, select = c("X1", "X2"))
#'
#' }
#' @method plot bsimSetup
#' @name plot.bsimSetup
#' @export
plot.bsimSetup <- function(x, select = NULL, ...) {
  value <- 0
  X <- x$input$origdata$x
  y <- x$input$origdata$y

  if (is.null(select)) {
    select <- seq_len(ncol(X))
  }

  X <- X[, select, drop = FALSE]

  df <- data.frame(y = rep(y, ncol(X)),
                   value = as.vector(X),
                   variable = rep(colnames(X), each = length(y)))


  p <- ggplot(df, aes(x = value, y = y)) +
    geom_point(alpha = 0.5) +
    geom_smooth() +
    facet_wrap(~ variable, scales = "free_x") +
    labs(x = "Predictor", y = "Response")

  print(p)
  invisible(p)

}



#' Prediction Method for `BayesSIM`
#'
#' Generate predictions from a fitted Bayesian single-index model.
#'
#' @param object A fitted object of \code{BayesSIM} or individual model.
#' @param x An object of class "bsimPred" which is the result of \code{predict}.
#' @param newdata Optional data frame for which predictions should be made.
#'   If `NULL`, predictions are returned for the training data.
#' @param se.fit A logical value indicating whether standard errors are required.
#'   Default is `FALSE`.
#' @param type Character string specifying the type of prediction. `"response"`
#'   is the default.
#'   \describe{
#'     \item{`"response"`}{
#'       Posterior predictive summaries of the response \eqn{Y}.
#'       This corresponds to draws from the posterior predictive distribution
#'       \eqn{Y^{(m)} \sim N(f(X'\theta^{(m)}), \sigma^{2(m)})}
#'       and therefore incorporates both the uncertainty in the link function
#'       and the variability of the error term for each \eqn{m^{th}} MCMC sample.}
#'     \item{`"latent"`}{
#'       Posterior summaries of the latent mean structure
#'       \eqn{E(Y \mid X) = f^{(m)}(t^{(m)})}, where \eqn{t^{(m)} = X'\theta^{(m)}}.
#'       Unlike `"response"`, it excludes the noise term and calculated by
#'       \eqn{f^{(m)}(X'\theta^{(m)})} for each \eqn{m^{th}} MCMC sample of
#'       \eqn{\theta}.}
#'     \item{`"index"`}{
#'       Posterior summaries of the single index \eqn{t^{(m)} = X'\theta^{(m)}}.}
#'   }
#'
#' @param method Character string determining the posterior summary used for
#'   point predictions. Options are `"mean"` or `"median"`. Default is `"mean"`.
#'
#' @param interval Character string indicating whether a credible interval should be returned. Default is `"none"`.
#'   \describe{
#'     \item{`"none"`}{Return only point predictions.}
#'     \item{`"credible"`}{Return a \eqn{100 \times \text{level}\%} credible interval.}
#'   }
#'
#' @param level Numeric value between 0 and 1 specifying the credible level.
#'   `level = 0.95` yields a 95% credible interval. Default is `0.95`.
#'
#' @param ... Additional arguments.
#'
#' @details
#' This method extracts MCMC posterior samples stored in a BayesSIM object
#' and computes posterior summaries of:
#' \itemize{
#'   \item the **posterior predictive response** \eqn{Y \mid X} (type `"response"`),
#'   \item the **latent link function** evaluation \eqn{E(Y \mid X) = f(X'\theta)} (type `"latent"`), or
#'   \item the **single index** \eqn{X'\theta} (type `"index"`).
#' }
#'
#' The key distinction is that `"response"` incorporates the posterior
#' variability of the error term \eqn{\epsilon}, whereas `"latent"` represents
#' the noiseless conditional expectation \eqn{E(Y \mid X)} computed directly
#' from the link function and the posterior draws of \eqn{\theta}.
#'
#' When `interval = "credible"`, the returned object includes lower and upper
#' credible bounds computed via posterior quantiles for the chosen prediction
#' scale.
#'
#' If `newdata` is supplied, predictions are evaluated at the new covariate
#' values by computing the corresponding posterior index \eqn{t = X'\theta}
#' and applying the link function.
#'
#' @return
#' A list containing at least the following components:
#' \describe{
#'   \item{`fitted`}{Posterior mean or median predictions. If \code{se.fit = TRUE} or
#'   \code{interval = "credible"}, standard error and lower, upper bounds of the credible interval is also included.}
#'   \item{`truey`}{True response value of test data. When true response value is not available, \code{NULL} is saved.}
#'   \item{`idxValue`}{Linear index value is saved where \eqn{\theta} is estimated by \code{method}.}
#'   \item{`level`}{Credible level.}
#' }
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Check median index vector estimates with standard errors
#' coef(fit_one, method = "median", se = TRUE)
#'
#' # Fitted index values of median prediction
#' fitted(fit_one, type = "linpred", method = "median")
#'
#' # Residuals of median prediction
#' residuals(fit_one, method = "median")
#'
#' # Summary of the model
#' summary(fit_one)
#'
#' # Convergence diagnostics
#' nimTraceplot(fit_one)
#'
#' # Goodness of fit
#' GOF(fit_one)
#'
#' # Fitted plot
#' plot(fit_one)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' plot(pred)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#' @method predict bsim
#' @name predict.bsim
#' @export
predict.bsim <- function(object, newdata = NULL,
                         se.fit = FALSE,
                         type = c("response", "latent", "index"),
                         method = c("mean", "median"),
                         interval = c("none", "credible"),
                         level = 0.95, ...){
  start1 <- Sys.time()
  truey <- 0; pred <- 0; LB <- 0; UB <- 0

  # environment
  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))

  # aa_bspline_ver3
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # aa_bspline_ver3
    "SplineState", "any_duplicated", "mat_wo_col1", "update_spline_df", "update_x_index",
    "update_knot_sequence", "get_basis_simple", "simplify_knots", "get_inside_x",
    "gen_default_internal_knots", "SplineBase1", "SplineBase2", "basis", "bsNimble", "bsBasis",

    # bsFisher
    "pred_bsplineFisher",

    # bsSphere
    "pred_bsplineSphere","expcovTest_gpSphere",
    "expcovTest_gpPolar","expcovnn_gpSpike",

    # extra
    "expcov_gpPolar", "expcov_gpSphere", "expcov_gpSpike",
    "transX_fisher", "transX_sp", "Xlinear", "alphaTheta",

    # distribution
    "dvMFnim", "dvMFnim", "dunitSphere", "runitSphere"

  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())

  suppressMessages(
    nimble::registerDistributions(list(
      dvMFnim = list(
        BUGSdist = "dvMFnim(theta)",
        types    = c("value = double(1)", "theta = double(1)"),
        discrete = FALSE
      )
    ), verbose = FALSE)
  )

  suppressMessages(
    nimble::registerDistributions(list(
      dKnotsSimple = list(
        BUGSdist = "dKnotsSimple(a, b, k, alpha)",
        types = c("value = double(1)", "a = double(0)", "b = double(0)", "k = double(0)",
                  "alpha = double(1)"),
        discrete = FALSE
      )
    ), verbose = FALSE)
  )

  suppressMessages(
    nimble::registerDistributions(list(
      dunitSphere = list(
        BUGSdist     = "dunitSphere(dim)",
        types        = c("value = double(1)",
                         "dim   = double(0)"),
        discrete     = FALSE
      )
    ), verbose = FALSE)

  )


  type <- match.arg(type)
  method <- match.arg(method)
  interval <- match.arg(interval)

  # check - modify
  # if (is.null(object$samples)){
  #   stop("No MCMC samples available. Make sure 'sample = TRUE'")
  # }

  if (!is.character(method) || length(method) != 1){
    stop("'method' argument should be correctly specified.")
  }

  if (!is.logical(se.fit)){
    stop("'se.fit' argument should be correctly specified.")
  }

  if (!is.numeric(level) || length(level) != 1){
    stop("'level' argument should be correctly specified.")
  }

  if (!(type %in% c("index", "latent", "response"))){
    stop("'type' arugment is incorrect.")
  }

  # ------------------------------------------------------------------
  # Samples ready
  nchain <- object$input$samplingOptions$nchain
  samples <- sampleBind(object$samples, nchain)

  # Data formation for new data set
  if (!is.null(newdata)){ # test data
    # test new data
    if (!is.data.frame(newdata)){
      stop("newdata should be data.frame.")
    }

    formula <- as.character(object$input$formula)
    class.name <- formula[2]
    data.name <- strsplit(formula[3]," \\+ ")[[1]]
    # int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])

    if(data.name[1] == "."){
      allX <- colnames(newdata)[colnames(newdata) != class.name]
      tot.name <- c(class.name, allX)
    } else{
      tot.name <- c(class.name, data.name)
    }
    varset <- setdiff(tot.name, class.name)
    newdataX <- newdata[ ,varset]

    if (!(class.name %in% colnames(newdata))) { # new Y is not included
      predIncluded <- FALSE
      newdataY <- NULL
    } else{ # new Y is included
      predIncluded <- TRUE
      newdataY <- as.vector(newdata[,class.name])
    }

  } else{
    predIncluded <- TRUE
    newdataX <- object$input$origdata$x
    newdataY <- object$input$origdata$y
  }

  train_p <- ncol(newdataX)
  # N <- nrow(newdataX)
  N <- nrow(object$input$origdata$x)
  trainY <- as.vector(object$input$origdata$y)


  ## newdataY, newdataX exists (prediction starts) ----------------------------
  nsamp <- nrow(samples)
  namesIndex <- paste0("index[", 1:train_p, "]")
  namesXlin <- paste0("Xlin[", 1:N, "]") ##

  indexSample <- samples[, namesIndex]
  XlinSample <- samples[, namesXlin]
  sigma2_samples <- samples[, "sigma2"]
  newdataMat <- as.matrix(newdataX)

  indexValueMatrix <- indexSample %*% t(newdataX) # N(MCMCsamp) * nrow(X)

  # What - posterior samples
  if (type == "index"){ # newdataX %*% indexSample
    testPred <- indexValueMatrix # N(MCMCsamp) * nrow(X)
  }

  if (type %in% c("response", "latent")){
    # B-spline
    if (object$modelName == "bsSphere"){
      maxknots <- object$input$prior$link$knots$maxknots
      k <- samples[ ,"k"]
      knotsSample <- samples[ ,paste0("knots[", 1:maxknots, "]")]
      dfSample <- samples[ ,"numBasis"]
      degree <- object$input$prior$link$basis$degree
      namesBeta <- paste0("beta[", 1:maxknots, ", 1]")
      betaSample <- samples[,namesBeta]
      mina <- samples[ ,"a_alpha"]
      maxb <- samples[ ,"b_alpha"]


      message("== Compiling functions ==")
      suppressMessages(
        cpred_bsplineSphere <- compileNimble(pred_bsplineSphere)
      )

      if (type == "latent"){

        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_bsplineSphere(newdataMat, indexSample,
                                        knotsSample, XlinSample,
                                        betaSample, sigma2_samples, k,
                                        dfSample,
                                        mina, maxb, nsamp,degree,
                                        prediction = 1)
        end2 <- Sys.time()


      } else if (type == "response"){
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_bsplineSphere(newdataMat, indexSample,
                                        knotsSample, XlinSample,
                                        betaSample, sigma2_samples, k,
                                        dfSample,
                                        mina, maxb, nsamp,degree,
                                        prediction = 2)
        end2 <- Sys.time()

      }

    } else if (object$modelName %in% c("bsFisher", "bsPolar", "bsSpike")){ # c("bsFisher", "bsPolar", "bsSpike")
      df <- object$input$prior$link$basis$df
      degree <- object$input$prior$link$basis$degree
      delta <- object$input$prior$link$basis$delta
      n_internal_knots <- df - degree
      prob_vec <- seq(0, 1, length.out = n_internal_knots+2)[2:(n_internal_knots+1)]
      knotsSample <- t(apply(XlinSample, 1, function(x) quantile(x, prob_vec))) # n_internal_knots * nsamp
      namesBeta <- paste0("beta[", 1:df, "]")
      betaSample <- samples[ ,namesBeta]
      end1 <- Sys.time()


      message("== Compiling functions ==")
      suppressMessages(
        cpred_bsplineFisher <- compileNimble(pred_bsplineFisher)
      )

      if (type == "latent"){
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_bsplineFisher(newdataMat, indexSample,
                                        knotsSample, XlinSample,
                                        betaSample, sigma2_samples,
                                        nsamp, df, degree, delta,
                                        prediction = 1)
        end2 <- Sys.time()
      } else if (type == "response"){
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_bsplineFisher(newdataMat, indexSample,
                                        knotsSample, XlinSample,
                                        betaSample, sigma2_samples,
                                        nsamp, df, degree, delta,
                                        prediction = 2)
        end2 <- Sys.time()
      }
    }

    # Gp
    if (object$modelName %in% c("gpSphere", "gpFisher")){
      # Hyper-parameters of expcov
      lengthSample <- samples[ ,"lengthscale"]
      ampSample <- samples[ ,"amp"]
      # newdataMat <- as.matrix(newdataX)

      message("== Compiling functions ==")
      suppressMessages(
        cpred_gpSphere <- compileNimble(pred_gpSphere)
      )

      if (type == "latent"){ # latent
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_gpSphere(newdataMat, nsamp, trainY, indexSample,
                                   XlinSample, sigma2_samples,
                                   lengthSample, ampSample,
                                   prediction = 1)
        end2 <- Sys.time()
      } else{# response
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_gpSphere(newdataMat, nsamp, trainY, indexSample,
                                   XlinSample, sigma2_samples,
                                   lengthSample, ampSample,
                                   prediction = 2)
        end2 <- Sys.time()
      }

    } else if (object$modelName == "gpPolar"){
      # Hyper-parameters of expcov
      kappaSample <- samples[ , "kappa"]
      newdataMat <- as.matrix(newdataX)

      message("== Compiling functions ==")
      suppressMessages(
        cpred_gpPolar <- compileNimble(pred_gpPolar)
      )

      if (type == "latent"){
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_gpPolar(newdataMat, nsamp, trainY, indexSample,
                                  XlinSample, sigma2_samples,
                                  kappaSample, prediction = 1)
        end2 <- Sys.time()
      } else{ # response
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_gpPolar(newdataMat, nsamp, trainY, indexSample,
                                  XlinSample, sigma2_samples,
                                  kappaSample, prediction = 2)
        end2 <- Sys.time()
      }

    } else if (object$modelName == "gpSpike"){
      # namesIndex <- paste0("index[", 1:train_p, "]")
      namesIndexstar <- paste0("indexstar[", 1:train_p, "]")
      indexstarSample <- samples[, namesIndexstar]
      invlambdaSample <- samples[,"invlambda"]
      newdataMat <- as.matrix(newdataX)
      # newdataMat <- as.matrix(scale(newdataX))

      message("== Compiling functions ==")
      suppressMessages(
        cpred_gpSpike <- compileNimble(pred_gpSpike)
      )

      if (type == "latent"){
        # latent
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_gpSpike(newdataMat, nsamp, trainY, indexstarSample,
                                  XlinSample, sigma2_samples, invlambdaSample,
                                  prediction = 1)
        end2 <- Sys.time()
      } else{ # response
        start2 <- Sys.time()
        message("== Computing predicted values ==")
        testPred <- cpred_gpSpike(newdataMat, nsamp, trainY, indexstarSample,
                                  XlinSample, sigma2_samples, invlambdaSample,
                                  prediction = 2)
        end2 <- Sys.time()
      }

    }


  }
  # testpred: samples

  # Compute results
  alpha <- 1-level
  fittedResult <- data.frame(mean = apply(testPred, 2, mean),
                             median = apply(testPred, 2, median),
                             SE = apply(testPred, 2, sd),
                             LB = apply(testPred, 2, quantile, probs = alpha/2, na.rm = TRUE),
                             UB = apply(testPred, 2, quantile, probs = 1-alpha/2, na.rm = TRUE))

  # interval, se

  if (se.fit){
    Name <- c(method, "SE")
  } else{
    Name <- method
  }

  if (interval == "credible"){
    Name <- c(Name, "LB", "UB")
  }
  end1 <- Sys.time()

  summaryResult <- list(fitted = NULL, truey = NULL,
                        idxValue = NULL, level = level,
                        type = type)


  summaryResult$fitted <- as.data.frame(fittedResult[ ,Name])
  summaryResult$truey <- newdataY # if predIncluded = FALSE, truey = NULL
  if (method == "mean"){
    colnames(summaryResult$fitted)[1] <- "Mean"
    summaryResult$idxValue <- apply(indexValueMatrix, 2, mean)
  } else if (method == "median"){
    colnames(summaryResult$fitted)[1] <- "Median"
    summaryResult$idxValue <- apply(indexValueMatrix, 2, median)
  }

  class(summaryResult) <- "bsimPred"
  invisible(summaryResult)

}

#' @rdname predict.bsim
#' @export
print.bsimPred <- function(x, ...){
  cat("Prediction object of BayesSIM\n")
  n <- nrow(x$fitted)
  cat("  n:", n, "\n")

  cat("\nPrediction summary:\n")

  if (is.data.frame(x$fitted)) {

    f_names <- names(x$fitted)

    predType <- x$type
    if ("response" == predType){
      cat("  - Type : Response \n")
    } else if ("latent" == predType){
      cat("  - Type : Latent \n")
    } else if ("index" == predType){
      cat("  - Type : Index \n")
    }


    if ("Mean" %in% f_names){
      cat("  - Mean : Posterior mean prediction\n")
    } else if ("Median" %in% f_names){
      cat("  - Median : Posterior median prediction\n")
    }


    if ("SE" %in% f_names)
      cat("  - SE   : Posterior standard error\n")

    if (all(c("LB", "UB") %in% f_names)) {
      cat("  - LB/UB: Credible interval bounds (Level:", x$level, ")\n")
      # cat("    Interval level:", x$level, "\n")
    } else {
      cat("  (Credible intervals are not stored)\n")
    }

  }

  if (length(f_names) == 1){
    cat("  (No uncertainty information available)\n")
  }

  invisible(x)

}


#' Summarize Predictions
#' @name summary.bsimPred
#' @description
#' Provides a \code{summary} for `BayesSIM` prediction.
#'
#' @param object An object of class "bsimPred" which is the result of \code{predict}.
#' @param x A summary output of class "bsimPred".
#' @param digits The minimum number of significant digits to be printed.
#' @param ... Further arguments passed.
#'
#' @details
#' Performance of prediction with point predicted values.
#'
#' @return
#' Metrics are including RMSE, MAE, Mean bias, and coverage. In terms of coverage, it is only available
#' when the "bsimPred" includes interval, corresponding to \code{interval = "credible"} options for \code{predict} function.
#' It indicates mean coverage rate of data points whether confidence intervals include true response values.
#' In this case, all chains are combined for the computation.
#'
#' @method summary bsimPred
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' # Prediction with 95% credible interval at new data
#' newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
#' pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
#' summary(pred)
#' plot(pred)
#'
#' }
#' @export
summary.bsimPred <- function(object, ...){

  y_pred <- object$fitted[,1]

  # 1. summary statistics of predictions
  dist_stats <- list(
    n = length(y_pred),
    mean = mean(y_pred, na.rm = TRUE),
    sd = sd(y_pred, na.rm = TRUE),
    range = range(y_pred, na.rm = TRUE),
    quartiles = quantile(y_pred, probs = c(0.25, 0.5, 0.75), na.rm = TRUE)
  )

  # 2. Metrics
  metrics <- NULL
  y_true <- object$truey
  if (!is.null(y_true)) {
    rmse <- sqrt(mean((y_true - y_pred)^2, na.rm = TRUE))
    mae <- mean(abs(y_true - y_pred), na.rm = TRUE)
    bias <- mean(y_pred - y_true, na.rm = TRUE)
    metrics <- list(RMSE = rmse, MAE = mae, Mean_bias = bias)

    if ("LB" %in% names(object$fitted) &
        "UB" %in% names(object$fitted)){
      lower <- object$fitted$LB
      upper <- object$fitted$UB
      cover <- mean(y_true >= lower & y_true <= upper, na.rm = TRUE)
      metrics$Coverage <- cover
      metrics$level <- object$level
    }
  } else{
    metrics <- list(RMSE = NULL, MAE = NULL, Mean_bias = NULL)
    if ("LB" %in% names(object$fitted) &
        "UB" %in% names(object$fitted)){
      metrics$Coverage <- NULL
    }
    metrics$level <- object$level
  }

  out <- list(
    dist_stats = dist_stats,
    metrics = metrics,
    has_truth = !is.null(y_true)
  )
  class(out) <- "summary.bsimPred"
  return(out)

}

#' @rdname summary.bsimPred
#' @export
print.summary.bsimPred <- function(x, digits = 3, ...){

  cat("\n==========================================\n")
  cat("       Model Prediction Summary           \n")
  cat("==========================================\n")

  cat(paste0("[Observations] : ", x$dist_stats$n, "\n"))
  cat("[Distribution] :\n")
  cat(paste0("  - Mean (SD)  : ", round(x$dist_stats$mean, digits),
             " (", round(x$dist_stats$sd, digits), ")\n"))
  cat(paste0("  - Range      : ", round(x$dist_stats$range[1], digits),
             " to ", round(x$dist_stats$range[2], digits), "\n"))
  cat(paste0("  - Percentiles: 25% = ", round(x$dist_stats$quartiles[1], digits),
             ", 50% = ", round(x$dist_stats$quartiles[2], digits),
             ", 75% = ", round(x$dist_stats$quartiles[3], digits), "\n"))

  cat("\n------------------------------------------\n")

  if (x$has_truth) {
    cat("[Performance Metrics] :\n")
    cat(paste0("  - RMSE       : ", round(x$metrics$RMSE, digits), "\n"))
    cat(paste0("  - MAE        : ", round(x$metrics$MAE, digits), "\n"))
    cat(paste0("  - Bias       : ", round(x$metrics$Mean_bias, digits), "\n"))
  } else {
    cat("[Note] :\n")
    cat("  True target values (y) were not provided.\n")
    cat("  Performance metrics are unavailable.\n")
  }

  cat("==========================================\n\n")

  invisible(x)
}

#' Extract the Result Data.frame
#' @description
#' Extracts fitted result that contains posterior mean/median prediction of response variable, and
#' standard error, credible interval of each data point in data.frame.
#' @param x An object of class "bsimPred" which is the result of \code{predict.bsim}.
#' @param ... Additional arguments passed to other methods.
#' @return data.frame of prediction information.
#' @seealso
#' \code{\link{predict.bsim}}
#' @examples
#' \donttest{
#' data("DATA1")
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#' fit_one  <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#' pred <- predict(fit_one, se.fit = TRUE, interval = "credible",level = 0.8)
#' results <- as.data.frame(pred)
#' summary(results)
#' }
#'
#' @export
as.data.frame.bsimPred <- function(x, ...){
  invisible(as.data.frame(x$fitted))
}

# predict structure
#' @rdname plot.bsim
#' @export
plot.bsimPred <- function(x, ...){ # predict object

  # object: predict object
  if (sum(c("LB", "UB") %in% colnames(x$fitted)) == 2){
    interval <- TRUE
  } else{
    interval <- FALSE
  }

  plot_bsim_fitted(x, interval)

}



#' Retrieve Monitorable Variables
#'
#' @description
#' Functions for retrieving the variables that can be monitored.
#'
#' @param object A fitted object of \code{BayesSIM}, \code{BayesSIM_setup} or individual model.
#' @param type Options for variables. By default, `type = "name"` is used that it only prints the name of the node.
#' If you put name of the nodes, the MCMC outputs gave you all elements of the variable, in case of the vector.
#' If `type = "list"`, the dimension of the nodes are printed. If you put name and dimension of the nodes, only specific location of vector or matrix can be seen in `summary` or `nimTraceplot`.
#'
#' @details
#' The function returns a list of variables that can be used in \code{monitors2} in the \code{bayesSIM} function.
#' You can also refer to \code{\link{getModelDef}} to understand the model structure and designate necessary variables.
#' Stochastic nodes of the model are recommended to be monitored.
#'
#' @return A vector of variables that can be monitored in the model.
#' @seealso
#' \code{\link{getModelDef}}
#'
#' @examples
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#'
#' # Get monitorable variables
#' getVarMonitor(models)
#' # Get the list of variables with dimension
#' getVarMonitor(models, type = "list")
#'
#' @export
getVarMonitor <- function(object, type = c("name", "list")){
  type <- match.arg(type)
  possibleList <- object$defModel$getNodeNames(stochOnly = TRUE, includeData = FALSE)
  tempList <- expand_monitor(possibleList)
  if (type == "name"){
    return(tempList$baseName)
  } else if (type == "list"){
    return(tempList$out)
  } else{
    stop("'type' argument is wrong.")
  }


}

#' Get Definition of the Model
#'
#' @description
#' Functions for identifying definition of the \pkg{nimble} model.
#'
#' @param object A fitted object of \code{BayesSIM} or individual model.
#'
#' @details
#' The function that contain Bayes SIM model structure in \pkg{nimble}.
#' This function is for advanced users.
#' There are several functions used in the model definition.
#' \itemize{
#' \item `transX_fisher`, `transX_sp`: Making B-spline basis.
#' \item `dvMFnim`: Distribution of von Mises-Fisher.
#' \item `dKnotsSimple`: Distribution of the free knots for `bsSphere`.
#' \item `dunitSphere`: Distribution of unit sphere.
#' \item `alphaTheta`: One-to-one polar transformation. Making index vector from individual angular vector `psi`.
#' \item `expcov_gpSphere`, `expcov_gpPolar`, `expcov_gpSpike`: Covariance kernel of each model.
#' `expcov_gpSphere` uses squared-exponential kernel, `expcov_gpPolar` uses OU process kernel, and `expcov_gpSpike` uses squared-exponential including its own parameter, \eqn{\lambda^{-1}}.
#' \item `Xlinear`: Making linear combination with index vector.
#' }
#'
#' @return BUGS code of the model definition.
#'
#'
#' @seealso
#' \code{\link{getVarMonitor}}
#'
#' @examples
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#' models <- bsFisher_setup(y ~ ., data = simdata2)
#' # Get model definition
#' getModelDef(models)
#'
#' @export
getModelDef <- function(object){
  return(object$defModel$modelDef$BUGScode)
}


#' Get Initial Value of the Model
#'
#' @description
#' Functions for getting list of initial values of the \pkg{nimble} model.
#'
#' @param object A fitted object of \code{BayesSIM}, `BayesSIM_setup` or individual model.
#'
#' @details
#' The list of initial values are returned. This can be helpful to use when you use `BayesSIM_setup`.
#' You should be aware of that if you want to get more than 1 chain of MCMC samples, you should change `nchain` argument in `BayesSIM_setup`.
#' The output of initial values would be different, depending on the number of chain.
#'
#' You can apply `BayesSIM` object when you want to check the list of initial values.
#'
#' @return BUGS code of the model definition.
#'
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#'
#' # Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#'
#'
#' @export
getInit <- function(object){

  return(object$input$init)
}


#' Compile a 'nimble' Model and Its MCMC
#'
#' @description
#' Compiles a nimble model object and a corresponding (uncompiled) MCMC
#' algorithm and returns the compiled pair.
#'
#' @param object Class `BayesSIM_setup` object
#'
#' @details
#' The function first compiles `nimble` model object, then compiles `nimble` sampler.
#' Both compiled model and compiled MCMC samplers are returned as a list.
#'
#' @return A list with two elements:
#' \describe{
#'   \item{\code{model}}{the compiled NIMBLE model (external pointer object).}
#'   \item{\code{mcmc}}{the compiled MCMC function/algorithm bound to the model.}
#' }
#'
#' @seealso
#' \code{\link[nimble]{nimbleModel}},
#' \code{\link[nimble]{configureMCMC}},
#' \code{\link[nimble]{buildMCMC}},
#' \code{\link[nimble]{compileNimble}},
#' \code{\link[nimble]{runMCMC}}
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#'
#' @export
compileModelAndMCMC <- function(object) {
  model <- object$defModel; mcmc <- object$defSampler
  # global assign
  envobj <- ls(envir=.GlobalEnv)
  on.exit(rm(list=ls(envir=.GlobalEnv)[which(!ls(envir=.GlobalEnv)%in%envobj)],envir=.GlobalEnv))


  # aa_bspline_ver3
  .fns <- c(
    # a_common
    "quickSortOrderIndexOnly", "nimOrder", "nimSort",
    "sampleQuantile_nim", "quantile_nimble",

    # aa_bspline_ver3
    "SplineState", "any_duplicated", "mat_wo_col1", "update_spline_df", "update_x_index",
    "update_knot_sequence", "get_basis_simple", "simplify_knots", "get_inside_x",
    "gen_default_internal_knots", "SplineBase1", "SplineBase2", "basis", "bsNimble", "bsBasis",

    # bsFisher
    "postll_bspline_fisher","nimNorm","rW","besselI_nimble","Stheta",
    "estBeta_fisher","gvcCV","transX_fisher",
    "pred_bsplineFisher","indexSampler_bspline_fisher","betaSampler_bspline_fisher",

    # bsSphere
    "transX_sp","estBetaInit_sp","computeSig","ComputeS","logdet_nim",
    "postll_bspline_sphere","postll_knots","betaFunction","computeA1_nu1",
    "computeA1_nu0","computeA2_add","computeA2_delete","rnormTrun","newKnots",
    "prop_add","prop_delete","rtruncnorm","pred_bsplineSphere",
    "nuSampler_bspline_sphere","indexSampler_bspline_sphere","knotsSampler_bspline_sphere",
    "betaSampler_bspline_sphere","sigma2Sampler_bspline_sphere",

    # gpPolar
    "alphaTheta","Xlinear","invcov","expcov_gpPolar","expcovTest_gpPolar",
    "obj_btt_theta","thetaPrior","pred_gpPolar","gibbsSampler_sigma2",
    "gibbsSampler_kappa","MH_thetaeta",

    # gpspike
    "expcov_gpSpike","expcovnn_gpSpike","computeA1","computeA2",
    "llFunLambda","llFunThetaV1","llFunThetaV2","transitionTheta",
    "multiplyMatrixByConstant","pred_gpSpike",
    "SamplingLambda_gp_spike","SamplingThetaV_gp_spike",
    "gibbsSigma2_gp_spike","gibbsGam_gp_spike",

    # gpSphere
    "expcov_gpSphere","expcovTest_gpSphere","conBeta","obj_btt",
    "obj_btt_EB","pred_gpSphere","indexSampler_gpSphere","sigma2Sampler_gpSphere",
    "optSampler",

    # utils
    "pred_fitted",
    # distribution
    "dvMFnim", "rvMFnim", "dunitSphere", "runitSphere","dKnotsSimple", "rKnotsSimple"

  )

  pkg <- "BayesSIM"
  ns <- asNamespace(pkg)
  list2env(mget(.fns, envir = ns, inherits = FALSE), envir = globalenv())

  suppressMessages(
    nimble::registerDistributions(list(
      dvMFnim = list(
        BUGSdist = "dvMFnim(theta)",
        types    = c("value = double(1)", "theta = double(1)"),
        discrete = FALSE
      )
    ), verbose = FALSE)
  )

  suppressMessages(
    nimble::registerDistributions(list(
      dKnotsSimple = list(
        BUGSdist = "dKnotsSimple(a, b, k, alpha)",
        types = c("value = double(1)", "a = double(0)", "b = double(0)", "k = double(0)",
                  "alpha = double(1)"),
        discrete = FALSE
      )
    ), verbose = FALSE)
  )

  suppressMessages(
    nimble::registerDistributions(list(
      dunitSphere = list(
        BUGSdist     = "dunitSphere(dim)",
        types        = c("value = double(1)",
                         "dim   = double(0)"),
        discrete     = FALSE
      )
    ), verbose = FALSE)

  )

  message("== Compiling model ==")
  suppressMessages(Cmodel <- nimble::compileNimble(model))
  message("== Compiling samplers ==")
  suppressMessages(Cmcmc  <- nimble::compileNimble(mcmc, project = model, resetFunctions = TRUE))
  return(list(model = Cmodel, sampler = Cmcmc))
}


#' Get nimbleModel and nimbleSampler Object from the Result of `compileModelAndMCMC`
#' @description
#' Return compiled nimble model object and a corresponding MCMC samplers.
#'
#'
#' @param object The result of `compileModelAndMCMC` function.
#'
#' @return
#' `getModel` returns compiled Nimble model object.
#' `getSampler` returns compiled Nimble sampler object, directly using in `runMCMC` function.
#'
#' @seealso
#' \code{\link[nimble]{nimbleModel}},
#' \code{\link[nimble]{configureMCMC}},
#' \code{\link[nimble]{buildMCMC}},
#' \code{\link[nimble]{compileNimble}},
#' \code{\link[nimble]{runMCMC}}
#'
#' @examples
#' \donttest{
#' simdata2 <- data.frame(DATA1$X, y = DATA1$y)
#'
#' # 1. One tool version
#' fit_one <- BayesSIM(y ~ ., data = simdata2,
#'                     niter = 5000, nburnin = 1000, nchain = 1)
#'
#'
#' # 2. Split version
#' models <- BayesSIM_setup(y ~ ., data = simdata2)
#' Ccompile <- compileModelAndMCMC(models)
#' nimSampler <- getSampler(Ccompile)
#' initList <- getInit(models)
#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
#'                     nchains = 1, setSeed = TRUE, inits = initList,
#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
#'
#' # "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
#' fit_split <- as_bsim(models, mcmc.out)
#'
#' }
#' @name getFunction
#' @export
getModel <- function(object){
  return(object$model)
}

#' @rdname getFunction
#' @export
getSampler <- function(object){
  return(object$sampler)
}

# tests/testthat/test-bayesim-generics.R
test_that("bsim and bsimSetup constructors create correct classes", {
  dummy <- bsim(
    coefficients = c(a = 1, b = 2),
    ses          = c(0.1, 0.2),
    residuals    = c(0.5, -0.5),
    fitted.values = c(1.5, 1.5),
    linear.predictors = c(0.2, 0.3),
    gof          = 0.25,
    input        = list(),
    samples      = NULL,
    model        = NULL,
    sampler      = NULL,
    modelName    = "bsFisher"
  )

  expect_s3_class(dummy, "bsim")
  expect_equal(dummy$modelName, "bsFisher")
  expect_true("coefficients" %in% names(dummy))

  setup <- bsimSetup(
    coefficients = NULL,
    ses          = NULL,
    residuals    = NULL,
    fitted.values = NULL,
    linear.predictors = NULL,
    gof          = NULL,
    input        = list(foo = 1),
    samples      = NULL,
    model        = "model",
    sampler      = "sampler",
    modelName    = "bsFisher"
  )

  expect_s3_class(setup, "bsimSetup")
  expect_equal(setup$input$foo, 1)
  expect_equal(setup$modelName, "bsFisher")
})

test_that("bsim.predict constructor builds bsimPred object", {
  obj <- bsim.predict(
    fitted   = data.frame(mean = 1:3),
    truey    = 1:3,
    idxValue = c(0.1, 0.2, 0.3),
    level    = 0.95,
    type = "response"
  )

  expect_s3_class(obj, "bsimPred")
  expect_equal(obj$level, 0.95)
  expect_equal(length(obj$idxValue), 3)
})

test_that("as_bsim combines setup and samples using sampleBind and bsimFit_pointest", {
  skip_if_not_installed("coda")

  # dummy setup object (bsimSetup-like)
  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  setup <- list(
    input = list(
      origdata = list(x = X, y = Y),
      samplingOptions = list(nchain = 1),
      formula = c("y", "~", "x1 + x2")
    ),
    defModel   = "model_stub",
    defSampler = "sampler_stub",
    modelName  = "bsFisher"
  )
  class(setup) <- "bsimSetup"

  # mcmc.out with "samples" element
  samp_mat <- cbind(
    "index[1]" = rnorm(10),
    "index[2]" = rnorm(10),
    "sigma2"   = rexp(10)
  )
  mcmc_samples <- coda::mcmc(samp_mat)
  mcmc_out <- list(samples = mcmc_samples)

  env_as <- environment(as_bsim)

  testthat::local_mocked_bindings(
    sampleBind = function(x, nchain) x,
    bsimFit_pointest = function(samples, X, Y) {
      list(
        coefficients       = c(index1 = 0.5, index2 = -0.5),
        ses_coef           = c(0.1, 0.2),
        se                 = c(0.1, 0.2),
        residuals          = as.numeric(Y) - mean(as.numeric(Y)),
        fitted.values      = rep(mean(as.numeric(Y)), length(Y)),
        linear.predictors  = X %*% c(0.5, -0.5),
        gof                = 0.123
      )
    },
    .env = env_as
  )

  fit <- as_bsim(setup, mcmc_out)

  expect_s3_class(fit, "bsim")
  expect_equal(fit$modelName, "bsFisher")
  expect_true(all(c("coefficients", "residuals", "fitted.values") %in% names(fit)))
  expect_true(inherits(fit$samples, "mcmc"))
})

test_that("print.bsim prints header and coefficients when samples exist", {
  skip_if_not_installed("coda")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  obj <- list(
    coefficients = c(index1 = 0.5, index2 = -0.5),
    ses_coef     = c(0.1, 0.2),
    residuals    = c(0.1, -0.1),
    fitted.values = c(1, 2),
    linear.predictors = c(0.3, 0.4),
    gof          = 0.25,
    input = list(
      origdata = list(x = X, y = Y),
      formula  = c("y", "~", "x1 + x2")
    ),
    samples   = coda::mcmc(matrix(rnorm(20), ncol = 2)),
    modelName = "bsFisher"
  )
  class(obj) <- "bsim"

  expect_output(
    print(obj),
    "BayesSIM model"
  )
  expect_output(
    print(obj),
    "Single-index estimates:"
  )
})


test_that("coef.bsim returns mean coefficients and SE when requested", {
  xData <- matrix(rnorm(4), ncol = 2)
  colnames(xData) <- c("X1", "X2")
  obj <- list(
    coefficients = c(X1 = 0.5, X2 = -0.5),
    ses_coef     = c(0.1, 0.2),
    input = list(
      origdata = list(x = xData),
      samplingOptions = list(nchain = 1)
    ),
    samples = NULL
  )
  class(obj) <- "bsim"

  # method = "mean"
  co <- coef.bsim(obj, method = "mean", se = FALSE)
  expect_equal(co, obj$coefficients)

  co_se <- coef.bsim(obj, method = "mean", se = TRUE)
  expect_true(is.matrix(co_se))
  expect_equal(rownames(co_se), c("Estimate (mean)", "SE"))
})

test_that("coef.bsim with median uses sampleBind and MAD", {
  skip_if_not_installed("coda")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")

  samp_mat <- cbind(
    "index[1]" = rnorm(20, 1),
    "index[2]" = rnorm(20, -1),
    "sigma2"   = rexp(20)
  )
  samples <- coda::mcmc(samp_mat)

  obj <- list(
    coefficients = c(index1 = 0.5, index2 = -0.5),
    ses_coef     = c(0.1, 0.2),
    input = list(
      origdata = list(x = X),
      samplingOptions = list(nchain = 1)
    ),
    samples = samples
  )
  class(obj) <- "bsim"

  env_coef <- environment(coef.bsim)
  testthat::local_mocked_bindings(
    sampleBind = function(x, nchain) x,
    .env = env_coef
  )

  med <- coef.bsim(obj, method = "median", se = FALSE)
  expect_equal(length(med), 2)
  expect_equal(names(med), colnames(X))
})


test_that("residuals.bsim returns mean and median residuals", {
  skip_if_not_installed("coda")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  samp_mat <- cbind(
    "linkFunction[1, 1]" = rnorm(20),
    "linkFunction[2, 1]" = rnorm(20)
  )
  samples <- coda::mcmc(samp_mat)

  obj <- list(
    residuals = c(0.1, -0.1),
    input = list(
      origdata = list(x = X, y = Y),
      samplingOptions = list(nchain = 1)
    ),
    samples = samples
  )
  class(obj) <- "bsim"

  env_res <- environment(residuals.bsim)
  testthat::local_mocked_bindings(
    sampleBind = function(x, nchain) x,
    .env = env_res
  )

  # mean
  r_mean <- residuals(obj, method = "mean")
  expect_equal(r_mean, obj$residuals)

  # median
  r_med <- residuals(obj, method = "median")
  expect_length(r_med, 2)
})

test_that("fitted.bsim works for response/linpred and mean/median", {
  skip_if_not_installed("coda")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  samp_mat <- cbind(
    "linkFunction[1, 1]" = rnorm(20),
    "linkFunction[2, 1]" = rnorm(20),
    "Xlin[1]"            = rnorm(20),
    "Xlin[2]"            = rnorm(20)
  )
  samples <- coda::mcmc(samp_mat)

  obj <- list(
    fitted.values     = c(1, 2),
    linear.predictors = c(0.3, 0.4),
    input = list(
      origdata = list(x = X, y = Y),
      samplingOptions = list(nchain = 1)
    ),
    samples   = samples
  )
  class(obj) <- "bsim"

  env_fit <- environment(fitted.bsim)
  testthat::local_mocked_bindings(
    sampleBind = function(x, nchain) x,
    .env = env_fit
  )

  # response, mean
  fr <- fitted(obj, type = "latent", method = "mean")
  expect_equal(fr, obj$fitted.values)

  # response, median
  fr_med <- fitted(obj, type = "latent", method = "median")
  expect_length(fr_med, 2)

  # linpred, mean
  fl <- fitted(obj, type = "linpred", method = "mean")
  expect_length(fl, 2)

  # linpred, median
  fl_med <- fitted(obj, type = "linpred", method = "median")
  expect_length(fl_med, 2)
})

test_that("summary.bsim produces summary.bsim object and print works", {
  skip_if_not_installed("coda")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  samp_mat <- cbind(
    "index[1]" = rnorm(40),
    "index[2]" = rnorm(40),
    "sigma2"   = rexp(40)
  )
  samples <- coda::mcmc(samp_mat)

  obj <- list(
    input = list(
      origdata = list(x = X, y = Y),
      samplingOptions = list(nchain = 1, monitors = NULL)
    ),
    samples   = samples,
    modelName = "bsFisher"
  )
  class(obj) <- "bsim"

  s <- summary(obj)
  expect_s3_class(s, "summary.bsim")
  expect_true("all.chain" %in% names(s))
  expect_true(is.data.frame(s$all.chain))

  expect_output(
    print(s),
    "Summary statistics of all chains"
  )
})

test_that("nimTraceplot returns a ggplot object", {
  skip_if_not_installed("coda")
  skip_if_not_installed("ggplot2")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  samp_mat <- cbind(
    "index[1]" = rnorm(40),
    "index[2]" = rnorm(40),
    "sigma2"   = rexp(40)
  )
  samples <- coda::mcmc(samp_mat)

  obj <- list(
    input = list(
      origdata = list(x = X, y = Y),
      samplingOptions = list(nchain = 1, monitors = NULL)
    ),
    samples   = samples,
    modelName = "bsFisher"
  )
  class(obj) <- "bsim"

  p <- nimTraceplot(obj)
  expect_s3_class(p, "ggplot")
})

test_that("plot.bsim calls predict.bsim and plot_bsim_fitted with correct interval flag", {
  skip_if_not_installed("ggplot2")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  obj <- list(
    input = list(
      origdata = list(x = X, y = Y),
      samplingOptions = list(nchain = 1, monitors = NULL),
      formula = c("y", "~", "x1 + x2")
    ),
    samples   = NULL,
    modelName = "bsFisher"
  )
  class(obj) <- "bsim"

  env_plot <- environment(plot.bsim)

  testthat::local_mocked_bindings(
    predict.bsim = function(object, ...) {
      bsim.predict(
        fitted   = data.frame(mean = c(1, 2)),
        truey    = c(1, 2),
        idxValue = c(0.1, 0.2),
        level    = 0.95,
        type = "response"
      )
    },
    plot_bsim_fitted = function(pred, interval) {
      list(pred = pred, interval = interval)
    },
    .env = env_plot
  )

  res1 <- plot(obj, method = "mean", interval = TRUE)
  expect_true(is.list(res1))
  expect_true(res1$interval)

  res2 <- plot(obj, method = "mean", interval = FALSE)
  expect_false(res2$interval)
})

test_that("predict.bsim with type = 'index' returns bsimPred object", {
  skip_if_not_installed("coda")
  skip_if_not_installed("nimble")

  X <- matrix(rnorm(4), ncol = 2)
  colnames(X) <- c("x1", "x2")
  Y <- matrix(rnorm(2), ncol = 1)

  samp_mat <- cbind(
    "index[1]" = rnorm(20),
    "index[2]" = rnorm(20),
    "Xlin[1]"  = rnorm(20),
    "Xlin[2]"  = rnorm(20),
    "sigma2"   = rexp(20)
  )
  samples <- coda::mcmc(samp_mat)

  obj <- list(
    input = list(
      origdata = list(x = X, y = Y),
      samplingOptions = list(nchain = 1, monitors = NULL),
      formula = c("y", "~", "x1 + x2"),
      prior   = list()
    ),
    samples   = samples,
    modelName = "bsFisher"
  )
  class(obj) <- "bsim"

  env_pred <- environment(predict.bsim)
  testthat::local_mocked_bindings(
    sampleBind = function(x, nchain) x,
    .env = env_pred
  )

  res <- predict.bsim(obj, type = "index", method = "mean",
                      interval = "none", level = 0.9)
  expect_s3_class(res, "bsimPred")
  expect_equal(length(res$idxValue), nrow(X))
})

# test_that("predict.bsim validates arguments", {
#   skip_if_not_installed("nimble")
#   skip_if_not_installed("coda")
#
#   X <- matrix(rnorm(4), ncol = 2)
#   colnames(X) <- c("x1", "x2")
#   Y <- matrix(rnorm(2), ncol = 1)
#
#   samp_mat <- cbind(
#     "index[1]" = rnorm(20),
#     "index[2]" = rnorm(20),
#     "Xlin[1]"  = rnorm(20),
#     "Xlin[2]"  = rnorm(20),
#     "sigma2"   = rexp(20)
#   )
#   samples <- coda::mcmc(samp_mat)
#
#   obj <- list(
#     input = list(
#       origdata = list(x = X, y = Y),
#       samplingOptions = list(nchain = 1, monitors = NULL),
#       formula = c("y", "~", "x1 + x2")
#     ),
#     samples   = samples,
#     modelName = "bsFisher"
#   )
#   class(obj) <- "bsim"
#
#   env_pred <- environment(predict.bsim)
#   testthat::local_mocked_bindings(
#     sampleBind = function(x, nchain) x,
#     .env = env_pred
#   )
#
#   expect_error(
#     predict.bsim(obj, newdata = matrix(1, 2, 2)),
#     "newdata should be data.frame."
#   )
# })

test_that("getVarMonitor returns correct name/list from expand_monitor", {
  obj <- list(
    defModel = list(
      getNodeNames = function(stochOnly = TRUE, includeData = FALSE) {
        c("index[1]", "index[2]", "sigma2")
      }
    )
  )

  env_getVar <- environment(getVarMonitor)
  testthat::local_mocked_bindings(
    expand_monitor = function(possibleList) {
      list(
        baseName = paste0(possibleList, "_base"),
        out      = paste0(possibleList, "_out")
      )
    },
    .env = env_getVar
  )

  # type = "name"
  nm <- getVarMonitor(obj, type = "name")
  expect_equal(nm, c("index[1]_base", "index[2]_base", "sigma2_base"))

  # type = "list"
  ls <- getVarMonitor(obj, type = "list")
  expect_equal(ls, c("index[1]_out", "index[2]_out", "sigma2_out"))
})

test_that("getModelDef returns BUGScode", {
  obj <- list(
    defModel = list(
      modelDef = list(
        BUGScode = "model { y ~ dnorm(0,1) }"
      )
    )
  )
  expect_equal(getModelDef(obj), "model { y ~ dnorm(0,1) }")
})

test_that("getInit returns initial list", {
  obj <- list(input = list(init = list(a = 1, b = 2)))
  expect_equal(getInit(obj), list(a = 1, b = 2))
})

test_that("compileModelAndMCMC compiles nimble model and sampler", {
  skip_if_not_installed("nimble")
  skip_on_cran()

  code <- nimble::nimbleCode({
    for (i in 1:N) {
      y[i] ~ dnorm(mu, 1)
    }
    mu ~ dnorm(0, 1)
  })

  const <- list(N = 3)
  dat   <- list(y = rnorm(3))
  inits <- list(mu = 0)

  model <- nimble::nimbleModel(code, constants = const, data = dat, inits = inits)
  conf  <- nimble::configureMCMC(model)
  mcmc  <- nimble::buildMCMC(conf)

  fullmodel <- list(defModel = model, defSampler = mcmc)
  res <- compileModelAndMCMC(fullmodel)

  expect_true("model" %in% names(res))
  expect_true("sampler" %in% names(res))
})

test_that("getModel and getSampler extract compiled objects", {
  obj <- list(
    model   = "compiled_model_stub",
    sampler = "compiled_sampler_stub"
  )

  expect_equal(getModel(obj), "compiled_model_stub")
  expect_equal(getSampler(obj), "compiled_sampler_stub")
})

# expcov: I + lambda^(-1)*K
expcov_gpSpike <- nimbleFunction(
  run = function(vec = double(1), invlambda= double(0)){
    returnType(double(2))
    n <- length(vec)
    result <- matrix(nrow = n, ncol = n, init = FALSE)
    for(i in 1:(n-1)){
      for(j in (i+1):n){
        result[i, j] <- exp(-(vec[i] - vec[j])^2) * invlambda
        result[j, i] <- result[i, j]
      }
      result[i, i] <- invlambda
    }
    result[n, n] <- invlambda
    result <- result + diag(rep(1, n))

    return(result)

  }
)


## 2. pred cov - inverse lambda kernel
expcovnn_gpSpike <- nimbleFunction(
  run = function(vec1 = double(1), vec2 = double(1), invlambda= double(0)){
    returnType(double(2))
    n <- length(vec1); k <- length(vec2)
    result <- matrix(nrow = n, ncol = k, init = FALSE)
    for(i in 1:n){
      for(j in 1:k){
        result[i, j] <- exp(-(vec1[i] - vec2[j])^2) * invlambda
      }
    }
    # result = (result + t(result))/2

    return(result)

  })



computeA1 <- nimbleFunction(
  run = function(y = double(2), expcov = double(2)){
    returnType(double(0))
    computeA1 <- t(y) %*% inverse(expcov) %*% y

    return(computeA1[1,1])
  }
)


computeA2 <- nimbleFunction(
  run = function(y = double(2), z = double(2), expcov = double(2)){
    returnType(double(0))
    invCov <- inverse(expcov)
    part1 <- t(y) %*% invCov %*% z
    part2 <- inverse(t(z) %*% invCov %*% z)
    part3 <- t(z) %*% invCov %*% y

    computeA2 <- part1 %*% part2 %*% part3
    result <- computeA2[1,1]
    return(result)
  }
)
## 2. lambda posterior log - likelihood
llFunLambda <- nimbleFunction(
  setup = function(model){
    a_sig <- model$getConstants()$a_sig
    b_sig <- model$getConstants()$b_sig
    a_lam <- model$getConstants()$a_lam
    b_lam <- model$getConstants()$b_lam
    N <- model$getConstants()$N
    # pz <- model$getConstants()$z
  },
  run = function(){
    returnType(double(0))
    invlambda <- model$invlambda # target
    # z <- model$Z
    y <- model$Y
    expcovtemp <- model$Ki
    sqrt_expcovtemp <- log(diag(chol(expcovtemp)))

    # matPart3 <- t(z) %*% inverse(expcovtemp) %*% z
    # sqrt_matPart3 <- log(diag(chol(matPart3)))


    part1 <- log(invlambda) + dgamma(invlambda, shape = a_lam, rate = b_lam, log = TRUE)
    part2 <- 0
    # part3 <- 0
    for (i in 1:N){
      part2 <- part2 + sqrt_expcovtemp[i]
    }
    # for (j in 1:pz){
    #   part3 <- part3 + sqrt_matPart3[j]
    # }
    part4 <- -(a_sig + (N)/2)*log(b_sig + 0.5*(computeA1(y, expcovtemp)))

    total <- part1 - part2 + part4

    # nimPrint(total)

    return(total[1])
  }
)

# 3. theta and delta posterior log - likelihood
llFunThetaV1 <- nimbleFunction(
  setup = function(model){
    p <- model$getConstants()$p
    a0 <- model$getConstants()$a0
    b0 <- model$getConstants()$b0
    a_sig <- model$getConstants()$a_sig
    b_sig <- model$getConstants()$b_sig
    a_lam <- model$getConstants()$a_lam
    b_lam <- model$getConstants()$b_lam
    theta_sig <- model$getConstants()$sigma_theta
    N <- model$getConstants()$N
    # pz <- model$getConstants()$z
  },
  run = function(idx = integer(0)){
    returnType(double(0))
    # z <- model$Z
    y <- model$Y
    vi <- model$nu; sumvi <- 0
    theta_raw <- model$index_raw
    for (i in 1:length(vi)){
      sumvi <- sumvi + vi[i]
    }
    expcovtemp <- model$Ki
    sqrt_expcovtemp <- log(diag(chol(expcovtemp)))
    # matPart3 <- t(z) %*% inverse(expcovtemp) %*% z
    # sqrt_matPart3 <- log(diag(chol(matPart3)))

    part1 <- lgamma(sumvi + a0)+lgamma(p-sumvi+b0)
    # print(part1)
    part2 <- dnorm(theta_raw[idx], 0, theta_sig, log = TRUE) * vi[idx]
    # print(part2)
    part3 <- 0
    # part4 <- 0
    for (i in 1:N){
      part3 <- part3 + sqrt_expcovtemp[i]
    }
    # print(part3)
    # for (j in 1:pz){
    #   part4 <- part4 + sqrt_matPart3[j]
    # }
    # print(part4)
    part5 <- -(a_sig + (N)*0.5)*log(b_sig + 0.5*(computeA1(y, expcovtemp)))
    # print(part5)
    total <- part1 + part2 - part3 + part5
    # total <- part2 - part3 - part4 + part5

    # nimPrint(total)

    return(total)
  }
)

llFunThetaV2 <- nimbleFunction(
  setup = function(model){
    p <- model$getConstants()$p
    a0 <- model$getConstants()$a0
    b0 <- model$getConstants()$b0
    a_sig <- model$getConstants()$a_sig
    b_sig <- model$getConstants()$b_sig
    a_lam <- model$getConstants()$a_lam
    b_lam <- model$getConstants()$b_lam
    theta_sig <- model$getConstants()$sigma_theta
    N <- model$getConstants()$N
    # pz <- model$getConstants()$z
  },
  run = function(idx = integer(0)){
    returnType(double(0))
    # z <- model$Z
    y <- model$Y
    vi <- model$nu; sumvi <- 0
    theta_raw <- model$index_raw
    for (i in 1:length(vi)){
      sumvi <- sumvi + vi[i]
    }
    expcovtemp <- model$Ki
    sqrt_expcovtemp <- log(diag(chol(expcovtemp)))
    # matPart3 <- t(z) %*% inverse(expcovtemp) %*% z
    # sqrt_matPart3 <- log(diag(chol(matPart3)))

    # part1 <- lgamma(sumvi + a0)+lgamma(p-sumvi+b0)
    part2 <- dnorm(theta_raw[idx], 0, theta_sig, log = TRUE)
    # print(part2)
    part3 <- 0
    # part4 <- 0
    for (i in 1:N){
      part3 <- part3 + sqrt_expcovtemp[i]
    }
    # print(part3)
    # for (j in 1:pz){
    #   part4 <- part4 + sqrt_matPart3[j]
    # }
    # print(part4)
    part5 <- -(a_sig + (N)*0.5)*log(b_sig + 0.5*(computeA1(y, expcovtemp)))
    # print(part5)
    # total <- part1 + part2 - part3 - part4 + part5
    total <- part2 - part3 + part5

    # nimPrint(total)

    return(total)
  }
)


# 4. transition kernel

transitionTheta <- nimbleFunction(
  run = function(theta = double(0), theta_sig = double(0), delta = integer(0), sumv = double(0)){
    returnType(double(0))
    part1 <- dnorm(theta, 0, theta_sig, log = TRUE) * delta
    part2 <- 0
    if (sumv == 0){
      part2 <- log(1)
    } else{
      part2 <- log(0.5)
    }

    total <- part1 + part2
    return(total)
  }
)


# 5. multiply scalar in matrix
multiplyMatrixByConstant <- nimbleFunction(
  run = function(mat = double(2), c = double(0)) {
    returnType(double(2))

    nrow <- nimDim(mat)[1]
    ncol <- nimDim(mat)[2]
    result <- matrix(0, nrow, ncol)

    for(i in 1:nrow){
      for(j in 1:ncol){
        result[i, j] <- mat[i, j] * c
      }
    }
    return(result)
  }
)

# Initial value functions
initfunction_gpSpike <- function(x, y, p,
                                 pi, nu, index, inv_lambda, sigma2,
                                 setSeed){
  if (setSeed != FALSE){
    set.seed(setSeed)
  }
 # Initialize
  ## pi, nu, theta
  init_pi <- pi
  if (length(pi) >= 2 || !is.numeric(pi)){
    stop("Error: Initial value of pi should be scalar.")
  }
  if (pi < 0){
    stop("Error: Initial value of pi should be positive.")
  }

  ## nu, theta
  if (!is.null(nu) & (length(nu) != p)){
    stop("Error: Incorrect dimension on initial value of nu.")
  }

  if (!is.null(nu) & (!all(nu %in% c(0, 1)))){
    stop("Error: Delta should consist of 0 or 1.")
  }

  if (is.null(nu) & is.null(index)){
    init_nu <- rbinom(p, 1 ,prob = init_pi)
  } else if (is.null(nu) & !is.null(index)){
    init_nu <- as.numeric(index!=0)
  } else{
    init_nu <- nu
  }


  if (!is.null(index) & (length(index) != p)){
    stop("Error: Incorrect dimension on initial value of index")
  }

  if (is.null(index)){
    nonzero <- sum(init_nu)
    nonzeroidx <- which(init_nu == 1)
    temp <- rtruncnorm(n = sum(nonzeroidx), 0, 1, lower = 0, upper = Inf)
    init_index <- rep(0, p)
    init_index[nonzeroidx] <-temp[nonzeroidx]
  } else{
    bad_zero  <- which(init_nu == 0 & index != 0)
    bad_nonzero <- which(init_nu == 1 & index == 0)

    if (length(bad_zero) > 0) {
      warning(sprintf(
        "Theta is nonzero at positions where nu = 0: %s",
        paste(bad_zero, collapse = ", ")
      ))
    }

    if (length(bad_nonzero) > 0) {
      warning(sprintf(
        "Theta is zero at positions where nu = 1: %s",
        paste(bad_nonzero, collapse = ", ")
      ))
    }

    init_index <- index * init_nu
  }

  if (sum(init_index) < 0){
    init_index <- (-1) * init_index
  }

  init_Xlin <- as.vector(x %*% matrix(init_index, nrow = p))

  ## inv_lambda
  init_invlambda <- ifelse(is.null(inv_lambda), exp(rnorm(1)*sqrt(1)/10), inv_lambda)

  if (length(init_invlambda) >= 2 || !is.numeric(init_invlambda)){
    stop("Error: Initial value of inverse lambda should be scalar.")
  }
  if ((init_invlambda < 0)){
    stop("Error: Initial value of inverse lambda should be positive.")
  }


  ## sigma2
  init_sigma2 <- ifelse(is.null(sigma2), 1, sigma2)
  if (length(init_sigma2) >= 2 || !is.numeric(init_sigma2)){
    stop("Error: Initial value of sigma2 should be scalar.")
  }
  if (init_sigma2 < 0){
    stop("Error: Initial value of sigma2 should be positive.")
  }

  init_Ki <- expcov_gpSpike(init_Xlin, init_invlambda)
  init_cov <- init_Ki * init_sigma2

  return(list(pi = init_pi, nu = init_nu,
              index_raw = init_index,
              indexstar = init_index,
              index = init_index,
              Xlin = init_Xlin,
              Ki = init_Ki, invlambda = init_invlambda,
              sigma2 = init_sigma2, cov = init_cov))
}


# Prediction
pred_gpSpike <- nimbleFunction(
  run = function(newdata = double(2), nsamp = integer(0), y = double(1),
                 indexSample = double(2),
                 XlinSample = double(2), sigma2_samples = double(1),
                 invlambdaSample = double(1), prediction = integer(0)){
    returnType(double(2))

    new_ncol <- nimDim(newdata)[1]
    orig_ncol <- length(y)

    testPred <- matrix(0, nrow = nsamp, ncol = new_ncol)
    for (i in 1:nsamp){
      currInvlambda <- invlambdaSample[i]
      currSigma <- sigma2_samples[i]
      # 1. compute index
      sampleZ <- newdata %*% matrix(indexSample[i, ], ncol = 1)

      # 2. compute covariance
      ## 1) C(ori, ori) - I + invlambda*Koo
      cov_ori_ori <- expcov_gpSpike(XlinSample[i,], currInvlambda)
      ## 2) C(new, ori) - Kno * invlambda
      cov_new_ori <- expcovnn_gpSpike(sampleZ[,1], XlinSample[i,], currInvlambda)
      ## 3) c(new, new) - knn * invlambda
      cov_new_new <- expcovnn_gpSpike(sampleZ[,1], sampleZ[,1], currInvlambda)

      # 3. compute mu, Sigma
      midMatrix <- inverse(cov_ori_ori)
      mu <- cov_new_ori %*% midMatrix %*% matrix(y, ncol = 1)
      Sigcov <- cov_new_new - cov_new_ori %*% midMatrix %*%  t(cov_new_ori)

      # 4. Sampling
      if (prediction == 1){ # latent
        cholpredcov <- chol(Sigcov)
        testPred[i,] <- t(rmnorm_chol(1, mean = mu[,1], cholesky = cholpredcov,
                                      prec_param  = FALSE))

      } else{ # response
        predcov <- Sigcov * currSigma + diag(rep(currSigma, new_ncol))
        cholpredcov <- chol(predcov)
        testPred[i,] <- t(rmnorm_chol(1, mean = mu[,1], cholesky = cholpredcov,
                                      prec_param  = FALSE))
      }


    }
    return(testPred)
  }
)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/initial-params.R
\name{init_param}
\alias{init_param}
\title{Build an Initial Value List for BayesSIM Models}
\usage{
init_param(
  indexprior,
  link,
  index = NULL,
  index_nu = NULL,
  index_psi = NULL,
  index_pi = NULL,
  link_beta = NULL,
  link_k = NULL,
  link_knots = NULL,
  link_lengthscale = NULL,
  link_amp = NULL,
  link_kappa = NULL,
  link_inv_lambda = NULL,
  sigma2 = NULL
)
}
\arguments{
\item{indexprior}{Character scalar indicating the prior for the index.
Typically one of \code{"fisher"}, \code{"sphere"}, \code{"polar"}, or \code{"spike"}.
The valid options mirror those used in the corresponding model functions.}

\item{link}{Character scalar indicating the link function family.
Typically \code{"bspline"} for B-spline link functions or \code{"gp"} for Gaussian
process link functions. The valid options mirror those used in the
corresponding model functions.}

\item{index, index_nu, index_psi, index_pi}{Optional initial values for index and related parameter values.}

\item{link_beta, link_k, link_knots, link_lengthscale, link_amp, link_kappa, link_inv_lambda}{Optional initial values for components under link functions.}

\item{sigma2}{Optional numeric scalar giving the initial value of \eqn{\sigma^2}.}
}
\value{
A nested list with components \code{index}, \code{link}, and
\code{sigma2}.
}
\description{
\code{init_param} is a convenience helper that constructs a nested initial value list
for a given combination of index vector and link function.
It starts from the model-specific default prior, and then overwrites only those components for
which the user supplies non-\code{NULL} arguments.

This allows users to modify selected hyper-parameters without having to know
or manually reconstruct the full nested prior list structure.
}
\details{
\code{init_param(indexprior, link)} can be used to obtain the random initial values
list for the requested combination of index prior and link function.
For any argument that is not \code{NULL}, the corresponding field in the nested prior list is overwritten.

The detailed meaning and recommended choices for each initial values depend
on the specific model, index vector and link function.
For those details, please refer to the documentation of the corresponding
model-fitting functions.
}
\examples{
## Default initial values for Fisher index + B-spline link:
i0 <- init_param("fisher", "bspline")

## Modify only a few initial values:
i1 <- init_param(
  indexprior = "fisher",
  link       = "bspline",
  index      = c(1, 0, 0),      # initial direction of the index
  link_beta  = rep(0, 21),      # initial values for spline coefficients
  sigma2     = 0.1              # initial value for sigma^2
)

## Example with GP link:
i2 <- init_param(
  indexprior        = "sphere",
  link              = "gp",
  link_lengthscale  = 0.2,      # initial GP length-scale
  link_amp          = 1.5,      # initial GP amplitude
  sigma2            = 1         # initial variance
)

}
\seealso{
\code{\link[=bsFisher]{bsFisher()}}, \code{\link[=bsSphere]{bsSphere()}}, \code{\link[=bsPolar]{bsPolar()}}, \code{\link[=bsSpike]{bsSpike()}},
\code{\link[=gpFisher]{gpFisher()}}, \code{\link[=gpSphere]{gpSphere()}}, \code{\link[=gpPolar]{gpPolar()}}, \code{\link[=gpPolarHigh]{gpPolarHigh()}}, \code{\link[=gpSpike]{gpSpike()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{plot.bsim}
\alias{plot.bsim}
\alias{plot.bsimPred}
\title{Plot Method for \code{BayesSIM}}
\usage{
\method{plot}{bsim}(x, method = c("mean", "median"), interval = TRUE, alpha = 0.95, ...)

\method{plot}{bsimPred}(x, ...)
}
\arguments{
\item{x}{A fitted object of \code{BayesSIM} or individual model.}

\item{method}{Character string specifying the summary used for the
posterior fitted values. Options are \code{"mean"} or \code{"median"}.
Default is \code{"mean"}.}

\item{interval}{A logical value indicating whether a credible interval is included in the fitted plot. Default is \code{TRUE}.}

\item{alpha}{Numeric value between 0 and 1 specifying the credible level.
By default, \code{alpha = 0.95} produces a 95\% credible interval.}

\item{...}{Additional arguments passed to underlying plotting functions.}
}
\value{
The output consists of two plots:
\enumerate{
\item \strong{Observed vs Predicted plot}: a diagnostic scatter plot comparing
actual outcomes with posterior fitted values to visually assess model fit.
\item \strong{Fitted curve plot}: posterior \eqn{\hat{y}} as a function of the
estimated single index, optionally with \eqn{100 \times \alpha} \% credible intervals.
}
}
\description{
Produce diagnostic plots for a fitted Bayesian single-index model.
}
\details{
The function internally calls \code{\link[=predict]{predict()}} on the fitted model object
to obtain posterior summaries of \eqn{\hat{y}}. Predicted value of \eqn{y} is \eqn{\hat{f}(X'\hat{\theta})}.
\itemize{
\item If \code{interval = TRUE}, the function requests posterior credible intervals
and overlays them on the fitted curve.
\item If \code{interval = FALSE}, only the posterior mean or median curve is drawn.
}
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

# 1. One tool version
fit_one <- BayesSIM(y ~ ., data = simdata2,
                    niter = 5000, nburnin = 1000, nchain = 1)

# Check median index vector estimates with standard errors
coef(fit_one, method = "median", se = TRUE)

# Fitted index values of median prediction
fitted(fit_one, type = "linpred", method = "median")

# Residuals of median prediction
residuals(fit_one, method = "median")

# Summary of the model
summary(fit_one)

# Convergence diagnostics
nimTraceplot(fit_one)

# Goodness of fit
GOF(fit_one)

# Fitted plot
plot(fit_one)

# Prediction with 95\% credible interval at new data
newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
plot(pred)


# 2. Split version
models <- BayesSIM_setup(y ~ ., data = simdata2)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)

# "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
fit_split <- as_bsim(models, mcmc.out)

}
}
\seealso{
\code{\link[=predict.bsim]{predict.bsim()}}, \code{\link[=summary.bsim]{summary.bsim()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpspike.R
\name{gpSpike}
\alias{gpSpike}
\alias{gpSpike_setup}
\title{Bayesian Single-Index Regression with Gaussian Process Link and Spike-and-Slab Prior}
\usage{
gpSpike(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

gpSpike_setup(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)
}
\arguments{
\item{formula}{an object of class \link{formula}. Interaction term is not allowed for single-index model.}

\item{data}{an data frame.}

\item{prior}{Optional named list of prior settings. Further descriptions are in \strong{Details} section.}

\item{init}{Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
Further descriptions are in \strong{Details} section.}

\item{monitors}{Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
then inspect the variable names stored in the model object using \link{getVarMonitor}.}

\item{niter}{Integer. Total MCMC iterations (default \code{10000}).}

\item{nburnin}{Integer. Burn-in iterations (default \code{1000}).}

\item{thin}{Integer. Thinning for monitors (default \code{1}).}

\item{nchain}{Integer. Number of MCMC chains (default \code{1}).}

\item{setSeed}{Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.}
}
\value{
A \code{list} typically containing:
\describe{
\item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
\item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
\item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
Return list of \code{model_setup} does not include it.}
\item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
\item{\code{defModel}}{Nimble model object.}
\item{\code{defSampler}}{Nimble sampler object.}
\item{\code{modelName}}{Name of the model.}
}
}
\description{
Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n},
where index vector \eqn{\theta} has a spike and slab prior and
the link \eqn{f(\cdot)} is represented by Gaussian process and the
}
\details{
\strong{Model} The single–index model is specified as \eqn{Y_i = f(X_i' \theta) + \epsilon_i},
where \eqn{\theta} is a p-dimensional index vector subject to a spike-and-slab
prior for variable selection. The link function \eqn{f(\cdot)} is modeled
using a Gaussian process prior with zero mean and squared exponential covariance
kernel \eqn{K(x_1, x_2) = \exp\{-\rho {(x_1 - x_2)^{T}\theta}^2\}},
where \eqn{\rho} determines the smoothness of \eqn{f}.
The covariance kernel is re-parameterized to \eqn{\exp\{-{(x_1 - x_2)^{T}\theta^{*}}^2\}} where
\eqn{\rho = ||\theta^{*}||} and
\eqn{\theta = ||\theta||^{-1}\theta^{*}}.
Therefore, \eqn{\theta^{*}} is sampled in MCMC.

\strong{Priors}
\itemize{
\item The variable selection indicator \eqn{\nu} has Beta–Bernoulli hierarchy prior. Beta hyper-prior on the Bernoulli–inclusion probability \eqn{w},
inducing \eqn{p(\nu) \propto \mathrm{Beta}(r_1 + n_\nu, r_2 + p - n_\nu)} where  \eqn{n_\nu = \Sigma_{i=1}^{p}I(\nu_i = 1)}.
\eqn{r_1, r_2} are shape and rate parameter of beta distribution.
\item Slab coefficients \eqn{\theta} have normal distribution with zero mean and \eqn{\sigma_\theta^2} variance.
\item GP precision \eqn{\lambda^{-1}} follows gamma distribution with shape parameter \eqn{a_\lambda}, and rate parameter \eqn{b_\lambda}.
\item Error precision \eqn{(\sigma^2)^{-1}} follows gamma distribution with shape parameter \eqn{a_\sigma}, and rate parameter \eqn{b_\sigma}.
}

\strong{Sampling} A random walk Metropolis algorithm is used to sample \eqn{\lambda^{-1}}
and a Metropolis-Hastings algorithm is used for the main parameters \eqn{(\theta^{*}, \nu)}.
The variance \eqn{\sigma^2} is directly sampled from posterior distribution.
\eqn{f} is not directly sampled by MCMC.

\strong{Prior hyper-parameters}
These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
\enumerate{
\item Index vector: \code{index_r1, index_r2} gives the shape and rate parameter of beta-binomial prior, respectively.
For slab prior, normal distribution with zero mean is assigned for selected variables \eqn{\theta}. \code{index_sigma_theta} is for variance of \eqn{\theta}, and it is assigned 0.25 by default.
\item Link function: Inverse gamma prior is assigned for hyper-parameters \eqn{\lambda^{-1}}
\code{link_inv_lambda_shape} is shape parameter and \code{link_inv_lambda_rate} is rate parameter of inverse gamma distribution.
(default \code{link_inv_lambda_shape = 1, link_inv_lambda_rate = 0.1})
\item Error variance (\code{sigma2}): An Inverse gamma prior is assigned to \eqn{\sigma^2}
where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
(default \code{sigma2_shape = 0.001, sigma2_rate = 100})
}

\strong{Initial values}
These are the initial values set in the function. You can define new values for each initial value in \link{init_param}
\enumerate{
\item Index vector:
\itemize{
\item{\code{index_pi}: Initial selecting variable probability. (default: \code{0.5})}
\item{\code{index_nu}: Initial vector of inclusion indicators . By default, each \code{index_nu} is randomly drawn by  \eqn{Bernoulli(1/2)}}
\item{\code{index}: Initial vector of index. By default, each element of index vector, which is chosen by indicator, is proposed by normal distribution.}
}
\item Link function: Initial scalar of lambda (\code{link_inv_lambda}) for covariance kernel of Gaussian process.
\item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{0.01})

}
}
\examples{
\donttest{
set.seed(123)
n <- 200; d <- 4
theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
f <- function(u) u^2 * exp(u)
sigma <- 0.5
X <- matrix(runif(n * d, -1, 1), nrow = n)
index_vals <- as.vector(X \%*\% theta)
y <- f(index_vals) + rnorm(n, 0, sigma)
simdata <- data.frame(x = X, y = y)
colnames(simdata) <- c(paste0("X", 1:4), "y")

# One tool version
fit1 <- gpSpike(y ~ ., data = simdata,
               niter = 5000, nburnin = 1000)

# Split version
models <- gpSpike_setup(y ~ ., data = simdata)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)
fit2 <- as_bsim(models, mcmc.out)
summary(fit2)
}

}
\references{
McGee, G., Wilson, A., Webster, T. F., & Coull, B. A. (2023).
Bayesian multiple index models for environmental mixtures.
\emph{Biometrics}, 79(1), 462-474.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{plot.bsimSetup}
\alias{plot.bsimSetup}
\title{Plot Method for \code{BayesSIM_setup}}
\usage{
\method{plot}{bsimSetup}(x, select = NULL, ...)
}
\arguments{
\item{x}{A fitted object of \code{BayesSIM} or individual model.}

\item{select}{A vector of index or name of variables in data.}

\item{...}{Additional arguments passed to underlying plotting functions.}
}
\value{
A \code{ggplot} object is returned invisibly, allowing further modification if needed.
}
\description{
Produce diagnostic plots for a raw data.
}
\details{
The function displays scatter plots of the response variable against
each predictor variable. Each panel includes a smoothed trend line
to help assess the marginal relationship before fitting the model.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

models <- BayesSIM_setup(y ~ ., data = simdata2)

plot(models, select = c("X1", "X2"))

}
}
\seealso{
\code{\link[=summary.bsimSetup]{summary.bsimSetup()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{summary.bsim}
\alias{summary.bsim}
\alias{print.summary.bsim}
\title{Summarize \code{BayesSIM}}
\usage{
\method{summary}{bsim}(object, ...)

\method{print}{summary.bsim}(x, digits = 3, ...)
}
\arguments{
\item{object}{A fitted object of \code{BayesSIM} or individual model.}

\item{...}{Further arguments passed.}

\item{x}{A summary output of \code{BayesSIM} or individual model.}

\item{digits}{The minimum number of significant digits to be printed.}
}
\value{
The function summarizes posterior MCMC samples by reporting key statistics, including:
\itemize{
\item Posterior mean and median
\item Empirical standard deviation
\item 95\% credible interval (lower and upper quantiles)
\item Potential scale reduction factor (\code{Rhat}) for multiple chains
\item Effective sample size (\code{ESS})
}

By default, the index vector and error variance are only included in the summary.
If variable selection methods are used, such as uniform sphere and spike-and-slab prior,
the indicator vector (\code{nu}) is also included.
Note that the potential scale reduction factor for \code{nu} can be reported as
\code{NaN} or \code{Inf}, since the indicator rarely changes during the MCMC run.

If the model is fitted with single chain, both \code{all.chain} and \code{chain} have identical information.
}
\description{
Provides a \code{summary} for \code{BayesSIM}.
}
\details{
A \code{list} of summary statistics for MCMC samples, including \code{data.frame} table for the results.
Each row corresponds to a model parameter, and columns report the statistics.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

# 1. One tool version
fit_one <- BayesSIM(y ~ ., data = simdata2,
                    niter = 5000, nburnin = 1000, nchain = 1)

# Check median index vector estimates with standard errors
coef(fit_one, method = "median", se = TRUE)

# Fitted index values of median prediction
fitted(fit_one, type = "linpred", method = "median")

# Residuals of median prediction
residuals(fit_one, method = "median")

# Summary of the model
summary(fit_one)

# Convergence diagnostics
nimTraceplot(fit_one)

# Goodness of fit
GOF(fit_one)

# Fitted plot
plot(fit_one)

# Prediction with 95\% credible interval at new data
newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
plot(pred)


# 2. Split version
models <- BayesSIM_setup(y ~ ., data = simdata2)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)

# "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
fit_split <- as_bsim(models, mcmc.out)

}

}
\seealso{
\code{\link[coda]{gelman.diag}}, \code{\link[coda]{effectiveSize}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{getModelDef}
\alias{getModelDef}
\title{Get Definition of the Model}
\usage{
getModelDef(object)
}
\arguments{
\item{object}{A fitted object of \code{BayesSIM} or individual model.}
}
\value{
BUGS code of the model definition.
}
\description{
Functions for identifying definition of the \pkg{nimble} model.
}
\details{
The function that contain Bayes SIM model structure in \pkg{nimble}.
This function is for advanced users.
There are several functions used in the model definition.
\itemize{
\item \code{transX_fisher}, \code{transX_sp}: Making B-spline basis.
\item \code{dvMFnim}: Distribution of von Mises-Fisher.
\item \code{dKnotsSimple}: Distribution of the free knots for \code{bsSphere}.
\item \code{dunitSphere}: Distribution of unit sphere.
\item \code{alphaTheta}: One-to-one polar transformation. Making index vector from individual angular vector \code{psi}.
\item \code{expcov_gpSphere}, \code{expcov_gpPolar}, \code{expcov_gpSpike}: Covariance kernel of each model.
\code{expcov_gpSphere} uses squared-exponential kernel, \code{expcov_gpPolar} uses OU process kernel, and \code{expcov_gpSpike} uses squared-exponential including its own parameter, \eqn{\lambda^{-1}}.
\item \code{Xlinear}: Making linear combination with index vector.
}
}
\examples{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)
models <- bsFisher_setup(y ~ ., data = simdata2)
# Get model definition
getModelDef(models)

}
\seealso{
\code{\link{getVarMonitor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{as_bsim}
\alias{as_bsim}
\alias{as_bsim.bsimSetup}
\title{Construct a Fitted Model Object from Model Setup and MCMC Output}
\usage{
as_bsim(object, mcmc.out, ...)

\method{as_bsim}{bsimSetup}(object, mcmc.out, ...)
}
\arguments{
\item{object}{A \code{BayesSIM} setup object, typically the output of a
\code{_setup} function.}

\item{mcmc.out}{MCMC output corresponding to the result of a call to \code{runMCMC()}.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
An object of class \code{"bsim"} containing posterior samples,
point estimates, fitted values, and related model information.
}
\description{
Create a fitted \code{bsim} object by combining a \code{BayesSIM}
setup object with MCMC samples returned by \code{runMCMC()}.
}
\details{
This function is mainly intended for workflows where the model structure
and the MCMC sampling are performed separately. It collects the MCMC draws across chains, and returns an object of class \code{"bsim"}
that can be used with generic functions such as \code{summary()}, \code{plot()}, and \code{predict()}.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)
models <- BayesSIM_setup(y ~ ., data = simdata2)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                   nchains = 1, setSeed = TRUE, inits = initList,
                   summary = TRUE, samplesAsCodaMCMC = TRUE)
fit2 <- as_bsim(models, mcmc.out)
summary(fit2)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{getVarMonitor}
\alias{getVarMonitor}
\title{Retrieve Monitorable Variables}
\usage{
getVarMonitor(object, type = c("name", "list"))
}
\arguments{
\item{object}{A fitted object of \code{BayesSIM}, \code{BayesSIM_setup} or individual model.}

\item{type}{Options for variables. By default, \code{type = "name"} is used that it only prints the name of the node.
If you put name of the nodes, the MCMC outputs gave you all elements of the variable, in case of the vector.
If \code{type = "list"}, the dimension of the nodes are printed. If you put name and dimension of the nodes, only specific location of vector or matrix can be seen in \code{summary} or \code{nimTraceplot}.}
}
\value{
A vector of variables that can be monitored in the model.
}
\description{
Functions for retrieving the variables that can be monitored.
}
\details{
The function returns a list of variables that can be used in \code{monitors2} in the \code{bayesSIM} function.
You can also refer to \code{\link{getModelDef}} to understand the model structure and designate necessary variables.
Stochastic nodes of the model are recommended to be monitored.
}
\examples{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)
models <- BayesSIM_setup(y ~ ., data = simdata2)

# Get monitorable variables
getVarMonitor(models)
# Get the list of variables with dimension
getVarMonitor(models, type = "list")

}
\seealso{
\code{\link{getModelDef}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{getInit}
\alias{getInit}
\title{Get Initial Value of the Model}
\usage{
getInit(object)
}
\arguments{
\item{object}{A fitted object of \code{BayesSIM}, \code{BayesSIM_setup} or individual model.}
}
\value{
BUGS code of the model definition.
}
\description{
Functions for getting list of initial values of the \pkg{nimble} model.
}
\details{
The list of initial values are returned. This can be helpful to use when you use \code{BayesSIM_setup}.
You should be aware of that if you want to get more than 1 chain of MCMC samples, you should change \code{nchain} argument in \code{BayesSIM_setup}.
The output of initial values would be different, depending on the number of chain.

You can apply \code{BayesSIM} object when you want to check the list of initial values.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)


# Split version
models <- BayesSIM_setup(y ~ ., data = simdata2)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)

# "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
fit_split <- as_bsim(models, mcmc.out)

}


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{summary.bsimSetup}
\alias{summary.bsimSetup}
\alias{print.summary.bsimSetup}
\title{Summarize \code{BayesSIM_setup}}
\usage{
\method{summary}{bsimSetup}(object, ...)

\method{print}{summary.bsimSetup}(x, digits = 1, ...)
}
\arguments{
\item{object}{A fitted object of \code{BayesSIM_setup}.}

\item{...}{Further arguments passed.}

\item{x}{A summary output of \code{BayesSIM_setup}.}

\item{digits}{The minimum number of significant digits to be printed.}
}
\value{
An object of class \code{summary.bsimSetup}, which is a list containing:
\item{modelName}{The name of the fitted model.}
\item{n}{The number of observations.}
\item{p}{The number of predictors.}
\item{prior}{A list of prior settings used in the model.}
\item{samplingOptions}{A list of MCMC sampling options, including \code{niter}, \code{nburnin}, \code{thin}, and \code{nchain}.}
\item{model}{The compiled nimble model object.}
}
\description{
Provides a \code{summary} for \code{BayesSIM_setup}.
}
\details{
Summarize the model setup information of an object created by
a setup function (e.g., \code{\link{BayesSIM_setup}}),
including the model name, data dimensions, prior settings, and sampling options.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

models <- BayesSIM_setup(y ~ ., data = simdata2)
summary(models)

Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)

# "fit_split" becomes the class of "bsim".
fit_split <- as_bsim(models, mcmc.out)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpPolar.R, R/gpPolarTemp.R
\name{gpPolar}
\alias{gpPolar}
\alias{gpPolar_setup}
\alias{gpPolarHigh}
\alias{gpPolarHigh_setup}
\title{Bayesian Single-Index Regression with Gaussian Process Link and One-to-One Polar Transformation}
\usage{
gpPolar(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

gpPolar_setup(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

gpPolarHigh(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

gpPolarHigh_setup(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)
}
\arguments{
\item{formula}{an object of class \link{formula}. Interaction term is not allowed for single-index model.}

\item{data}{an data frame.}

\item{prior}{Optional named list of prior settings. Further descriptions are in \strong{Details} section.}

\item{init}{Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
Further descriptions are in \strong{Details} section.}

\item{monitors}{Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
then inspect the variable names stored in the model object using \link{getVarMonitor}.}

\item{niter}{Integer. Total MCMC iterations (default \code{10000}).}

\item{nburnin}{Integer. Burn-in iterations (default \code{1000}).}

\item{thin}{Integer. Thinning for monitors (default \code{1}).}

\item{nchain}{Integer. Number of MCMC chains (default \code{1}).}

\item{setSeed}{Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.}
}
\value{
A \code{list} typically containing:
\describe{
\item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
\item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
\item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
Return list of \code{model_setup} does not include it.}
\item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
\item{\code{defModel}}{Nimble model object.}
\item{\code{defSampler}}{Nimble sampler object.}
\item{\code{modelName}}{Name of the model.}
}
}
\description{
Fits a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}, where
the index \eqn{\theta} is specified and computed via a one-to-one polar
transformation, and the link \eqn{f(\cdot)} is represented with a Gaussian process.
}
\details{
\strong{Model} The single–index model is specified as \eqn{Y_i = f(X_i'{\theta}) + \epsilon_i},
where the index vector \eqn{\theta} lies on the unit sphere with (\eqn{\|\theta\|_2=1}) with non-zero first component
to ensure identifiability and is parameterized via a one-to-one polar transformation with angle \eqn{\psi_1,...,\psi_{p-1}}.

The mapping is
\deqn{
\begin{aligned}
\theta_1 &= \sin(\psi_1),\\
\theta_i &= \Big(\prod_{j=1}^{i-1}\cos(\psi_j)\Big)\sin(\psi_i), \quad i=2,\dots,p-1,\\
\theta_p &= \prod_{j=1}^{p-1}\cos(\psi_j).
\end{aligned}
}
The vector is then scaled to unit length.

Sampling is  performed on the angular parameters \eqn{\theta} defining
the index vector. The link function \eqn{f(\cdot)} is modeled by a Gaussian process
prior with zero mean and an Ornstein–Uhlenbeck (OU) covariance kernel
\eqn{\exp(-\kappa \cdot |t_i - t_j|), i, j = 1,\ldots, n}, where \eqn{\kappa} is a bandwidth (smoothness)
parameter and \eqn{t_i, t_j} is index value (\eqn{t_i = X_i'\theta}).

\strong{Priors}
\itemize{
\item \eqn{\psi} is \eqn{p-1} dimension of polar angle of index vector and re-scaled Beta distribution on \eqn{[0, \pi]} is assigned for prior.
\item Prior for \eqn{\kappa} (bandwidth parameter) is discrete uniform of equally spaced grid points in \eqn{[\kappa_{\text{min}}, \kappa_{\text{max}}}].
\item Inverse gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.

}

\strong{Sampling} For \code{gpPolar}, \eqn{\theta} is sampled by Metropolis-Hastings and updated with \eqn{f},
\eqn{\kappa} is chosen by grid search method that maximizes likelihood,
\eqn{\sigma^2} are sampled from their full conditional
distributions using Gibbs sampling.
Since \eqn{\kappa} is sampled by grid search, more than 5 dimension of variables \code{gpPolarHigh} is recommended.
For \code{gpPolarHigh}, all sampling parameters' samplers are assigned by nimble.

\strong{Prior hyper-parameters}
These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
\enumerate{
\item Index vector: Only shape parameter \code{index_psi_alpha} of \eqn{p-1} dimension vector is needed since rate parameters is computed to satisfy \eqn{\theta_{j, \text{MAP}}}.
By default, the shape parameter for each element of the polar vector is set to \code{5000}.

\item Link function:
\code{link_kappa_min} is minimum value of kappa (default \code{0.5}), \code{link_kappa_max} is maximum value of kappa (default \code{4}),
and \code{link_kappa_grid_width} is space (default \code{0.1}).
\item Error variance (\code{sigma2}): An Inverse gamma prior is assigned to \eqn{\sigma^2}
where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
(default \code{sigma2_shape = 2, sigma2_rate = 0.01})
}

\strong{Initial values}
These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
\enumerate{

\item Index vector: Initial vector of polar angle \code{index_psi} with \eqn{p-1} dimension. Each element of angle is between 0 and \eqn{\pi}.
\item Link function: Initial scalar scale parameter of covariance kernel \code{link_kappa}. (default: \code{2})
\item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{0.01})
}
}
\examples{
\donttest{
library(MASS)
N <- 100    # Sample Size
p <- 3
mu <- c(0,0,0)
rho <- 0
Cx <- rbind(c(1,rho,rho), c(rho,1,rho), c(rho, rho,1))
X <- mvrnorm(n = N, mu=mu, Sigma=Cx, tol=1e-8)
alpha <- c(1,1,1)
alpha <- alpha/sqrt(sum(alpha^2))
z <- matrix(0,N)
z <- X \%*\% alpha
z <- z[,1]
sigma <- 0.3
f <- exp(z)
y <- f + rnorm(N, 0, sd=sigma) # adding noise
y <- y-mean(y)
f_all <- f
x_all <- X
y_all <- y
simdata <- cbind(x_all, y, f)
simdata <- as.data.frame(simdata)
colnames(simdata) = c('x1', 'x2', 'x3', 'y','f')

prior_gpPolar <- prior_param(indexprior = "polar", link = "gp",
                            link_kappa_max = 2, link_kappa_grid_width = 0.05)

# One tool version
fit1 <- gpPolar(y ~ x1 + x2 + x3, data = simdata,
                prior = prior_gpPolar,
                niter = 3000, nburnin = 2000, nchain = 1)
fit2 <- gpPolarHigh(y ~ x1 + x2 + x3, data = simdata,
                    niter = 3000, nburnin = 2000, nchain = 1)

# Split version
models1 <- gpPolar_setup(y ~ x1 + x2 + x3, data = simdata,
                         prior = prior_gpPolar)
models2 <- gpPolarHigh_setup(y ~ x1 + x2 + x3, data = simdata,
                         prior = prior_gpPolar)
Ccompile1 <- compileModelAndMCMC(models1)
Ccompile2 <- compileModelAndMCMC(models2)
sampler1 <- getSampler(Ccompile1)
sampler2 <- getSampler(Ccompile2)
initList1 <- getInit(models1)
initList2 <- getInit(models2)
mcmc.out1 <- runMCMC(sampler1, niter = 3000, nburnin = 2000, thin = 1,
                    nchains = 1, setSeed = TRUE, init = initList1,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)
mcmc.out2 <- runMCMC(sampler2, niter = 3000, nburnin = 2000, thin = 1,
                    nchains = 1, setSeed = TRUE, init = initList2,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)
fit1_split <- as_bsim(models1, mcmc.out1)
fit2_split <- as_bsim(models2, mcmc.out2)
}

}
\references{
Dhara, K., Lipsitz, S., Pati, D., & Sinha, D. (2019). A new Bayesian single index model with or without covariates missing at random.
\emph{Bayesian analysis}, 15(3), 759.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{summary.bsimPred}
\alias{summary.bsimPred}
\alias{print.summary.bsimPred}
\title{Summarize Predictions}
\usage{
\method{summary}{bsimPred}(object, ...)

\method{print}{summary.bsimPred}(x, digits = 3, ...)
}
\arguments{
\item{object}{An object of class "bsimPred" which is the result of \code{predict}.}

\item{...}{Further arguments passed.}

\item{x}{A summary output of class "bsimPred".}

\item{digits}{The minimum number of significant digits to be printed.}
}
\value{
Metrics are including RMSE, MAE, Mean bias, and coverage. In terms of coverage, it is only available
when the "bsimPred" includes interval, corresponding to \code{interval = "credible"} options for \code{predict} function.
It indicates mean coverage rate of data points whether confidence intervals include true response values.
In this case, all chains are combined for the computation.
}
\description{
Provides a \code{summary} for \code{BayesSIM} prediction.
}
\details{
Performance of prediction with point predicted values.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

fit_one <- BayesSIM(y ~ ., data = simdata2,
                    niter = 5000, nburnin = 1000, nchain = 1)

# Prediction with 95\% credible interval at new data
newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
summary(pred)
plot(pred)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpFisher.R
\name{gpFisher}
\alias{gpFisher}
\alias{gpFisher_setup}
\title{Bayesian Single-Index Regression with Gaussian Process Link and von Mises-Fisher Prior}
\usage{
gpFisher(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

gpFisher_setup(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)
}
\arguments{
\item{formula}{an object of class \link{formula}. Interaction term is not allowed for single-index model.}

\item{data}{an data frame.}

\item{prior}{Optional named list of prior settings. Further descriptions are in \strong{Details} section.}

\item{init}{Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
Further descriptions are in \strong{Details} section.}

\item{monitors}{Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
then inspect the variable names stored in the model object using \link{getVarMonitor}.}

\item{niter}{Integer. Total MCMC iterations (default \code{10000}).}

\item{nburnin}{Integer. Burn-in iterations (default \code{1000}).}

\item{thin}{Integer. Thinning for monitors (default \code{1}).}

\item{nchain}{Integer. Number of MCMC chains (default \code{1}).}

\item{setSeed}{Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.}
}
\value{
A \code{list} typically containing:
\describe{
\item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
\item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
\item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
Return list of \code{model_setup} does not include it.}
\item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
\item{\code{defModel}}{Nimble model object.}
\item{\code{defSampler}}{Nimble sampler object.}
\item{\code{modelName}}{Name of the model.}
}
}
\description{
Fits a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}, where
the index \eqn{\theta} lies on the unit sphere with von Mises-Fisher prior, and the link \eqn{f(\cdot)} is represented
with Gaussian process.
}
\details{
\strong{Model} The single-index model uses Gaussian process with zero mean and and covariance kernel \eqn{\eta \cdot \text{exp}(-\frac{(t_i-t_j)^2}{l})} as a link function, where \eqn{t_i, t_j, j = 1, \ldots, n} is index value.
Index vector should be length 1.

\strong{Priors}
\itemize{
\item von Mises–Fisher prior on the index \eqn{\theta} with direction and concentration.
\item Covariance kernel: Amplitude, \eqn{\eta}, follows log normal distribution with mean \eqn{a_\eta} and variance \eqn{b_\eta}.
Length-scale parameter follows gamma distribution with shape parameter \eqn{\alpha_l} and rate parameter \eqn{\beta_l}.
\item Inverse gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.

}

\strong{Sampling} All sampling parameters' samplers are assigned by nimble.

\strong{Prior hyper-parameters}
These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
\enumerate{
\item Index vector: von Mises--Fisher prior for the projection vector \eqn{\theta} (\code{index}).
\code{index_direction} is a unit direction vector of the von Mises--Fisher distribution.
By default, the estimated vector from projection pursuit regression is assigned.
\code{index_dispersion} is the positive concentration parameter. By default, \code{150} is assigned.

\item Link function:
\itemize{
\item{Length-scale:Gamma distribution is assigned for length-scale parameter, \eqn{l}.
\code{link_lengthscale_shape} is shape parameter (default \code{1/8}) and \code{link_lengthscale_rate} is rate parameter of \code{lengthscale}. (default \code{1/8})}
\item{Amplitude: Log-normal distribution is assigned for amplitude parameter, \eqn{\eta}.
\code{link_amp_a} is mean (default \code{-1}), and \code{link_amp_b} is variance. (default \code{1})}
}

\item Error variance (\code{sigma2}): An inverse-gamma prior is assigned to \eqn{\sigma^2}
where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
(default \code{sigma2_shape = 1, sigma2_rate = 1})
}

\strong{Initial values}
These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
\enumerate{
\item Index vector (\code{index}): Initial unit index vector \eqn{\theta}. By default, the vector is sampled from the von Mises--Fisher prior.
\item Link function: \code{link_lengthscale} is initial scalar length-scale parameter. (default: \code{0.1})
\code{link_amp} is initial scalar amplitude parameter. (default: \code{1})
\item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{1})
}
}
\examples{
\donttest{
set.seed(123)
N <- 60; p <- 2
x1 <- runif(N, -3, 5)
x2 <- runif(N, -3, 5)
beta1 <- 0.45; beta2 <- sqrt(1-beta1^2)
sigma <- sqrt(0.0036)
xlin <- x1*beta1 + x2*beta2
eta <- 0.1*xlin + sin(0.5*xlin)^2
y <- rnorm(N, eta, sigma)
x <- matrix(c(x1, x2), ncol = 2)
simdata <- data.frame(x = x, y = y)
colnames(simdata) <- c("X1", "X2", "y")

# One tool version
fit1 <- gpFisher(y ~ ., data = simdata, nchain = 1, niter = 1000, nburnin = 100)

# Split version
models <- gpFisher_setup(y ~ ., data = simdata, nchain = 1)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 1000, nburnin = 100, thin = 1,
                   nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)
fit2 <- as_bsim(models, mcmc.out)
summary(fit2)
}

}
\references{
Antoniadis, A., Grégoire, G., & McKeague, I. W. (2004).
Bayesian estimation in single-index models. \emph{Statistica Sinica}, 1147-1164.

Choi, T., Shi, J. Q., & Wang, B. (2011).
A Gaussian process regression approach to a single-index model.
\emph{Journal of Nonparametric Statistics}, 23(1), 21-36.

Hornik, K., & Grün, B. (2014). movMF: an R package for fitting mixtures of von Mises-Fisher distributions.
\emph{Journal of Statistical Software}, 58, 1-31.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{compileModelAndMCMC}
\alias{compileModelAndMCMC}
\title{Compile a 'nimble' Model and Its MCMC}
\usage{
compileModelAndMCMC(object)
}
\arguments{
\item{object}{Class \code{BayesSIM_setup} object}
}
\value{
A list with two elements:
\describe{
\item{\code{model}}{the compiled NIMBLE model (external pointer object).}
\item{\code{mcmc}}{the compiled MCMC function/algorithm bound to the model.}
}
}
\description{
Compiles a nimble model object and a corresponding (uncompiled) MCMC
algorithm and returns the compiled pair.
}
\details{
The function first compiles \code{nimble} model object, then compiles \code{nimble} sampler.
Both compiled model and compiled MCMC samplers are returned as a list.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

# 1. One tool version
fit_one <- BayesSIM(y ~ ., data = simdata2,
                    niter = 5000, nburnin = 1000, nchain = 1)


# 2. Split version
models <- BayesSIM_setup(y ~ ., data = simdata2)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)

# "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
fit_split <- as_bsim(models, mcmc.out)

}

}
\seealso{
\code{\link[nimble]{nimbleModel}},
\code{\link[nimble]{configureMCMC}},
\code{\link[nimble]{buildMCMC}},
\code{\link[nimble]{compileNimble}},
\code{\link[nimble]{runMCMC}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior-params.R
\name{prior_param}
\alias{prior_param}
\title{Build a Prior List for BayesSIM Models}
\usage{
prior_param(
  indexprior,
  link,
  index_direction = NULL,
  index_dispersion = NULL,
  index_nu_r1 = NULL,
  index_nu_r2 = NULL,
  index_psi_alpha = NULL,
  index_sigma_theta = NULL,
  index_r1 = NULL,
  index_r2 = NULL,
  link_basis_df = NULL,
  link_basis_degree = NULL,
  link_basis_delta = NULL,
  link_knots_lambda_k = NULL,
  link_knots_maxknots = NULL,
  link_beta_mu = NULL,
  link_beta_cov = NULL,
  link_beta_tau = NULL,
  link_beta_Sigma0 = NULL,
  link_lengthscale_shape = NULL,
  link_lengthscale_rate = NULL,
  link_amp_a = NULL,
  link_amp_b = NULL,
  link_kappa_min = NULL,
  link_kappa_max = NULL,
  link_kappa_grid_width = NULL,
  link_inv_lambda_shape = NULL,
  link_inv_lambda_rate = NULL,
  sigma2_shape = NULL,
  sigma2_rate = NULL
)
}
\arguments{
\item{indexprior}{Character scalar indicating the prior for the index.
Typically one of \code{"fisher"}, \code{"sphere"}, \code{"polar"}, or \code{"spike"}.
The valid options mirror those used in the corresponding model functions.}

\item{link}{Character scalar indicating the link function family.
Typically \code{"bspline"} for B-spline link functions or \code{"gp"} for Gaussian
process link functions. The valid options mirror those used in the
corresponding model functions.}

\item{index_direction, index_dispersion, index_nu_r1, index_nu_r2, index_psi_alpha, index_sigma_theta, index_r1, index_r2}{Optional overrides for hyper-parameters related to the index prior.}

\item{link_basis_df, link_basis_degree, link_basis_delta}{Optional overrides for the B-spline basis hyper-parameters, such as the effective degrees of freedom,
spline degree, and penalty parameter.}

\item{link_knots_lambda_k, link_knots_maxknots}{Optional overrides for the B-spline knot-selection hyper-parameters in, used for models with adaptive knot placement.}

\item{link_beta_mu, link_beta_cov, link_beta_tau, link_beta_Sigma0}{Optional overrides for the prior on spline coefficients. The detailed interpretation of these
hyper-parameters depends on the specific model and is described in the
documentation of each model-fitting function.}

\item{link_lengthscale_shape, link_lengthscale_rate}{Optional overrides for the hyper-parameters of the GP length-scale prior.}

\item{link_amp_a, link_amp_b}{Optional overrides for the hyper-parameters of the GP amplitude prior.}

\item{link_kappa_min, link_kappa_max, link_kappa_grid_width}{Optional overrides for the hyper-parameters in used in models with polar index and GP-type link,
to control the grid or support for the concentration parameter \eqn{\kappa} in \code{gpPolar}.}

\item{link_inv_lambda_shape, link_inv_lambda_rate}{Optional overrides for spike-and-slab–type GP link priors.}

\item{sigma2_shape, sigma2_rate}{Optional overrides for the inverse-gamma prior on the observation
variance \eqn{\sigma^2}.}
}
\value{
A nested list with top-level elements \code{index}, \code{link}, and
\code{sigma2}, suitable for passing to the \code{prior} argument of the
various BayesSIM model fitting functions.
}
\description{
\code{prior_param} is a convenience helper that constructs a nested prior list
for a given combination of index prior and link function.
It starts from the model-specific default prior, and then overwrites only those components for
which the user supplies non-\code{NULL} arguments.

This allows users to modify selected hyper-parameters without having to know
or manually reconstruct the full nested prior list structure.
}
\details{
\code{prior_param(indexprior, link)} can be used to obtain the default prior
list for the requested combination of index prior and link function.
For any argument that is not \code{NULL}, the corresponding field in the nested prior list is overwritten.

The detailed meaning and recommended choices for each hyper-parameter depend
on the specific model, prior of index vector and link function.
For those details, please refer to the documentation of the corresponding
model-fitting functions.
}
\examples{
## Default prior for Fisher index + B-spline link:
p0 <- prior_param("fisher", "bspline")

## Modify only a few hyper-parameters:
p1 <- prior_param(
  indexprior          = "fisher",
  link                = "bspline",
  sigma2_shape        = 0.5,
  link_basis_df       = 30,
  index_direction     = c(1, 0, 0)
)
}
\seealso{
\code{\link[=bsFisher]{bsFisher()}}, \code{\link[=bsSphere]{bsSphere()}}, \code{\link[=bsPolar]{bsPolar()}}, \code{\link[=bsSpike]{bsSpike()}},
\code{\link[=gpFisher]{gpFisher()}}, \code{\link[=gpSphere]{gpSphere()}}, \code{\link[=gpPolar]{gpPolar()}}, \code{\link[=gpPolarHigh]{gpPolarHigh()}}, \code{\link[=gpSpike]{gpSpike()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{as.data.frame.bsimPred}
\alias{as.data.frame.bsimPred}
\title{Extract the Result Data.frame}
\usage{
\method{as.data.frame}{bsimPred}(x, ...)
}
\arguments{
\item{x}{An object of class "bsimPred" which is the result of \code{predict.bsim}.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
data.frame of prediction information.
}
\description{
Extracts fitted result that contains posterior mean/median prediction of response variable, and
standard error, credible interval of each data point in data.frame.
}
\examples{
\donttest{
data("DATA1")
simdata2 <- data.frame(DATA1$X, y = DATA1$y)
fit_one  <- BayesSIM(y ~ ., data = simdata2,
                    niter = 5000, nburnin = 1000, nchain = 1)

pred <- predict(fit_one, se.fit = TRUE, interval = "credible",level = 0.8)
results <- as.data.frame(pred)
summary(results)
}

}
\seealso{
\code{\link{predict.bsim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bsSphere.R
\name{bsSphere}
\alias{bsSphere}
\alias{bsSphere_setup}
\title{Bayesian Single-Index Regression with B-Spline Link and Half-Unit Sphere Prior}
\usage{
bsSphere(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

bsSphere_setup(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)
}
\arguments{
\item{formula}{an object of class \link{formula}. Interaction term is not allowed for single-index model.}

\item{data}{an data frame.}

\item{prior}{Optional named list of prior settings. Further descriptions are in \strong{Details} section.}

\item{init}{Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
Further descriptions are in \strong{Details} section.}

\item{monitors}{Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
then inspect the variable names stored in the model object using \link{getVarMonitor}.}

\item{niter}{Integer. Total MCMC iterations (default \code{10000}).}

\item{nburnin}{Integer. Burn-in iterations (default \code{1000}).}

\item{thin}{Integer. Thinning for monitors (default \code{1}).}

\item{nchain}{Integer. Number of MCMC chains (default \code{1}).}

\item{setSeed}{Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.}
}
\value{
A \code{list} typically containing:
\describe{
\item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
\item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
\item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
Return list of \code{model_setup} does not include it.}
\item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
\item{\code{defModel}}{Nimble model object.}
\item{\code{defSampler}}{Nimble sampler object.}
\item{\code{modelName}}{Name of the model.}
}
}
\description{
Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}
where the link \eqn{f(\cdot)} is represented by B-spline link and the
index vector \eqn{\theta} is on half-unit sphere.
}
\details{
\strong{Model} The single–index model uses a \eqn{m}-order polynomial spline with \eqn{k} interior knots and intercept as follows:
\deqn{f(t) = \sum_{j=1}^{1+m+k} B_j(t)\,\beta_j} on \eqn{[a, b]} with \eqn{t_i = X_i' \theta, i = 1,\cdots, n}
and \eqn{\|\theta\|_2 = 1}. \eqn{\{\beta_j\}_{j=1}^{m+k+1}} are spline coefficients and \eqn{a_\theta}, \eqn{ b_\theta} are boundary knots where \eqn{a_{\theta} = min(t_i, i = 1, \cdots, n)},
and \eqn{b_{\theta} = max(t_i, i = 1,\cdots, n)}. Variable selection is encoded by a binary vector \eqn{\nu}, equivalently
setting components of \eqn{\theta} to zero when \eqn{\nu_i = 0}.

\strong{Priors}
\itemize{
\item The variable selection indicator \eqn{\nu} has Beta–Bernoulli hierarchy prior. Beta hyper-prior on the Bernoulli–inclusion probability \eqn{w},
inducing \eqn{p(\nu) \propto \mathrm{Beta}(r_1 + n_\nu, r_2 + p - n_\nu)} where  \eqn{n_\nu = \Sigma_{i=1}^{p}I(\nu_i = 1)}.
\eqn{r_1, r_2} are shape and rate parameter of beta distribution.
\item Free‑knot prior: the number of knots \eqn{k} with mean \eqn{\lambda_k}. The knot locations \eqn{\xi_i, i = 1,...,k} have a Dirichlet prior on the scaled interval \eqn{[0, 1]}.
\item Index vector prior is uniform on the half‑sphere of dimension \eqn{n_\nu} with first nonzero positive.
\item Conjugate normal–inverse-gamma on \eqn{(\beta, \sigma^2)} enables analytic integration for RJMCMC with covariance \eqn{\tau \Sigma_0}.
}

\strong{Sampling} Posterior exploration follows a hybrid RJMCMC with six move types:
add/remove predictor \eqn{\nu}, update \eqn{\theta}, add/delete/relocate a knot. The \eqn{\theta} update is a random‑walk
Metropolis via local rotations in a two‑coordinate subspace. Knot changes use simple proposals with tractable acceptance ratios.
Further sampling method is described in Wang (2009).

\strong{Prior hyper-parameters}
These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
\enumerate{
\item Index vector: \code{index_nu_r1, index_nu_r2} gives the shape and rate parameter of beta-binomial prior, respectively. (default \code{index_nu_r1 = 1, index_nu_r2 = 1}).
\item Link function: B-spline knots, basis and coefficient setup.
\itemize{
\item{knots: Free-knot prior for the spline. \code{link_knots_lambda_k} is the Poisson mean for the number of
interior knot \eqn{k} (default \code{5}). \code{link_knots_maxknots} is the maximum number of interior knots.
If \code{link_knots_maxknots} is \code{NULL}, the number of interior knots is randomly drawn from a Poisson distribution.}
\item{basis: For the basis of B-spline,  \code{link_basis_degree} is the spline
degree (default \code{2}).}
\item{beta: For the coefficient of B-spline,
By default, \code{link_beta_mu} is a zero vector, \code{link_beta_tau} is set to the sample size,
and \code{link_beta_Sigma0} is the identity matrix of dimension \eqn{1 + k + m},
where \eqn{k} is the number of interior knots and \eqn{m} is the spline order.}}
\item Error variance (\code{sigma2}): Inverse gamma prior is assigned to \eqn{\sigma^2}
where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
Small values for shape and rate parameters yield a weakly-informative prior on \eqn{\sigma^{2}}. (default \code{sigma2_shape = 0.001, sigma2_rate = 0.001})
}

\strong{Initial values}
These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
\enumerate{
\item Index vector: \code{index_nu} is binary vector indicating active predictors for the index.
\code{index} is initial unit-norm index vector \eqn{\theta} (automatically normalized if necessary, with the first nonzero element made positive for identifiability).
The vector length must match the number of predictor.
Ideally, positions where \code{index_nu} has a value of 1 should correspond to nonzero elements in \eqn{\theta}; elements corresponding to \code{index_nu} = 0 will be set to zero.
\item Link function: \code{link_k} is initial number of interior knots. \code{link_knots} is initial vector of interior knot positions in \eqn{[0, 1]}, automatically scaled to the true boundary.
Length of this vector should be equal to the initial value of \code{k}.
\code{link_beta} is initial vector of spline coefficients. Length should be equal to the initial number of basis functions with intercept (\eqn{1 + k + m}).
\item Error variance (\code{sigma2}): Initial scalar error variance. (default \code{0.01})
}
}
\examples{
\donttest{
set.seed(123)
n <- 200; d <- 4
theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
f <- function(u) u^2 * exp(u)
sigma <- 0.5
X <- matrix(runif(n * d, -1, 1), nrow = n)
index_vals <- as.vector(X \%*\% theta)
y <- f(index_vals) + rnorm(n, 0, sigma)
simdata <- data.frame(x = X, y = y)
colnames(simdata) <- c(paste0("X", 1:4), "y")

# One tool version
fit1 <- bsSphere(y ~ ., data = simdata,
                 niter = 5000, nburnin = 1000, nchain = 1)

# Split version
models <- bsSphere_setup(y ~ ., data = simdata)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                   nchains = 1, setSeed = TRUE, inits = initList,
                   samplesAsCodaMCMC = TRUE)
fit2 <- as_bsim(models, mcmc.out)
summary(fit2)
}

}
\references{
Wang, H.-B. (2009). Bayesian estimation and variable selection for single index models.
\emph{Computational Statistics & Data Analysis}, 53, 2617–2627.

Hornik, K., & Grün, B. (2014). movMF: an R package for fitting mixtures of von Mises-Fisher distributions.
\emph{Journal of Statistical Software}, 58, 1-31.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesSIM-package.R, R/zzz.R
\docType{package}
\name{BayesSIM-package}
\alias{BayesSIM-package}
\title{BayesSIM: Integrated Interface of Bayesian Single Index Models using 'nimble'}
\description{
Provides tools for fitting Bayesian single index models with flexible choices of priors for both the index and the link function. The package implements model estimation and posterior inference using efficient MCMC algorithms built on the 'nimble' framework, allowing users to specify, extend, and simulate models in a unified and reproducible manner. The following methods are implemented in the package: Antoniadis et al. (2004) \url{https://www.jstor.org/stable/24307224}, Wang (2009) \doi{10.1016/j.csda.2008.12.010}, Choi et al. (2011) \doi{10.1080/10485251003768019}, Dhara et al. (2019) \doi{10.1214/19-BA1170}, McGee et al. (2023) \doi{10.1111/biom.13569}.

Provides tools for fitting Bayesian single index models with flexible choices of priors for both the index and the link function. The package implements model estimation and posterior inference using efficient MCMC algorithms built on the 'nimble' framework, allowing users to specify, extend, and simulate models in a unified and reproducible manner. The following methods are implemented in the package: Antoniadis et al. (2004) \url{https://www.jstor.org/stable/24307224}, Wang (2009) \doi{10.1016/j.csda.2008.12.010}, Choi et al. (2011) \doi{10.1080/10485251003768019}, Dhara et al. (2019) \doi{10.1214/19-BA1170}, McGee et al. (2023) \doi{10.1111/biom.13569}.
}
\author{
\strong{Maintainer}: Seowoo Jung \email{jsw1347@ewha.ac.kr}

Authors:
\itemize{
  \item Seowoo Jung \email{jsw1347@ewha.ac.kr}
  \item Eun-kyung Lee \email{lee.eunk@ewha.ac.kr}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpSphere-united.R
\name{gpSphere}
\alias{gpSphere}
\alias{gpSphere_setup}
\title{Bayesian Single-Index Regression with Gaussian Process Link and Unit Sphere Prior}
\usage{
gpSphere(
  formula,
  data,
  prior = NULL,
  init = NULL,
  method = "FB",
  lowerB = NULL,
  upperB = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

gpSphere_setup(
  formula,
  data,
  prior = NULL,
  init = NULL,
  method = "FB",
  lowerB = NULL,
  upperB = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)
}
\arguments{
\item{formula}{an object of class \link{formula}. Interaction term is not allowed for single-index model.}

\item{data}{an data frame.}

\item{prior}{Optional named list of prior settings. Further descriptions are in \strong{Details} section.}

\item{init}{Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
Further descriptions are in \strong{Details} section.}

\item{method}{Character, \code{gpSphere} model has 3 different types of sampling method, fully Bayesian method (\code{"FB"}), empirical Bayes approach (\code{"EB"}), and empirical Gibbs sampler (\code{"EG"}).
Assign one sampler method. Empirical sampling approach is recommended in high-dimensional data. By default, fully Bayesian approach is assigned.}

\item{lowerB}{This parameter is only for \code{gpSphere} model. Numeric vector of element-wise lower bounds for the \code{"L-BFGS-B"} method.
When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
By default, the lower bounds are \code{-1} for the index vector and \code{-1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.}

\item{upperB}{This parameter is only for \code{gpSphere} model. Numeric vector of element-wise upper bounds for the \code{"L-BFGS-B"} method.
When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
By default, the upper bounds are \code{1} for the index vector and \code{1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.}

\item{monitors}{Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
then inspect the variable names stored in the model object using \link{getVarMonitor}.}

\item{niter}{Integer. Total MCMC iterations (default \code{10000}).}

\item{nburnin}{Integer. Burn-in iterations (default \code{1000}).}

\item{thin}{Integer. Thinning for monitors (default \code{1}).}

\item{nchain}{Integer. Number of MCMC chains (default \code{1}).}

\item{setSeed}{Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.}
}
\value{
A \code{list} typically containing:
\describe{
\item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
\item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
\item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
Return list of \code{model_setup} does not include it.}
\item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
\item{\code{defModel}}{Nimble model object.}
\item{\code{defSampler}}{Nimble sampler object.}
\item{\code{modelName}}{Name of the model.}
}
}
\description{
Fits a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}, where
the index \eqn{\theta} lies on the unit sphere, and the link \eqn{f(\cdot)} is represented
with Gaussian process.
}
\details{
\strong{Model} The single-index model uses Gaussian process with zero mean and and covariance kernel \eqn{\eta \cdot \text{exp}(-\frac{(t_i-t_j)^2}{l})} as a link function, where \eqn{t_i, t_j, j = 1, \ldots, n} is index value.
Index vector should be length 1.

\strong{Priors}
\itemize{
\item von Mises–Fisher prior on the index \eqn{\theta} with direction and concentration.
\item Covariance kernel: Amplitude, \eqn{\eta}, follows log normal distribution with mean \eqn{a_\eta} and variance \eqn{b_\eta}.
Length-scale parameter follows gamma distribution with shape parameter \eqn{\alpha_l} and rate parameter \eqn{\beta_l}.
\item Inverse-Gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.

}

\strong{Sampling} In the fully Bayesian approach, \eqn{\theta}, \eqn{l}, and \eqn{\eta}
are updated via the Metropolis–Hastings algorithm, while \eqn{f} and
\eqn{\sigma^2} are sampled using Gibbs sampling.

In the empirical Bayes approach, \eqn{\theta}, \eqn{l}, \eqn{\eta},
and \eqn{\sigma^2} are estimated by maximum a posteriori (MAP), and
\eqn{f} is sampled from its full conditional posterior distribution.

In the empirical Gibbs sampler, \eqn{\theta}, \eqn{l}, and \eqn{\eta}
are estimated by MAP, whereas \eqn{f} and \eqn{\sigma^2} are sampled
via Gibbs sampling.

For estimation via MAP, effective sample size or potential scale reduction factor is meaningless.

\strong{Prior hyper-parameters}
These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
\enumerate{
\item Index vector: Nothing to assign.
\item Link function:
\itemize{
\item{Length-scale:Gamma distribution is assigned for length-scale parameter, \eqn{l}.
\code{link_lengthscale_shape} is shape parameter (default \code{1/8}) and \code{link_lengthscale_rate} is rate parameter of \code{lengthscale}. (default \code{1/8})}
\item{Amplitude: Log-normal distribution is assigned for amplitude parameter, \eqn{\eta}.
\code{link_amp_a} is mean (default \code{-1}), and \code{link_amp_b} is variance. (default \code{1})}
}

\item Error variance (\code{sigma2}): inverse gamma prior is assigned to \eqn{\sigma^2}
where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
(default \code{sigma2_shape = 1, sigma2_rate = 1})

}

\strong{Initial values}
These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
\enumerate{
\item Index vector (\code{index}): Initial unit index vector. By default, vector is randomly drawn from normal distribution and standardized.
\item Link function: \code{link_lengthscale} is initial scalar length-scale parameter. (default: \code{0.1})
\code{link_amp} is initial scalar amplitude parameter. (default: \code{1})
\item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{1})

}
}
\examples{
\donttest{
set.seed(123)
n <- 200; d <- 4
theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
f <- function(u) u^2 * exp(u)
sigma <- 0.5
X <- matrix(runif(n * d, -1, 1), nrow = n)
index_vals <- as.vector(X \%*\% theta)
y <- f(index_vals) + rnorm(n, 0, sigma)
simdata <- data.frame(x = X, y = y)
colnames(simdata) <- c(paste0("X", 1:4), "y")

# One tool version
fit1 <- gpSphere(y ~ ., method = "EB", data = simdata,
                 niter = 1000, nburnin = 100)

# Split version
models <- gpSphere_setup(y ~ ., method = "EB", data = simdata)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 1000, nburnin = 100, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)
fit2 <- as_bsim(models, mcmc.out)
# The estimates computed by MAP - standard error of the esitmate is meaningless.
summary(fit2)
}
}
\references{
Choi, T., Shi, J. Q., & Wang, B. (2011).
A Gaussian process regression approach to a single-index model.
\emph{Journal of Nonparametric Statistics}, 23(1), 21-36.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bsSpike.R
\name{bsSpike}
\alias{bsSpike}
\alias{bsSpike_setup}
\title{Bayesian Single-Index Regression with B-Spline Link and Spike-and-Slab Prior}
\usage{
bsSpike(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

bsSpike_setup(
  formula,
  data,
  prior = NULL,
  init = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)
}
\arguments{
\item{formula}{an object of class \link{formula}. Interaction term is not allowed for single-index model.}

\item{data}{an data frame.}

\item{prior}{Optional named list of prior settings. Further descriptions are in \strong{Details} section.}

\item{init}{Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
Further descriptions are in \strong{Details} section.}

\item{monitors}{Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
then inspect the variable names stored in the model object using \link{getVarMonitor}.}

\item{niter}{Integer. Total MCMC iterations (default \code{10000}).}

\item{nburnin}{Integer. Burn-in iterations (default \code{1000}).}

\item{thin}{Integer. Thinning for monitors (default \code{1}).}

\item{nchain}{Integer. Number of MCMC chains (default \code{1}).}

\item{setSeed}{Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.}
}
\value{
A \code{list} typically containing:
\describe{
\item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
\item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
\item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
Return list of \code{model_setup} does not include it.}
\item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
\item{\code{defModel}}{Nimble model object.}
\item{\code{defSampler}}{Nimble sampler object.}
\item{\code{modelName}}{Name of the model.}
}
}
\description{
Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}
where the link \eqn{f(\cdot)} is represented by B-spline link function and the
index vector \eqn{\theta} has spike-and-slab prior.
}
\details{
\strong{Model} The single–index model uses a \eqn{m}-order polynomial spline with \eqn{k} interior knots as follows:
\deqn{f(t) = \sum_{j=1}^{m+k} B_j(t)\,\beta_j} on \eqn{[a, b]} with \eqn{t_i = X_i' \theta, i = 1,\cdots, n}
and \eqn{\|\theta\|_2 = 1}. \eqn{\{\beta_j\}_{j=1}^{m+k}} are spline coefficient and \eqn{a_\theta} and \eqn{ b_\theta} are boundary knots where \eqn{a_\theta = min(t_i, i = 1, \cdots, n) - \delta},
and \eqn{b_\theta = max(t_i, i = 1,\cdots, n) + \delta}. \eqn{\theta} is a p-dimensional index vector subject to a spike-and-slab
prior for variable selection with binary indicator variable \eqn{\nu}.

\strong{Priors}
\itemize{
\item The variable selection indicator \eqn{\nu} has Beta–Bernoulli hierarchy prior. Beta hyper-prior on the Bernoulli–inclusion probability \eqn{w},
inducing \eqn{p(\nu) \propto \mathrm{Beta}(r_1 + n_\nu, r_2 + p - n_\nu)} where  \eqn{n_\nu = \Sigma_{i=1}^{p}I(\nu_i = 1)}.
\eqn{r_1, r_2} are shape and rate parameter of beta distribution.
\item Slab coefficients \eqn{\theta} have normal distribution with zero mean and \eqn{\sigma_\theta^2} variance.
\item Conditioned on \eqn{\theta} and \eqn{\sigma^2}, the link coefficients \eqn{\beta = (\beta_1,\ldots,\beta_{m+k})^\top} follow
normal distribution with estimated mean vector \eqn{\hat{\beta}_{\theta} = (X_{\theta}'X_{\theta})^{-1}X_{\theta}'Y} and
covariance \eqn{\sigma^2 (X_{\theta}^\top X_{\theta})^{-1}}, where \eqn{X_{\theta}} is the B-spline basis design matrix.
\item Inverse gamma prior on \eqn{\sigma^2} with shape parameter \eqn{a_\sigma} and rate parameter \eqn{b_\sigma}.
}

\strong{Sampling}
Samplers are automatically assigned by nimble.

\strong{Prior hyper-parameters}
These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
\enumerate{
\item Index vector: \code{index_nu_r1, index_nu_r2} gives the shape and rate parameter of beta-binomial prior, respectively.
For slab prior, normal distribution with zero mean is assigned for selected variables \eqn{\theta}. \code{index_sigma_theta} is for variance of \eqn{\theta}, and it is assigned 0.25 by default.
\item Link function: B-spline basis and coefficient of B-spline setup.
\itemize{
\item{basis: For the basis of B-spline, \code{link_basis_df} is the number of basis functions (default \code{21}), \code{link_basis_degree} is the spline degree (default \code{2}) and \code{link_basis_delta} is a small jitter for boundary-knot spacing control (default \code{0.01}).}
\item{beta: For the coefficient of B-spline, multivariate normal prior is assigned with mean \code{link_beta_mu}, and covariance \code{link_beta_cov}. By default, \eqn{\mathcal{N}_p\!\big(0, \mathrm{I}_p\big)}}
}
\item Error variance (\code{sigma2}):  Inverse gamma prior is assigned to \eqn{\sigma^2}
where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
(default \code{sigma2_shape = 0.001, sigma2_rate = 100})
}

\strong{Initial values}
These are the initial values set in the function. You can define new values for each initial value in \link{init_param}.
\enumerate{
\item Index vector:
\itemize{
\item{\code{index_pi} Initial selecting variable probability. (default: \code{0.5})}
\item{\code{index_nu} Initial vector of inclusion indicators . By default, each nu is randomly drawn by  \eqn{Bernoulli(1/2)}}
\item{\code{index} Initial vector of index. By default, each element of index vector, which is chosen by \eqn{\nu}, is proposed by normal distribution.}
}
\item Link function: Initial spline coefficients (\code{link_beta}). By default,
\eqn{\big(X_{\theta}^\top X_{\theta} + \rho\, \mathrm{I}\big)^{-1} X_{\theta}^\top Y} is computed,
where \eqn{X_{\theta}} is the B-spline basis design matrix.
\item Error variance (\code{sigma2}): Initial scalar error variance (default \code{0.01}).

}
}
\examples{
\donttest{
set.seed(123)
n <- 200; d <- 4
theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
f <- function(u) u^2 * exp(u)
sigma <- 0.5
X <- matrix(runif(n * d, -1, 1), nrow = n)
index_vals <- as.vector(X \%*\% theta)
y <- f(index_vals) + rnorm(n, 0, sigma)
simdata <- data.frame(x = X, y = y)
colnames(simdata) <- c(paste0("X", 1:4), "y")

# One tool version
fit1 <- bsSpike(y ~ ., data = simdata,
                niter = 5000, nburnin = 1000, nchain = 1)

# Split version
models <- bsSpike_setup(y ~ ., data = simdata)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                   nchains = 1, setSeed = TRUE, inits = initList,
                   summary = TRUE, samplesAsCodaMCMC = TRUE)
fit2 <- as_bsim(models, mcmc.out)
summary(fit2)
}

}
\references{
Antoniadis, A., Grégoire, G., & McKeague, I. W. (2004).
Bayesian estimation in single-index models. \emph{Statistica Sinica}, 1147-1164.

Hornik, K., & Grün, B. (2014). movMF: an R package for fitting mixtures of von Mises-Fisher distributions.
\emph{Journal of Statistical Software}, 58, 1-31.

McGee, G., Wilson, A., Webster, T. F., & Coull, B. A. (2023).
Bayesian multiple index models for environmental mixtures.
\emph{Biometrics}, 79(1), 462-474.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{getFunction}
\alias{getFunction}
\alias{getModel}
\alias{getSampler}
\title{Get nimbleModel and nimbleSampler Object from the Result of \code{compileModelAndMCMC}}
\usage{
getModel(object)

getSampler(object)
}
\arguments{
\item{object}{The result of \code{compileModelAndMCMC} function.}
}
\value{
\code{getModel} returns compiled Nimble model object.
\code{getSampler} returns compiled Nimble sampler object, directly using in \code{runMCMC} function.
}
\description{
Return compiled nimble model object and a corresponding MCMC samplers.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

# 1. One tool version
fit_one <- BayesSIM(y ~ ., data = simdata2,
                    niter = 5000, nburnin = 1000, nchain = 1)


# 2. Split version
models <- BayesSIM_setup(y ~ ., data = simdata2)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)

# "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
fit_split <- as_bsim(models, mcmc.out)

}
}
\seealso{
\code{\link[nimble]{nimbleModel}},
\code{\link[nimble]{configureMCMC}},
\code{\link[nimble]{buildMCMC}},
\code{\link[nimble]{compileNimble}},
\code{\link[nimble]{runMCMC}}
}

\name{concrete}
\alias{concrete}
\docType{data}
\title{UCI Concrete Compressive Strength (n = 1030, p = 8)}
\usage{data(concrete)}
\format{
Numeric data.frame of size 1030 \eqn{\times} 8.
\describe{
\item{cement}{Numeric. Cement content (kg/m\eqn{^3}).}
  \item{blast_furnace_slag}{Numeric. Blast furnace slag (kg/m\eqn{^3}).}
  \item{fly_ash}{Numeric. Fly ash (kg/m\eqn{^3}).}
  \item{water}{Numeric. Mixing water (kg/m\eqn{^3}).}
  \item{superplasticizer}{Numeric. Superplasticizer (kg/m\eqn{^3}).}
  \item{coarse_aggreate}{Numeric. Coarse aggregate (kg/m\eqn{^3}).}
  \item{fine_aggregate}{Numeric. Fine aggregate (kg/m\eqn{^3}).}
  \item{age}{Numeric. Curing age (days; typically 1--365).}
  \item{strength}{Numeric. Concrete compressive strength (MPa).}
}
}
\description{
Concrete compressive strength dataset from the UCI Machine Learning Repository. No missing variables and there are 8 quantitative inputs and 1 quantitative output.
}
\details{
\strong{Source}
Concrete Compressive Strength in UCI Machine Learning Repository. This data is integrated by experimental data from 17 different sources to check the realiability of the strength. This dataset compiles experimental concrete mixes from multiple studies and is used to predict compressive strength and quantify how mixture ingredients and curing age affect that strength.

\strong{Variables.}
\itemize{
  \item Cement, Blast Furnace Slag, Fly Ash, Water, Superplasticizer, Coarse Aggregate, Fine Aggregate: quantities in kg per m\eqn{^3} of mixture.
  \item Age: curing time in days (1--365).
  \item Target(strength): compressive strength in MPa.
}

}
\references{
Yeh, I. (1998). Concrete Compressive Strength [Dataset]. UCI Machine Learning Repository. https://doi.org/10.24432/C5PK67.

Yeh, I. (1998). Modeling of strength of high-performance concrete using artificial neural networks. Cement and Concrete research, 28(12), 1797-1808.
}
\examples{
data(concrete)
str(concrete)
plot(density(concrete$strength), main = "Concrete compressive strength (MPa)")
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generics.R
\name{predict.bsim}
\alias{predict.bsim}
\alias{print.bsimPred}
\title{Prediction Method for \code{BayesSIM}}
\usage{
\method{predict}{bsim}(
  object,
  newdata = NULL,
  se.fit = FALSE,
  type = c("response", "latent", "index"),
  method = c("mean", "median"),
  interval = c("none", "credible"),
  level = 0.95,
  ...
)

\method{print}{bsimPred}(x, ...)
}
\arguments{
\item{object}{A fitted object of \code{BayesSIM} or individual model.}

\item{newdata}{Optional data frame for which predictions should be made.
If \code{NULL}, predictions are returned for the training data.}

\item{se.fit}{A logical value indicating whether standard errors are required.
Default is \code{FALSE}.}

\item{type}{Character string specifying the type of prediction. \code{"response"}
is the default.
\describe{
\item{\code{"response"}}{
Posterior predictive summaries of the response \eqn{Y}.
This corresponds to draws from the posterior predictive distribution
\eqn{Y^{(m)} \sim N(f(X'\theta^{(m)}), \sigma^{2(m)})}
and therefore incorporates both the uncertainty in the link function
and the variability of the error term for each \eqn{m^{th}} MCMC sample.}
\item{\code{"latent"}}{
Posterior summaries of the latent mean structure
\eqn{E(Y \mid X) = f^{(m)}(t^{(m)})}, where \eqn{t^{(m)} = X'\theta^{(m)}}.
Unlike \code{"response"}, it excludes the noise term and calculated by
\eqn{f^{(m)}(X'\theta^{(m)})} for each \eqn{m^{th}} MCMC sample of
\eqn{\theta}.}
\item{\code{"index"}}{
Posterior summaries of the single index \eqn{t^{(m)} = X'\theta^{(m)}}.}
}}

\item{method}{Character string determining the posterior summary used for
point predictions. Options are \code{"mean"} or \code{"median"}. Default is \code{"mean"}.}

\item{interval}{Character string indicating whether a credible interval should be returned. Default is \code{"none"}.
\describe{
\item{\code{"none"}}{Return only point predictions.}
\item{\code{"credible"}}{Return a \eqn{100 \times \text{level}\%} credible interval.}
}}

\item{level}{Numeric value between 0 and 1 specifying the credible level.
\code{level = 0.95} yields a 95\% credible interval. Default is \code{0.95}.}

\item{...}{Additional arguments.}

\item{x}{An object of class "bsimPred" which is the result of \code{predict}.}
}
\value{
A list containing at least the following components:
\describe{
\item{\code{fitted}}{Posterior mean or median predictions. If \code{se.fit = TRUE} or
\code{interval = "credible"}, standard error and lower, upper bounds of the credible interval is also included.}
\item{\code{truey}}{True response value of test data. When true response value is not available, \code{NULL} is saved.}
\item{\code{idxValue}}{Linear index value is saved where \eqn{\theta} is estimated by \code{method}.}
\item{\code{level}}{Credible level.}
}
}
\description{
Generate predictions from a fitted Bayesian single-index model.
}
\details{
This method extracts MCMC posterior samples stored in a BayesSIM object
and computes posterior summaries of:
\itemize{
\item the \strong{posterior predictive response} \eqn{Y \mid X} (type \code{"response"}),
\item the \strong{latent link function} evaluation \eqn{E(Y \mid X) = f(X'\theta)} (type \code{"latent"}), or
\item the \strong{single index} \eqn{X'\theta} (type \code{"index"}).
}

The key distinction is that \code{"response"} incorporates the posterior
variability of the error term \eqn{\epsilon}, whereas \code{"latent"} represents
the noiseless conditional expectation \eqn{E(Y \mid X)} computed directly
from the link function and the posterior draws of \eqn{\theta}.

When \code{interval = "credible"}, the returned object includes lower and upper
credible bounds computed via posterior quantiles for the chosen prediction
scale.

If \code{newdata} is supplied, predictions are evaluated at the new covariate
values by computing the corresponding posterior index \eqn{t = X'\theta}
and applying the link function.
}
\examples{
\donttest{
simdata2 <- data.frame(DATA1$X, y = DATA1$y)

# 1. One tool version
fit_one <- BayesSIM(y ~ ., data = simdata2,
                    niter = 5000, nburnin = 1000, nchain = 1)

# Check median index vector estimates with standard errors
coef(fit_one, method = "median", se = TRUE)

# Fitted index values of median prediction
fitted(fit_one, type = "linpred", method = "median")

# Residuals of median prediction
residuals(fit_one, method = "median")

# Summary of the model
summary(fit_one)

# Convergence diagnostics
nimTraceplot(fit_one)

# Goodness of fit
GOF(fit_one)

# Fitted plot
plot(fit_one)

# Prediction with 95\% credible interval at new data
newx <- data.frame(X1 = rnorm(10), X2 = rnorm(10), X3 = rnorm(10), X4 = rnorm(10))
pred <- predict(fit_one, newdata = newx, interval = "credible", level = 0.95)
plot(pred)


# 2. Split version
models <- BayesSIM_setup(y ~ ., data = simdata2)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                    nchains = 1, setSeed = TRUE, inits = initList,
                    summary = TRUE, samplesAsCodaMCMC = TRUE)

# "fit_split" becomes exactly the same as the class of "fit_one" object and apply generic functions.
fit_split <- as_bsim(models, mcmc.out)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayesSIM.R, R/generics.R
\name{BayesSIM}
\alias{BayesSIM}
\alias{BayesSIM_setup}
\alias{print.bsim}
\alias{print.bsimSetup}
\title{Integrated Function for Bayesian Single-Index Regression}
\usage{
BayesSIM(
  formula,
  data,
  indexprior = "fisher",
  link = "bspline",
  prior = NULL,
  init = NULL,
  method = "FB",
  lowerB = NULL,
  upperB = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

BayesSIM_setup(
  formula,
  data,
  indexprior = "fisher",
  link = "bspline",
  prior = NULL,
  init = NULL,
  method = "FB",
  lowerB = NULL,
  upperB = NULL,
  monitors = NULL,
  niter = 10000,
  nburnin = 1000,
  thin = 1,
  nchain = 1,
  setSeed = FALSE
)

\method{print}{bsim}(x, digits = 3, ...)

\method{print}{bsimSetup}(x, digits = 3, ...)
}
\arguments{
\item{formula}{an object of class \link{formula}. Interaction term is not allowed for single-index model.}

\item{data}{an data frame.}

\item{indexprior}{Index vector prior among \code{"fisher"} (default), \code{"sphere"}, \code{"polar"}, \code{"spike"}.}

\item{link}{Link function among \code{"bspline"} (default), \code{"gp"}}

\item{prior}{Optional named list of prior settings. Further descriptions are in every specific model's \strong{Details} section.}

\item{init}{Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior or designated value.
Further descriptions are in every specific model's \strong{Details} section.}

\item{method}{Character, \code{gpSphere} model has 3 different types of sampling method, fully Bayesian method (\code{"FB"}), empirical Bayes approach (\code{"EB"}), and empirical Gibbs sampler (\code{"EG"}).
Assign one sampler method. Empirical sampling approach is recommended in high-dimensional data. By default, fully Bayesian approach is assigned.}

\item{lowerB}{This parameter is only for \code{gpSphere} model. Numeric vector of element-wise lower bounds for the \code{"L-BFGS-B"} method.
When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
By default, the lower bounds are \code{-1} for the index vector and \code{-1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.}

\item{upperB}{This parameter is only for \code{gpSphere} model. Numeric vector of element-wise upper bounds for the \code{"L-BFGS-B"} method.
When the empirical Bayes or Gibbs sampler method is used, the marginal likelihood is optimized via \code{optim(method = "L-BFGS-B")}.
The vector must be ordered as \code{c(index vector, lengthscale, amp, sigma2)}. Note that \code{sigma2} is included only for the empirical Bayes method (omit it for Gibbs).
By default, the upper bounds are \code{1} for the index vector and \code{1e2} for logarithm of \code{lengthscale}, \code{amp}, and (if present) \code{sigma2}.}

\item{monitors}{Optional character vector of additional monitor nodes. To check the names of the nodes, fit the \code{model_setup} function and
then inspect the variable names stored in the model object using \link{getVarMonitor}.}

\item{niter}{Integer. Total MCMC iterations (default \code{10000}).}

\item{nburnin}{Integer. Burn-in iterations (default \code{1000}).}

\item{thin}{Integer. Thinning for monitors (default \code{1}).}

\item{nchain}{Integer. Number of MCMC chains (default \code{1}).}

\item{setSeed}{Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC} \code{setSeed} argument.}

\item{x}{A fitted \code{BayesSIM} object.}

\item{digits}{Number of digits to display.}

\item{...}{Additional arguments.}
}
\value{
A \code{list} typically containing:
\describe{
\item{\code{coefficients}}{Mean estimates of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{ses}}{Mean standard error of index vector. Return list of \code{model_setup} does not include it.}
\item{\code{residuals}}{Residuals with mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{fitted.values}}{Mean estimates of fitted values. Return list of \code{model_setup} does not include it.}
\item{\code{linear.predictors}}{Mean estimates of single-index values. Return list of \code{model_setup} does not include it.}
\item{\code{gof}}{Goodness-of-fit. Return list of \code{model_setup} function does not include it.}
\item{\code{samples}}{Posterior draws of variables for computing fitted values of the model, including \eqn{\theta}, \eqn{\sigma^2}.
Return list of \code{model_setup} does not include it.}
\item{\code{input}}{List of data used in modeling, formula, input values for prior and initial values, and computation time without compiling.}
\item{\code{defModel}}{Nimble model object.}
\item{\code{defSampler}}{Nimble sampler object.}
\item{\code{modelName}}{Name of the model.}
}
}
\description{
Fitting a single–index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n} in single integrated function.
}
\details{
Integrated function for Bayesian single-index model. Default model is von-Mises Fisher distribution for index vector with B-spline link function.
}
\examples{
\donttest{
set.seed(123)
n <- 200; d <- 4
theta <- c(2, 1, 1, 1); theta <- theta / sqrt(sum(theta^2))
f <- function(u) u^2 * exp(u)
sigma <- 0.5
X <- matrix(runif(n * d, -1, 1), nrow = n)
index_vals <- as.vector(X \%*\% theta)
y <- f(index_vals) + rnorm(n, 0, sigma)
simdata <- data.frame(x = X, y = y)
colnames(simdata) <- c(paste0("X", 1:4), "y")

# One tool version - bsFisher
fit1 <- BayesSIM(y ~ ., data = simdata,
                 niter = 5000, nburnin = 1000,
                 nchain = 1)

# Split version- bsFisher
models <- BayesSIM_setup(y ~ ., data = simdata)
Ccompile <- compileModelAndMCMC(models)
nimSampler <- getSampler(Ccompile)
initList <- getInit(models)
mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
                   nchains = 1, setSeed = TRUE, inits = initList,
                   summary = TRUE, samplesAsCodaMCMC = TRUE)
fit2 <- as_bsim(models, mcmc.out)
summary(fit2)
}
}
\seealso{
\code{\link[=bsFisher]{bsFisher()}}, \code{\link[=bsSphere]{bsSphere()}}, \code{\link[=bsPolar]{bsPolar()}}, \code{\link[=bsSpike]{bsSpike()}},
\code{\link[=gpFisher]{gpFisher()}}, \code{\link[=gpSphere]{gpSphere()}}, \code{\link[=gpPolar]{gpPolar()}}, \code{\link[=gpPolarHigh]{gpPolarHigh()}}, \code{\link[=gpSpike]{gpSpike()}}
}

\name{DATA1}
\alias{DATA1}
\docType{data}
\title{Simulated Single–Index Data (n = 200, p = 4)}
\usage{data(DATA1)}
\format{
\describe{
  \item{X}{Numeric matrix of size 200 \eqn{\times} 4, entries i.i.d. \eqn{Unif(-1, 1)}}.
  \item{y}{Numeric vector of length 200.}
}
}
\description{
Synthetic data from a single–index model \eqn{y = f(X'\theta) + \varepsilon}
with \eqn{f(u) = u^2 \exp(u)} and \eqn{\varepsilon \sim N(0,\sigma^2)}.
The index vector is \eqn{\theta = (2,1,1,1) / \| (2,1,1,1) \|_2} and
\eqn{\sigma = 0.5}.
}
\examples{
data(DATA1)
str(DATA1)
}
\keyword{datasets}
